@@ -34,6 +34,7 @@
 #include "graphics/TerritoryTexture.h"
 #include "graphics/Unit.h"
 #include "graphics/UnitManager.h"
+#include "graphics/scripting/JSInterface_GameView.h"
 #include "lib/input.h"
 #include "lib/timer.h"
 #include "maths/BoundingBoxAligned.h"
@@ -54,10 +55,10 @@
 #include "ps/World.h"
 #include "renderer/Renderer.h"
 #include "renderer/WaterManager.h"
-#include "scripting/ScriptableObject.h"
 #include "simulation2/Simulation2.h"
 #include "simulation2/components/ICmpPosition.h"
 #include "simulation2/components/ICmpRangeManager.h"
+#include "lobby/IXmppClient.h"
 
 extern int g_xres, g_yres;
 
@@ -148,7 +149,7 @@ public:
 	float m_Smoothness;
 };
 
-class CGameViewImpl : public CJSObject<CGameViewImpl>
+class CGameViewImpl
 {
 	NONCOPYABLE(CGameViewImpl);
 public:
@@ -308,10 +309,25 @@ public:
 	CSmoothedValue Zoom;
 	CSmoothedValue RotateX; // inclination around x axis (relative to camera)
 	CSmoothedValue RotateY; // rotation around y (vertical) axis
-
-	static void ScriptingInit();
 };
 
+#define IMPLEMENT_BOOLEAN_SETTING(NAME) \
+bool CGameView::Get##NAME##Enabled() \
+{ \
+	return m->NAME; \
+} \
+\
+void CGameView::Set##NAME##Enabled(bool Enabled) \
+{ \
+	m->NAME = Enabled; \
+}
+
+IMPLEMENT_BOOLEAN_SETTING(Culling);
+IMPLEMENT_BOOLEAN_SETTING(LockCullCamera);
+IMPLEMENT_BOOLEAN_SETTING(ConstrainCamera);
+
+#undef IMPLEMENT_BOOLEAN_SETTING
+
 static void SetupCameraMatrixSmooth(CGameViewImpl* m, CMatrix3D* orientation)
 {
 	orientation->SetIdentity();
@@ -372,16 +388,6 @@ CObjectManager& CGameView::GetObjectManager() const
 	return m->ObjectManager;
 }
 
-JSObject* CGameView::GetScript()
-{
-	return m->GetScript();
-}
-
-/*static*/ void CGameView::ScriptingInit()
-{
-	return CGameViewImpl::ScriptingInit();
-}
-
 CCamera* CGameView::GetCamera()
 {
 	return &m->ViewCamera;
@@ -402,16 +408,6 @@ CTerritoryTexture& CGameView::GetTerritoryTexture()
 	return m->TerritoryTexture;
 }
 
-
-void CGameViewImpl::ScriptingInit()
-{
-	AddProperty(L"culling", &CGameViewImpl::Culling);
-	AddProperty(L"lockCullCamera", &CGameViewImpl::LockCullCamera);
-	AddProperty(L"constrainCamera", &CGameViewImpl::ConstrainCamera);
-
-	CJSObject<CGameViewImpl>::ScriptingInit("GameView");
-}
-
 int CGameView::Initialize()
 {
 	CFG_GET_VAL("view.scroll.speed", Float, m->ViewScrollSpeed);
@@ -968,6 +964,54 @@ float CGameView::GetCameraZ()
 	return pivot.Z;
 }
 
+float CGameView::GetCameraPosX()
+{
+	return m->PosX.GetValue();
+}
+
+float CGameView::GetCameraPosY()
+{
+	return m->PosY.GetValue();
+}
+
+float CGameView::GetCameraPosZ()
+{
+	return m->PosZ.GetValue();
+}
+
+float CGameView::GetCameraRotX()
+{
+	return m->RotateX.GetValue();
+}
+
+float CGameView::GetCameraRotY()
+{
+	return m->RotateY.GetValue();
+}
+
+float CGameView::GetCameraZoom()
+{
+	return m->Zoom.GetValue();
+}
+
+void CGameView::SetCamera(CVector3D Pos, float RotX, float RotY, float zoom)
+{
+	m->PosX.SetValue(Pos.X);
+	m->PosY.SetValue(Pos.Y);
+	m->PosZ.SetValue(Pos.Z);
+	m->RotateX.SetValue(RotX);
+	m->RotateY.SetValue(RotY);
+	m->Zoom.SetValue(zoom);
+
+	FocusHeight(m, false);
+
+	SetupCameraMatrixNonSmooth(m, &m->ViewCamera.m_Orientation);
+	m->ViewCamera.UpdateFrustum();
+
+	// Break out of following mode so the camera really moves to the target
+	m->FollowEntity = INVALID_ENTITY;
+}
+
 void CGameView::MoveCameraTarget(const CVector3D& target)
 {
 	// Maintain the same orientation and level of zoom, if we can
@@ -1089,7 +1133,9 @@ InReaction CGameView::HandleEvent(const SDL_Event_* ev)
 
 		if (hotkey == "wireframe")
 		{
-			if (g_Renderer.GetModelRenderMode() == SOLID)
+			if (g_XmppClient && g_rankedGame == true)
+				break;
+			else if (g_Renderer.GetModelRenderMode() == SOLID)
 			{
 				g_Renderer.SetTerrainRenderMode(EDGED_FACES);
 				g_Renderer.SetModelRenderMode(EDGED_FACES);
