@@ -30,7 +30,7 @@ class CCinemaManager;
 class CVector3D;
 struct SViewPort;
 
-struct JSObject;
+class JSObject;
 
 class CGameViewImpl;
 
@@ -84,6 +84,13 @@ public:
 
 	float GetCameraX();
 	float GetCameraZ();
+	float GetCameraPosX();
+	float GetCameraPosY();
+	float GetCameraPosZ();
+	float GetCameraRotX();
+	float GetCameraRotY();
+	float GetCameraZoom();
+	void SetCamera(CVector3D Pos, float RotX, float RotY, float Zoom);
 	void MoveCameraTarget(const CVector3D& target);
 	void ResetCameraTarget(const CVector3D& target);
 	void ResetCameraAngleZoom();
@@ -96,6 +103,16 @@ public:
 	float GetFar() const;
 	float GetFOV() const;
 	float GetCullFOV() const;
+	
+	#define DECLARE_BOOLEAN_SETTING(NAME) \
+	bool Get##NAME##Enabled(); \
+	void Set##NAME##Enabled(bool Enabled);
+
+	DECLARE_BOOLEAN_SETTING(Culling);
+	DECLARE_BOOLEAN_SETTING(LockCullCamera);
+	DECLARE_BOOLEAN_SETTING(ConstrainCamera);
+
+	#undef DECLARE_BOOLEAN_SETTING
 
 	// Set projection of current camera using near, far, and FOV values
 	void SetCameraProjection();
@@ -104,8 +121,6 @@ public:
 	CCinemaManager* GetCinema();
 
 	JSObject* GetScript();
-	static void ScriptingInit();
 };
 extern InReaction game_view_handler(const SDL_Event_* ev);
-
 #endif
