@@ -64,7 +64,7 @@ CMapReader::CMapReader()
 }
 
 // LoadMap: try to load the map from given file; reinitialise the scene to new data if successful
-void CMapReader::LoadMap(const VfsPath& pathname, CTerrain *pTerrain_,
+void CMapReader::LoadMap(const VfsPath& pathname,  const CScriptValRooted& settings, CTerrain *pTerrain_,
 						 WaterManager* pWaterMan_, SkyManager* pSkyMan_,
 						 CLightEnv *pLightEnv_, CGameView *pGameView_, CCinemaManager* pCinema_, CTriggerManager* pTrigMan_, CPostprocManager* pPostproc_,
 						 CSimulation2 *pSimulation2_, const CSimContext* pSimContext_, int playerID_, bool skipEntities)
@@ -83,6 +83,7 @@ void CMapReader::LoadMap(const VfsPath& pathname, CTerrain *pTerrain_,
 	m_PlayerID = playerID_;
 	m_SkipEntities = skipEntities;
 	m_StartingCameraTarget = INVALID_ENTITY;
+	m_ScriptSettings = settings;
 
 	filename_xml = pathname.ChangeExtension(L".xml");
 
@@ -116,8 +117,11 @@ void CMapReader::LoadMap(const VfsPath& pathname, CTerrain *pTerrain_,
 	if (pPostproc)
 		pPostproc->SetPostEffect(L"default");
 
-	// load map settings script
-	RegMemFun(this, &CMapReader::LoadScriptSettings, L"CMapReader::LoadScriptSettings", 50);
+	// load map or script settings script
+	if (settings.undefined())
+		RegMemFun(this, &CMapReader::LoadScriptSettings, L"CMapReader::LoadScriptSettings", 50);
+	else
+		RegMemFun(this, &CMapReader::LoadRMSettings, L"CMapReader::LoadRMSettings", 50);
 
 	// load player settings script (must be done before reading map)
 	RegMemFun(this, &CMapReader::LoadPlayerSettings, L"CMapReader::LoadPlayerSettings", 50);
@@ -138,7 +142,6 @@ void CMapReader::LoadMap(const VfsPath& pathname, CTerrain *pTerrain_,
 	RegMemFun(this, &CMapReader::DelayLoadFinished, L"CMapReader::DelayLoadFinished", 5);
 }
 
-
 // LoadRandomMap: try to load the map data; reinitialise the scene to new data if successful
 void CMapReader::LoadRandomMap(const CStrW& scriptFile, const CScriptValRooted& settings, CTerrain *pTerrain_,
 						 WaterManager* pWaterMan_, SkyManager* pSkyMan_,
@@ -580,7 +583,7 @@ void CXMLReader::ReadEnvironment(XMBElement parent)
 	EL(type);
 	EL(colour);
 	EL(height);
-	EL(shininess);
+	EL(shininess);	// for compatibility
 	EL(waviness);
 	EL(murkiness);
 	EL(tint);
@@ -714,11 +717,15 @@ void CXMLReader::ReadEnvironment(XMBElement parent)
 					// graphics are disabled
 					if (!m_MapReader.pWaterMan)
 						continue;
-
+					
 					if (element_name == el_type)
 					{
 						// TODO: implement this, when WaterManager supports it
 					}
+					else if (element_name == el_shininess)
+					{
+						// deprecated.
+					}
 #define READ_COLOUR(el, out) \
 					else if (element_name == el) \
 					{ \
@@ -737,7 +744,6 @@ void CXMLReader::ReadEnvironment(XMBElement parent)
 					} \
 
 					READ_COLOUR(el_colour, m_MapReader.pWaterMan->m_WaterColor)
-					READ_FLOAT(el_shininess, m_MapReader.pWaterMan->m_Shininess)
 					READ_FLOAT(el_waviness, m_MapReader.pWaterMan->m_Waviness)
 					READ_FLOAT(el_murkiness, m_MapReader.pWaterMan->m_Murkiness)
 					READ_COLOUR(el_tint, m_MapReader.pWaterMan->m_WaterTint)
@@ -1196,6 +1202,7 @@ int CMapReader::DelayLoadFinished()
 int CMapReader::LoadRMSettings()
 {
 	// copy random map settings over to sim
+	ENSURE(pSimulation2);
 	pSimulation2->SetMapSettings(m_ScriptSettings);
 
 	return 0;
@@ -1457,7 +1464,6 @@ int CMapReader::ParseEnvironment()
 		// TODO: Water type not implemented
 
 		GET_ENVIRONMENT_PROPERTY(waterBodyObj.get(), Colour, pWaterMan->m_WaterColor)
-		GET_ENVIRONMENT_PROPERTY(waterBodyObj.get(), Shininess, pWaterMan->m_Shininess)
 		GET_ENVIRONMENT_PROPERTY(waterBodyObj.get(), Waviness, pWaterMan->m_Waviness)
 		GET_ENVIRONMENT_PROPERTY(waterBodyObj.get(), Murkiness, pWaterMan->m_Murkiness)
 		GET_ENVIRONMENT_PROPERTY(waterBodyObj.get(), Tint, pWaterMan->m_WaterTint)
@@ -1465,6 +1471,30 @@ int CMapReader::ParseEnvironment()
 		GET_ENVIRONMENT_PROPERTY(waterBodyObj.get(), ReflectionTintStrength, pWaterMan->m_ReflectionTintStrength)
 	}
 
+	CScriptValRooted fogObject;
+	GET_ENVIRONMENT_PROPERTY(envObj.get(), Fog, fogObject);
+
+	GET_ENVIRONMENT_PROPERTY(fogObject.get(), FogFactor, m_LightEnv.m_FogFactor);
+	GET_ENVIRONMENT_PROPERTY(fogObject.get(), FogThickness, m_LightEnv.m_FogMax);
+
+	CColor fogColor;
+	GET_ENVIRONMENT_PROPERTY(fogObject.get(), FogColor, fogColor);
+	m_LightEnv.m_FogColor = RGBColor(fogColor.r, fogColor.g, fogColor.b);
+
+	CScriptValRooted postprocObject;
+	GET_ENVIRONMENT_PROPERTY(envObj.get(), Postproc, postprocObject);
+
+	std::wstring postProcEffect;
+	GET_ENVIRONMENT_PROPERTY(postprocObject.get(), PostprocEffect, postProcEffect);
+
+	if (pPostproc)
+		pPostproc->SetPostEffect(postProcEffect);
+
+	GET_ENVIRONMENT_PROPERTY(postprocObject.get(), Brightness, m_LightEnv.m_Brightness);
+	GET_ENVIRONMENT_PROPERTY(postprocObject.get(), Contrast, m_LightEnv.m_Contrast);
+	GET_ENVIRONMENT_PROPERTY(postprocObject.get(), Saturation, m_LightEnv.m_Saturation);
+	GET_ENVIRONMENT_PROPERTY(postprocObject.get(), Bloom, m_LightEnv.m_Bloom);
+
 	m_LightEnv.CalculateSunDirection();
 
 #undef GET_ENVIRONMENT_PROPERTY
