@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2013 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -71,6 +71,7 @@ bool CObjectBase::Load(const VfsPath& pathname)
 	AT(speed);
 	AT(event);
 	AT(load);
+	AT(sound);
 	AT(attachpoint);
 	AT(actor);
 	AT(frequency);
@@ -79,6 +80,9 @@ bool CObjectBase::Load(const VfsPath& pathname)
 	AT(angle);
 	AT(offsetx);
 	AT(offsetz);
+	AT(minheight);
+	AT(maxheight);
+	AT(selectable);
 	#undef AT
 	#undef EL
 
@@ -222,6 +226,11 @@ bool CObjectBase::Load(const VfsPath& pathname)
 									float pos = ae.Value.ToFloat();
 									anim.m_ActionPos2 = clamp(pos, 0.f, 1.f);
 								}
+								else if (ae.Name == at_sound)
+								{
+									float pos = ae.Value.ToFloat();
+									anim.m_SoundPos = clamp(pos, 0.f, 1.f);
+								}
 							}
 							currentVariant->m_Anims.push_back(anim);
 						}
@@ -240,6 +249,12 @@ bool CObjectBase::Load(const VfsPath& pathname)
 									prop.m_PropPointName = pe.Value;
 								else if (pe.Name == at_actor)
 									prop.m_ModelName = pe.Value.FromUTF8();
+								else if (pe.Name == at_minheight)
+									prop.m_minHeight = pe.Value.ToFloat();
+								else if (pe.Name == at_maxheight)
+									prop.m_maxHeight = pe.Value.ToFloat();
+								else if (pe.Name == at_selectable)
+									prop.m_selectable = pe.Value != "false";
 							}
 							currentVariant->m_Props.push_back(prop);
 						}
@@ -585,7 +600,7 @@ std::set<CStr> CObjectBase::CalculateRandomRemainingSelections(rng_t& rng, const
 		if (prop)
 		{
 			std::vector<std::set<CStr> > propInitialSelections = initialSelections;
-			if (remainingSelections.size() > 0)
+			if (!remainingSelections.empty())
 				propInitialSelections.push_back(remainingSelections);
 
 			std::set<CStr> propRemainingSelections = prop->CalculateRandomRemainingSelections(rng, propInitialSelections);
@@ -611,7 +626,7 @@ std::vector<std::vector<CStr> > CObjectBase::GetVariantGroups() const
 	// Set of objects already processed, so we don't do them more than once
 	std::set<const CObjectBase*> objectsProcessed;
 
-	while (objectsQueue.size())
+	while (!objectsQueue.empty())
 	{
 		const CObjectBase* obj = objectsQueue.front();
 		objectsQueue.pop();
