@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2013 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -39,8 +39,7 @@ public:
 	struct Anim
 	{
 		// constructor
-		Anim() : m_Speed(1.f), m_ActionPos(-1.f), m_ActionPos2(-1.f) {}
-
+		Anim() : m_Speed(1.f), m_ActionPos(-1.f), m_ActionPos2(-1.f), m_SoundPos(-1.f) {}
 		// name of the animation - "Idle", "Run", etc
 		CStr m_AnimName;
 		// filename of the animation - manidle.psa, manrun.psa, etc
@@ -51,14 +50,21 @@ public:
 		// happens, or -1.0 if unspecified
 		float m_ActionPos;
 		float m_ActionPos2;
+		float m_SoundPos;
 	};
 
 	struct Prop
 	{
+		// constructor
+		Prop() : m_minHeight(0.f), m_maxHeight(0.f), m_selectable(true) {}
 		// name of the prop point to attach to - "Prop01", "Prop02", "Head", "LeftHand", etc ..
 		CStr m_PropPointName;
 		// name of the model file - art/actors/props/sword.xml or whatever
 		CStrW m_ModelName;
+		// allow the prop to ajust the height from minHeight to maxHeight relative to the main model
+		float m_minHeight;
+		float m_maxHeight;
+		bool m_selectable;
 	};
 	
 	struct Samp
