@@ -24,11 +24,6 @@
 
 #include <sstream>
 
-size_t hash_value(const CStrIntern& v)
-{
-	return v.GetHash();
-}
-
 size_t hash_value(const CVector4D& v)
 {
 	size_t hash = 0;
@@ -104,10 +99,21 @@ typename CShaderParams<value_t>::SItems* CShaderParams<value_t>::GetInterned(con
 
 template<typename value_t>
 CShaderParams<value_t>::CShaderParams()
+{
+	*this = s_Empty;
+}
+
+template<typename value_t>
+CShaderParams<value_t>::CShaderParams(SItems* items) : m_Items(items)
+{
+}
+
+template<typename value_t>
+CShaderParams<value_t> CShaderParams<value_t>::CreateEmpty()
 {
 	SItems items;
 	items.RecalcHash();
-	m_Items = GetInterned(items);
+	return CShaderParams(GetInterned(items));
 }
 
 template<typename value_t>
@@ -180,9 +186,9 @@ void CShaderParams<value_t>::SItems::RecalcHash()
 }
 
 
-void CShaderDefines::Add(const char* name, const char* value)
+void CShaderDefines::Add(CStrIntern name, CStrIntern value)
 {
-	Set(CStrIntern(name), CStrIntern(value));
+	Set(name, value);
 }
 
 int CShaderDefines::GetInt(const char* name) const
@@ -250,16 +256,6 @@ void CShaderRenderQueries::Add(const char* name)
 	}
 }
 
-size_t CShaderRenderQueries::GetSize()
-{
-	return m_Items.size();
-}
-
-CShaderRenderQueries::RenderQuery CShaderRenderQueries::GetItem(size_t i)
-{
-	return m_Items[i];
-}
-
 void CShaderConditionalDefines::Add(const char* defname, const char* defvalue, int type, std::vector<float> &args)
 {
 	CondDefine cd;
@@ -271,21 +267,14 @@ void CShaderConditionalDefines::Add(const char* defname, const char* defvalue, i
 	m_Defines.push_back(cd);
 }
 
-size_t CShaderConditionalDefines::GetSize()
-{
-	return m_Defines.size();
-}
-
-CShaderConditionalDefines::CondDefine& CShaderConditionalDefines::GetItem(size_t i)
-{
-	return m_Defines[i];
-}
-
 
 // Explicit instantiations:
 
 template<> CShaderParams<CStrIntern>::InternedItems_t CShaderParams<CStrIntern>::s_InternedItems = CShaderParams<CStrIntern>::InternedItems_t();
 template<> CShaderParams<CVector4D>::InternedItems_t CShaderParams<CVector4D>::s_InternedItems = CShaderParams<CVector4D>::InternedItems_t();
 
+template<> CShaderParams<CStrIntern> CShaderParams<CStrIntern>::s_Empty = CShaderParams<CStrIntern>::CreateEmpty();
+template<> CShaderParams<CVector4D> CShaderParams<CVector4D>::s_Empty = CShaderParams<CVector4D>::CreateEmpty();
+
 template class CShaderParams<CStrIntern>;
 template class CShaderParams<CVector4D>;
