@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2013 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -203,8 +203,8 @@ const float* CTerrainTextureEntry::GetTextureMatrix()
 // calculate the coordinate of each alphamap within this packed texture
 void CTerrainTextureEntry::LoadAlphaMaps(VfsPath &amtype)
 {
-	const wchar_t* const key = (L"(alpha map composite" + amtype.string() + L")").c_str();
-	
+	std::wstring key = L"(alpha map composite" + amtype.string() + L")";
+
 	CTerrainTextureManager::TerrainAlphaMap::iterator it = g_TexMan.m_TerrainAlphas.find(amtype);
 	
 	if (it != g_TexMan.m_TerrainAlphas.end())
@@ -333,7 +333,7 @@ void CTerrainTextureEntry::LoadAlphaMaps(VfsPath &amtype)
 
 	// upload the composite texture
 	Tex t;
-	(void)tex_wrap(total_w, total_h, 8, TEX_GREY, data, 0, &t);
+	(void)t.wrap(total_w, total_h, 8, TEX_GREY, data, 0);
 	
 	// uncomment the following to save a png of the generated texture
 	// in the public/ directory, for debugging
