@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2013 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -48,6 +48,7 @@ struct TPhash
 		boost::hash_combine(seed, a.m_WrapS);
 		boost::hash_combine(seed, a.m_WrapT);
 		boost::hash_combine(seed, a.m_Aniso);
+		boost::hash_combine(seed, a.m_Format);
 		return seed;
 	}
 	std::size_t operator()(CTexturePtr const& a) const
@@ -64,7 +65,7 @@ struct TPequal_to
 	{
 		return a.m_Path == b.m_Path && a.m_Filter == b.m_Filter
 			&& a.m_WrapS == b.m_WrapS && a.m_WrapT == b.m_WrapT
-			&& a.m_Aniso == b.m_Aniso;
+			&& a.m_Aniso == b.m_Aniso && a.m_Format == b.m_Format;
 	}
 	bool operator()(CTexturePtr const& a, CTexturePtr const& b) const
 	{
@@ -93,7 +94,7 @@ public:
 			data.get()[1] = 64;
 			data.get()[2] = 64;
 			Tex t;
-			(void)tex_wrap(1, 1, 24, 0, data, 0, &t);
+			(void)t.wrap(1, 1, 24, 0, data, 0);
 
 			m_DefaultHandle = ogl_tex_wrap(&t, m_VFS, L"(default texture)");
 			(void)ogl_tex_set_filter(m_DefaultHandle, GL_LINEAR);
@@ -110,7 +111,7 @@ public:
 			data.get()[1] = 0;
 			data.get()[2] = 255;
 			Tex t;
-			(void)tex_wrap(1, 1, 24, 0, data, 0, &t);
+			(void)t.wrap(1, 1, 24, 0, data, 0);
 
 			m_ErrorHandle = ogl_tex_wrap(&t, m_VFS, L"(error texture)");
 			(void)ogl_tex_set_filter(m_ErrorHandle, GL_LINEAR);
@@ -215,7 +216,7 @@ public:
 		(void)ogl_tex_set_filter(h, filter);
 
 		// Upload to GL
-		if (!m_DisableGL && ogl_tex_upload(h) < 0)
+		if (!m_DisableGL && ogl_tex_upload(h, texture->m_Properties.m_Format) < 0)
 		{
 			LOGERROR(L"Texture failed to upload: \"%ls\"", texture->m_Properties.m_Path.string().c_str());
 
@@ -481,6 +482,14 @@ public:
 		return INFO::OK;
 	}
 
+	size_t GetBytesUploaded() const
+	{
+		size_t size = 0;
+		for (TextureCache::const_iterator it = m_TextureCache.begin(); it != m_TextureCache.end(); ++it)
+			size += (*it)->GetUploadedSize();
+		return size;
+	}
+
 private:
 	PIVFS m_VFS;
 	CCacheLoader m_CacheLoader;
@@ -492,7 +501,7 @@ private:
 	CTexturePtr m_ErrorTexture;
 
 	// Cache of all loaded textures
-	typedef boost::unordered_set<CTexturePtr, TPhash, TPequal_to > TextureCache;
+	typedef boost::unordered_set<CTexturePtr, TPhash, TPequal_to> TextureCache;
 	TextureCache m_TextureCache;
 	// TODO: we ought to expire unused textures from the cache eventually
 
@@ -609,6 +618,13 @@ u32 CTexture::GetBaseColour() const
 	return m_BaseColour;
 }
 
+size_t CTexture::GetUploadedSize() const
+{
+	size_t size = 0;
+	(void)ogl_tex_get_uploaded_size(m_Handle, &size);
+	return size;
+}
+
 
 // CTextureManager: forward all calls to impl:
 
@@ -641,3 +657,8 @@ bool CTextureManager::GenerateCachedTexture(const VfsPath& path, VfsPath& output
 {
 	return m->GenerateCachedTexture(path, outputPath);
 }
+
+size_t CTextureManager::GetBytesUploaded() const
+{
+	return m->GetBytesUploaded();
+}
