@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2014 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -35,21 +35,24 @@ CGUI
 #include "CRadioButton.h"
 #include "CInput.h"
 #include "CList.h"
+#include "COList.h"
 #include "CDropDown.h"
 #include "CProgressBar.h"
 #include "CTooltip.h"
 #include "MiniMap.h"
-#include "scripting/JSInterface_GUITypes.h"
+#include "scripting/ScriptFunctions.h"
 
+#include "graphics/FontMetrics.h"
 #include "graphics/ShaderManager.h"
 #include "graphics/TextRenderer.h"
 #include "lib/input.h"
 #include "lib/bits.h"
+#include "i18n/L10n.h"
 #include "lib/timer.h"
 #include "lib/sysdep/sysdep.h"
+#include "lib/utf8.h"
 #include "ps/CLogger.h"
 #include "ps/Filesystem.h"
-#include "ps/Font.h"
 #include "ps/Hotkey.h"
 #include "ps/Globals.h"
 #include "ps/Overlay.h"
@@ -57,19 +60,12 @@ CGUI
 #include "ps/Pyrogenesis.h"
 #include "ps/XML/Xeromyces.h"
 #include "renderer/Renderer.h"
-#include "scripting/ScriptingHost.h"
 #include "scriptinterface/ScriptInterface.h"
 
 extern int g_yres;
 
 const double SELECT_DBLCLICK_RATE = 0.5;
 
-void CGUI::ScriptingInit()
-{
-	JSI_IGUIObject::init();
-	JSI_GUITypes::init();
-}
-
 InReaction CGUI::HandleEvent(const SDL_Event_* ev)
 {
 	InReaction ret = IN_PASS;
@@ -135,14 +131,9 @@ InReaction CGUI::HandleEvent(const SDL_Event_* ev)
 		// pNearest will after this point at the hovered object, possibly NULL
 		pNearest = FindObjectUnderMouse();
 
-		// Is placed in the UpdateMouseOver function
-		//if (ev->ev.type == SDL_MOUSEMOTION && pNearest)
-		//	pNearest->ScriptEvent("mousemove");
-
 		// Now we'll call UpdateMouseOver on *all* objects,
 		//  we'll input the one hovered, and they will each
 		//  update their own data and send messages accordingly
-		
 		GUI<IGUIObject*>::RecurseObject(GUIRR_HIDDEN | GUIRR_GHOST, m_BaseObject, 
 										&IGUIObject::UpdateMouseOver, 
 										pNearest);
@@ -238,10 +229,10 @@ InReaction CGUI::HandleEvent(const SDL_Event_* ev)
 			GUI<>::RecurseObject(GUIRR_HIDDEN, m_BaseObject, 
 									&IGUIObject::ResetStates);
 
-			// It will have reset the mouse over of the current hovered, so we'll
-			//  have to restore that
-			if (pNearest)
-				pNearest->m_MouseHovering = true;
+			// Since the hover state will have been reset, we reload it.
+			GUI<IGUIObject*>::RecurseObject(GUIRR_HIDDEN | GUIRR_GHOST, m_BaseObject, 
+											&IGUIObject::UpdateMouseOver, 
+											pNearest);
 		}
 	}
 	catch (PSERROR_GUI& e)
@@ -320,83 +311,13 @@ void CGUI::SendEventToAll(const CStr& EventName)
 //  Constructor / Destructor
 //-------------------------------------------------------------------
 
-// To isolate the vars declared by each GUI page, we need to create
-// a pseudo-global object to declare them in. In particular, it must
-// have no parent object, so it must be declared with JS_NewGlobalObject.
-// But GUI scripts should have access to the real global's properties
-// (Array, undefined, getGUIObjectByName, etc), so we add a custom resolver
-// that defers to the real global object when necessary.
-
-static JSBool GetGlobalProperty(JSContext* cx, JSObject* UNUSED(obj), jsid id, jsval* vp)
-{
-	return JS_GetPropertyById(cx, g_ScriptingHost.GetGlobalObject(), id, vp);
-}
-
-static JSBool SetGlobalProperty(JSContext* cx, JSObject* UNUSED(obj), jsid id, JSBool UNUSED(strict), jsval* vp)
-{
-	return JS_SetPropertyById(cx, g_ScriptingHost.GetGlobalObject(), id, vp);
-}
-
-static JSBool ResolveGlobalProperty(JSContext* cx, JSObject* obj, jsid id, uintN flags, JSObject** objp)
-{
-	// This gets called when the property can't be resolved in the page_global object.
-
-	// Warning: The interaction between this resolution stuff and the JITs appears
-	// to be quite fragile, and I don't quite understand what the constraints are.
-	// If changing it, be careful to test with each JIT to make sure it works.
-	// (This code is somewhat based on GPSEE's module system.)
-
-	// Declarations and assignments shouldn't affect the real global
-	if (flags & (JSRESOLVE_DECLARING | JSRESOLVE_ASSIGNING))
-	{
-		// Can't be resolved - return NULL
-		*objp = NULL;
-		return JS_TRUE;
-	}
-
-	// Check whether the real global object defined this property
-	uintN attrs;
-	JSBool found;
-	if (!JS_GetPropertyAttrsGetterAndSetterById(cx, g_ScriptingHost.GetGlobalObject(), id, &attrs, &found, NULL, NULL))
-		return JS_FALSE;
-
-	if (!found)
-	{
-		// Not found on real global, so can't be resolved - return NULL
-		*objp = NULL;
-		return JS_TRUE;
-	}
-
-	// Retrieve the property value from the global
-	jsval v;
-	if (!JS_GetPropertyById(cx, g_ScriptingHost.GetGlobalObject(), id, &v))
-		return JS_FALSE;
-
-	// Add the global's property value onto this object, with getter/setter that will
-	// update the global's copy instead of this copy, and then return this object
-	if (!JS_DefinePropertyById(cx, obj, id, v, GetGlobalProperty, SetGlobalProperty, attrs))
-		return JS_FALSE;
-
-	*objp = obj;
-	return JS_TRUE;
-}
-
-static JSClass page_global_class = {
-	"page_global", JSCLASS_GLOBAL_FLAGS | JSCLASS_NEW_RESOLVE,
-	JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
-	JS_EnumerateStub, (JSResolveOp)ResolveGlobalProperty, JS_ConvertStub, JS_FinalizeStub,
-	NULL, NULL, NULL, NULL,
-	NULL, NULL, NULL, NULL
-};
-
-CGUI::CGUI() : m_MouseButtons(0), m_FocusedObject(NULL), m_InternalNameNumber(0)
+CGUI::CGUI(const shared_ptr<ScriptRuntime>& runtime) : m_MouseButtons(0), m_FocusedObject(NULL), m_InternalNameNumber(0)
 {
+	m_ScriptInterface.reset(new ScriptInterface("Engine", "GUIPage", runtime));
+	GuiScriptingInit(*m_ScriptInterface);
+	m_ScriptInterface->LoadGlobalScripts();
 	m_BaseObject = new CGUIDummyObject;
-	m_BaseObject->SetGUI(this);
-
-	// Construct the root object for all GUI JavaScript things
-	m_ScriptObject = JS_NewGlobalObject(g_ScriptingHost.getContext(), &page_global_class);
-	JS_AddObjectRoot(g_ScriptingHost.getContext(), &m_ScriptObject);
+	m_BaseObject->SetGUI(this);	
 }
 
 CGUI::~CGUI()
@@ -405,12 +326,6 @@ CGUI::~CGUI()
 
 	if (m_BaseObject)
 		delete m_BaseObject;
-
-	if (m_ScriptObject)
-	{
-		// Let it be garbage-collected
-		JS_RemoveObjectRoot(g_ScriptingHost.getContext(), &m_ScriptObject);
-	}
 }
 
 //-------------------------------------------------------------------
@@ -442,6 +357,7 @@ void CGUI::Initialize()
 	AddObjectType("minimap",        &CMiniMap::ConstructObject);
 	AddObjectType("input",			&CInput::ConstructObject);
 	AddObjectType("list",			&CList::ConstructObject);
+	AddObjectType("olist",			&COList::ConstructObject);
 	AddObjectType("dropdown",		&CDropDown::ConstructObject);
 	AddObjectType("tooltip",		&CTooltip::ConstructObject);
 }
@@ -483,8 +399,7 @@ void CGUI::Destroy()
 {
 	// We can use the map to delete all
 	//  now we don't want to cancel all if one Destroy fails
-	map_pObjects::iterator it;
-	for (it = m_pAllObjects.begin(); it != m_pAllObjects.end(); ++it)
+	for (map_pObjects::iterator it = m_pAllObjects.begin(); it != m_pAllObjects.end(); ++it)
 	{
 		try
 		{
@@ -500,12 +415,10 @@ void CGUI::Destroy()
 		delete it->second;
 	}
 
-	for (std::map<CStr, CGUISprite>::iterator it2 = m_Sprites.begin(); it2 != m_Sprites.end(); ++it2)
-		for (std::vector<SGUIImage>::iterator it3 = it2->second.m_Images.begin(); it3 != it2->second.m_Images.end(); ++it3)
-			delete it3->m_Effects;
-
 	// Clear all
 	m_pAllObjects.clear();
+	for(std::map<CStr, CGUISprite*>::iterator it = m_Sprites.begin(); it != m_Sprites.end(); ++it)
+		delete it->second;
 	m_Sprites.clear();
 	m_Icons.clear();
 }
@@ -642,10 +555,12 @@ struct SGenerateTextImage
 };
 
 SGUIText CGUI::GenerateText(const CGUIString &string,
-							const CStrW& Font, const float &Width, const float &BufferZone,
+							const CStrW& FontW, const float &Width, const float &BufferZone,
 							const IGUIObject *pObject)
 {
 	SGUIText Text; // object we're generating
+
+	CStrIntern Font(FontW.ToUTF8());
 	
 	if (string.m_Words.size() == 0)
 		return Text;
@@ -684,7 +599,7 @@ SGUIText CGUI::GenerateText(const CGUIString &string,
 		float prelim_line_height=0.f;
 
 		// Width and height of all text calls generated.
-		string.GenerateTextCall(Feedback, Font,
+		string.GenerateTextCall(this, Feedback, Font,
 								string.m_Words[i], string.m_Words[i+1],
 								FirstLine);
 
@@ -802,7 +717,7 @@ SGUIText CGUI::GenerateText(const CGUIString &string,
 				// Don't attach object, it'll suppress the errors
 				//  we want them to be reported in the final GenerateTextCall()
 				//  so that we don't get duplicates.
-				string.GenerateTextCall(Feedback2, Font,
+				string.GenerateTextCall(this, Feedback2, Font,
 										string.m_Words[j], string.m_Words[j+1],
 										FirstLine);
 
@@ -855,7 +770,7 @@ SGUIText CGUI::GenerateText(const CGUIString &string,
 				CGUIString::SFeedback Feedback2;
 
 				// Defaults
-				string.GenerateTextCall(Feedback2, Font,
+				string.GenerateTextCall(this, Feedback2, Font,
 										string.m_Words[j], string.m_Words[j+1], 
 										FirstLine, pObject);
 
@@ -941,17 +856,19 @@ SGUIText CGUI::GenerateText(const CGUIString &string,
 void CGUI::DrawText(SGUIText &Text, const CColor &DefaultColor, 
 					const CPos &pos, const float &z, const CRect &clipping)
 {
-	CShaderTechniquePtr tech = g_Renderer.GetShaderManager().LoadEffect("gui_text");
+	CShaderTechniquePtr tech = g_Renderer.GetShaderManager().LoadEffect(str_gui_text);
 
 	tech->BeginPass();
 
-	if (clipping != CRect())
+	bool isClipped = (clipping != CRect());
+	if (isClipped)
 	{
 		glEnable(GL_SCISSOR_TEST);
 		glScissor(clipping.left, g_yres - clipping.bottom, clipping.GetWidth(), clipping.GetHeight());
 	}
 
 	CTextRenderer textRenderer(tech->GetShader());
+	textRenderer.SetClippingRect(clipping);
 	textRenderer.Translate(0.0f, 0.0f, z);
 
 	for (std::vector<SGUIText::STextCall>::const_iterator it = Text.m_TextCalls.begin(); 
@@ -966,7 +883,7 @@ void CGUI::DrawText(SGUIText &Text, const CColor &DefaultColor,
 
 		textRenderer.Color(color);
 		textRenderer.Font(it->m_Font);
-		textRenderer.Put((float)(int)(pos.x+it->m_Pos.x), (float)(int)(pos.y+it->m_Pos.y), it->m_String.c_str());
+		textRenderer.Put((float)(int)(pos.x+it->m_Pos.x), (float)(int)(pos.y+it->m_Pos.y), &it->m_String);
 	}
 
 	textRenderer.Render();
@@ -978,7 +895,7 @@ void CGUI::DrawText(SGUIText &Text, const CColor &DefaultColor,
 		DrawSprite(it->m_Sprite, it->m_CellID, z, it->m_Area + pos);
 	}
 
-	if (clipping != CRect())
+	if (isClipped)
 		glDisable(GL_SCISSOR_TEST);
 
 	tech->EndPass();
@@ -1180,6 +1097,10 @@ void CGUI::Xeromyces_ReadObject(XMBElement Element, CXeromyces* pFile, IGUIObjec
 	ELMT(object);
 	ELMT(action);
 	ELMT(repeat);
+	ELMT(translatableAttribute);
+	ELMT(translate);
+	ELMT(attribute);
+	ELMT(keep);
 	ATTR(style);
 	ATTR(type);
 	ATTR(name);
@@ -1187,6 +1108,8 @@ void CGUI::Xeromyces_ReadObject(XMBElement Element, CXeromyces* pFile, IGUIObjec
 	ATTR(z);
 	ATTR(on);
 	ATTR(file);
+	ATTR(id);
+	ATTR(context);
 
 	//
 	//	Read Style and set defaults
@@ -1327,16 +1250,94 @@ void CGUI::Xeromyces_ReadObject(XMBElement Element, CXeromyces* pFile, IGUIObjec
 				code = scriptfile.DecodeUTF8(); // assume it's UTF-8
 			}
 
-			// Read the inline code (concatenating to the file code, if both are specified)
-			code += CStr(child.GetText());
+			XMBElementList grandchildren = child.GetChildNodes();
+			if (grandchildren.Count > 0) // The <action> element contains <keep> and <translate> tags.
+			{
+				for (int i = 0; i < grandchildren.Count; ++i)
+				{
+					XMBElement grandchild = grandchildren.Item(i);
+					if (grandchild.GetNodeName() == elmt_translate)
+					{
+						code += L10n::Instance().Translate(grandchild.GetText());
+					}
+					else if (grandchild.GetNodeName() == elmt_keep)
+					{
+						code += grandchild.GetText();
+					}
+				}
+			}
+			else // It’s pure JavaScript code.
+			{
+				// Read the inline code (concatenating to the file code, if both are specified)
+				code += CStr(child.GetText());
+			}
 
 			CStr action = CStr(child.GetAttributes().GetNamedItem(attr_on));
+			
+			// We need to set the GUI this object belongs to because RegisterScriptHandler requires an associated GUI.
+			object->SetGUI(this);
 			object->RegisterScriptHandler(action.LowerCase(), code, this);
 		}
 		else if (element_name == elmt_repeat)
 		{
 			Xeromyces_ReadRepeat(child, pFile, object, Paths);
 		}
+		else if (element_name == elmt_translatableAttribute)
+		{
+			// This is an element in the form “<translatableAttribute id="attributeName">attributeValue</translatableAttribute>”.
+			CStr attributeName(child.GetAttributes().GetNamedItem(attr_id)); // Read the attribute name.
+			if (!attributeName.empty())
+			{
+				CStr value(child.GetText());
+				if (!value.empty())
+				{
+					CStr context(child.GetAttributes().GetNamedItem(attr_context)); // Read the context if any.
+					if (!context.empty())
+					{
+						CStr translatedValue(L10n::Instance().TranslateWithContext(context, value));
+						object->SetSetting(attributeName, translatedValue.UnescapeBackslashes().FromUTF8(), true);
+					}
+					else
+					{
+						CStr translatedValue(L10n::Instance().Translate(value));
+						object->SetSetting(attributeName, translatedValue.UnescapeBackslashes().FromUTF8(), true);
+					}
+				}
+			}
+			else // Ignore.
+			{
+				LOGERROR(L"GUI: ‘attribute’ XML element with empty ‘id’ XML attribute found. (object: %hs)", object->GetPresentableName().c_str());
+			}
+		}
+		else if (element_name == elmt_attribute)
+		{
+			// This is an element in the form “<attribute id="attributeName"><keep>Don’t translate this part
+			// </keep><translate>but translate this one.</translate></attribute>”.
+			CStr attributeName(child.GetAttributes().GetNamedItem(attr_id)); // Read the attribute name.
+			if (!attributeName.empty())
+			{
+				CStr translatedValue;
+
+				XMBElementList grandchildren = child.GetChildNodes();
+				for (int i = 0; i < grandchildren.Count; ++i)
+				{
+					XMBElement grandchild = grandchildren.Item(i);
+					if (grandchild.GetNodeName() == elmt_translate)
+					{
+						translatedValue += L10n::Instance().Translate(grandchild.GetText());
+					}
+					else if (grandchild.GetNodeName() == elmt_keep)
+					{
+						translatedValue += grandchild.GetText();
+					}
+				}
+				object->SetSetting(attributeName, translatedValue.UnescapeBackslashes().FromUTF8(), true);
+			}
+			else // Ignore.
+			{
+				LOGERROR(L"GUI: ‘attribute’ XML element with empty ‘id’ XML attribute found. (object: %hs)", object->GetPresentableName().c_str());
+			}
+		}
 		else
 		{
 			// Try making the object read the tag.
@@ -1424,7 +1425,7 @@ void CGUI::Xeromyces_ReadScript(XMBElement Element, CXeromyces* pFile, boost::un
 		Paths.insert(file);
 		try
 		{
-			g_ScriptingHost.RunScript(file, m_ScriptObject);
+			m_ScriptInterface->LoadGlobalScriptFile(file);
 		}
 		catch (PSERROR_Scripting& e)
 		{
@@ -1437,7 +1438,7 @@ void CGUI::Xeromyces_ReadScript(XMBElement Element, CXeromyces* pFile, boost::un
 	{
 		CStr code (Element.GetText());
 		if (! code.empty())
-			g_ScriptingHost.RunMemScript(code.c_str(), code.length(), "Some XML file", Element.GetLineNumber(), m_ScriptObject);
+			m_ScriptInterface->LoadGlobalScript(L"Some XML file", code.FromUTF8());
 	}
 	catch (PSERROR_Scripting& e)
 	{
@@ -1448,7 +1449,7 @@ void CGUI::Xeromyces_ReadScript(XMBElement Element, CXeromyces* pFile, boost::un
 void CGUI::Xeromyces_ReadSprite(XMBElement Element, CXeromyces* pFile)
 {
 	// Sprite object we're adding
-	CGUISprite sprite;
+	CGUISprite* Sprite = new CGUISprite;
 	
 	// and what will be its reference name
 	CStr name;
@@ -1481,7 +1482,7 @@ void CGUI::Xeromyces_ReadSprite(XMBElement Element, CXeromyces* pFile)
 
 		if (ElementName == "image")
 		{
-			Xeromyces_ReadImage(child, pFile, sprite);
+			Xeromyces_ReadImage(child, pFile, *Sprite);
 		}
 		else if (ElementName == "effect")
 		{
@@ -1504,9 +1505,9 @@ void CGUI::Xeromyces_ReadSprite(XMBElement Element, CXeromyces* pFile)
 	// Apply the effects to every image (unless the image overrides it with
 	// different effects)
 	if (effects)
-		for (std::vector<SGUIImage>::iterator it = sprite.m_Images.begin(); it != sprite.m_Images.end(); ++it)
-			if (! it->m_Effects)
-				it->m_Effects = new SGUIImageEffects(*effects); // do a copy just so it can be deleted correctly later
+		for (std::vector<SGUIImage*>::iterator it = Sprite->m_Images.begin(); it != Sprite->m_Images.end(); ++it)
+			if (! (*it)->m_Effects)
+				(*it)->m_Effects = new SGUIImageEffects(*effects); // do a copy just so it can be deleted correctly later
 
 	delete effects;
 
@@ -1514,18 +1515,18 @@ void CGUI::Xeromyces_ReadSprite(XMBElement Element, CXeromyces* pFile)
 	//	Add Sprite
 	//
 
-	m_Sprites[name] = sprite;
+	m_Sprites[name] = Sprite;
 }
 
 void CGUI::Xeromyces_ReadImage(XMBElement Element, CXeromyces* pFile, CGUISprite &parent)
 {
 
 	// Image object we're adding
-	SGUIImage image;
+	SGUIImage* Image = new SGUIImage;
 	
 	// Set defaults to "0 0 100% 100%"
-	image.m_TextureSize = CClientArea(CRect(0, 0, 0, 0), CRect(0, 0, 100, 100));
-	image.m_Size = CClientArea(CRect(0, 0, 0, 0), CRect(0, 0, 100, 100));
+	Image->m_TextureSize = CClientArea(CRect(0, 0, 0, 0), CRect(0, 0, 100, 100));
+	Image->m_Size = CClientArea(CRect(0, 0, 0, 0), CRect(0, 0, 100, 100));
 	
 	// TODO Gee: Setup defaults here (or maybe they are in the SGUIImage ctor)
 
@@ -1543,7 +1544,7 @@ void CGUI::Xeromyces_ReadImage(XMBElement Element, CXeromyces* pFile, CGUISprite
 
 		if (attr_name == "texture")
 		{
-			image.m_TextureName = VfsPath("art/textures/ui") / attr_value;
+			Image->m_TextureName = VfsPath("art/textures/ui") / attr_value;
 		}
 		else
 		if (attr_name == "size")
@@ -1551,7 +1552,7 @@ void CGUI::Xeromyces_ReadImage(XMBElement Element, CXeromyces* pFile, CGUISprite
 			CClientArea ca;
 			if (!GUI<CClientArea>::ParseString(attr_value, ca))
 				LOGERROR(L"GUI: Error parsing '%hs' (\"%ls\")", attr_name.c_str(), attr_value.c_str());
-			else image.m_Size = ca;
+			else Image->m_Size = ca;
 		}
 		else
 		if (attr_name == "texture_size")
@@ -1559,7 +1560,7 @@ void CGUI::Xeromyces_ReadImage(XMBElement Element, CXeromyces* pFile, CGUISprite
 			CClientArea ca;
 			if (!GUI<CClientArea>::ParseString(attr_value, ca))
 				LOGERROR(L"GUI: Error parsing '%hs' (\"%ls\")", attr_name.c_str(), attr_value.c_str());
-			else image.m_TextureSize = ca;
+			else Image->m_TextureSize = ca;
 		}
 		else
 		if (attr_name == "real_texture_placement")
@@ -1567,7 +1568,7 @@ void CGUI::Xeromyces_ReadImage(XMBElement Element, CXeromyces* pFile, CGUISprite
 			CRect rect;
 			if (!GUI<CRect>::ParseString(attr_value, rect))
 				LOGERROR(L"GUI: Error parsing '%hs' (\"%ls\")", attr_name.c_str(), attr_value.c_str());
-			else image.m_TexturePlacementInFile = rect;
+			else Image->m_TexturePlacementInFile = rect;
 		}
 		else
 		if (attr_name == "cell_size")
@@ -1575,7 +1576,7 @@ void CGUI::Xeromyces_ReadImage(XMBElement Element, CXeromyces* pFile, CGUISprite
 			CSize size;
 			if (!GUI<CSize>::ParseString(attr_value, size))
 				LOGERROR(L"GUI: Error parsing '%hs' (\"%ls\")", attr_name.c_str(), attr_value.c_str());
-			else image.m_CellSize = size;
+			else Image->m_CellSize = size;
 		}
 		else
 		if (attr_name == "fixed_h_aspect_ratio")
@@ -1583,7 +1584,7 @@ void CGUI::Xeromyces_ReadImage(XMBElement Element, CXeromyces* pFile, CGUISprite
 			float val;
 			if (!GUI<float>::ParseString(attr_value, val))
 				LOGERROR(L"GUI: Error parsing '%hs' (\"%ls\")", attr_name.c_str(), attr_value.c_str());
-			else image.m_FixedHAspectRatio = val;
+			else Image->m_FixedHAspectRatio = val;
 		}
 		else
 		if (attr_name == "round_coordinates")
@@ -1591,17 +1592,17 @@ void CGUI::Xeromyces_ReadImage(XMBElement Element, CXeromyces* pFile, CGUISprite
 			bool b;
 			if (!GUI<bool>::ParseString(attr_value, b))
 				LOGERROR(L"GUI: Error parsing '%hs' (\"%ls\")", attr_name.c_str(), attr_value.c_str());
-			else image.m_RoundCoordinates = b;
+			else Image->m_RoundCoordinates = b;
 		}
 		else
 		if (attr_name == "wrap_mode")
 		{
 			if (attr_value == L"repeat")
-				image.m_WrapMode = GL_REPEAT;
+				Image->m_WrapMode = GL_REPEAT;
 			else if (attr_value == L"mirrored_repeat")
-				image.m_WrapMode = GL_MIRRORED_REPEAT;
+				Image->m_WrapMode = GL_MIRRORED_REPEAT;
 			else if (attr_value == L"clamp_to_edge")
-				image.m_WrapMode = GL_CLAMP_TO_EDGE;
+				Image->m_WrapMode = GL_CLAMP_TO_EDGE;
 			else
 				LOGERROR(L"GUI: Error parsing '%hs' (\"%ls\")", attr_name.c_str(), attr_value.c_str());
 		}
@@ -1611,7 +1612,7 @@ void CGUI::Xeromyces_ReadImage(XMBElement Element, CXeromyces* pFile, CGUISprite
 			float z_level;
 			if (!GUI<float>::ParseString(attr_value, z_level))
 				LOGERROR(L"GUI: Error parsing '%hs' (\"%ls\")", attr_name.c_str(), attr_value.c_str());
-			else image.m_DeltaZ = z_level/100.f;
+			else Image->m_DeltaZ = z_level/100.f;
 		}
 		else
 		if (attr_name == "backcolor")
@@ -1619,7 +1620,7 @@ void CGUI::Xeromyces_ReadImage(XMBElement Element, CXeromyces* pFile, CGUISprite
 			CColor color;
 			if (!GUI<CColor>::ParseString(attr_value, color))
 				LOGERROR(L"GUI: Error parsing '%hs' (\"%ls\")", attr_name.c_str(), attr_value.c_str());
-			else image.m_BackColor = color;
+			else Image->m_BackColor = color;
 		}
 		else
 		if (attr_name == "bordercolor")
@@ -1627,7 +1628,7 @@ void CGUI::Xeromyces_ReadImage(XMBElement Element, CXeromyces* pFile, CGUISprite
 			CColor color;
 			if (!GUI<CColor>::ParseString(attr_value, color))
 				LOGERROR(L"GUI: Error parsing '%hs' (\"%ls\")", attr_name.c_str(), attr_value.c_str());
-			else image.m_BorderColor = color;
+			else Image->m_BorderColor = color;
 		}
 		else
 		if (attr_name == "border")
@@ -1635,7 +1636,7 @@ void CGUI::Xeromyces_ReadImage(XMBElement Element, CXeromyces* pFile, CGUISprite
 			bool b;
 			if (!GUI<bool>::ParseString(attr_value, b))
 				LOGERROR(L"GUI: Error parsing '%hs' (\"%ls\")", attr_name.c_str(), attr_value.c_str());
-			else image.m_Border = b;
+			else Image->m_Border = b;
 		}
 		else
 		{
@@ -1651,14 +1652,14 @@ void CGUI::Xeromyces_ReadImage(XMBElement Element, CXeromyces* pFile, CGUISprite
 		CStr ElementName (pFile->GetElementString(child.GetNodeName()));
 		if (ElementName == "effect")
 		{
-			if (image.m_Effects)
+			if (Image->m_Effects)
 			{
 				LOGERROR(L"GUI <image> must not have more than one <effect>");
 			}
 			else
 			{
-				image.m_Effects = new SGUIImageEffects;
-				Xeromyces_ReadEffects(child, pFile, *image.m_Effects);
+				Image->m_Effects = new SGUIImageEffects;
+				Xeromyces_ReadEffects(child, pFile, *Image->m_Effects);
 			}
 		}
 		else
@@ -1671,7 +1672,7 @@ void CGUI::Xeromyces_ReadImage(XMBElement Element, CXeromyces* pFile, CGUISprite
 	//	Input
 	//
 
-	parent.AddImage(image);	
+	parent.AddImage(Image);	
 }
 
 void CGUI::Xeromyces_ReadEffects(XMBElement Element, CXeromyces* pFile, SGUIImageEffects &effects)
@@ -1708,7 +1709,7 @@ void CGUI::Xeromyces_ReadStyle(XMBElement Element, CXeromyces* pFile)
 	CStr name;
 	
 	//
-	//	Read Attributes
+	// Read Attributes
 	//
 
 	// Now we can iterate all attributes and store
@@ -1727,7 +1728,7 @@ void CGUI::Xeromyces_ReadStyle(XMBElement Element, CXeromyces* pFile)
 	}
 
 	//
-	//	Add to CGUI
+	// Add to CGUI
 	//
 
 	m_Styles[name] = style;
@@ -1738,9 +1739,15 @@ void CGUI::Xeromyces_ReadScrollBarStyle(XMBElement Element, CXeromyces* pFile)
 	// style object we're adding
 	SGUIScrollBarStyle scrollbar;
 	CStr name;
-	
+
+	// Setup some defaults.
+	scrollbar.m_MinimumBarSize = 0.f;
+	// Using 1.0e10 as a substitute for infinity
+	scrollbar.m_MaximumBarSize = 1.0e10;
+	scrollbar.m_UseEdgeButtons = false;
+
 	//
-	//	Read Attributes
+	// Read Attributes
 	//
 
 	// Now we can iterate all attributes and store
@@ -1749,7 +1756,7 @@ void CGUI::Xeromyces_ReadScrollBarStyle(XMBElement Element, CXeromyces* pFile)
 	{
 		XMBAttribute attr = attributes.Item(i);
 		CStr attr_name = pFile->GetAttributeString(attr.Name);
-		CStr attr_value (attr.Value);
+		CStr attr_value (attr.Value); 
 
 		if (attr_value == "null")
 			continue;
@@ -1757,6 +1764,14 @@ void CGUI::Xeromyces_ReadScrollBarStyle(XMBElement Element, CXeromyces* pFile)
 		if (attr_name == "name")
 			name = attr_value;
 		else
+		if (attr_name == "show_edge_buttons")
+		{
+			bool b;
+			if (!GUI<bool>::ParseString(attr_value.FromUTF8(), b))
+				LOGERROR(L"GUI: Error parsing '%hs' (\"%hs\")", attr_name.c_str(), attr_value.c_str());
+			else
+				scrollbar.m_UseEdgeButtons = b;
+		}
 		if (attr_name == "width")
 		{
 			float f;
