@@ -24,6 +24,7 @@ CInput
 #include "CInput.h"
 #include "CGUIScrollBarVertical.h"
 
+#include "graphics/FontMetrics.h"
 #include "graphics/ShaderManager.h"
 #include "graphics/TextRenderer.h"
 #include "lib/ogl.h"
@@ -31,7 +32,6 @@ CInput
 #include "lib/timer.h"
 #include "ps/CLogger.h"
 #include "ps/ConfigDB.h"
-#include "ps/Font.h"
 #include "ps/Globals.h"
 #include "ps/Hotkey.h"
 #include "renderer/Renderer.h"
@@ -51,6 +51,8 @@ CInput::CInput()
 	AddSetting(GUIST_CStrW,					"caption");
 	AddSetting(GUIST_int,					"cell_id");
 	AddSetting(GUIST_CStrW,					"font");
+	AddSetting(GUIST_CStrW,					"mask_char");
+	AddSetting(GUIST_bool,					"mask");
 	AddSetting(GUIST_int,					"max_length");
 	AddSetting(GUIST_bool,					"multiline");
 	AddSetting(GUIST_bool,					"scrollbar");
@@ -67,7 +69,6 @@ CInput::CInput()
 	// Add scroll-bar
 	CGUIScrollBarVertical * bar = new CGUIScrollBarVertical();
 	bar->SetRightAligned(true);
-	bar->SetUseEdgeButtons(true);
 	AddScrollBar(bar);
 }
 
@@ -98,7 +99,8 @@ InReaction CInput::ManuallyHandleEvent(const SDL_Event_* ev)
 		{
 			case SDLK_TAB: // '\t'
 				/* Auto Complete */
-				// TODO Gee: (2004-09-07) What to do with tab?
+				// We just send the tab event to JS and let it figure out autocomplete.
+				SendEvent(GUIM_TAB, "tab");
 				break;
 
 			case SDLK_BACKSPACE: // '\b'
@@ -391,7 +393,8 @@ InReaction CInput::ManuallyHandleEvent(const SDL_Event_* ev)
 				break;
 			/* END: Message History Lookup */
 
-			case '\r':
+			case SDLK_KP_ENTER:
+			case SDLK_RETURN:
 				// 'Return' should do a Press event for single liners (e.g. submitting forms)
 				//  otherwise a '\n' character will be added.
 				{
@@ -909,6 +912,7 @@ void CInput::HandleMessage(SGUIMessage &Message)
 			m_SelectingText = false;
 		}
 		break;
+
 	case GUIM_MOUSE_MOTION:
 		// If we just pressed down and started to move before releasing
 		//  this is one way of selecting larger portions of text.
@@ -920,30 +924,10 @@ void CInput::HandleMessage(SGUIMessage &Message)
 				m_SelectingText = false;
 			else
 				m_iBufferPos = GetMouseHoveringTextPosition();
-
 			UpdateAutoScroll();
 		}
-
 		break;
 
-	case GUIM_MOUSE_WHEEL_DOWN:
-		{
-		GetScrollBar(0).ScrollPlus();
-		// Since the scroll was changed, let's simulate a mouse movement
-		//  to check if scrollbar now is hovered
-		SGUIMessage msg(GUIM_MOUSE_MOTION);
-		HandleMessage(msg);
-		break;
-		}
-	case GUIM_MOUSE_WHEEL_UP:
-		{
-		GetScrollBar(0).ScrollMinus();
-		// Since the scroll was changed, let's simulate a mouse movement
-		//  to check if scrollbar now is hovered
-		SGUIMessage msg(GUIM_MOUSE_MOTION);
-		HandleMessage(msg);
-		break;
-		}
 	case GUIM_LOAD:
 		{
 		GetScrollBar(0).SetX( m_CachedActualSize.right );
@@ -1018,9 +1002,11 @@ void CInput::Draw()
 	bool scrollbar;
 	float buffer_zone;
 	bool multiline;
+	bool mask;
 	GUI<bool>::GetSetting(this, "scrollbar", scrollbar);
 	GUI<float>::GetSetting(this, "buffer_zone", buffer_zone);
 	GUI<bool>::GetSetting(this, "multiline", multiline);
+	GUI<bool>::GetSetting(this, "mask", mask);
 
 	if (scrollbar && multiline)
 	{
@@ -1030,16 +1016,27 @@ void CInput::Draw()
 
 	if (GetGUI())
 	{	
-		CStrW font_name;
+		CStrW font_name_w;
 		CColor color, color_selected;
 		//CStrW caption;
-		GUI<CStrW>::GetSetting(this, "font", font_name);
+		GUI<CStrW>::GetSetting(this, "font", font_name_w);
 		GUI<CColor>::GetSetting(this, "textcolor", color);
 		GUI<CColor>::GetSetting(this, "textcolor_selected", color_selected);
+		CStrIntern font_name(font_name_w.ToUTF8());
 		
 		// Get pointer of caption, it might be very large, and we don't
 		//  want to copy it continuously.
-		CStrW *pCaption = (CStrW*)m_Settings["caption"].m_pSetting;
+		CStrW *pCaption = NULL;
+		wchar_t mask_char = L'*';
+		if (mask)
+		{
+			CStrW maskStr;
+			GUI<CStrW>::GetSetting(this, "mask_char", maskStr);
+			if (maskStr.length() > 0)
+				mask_char = maskStr[0];
+		}
+		else
+			pCaption = (CStrW*)m_Settings["caption"].m_pSetting;
 
 		CGUISpriteInstance *sprite=NULL, *sprite_selectarea=NULL;
 		int cell_id;
@@ -1057,7 +1054,7 @@ void CInput::Draw()
 			scroll = GetScrollBar(0).GetPos();
 		}
 
-		CFont font(font_name);
+		CFontMetrics font(font_name);
 
 		// We'll have to setup clipping manually, since we're doing the rendering manually.
 		CRect cliparea(m_CachedActualSize);
@@ -1102,7 +1099,7 @@ void CInput::Draw()
 		float h = (float)font.GetHeight();
 		float ls = (float)font.GetLineSpacing();
 
-		CShaderTechniquePtr tech = g_Renderer.GetShaderManager().LoadEffect("gui_text");
+		CShaderTechniquePtr tech = g_Renderer.GetShaderManager().LoadEffect(str_gui_text);
 		
 		CTextRenderer textRenderer(tech->GetShader());
 		textRenderer.Font(font_name);
@@ -1247,7 +1244,12 @@ void CInput::Draw()
 					}
 
 					if (i < (int)it->m_ListOfX.size())
-						x_pointer += (float)font.GetCharacterWidth((*pCaption)[it->m_ListStart + i]);
+					{
+						if (!mask)
+							x_pointer += (float)font.GetCharacterWidth((*pCaption)[it->m_ListStart + i]);
+						else
+							x_pointer += (float)font.GetCharacterWidth(mask_char);
+					}
 				}
 
 				if (done)
@@ -1338,7 +1340,12 @@ void CInput::Draw()
 					}
 
 					if (i != (int)it->m_ListOfX.size())
-						textRenderer.PrintfAdvance(L"%lc", (*pCaption)[it->m_ListStart + i]);
+					{
+						if (!mask)
+							textRenderer.PrintfAdvance(L"%lc", (*pCaption)[it->m_ListStart + i]);
+						else
+							textRenderer.PrintfAdvance(L"%lc", mask_char);
+					}
 
 					// check it's now outside a one-liner, then we'll break
 					if (!multiline && i < (int)it->m_ListOfX.size())				
@@ -1378,13 +1385,25 @@ void CInput::Draw()
 void CInput::UpdateText(int from, int to_before, int to_after)
 {
 	CStrW caption;
-	CStrW font_name;
+	CStrW font_name_w;
 	float buffer_zone;
 	bool multiline;
-	GUI<CStrW>::GetSetting(this, "font", font_name);
+	bool mask;
+	GUI<CStrW>::GetSetting(this, "font", font_name_w);
 	GUI<CStrW>::GetSetting(this, "caption", caption);
 	GUI<float>::GetSetting(this, "buffer_zone", buffer_zone);
 	GUI<bool>::GetSetting(this, "multiline", multiline);
+	GUI<bool>::GetSetting(this, "mask", mask);
+	CStrIntern font_name(font_name_w.ToUTF8());
+
+	wchar_t mask_char = L'*';
+	if (mask)
+	{
+		CStrW maskStr;
+		GUI<CStrW>::GetSetting(this, "mask_char", maskStr);
+		if (maskStr.length() > 0)
+			mask_char = maskStr[0];
+	}
 
 	// Ensure positions are valid after caption changes
 	m_iBufferPos = std::min(m_iBufferPos, (int)caption.size());
@@ -1406,7 +1425,7 @@ void CInput::UpdateText(int from, int to_before, int to_after)
 	if (to_before == -1)
 		to = (int)caption.length();
 
-	CFont font(font_name);
+	CFontMetrics font(font_name);
 
 	std::list<SRow>::iterator current_line;
 
@@ -1575,7 +1594,10 @@ void CInput::UpdateText(int from, int to_before, int to_after)
 				caption[i] == L'-'*/)
 				last_word_started = i+1;
 
-			x_pos += (float)font.GetCharacterWidth(caption[i]);
+			if (!mask)
+				x_pos += (float)font.GetCharacterWidth(caption[i]);
+			else
+				x_pos += (float)font.GetCharacterWidth(mask_char);
 
 			if (x_pos >= GetTextAreaWidth() && multiline)
 			{
@@ -1765,10 +1787,11 @@ int CInput::GetMouseHoveringTextPosition()
 
 	if (multiline)
 	{
-		CStrW font_name;
+		CStrW font_name_w;
 		bool scrollbar;
-		GUI<CStrW>::GetSetting(this, "font", font_name);
+		GUI<CStrW>::GetSetting(this, "font", font_name_w);
 		GUI<bool>::GetSetting(this, "scrollbar", scrollbar);
+		CStrIntern font_name(font_name_w.ToUTF8());
 		
 		float scroll=0.f;
 		if (scrollbar)
@@ -1782,7 +1805,7 @@ int CInput::GetMouseHoveringTextPosition()
 
 		// Now get the height of the font.
 						// TODO: Get the real font
-		CFont font(font_name);
+		CFontMetrics font(font_name);
 		float spacing = (float)font.GetLineSpacing();
 		//float height = (float)font.GetHeight();	// unused
 
@@ -1917,10 +1940,11 @@ void CInput::UpdateAutoScroll()
 	// Autoscrolling up and down
 	if (multiline)
 	{
-		CStrW font_name;
+		CStrW font_name_w;
 		bool scrollbar;
-		GUI<CStrW>::GetSetting(this, "font", font_name);
+		GUI<CStrW>::GetSetting(this, "font", font_name_w);
 		GUI<bool>::GetSetting(this, "scrollbar", scrollbar);
+		CStrIntern font_name(font_name_w.ToUTF8());
 		
 		float scroll=0.f;
 		if (!scrollbar)
@@ -1930,7 +1954,7 @@ void CInput::UpdateAutoScroll()
 		
 		// Now get the height of the font.
 						// TODO: Get the real font
-		CFont font(font_name);
+		CFontMetrics font(font_name);
 		float spacing = (float)font.GetLineSpacing();
 		//float height = font.GetHeight();
 
