@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2014 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -22,6 +22,7 @@
 #include "graphics/ShaderManager.h"
 #include "graphics/TextureManager.h"
 #include "gui/GUIutil.h"
+#include "i18n/L10n.h"
 #include "lib/ogl.h"
 #include "lib/utf8.h"
 #include "lib/res/h_mgr.h"
@@ -56,7 +57,7 @@ DrawCalls& DrawCalls::operator=(const DrawCalls&)
 }
 
 
-void GUIRenderer::UpdateDrawCallCache(DrawCalls &Calls, const CStr& SpriteName, const CRect &Size, int CellID, std::map<CStr, CGUISprite> &Sprites)
+void GUIRenderer::UpdateDrawCallCache(DrawCalls &Calls, const CStr& SpriteName, const CRect &Size, int CellID, std::map<CStr, CGUISprite*> &Sprites)
 {
 	// This is called only when something has changed (like the size of the
 	// sprite), so it doesn't need to be particularly efficient.
@@ -71,7 +72,7 @@ void GUIRenderer::UpdateDrawCallCache(DrawCalls &Calls, const CStr& SpriteName,
 		return;
 
 
-	std::map<CStr, CGUISprite>::iterator it (Sprites.find(SpriteName));
+	std::map<CStr, CGUISprite*>::iterator it (Sprites.find(SpriteName));
 	if (it == Sprites.end())
 	{
 		// Sprite not found. Check whether this a special sprite:
@@ -84,26 +85,26 @@ void GUIRenderer::UpdateDrawCallCache(DrawCalls &Calls, const CStr& SpriteName,
 		if (SpriteName.substr(0, 10) == "stretched:")
 		{
 			// TODO: Should check (nicely) that this is a valid file?
-			SGUIImage Image;
+			SGUIImage* Image = new SGUIImage;
 			
 			// Allow grayscale images for disabled portraits
 			if (SpriteName.substr(10, 10) == "grayscale:")
 			{
-				Image.m_TextureName = VfsPath("art/textures/ui") / wstring_from_utf8(SpriteName.substr(20));
-				Image.m_Effects = new SGUIImageEffects;
-				Image.m_Effects->m_Greyscale = true;
+				Image->m_TextureName = VfsPath("art/textures/ui") / wstring_from_utf8(SpriteName.substr(20));
+				Image->m_Effects = new SGUIImageEffects;
+				Image->m_Effects->m_Greyscale = true;
 			}
 			else
 			{
-				Image.m_TextureName = VfsPath("art/textures/ui") / wstring_from_utf8(SpriteName.substr(10));
+				Image->m_TextureName = VfsPath("art/textures/ui") / wstring_from_utf8(SpriteName.substr(10));
 			}
 
 			CClientArea ca(CRect(0, 0, 0, 0), CRect(0, 0, 100, 100));
-			Image.m_Size = ca;
-			Image.m_TextureSize = ca;
+			Image->m_Size = ca;
+			Image->m_TextureSize = ca;
 
-			CGUISprite Sprite;
-			Sprite.AddImage(Image);
+			CGUISprite* Sprite = new CGUISprite;
+			Sprite->AddImage(Image);
 
 			Sprites[SpriteName] = Sprite;
 			
@@ -113,22 +114,22 @@ void GUIRenderer::UpdateDrawCallCache(DrawCalls &Calls, const CStr& SpriteName,
 		else if (SpriteName.substr(0, 8) == "cropped:")
 		{
 			// TODO: Should check (nicely) that this is a valid file?
-			SGUIImage Image;
+			SGUIImage* Image = new SGUIImage;
 
 			double xRatio = SpriteName.BeforeFirst(",").AfterLast("(").ToDouble();
 			double yRatio = SpriteName.BeforeFirst(")").AfterLast(",").ToDouble();
 			
 			int PathStart = SpriteName.Find(")") + 1;
 			
-			Image.m_TextureName = VfsPath("art/textures/ui") / wstring_from_utf8(SpriteName.substr(PathStart));
+			Image->m_TextureName = VfsPath("art/textures/ui") / wstring_from_utf8(SpriteName.substr(PathStart));
 
 			CClientArea ca(CRect(0, 0, 0, 0), CRect(0, 0, 100, 100));
 			CClientArea cb(CRect(0, 0, 0, 0), CRect(0, 0, 100/xRatio, 100/yRatio));
-			Image.m_Size = ca;
-			Image.m_TextureSize = cb;
+			Image->m_Size = ca;
+			Image->m_TextureSize = cb;
 
-			CGUISprite Sprite;
-			Sprite.AddImage(Image);
+			CGUISprite* Sprite = new CGUISprite;
+			Sprite->AddImage(Image);
 
 			Sprites[SpriteName] = Sprite;
 			
@@ -147,16 +148,16 @@ void GUIRenderer::UpdateDrawCallCache(DrawCalls &Calls, const CStr& SpriteName,
 				return;
 			}
 
-			SGUIImage image;
+			SGUIImage* Image = new SGUIImage;
 
-			image.m_BackColor = color;
+			Image->m_BackColor = color;
 
 			CClientArea ca(CRect(0, 0, 0, 0), CRect(0, 0, 100, 100));
-			image.m_Size = ca;
-			image.m_TextureSize = ca;
+			Image->m_Size = ca;
+			Image->m_TextureSize = ca;
 
-			CGUISprite Sprite;
-			Sprite.AddImage(image);
+			CGUISprite* Sprite = new CGUISprite;
+			Sprite->AddImage(Image);
 
 			Sprites[SpriteName] = Sprite;
 
@@ -171,16 +172,16 @@ void GUIRenderer::UpdateDrawCallCache(DrawCalls &Calls, const CStr& SpriteName,
 		}
 	}
 
-	Calls.reserve(it->second.m_Images.size());
+	Calls.reserve(it->second->m_Images.size());
 
 	// Iterate through all the sprite's images, loading the texture and
 	// calculating the texture coordinates
-	std::vector<SGUIImage>::const_iterator cit;
-	for (cit = it->second.m_Images.begin(); cit != it->second.m_Images.end(); ++cit)
+	std::vector<SGUIImage*>::const_iterator cit;
+	for (cit = it->second->m_Images.begin(); cit != it->second->m_Images.end(); ++cit)
 	{
-		SDrawCall Call(&*cit); // pointers are safe since we never modify sprites/images after startup
+		SDrawCall Call(*cit); // pointers are safe since we never modify sprites/images after startup
 
-		CRect ObjectSize = cit->m_Size.GetClientArea(Size);
+		CRect ObjectSize = (*cit)->m_Size.GetClientArea(Size);
 
 		if (ObjectSize.GetWidth() == 0.0 || ObjectSize.GetHeight() == 0.0)
 		{
@@ -189,7 +190,7 @@ void GUIRenderer::UpdateDrawCallCache(DrawCalls &Calls, const CStr& SpriteName,
 		}
 
 		Call.m_Vertices = ObjectSize;
-		if (cit->m_RoundCoordinates)
+		if ((*cit)->m_RoundCoordinates)
 		{
 			// Round the vertex coordinates to integers, to avoid ugly filtering artifacts
 			Call.m_Vertices.left = (int)(Call.m_Vertices.left + 0.5f);
@@ -198,10 +199,10 @@ void GUIRenderer::UpdateDrawCallCache(DrawCalls &Calls, const CStr& SpriteName,
 			Call.m_Vertices.bottom = (int)(Call.m_Vertices.bottom + 0.5f);
 		}
 
-		if (! cit->m_TextureName.empty())
+		if (!(*cit)->m_TextureName.empty())
 		{
-			CTextureProperties textureProps(cit->m_TextureName);
-			textureProps.SetWrap(cit->m_WrapMode);
+			CTextureProperties textureProps(L10n::Instance().LocalizePath((*cit)->m_TextureName));
+			textureProps.SetWrap((*cit)->m_WrapMode);
 			CTexturePtr texture = g_Renderer.GetTextureManager().CreateTexture(textureProps);
 			texture->Prefetch();
 			Call.m_HasTexture = true;
@@ -216,40 +217,40 @@ void GUIRenderer::UpdateDrawCallCache(DrawCalls &Calls, const CStr& SpriteName,
 		{
 			Call.m_HasTexture = false;
 			// Enable blending if it's transparent (allowing a little error in the calculations)
-			Call.m_EnableBlending = !(fabs(cit->m_BackColor.a - 1.0f) < 0.0000001f);
+			Call.m_EnableBlending = !(fabs((*cit)->m_BackColor.a - 1.0f) < 0.0000001f);
 		}
 
-		Call.m_BackColor = cit->m_BackColor;
-		Call.m_BorderColor = cit->m_Border ? cit->m_BorderColor : CColor();
-		Call.m_DeltaZ = cit->m_DeltaZ;
+		Call.m_BackColor = (*cit)->m_BackColor;
+		Call.m_BorderColor = (*cit)->m_Border ? (*cit)->m_BorderColor : CColor();
+		Call.m_DeltaZ = (*cit)->m_DeltaZ;
 
 		if (!Call.m_HasTexture)
 		{
-			Call.m_Shader = g_Renderer.GetShaderManager().LoadEffect("gui_solid");
+			Call.m_Shader = g_Renderer.GetShaderManager().LoadEffect(str_gui_solid);
 		}
-		else if (cit->m_Effects)
+		else if ((*cit)->m_Effects)
 		{
-			if (cit->m_Effects->m_AddColor != CColor())
+			if ((*cit)->m_Effects->m_AddColor != CColor())
 			{
-				Call.m_Shader = g_Renderer.GetShaderManager().LoadEffect("gui_add");
-				Call.m_ShaderColorParameter = cit->m_Effects->m_AddColor;
+				Call.m_Shader = g_Renderer.GetShaderManager().LoadEffect(str_gui_add);
+				Call.m_ShaderColorParameter = (*cit)->m_Effects->m_AddColor;
 				// Always enable blending if something's being subtracted from
 				// the alpha channel
-				if (cit->m_Effects->m_AddColor.a < 0.f)
+				if ((*cit)->m_Effects->m_AddColor.a < 0.f)
 					Call.m_EnableBlending = true;
 			}
-			else if (cit->m_Effects->m_Greyscale)
+			else if ((*cit)->m_Effects->m_Greyscale)
 			{
-				Call.m_Shader = g_Renderer.GetShaderManager().LoadEffect("gui_grayscale");
+				Call.m_Shader = g_Renderer.GetShaderManager().LoadEffect(str_gui_grayscale);
 			}
 			else /* Slight confusion - why no effects? */
 			{
-				Call.m_Shader = g_Renderer.GetShaderManager().LoadEffect("gui_basic");
+				Call.m_Shader = g_Renderer.GetShaderManager().LoadEffect(str_gui_basic);
 			}
 		}
 		else
 		{
-			Call.m_Shader = g_Renderer.GetShaderManager().LoadEffect("gui_basic");
+			Call.m_Shader = g_Renderer.GetShaderManager().LoadEffect(str_gui_basic);
 		}
 
 		Calls.push_back(Call);
@@ -350,12 +351,12 @@ void GUIRenderer::Draw(DrawCalls &Calls, float Z)
 	{
 		cit->m_Shader->BeginPass();
 		CShaderProgramPtr shader = cit->m_Shader->GetShader();
-		shader->Uniform("transform", matrix);
+		shader->Uniform(str_transform, matrix);
 
 		if (cit->m_HasTexture)
 		{
-			shader->Uniform("color", cit->m_ShaderColorParameter);
-			shader->BindTexture("tex", cit->m_Texture);
+			shader->Uniform(str_color, cit->m_ShaderColorParameter);
+			shader->BindTexture(str_tex, cit->m_Texture);
 
 			if (cit->m_EnableBlending || cit->m_Texture->HasAlpha()) // (shouldn't call HasAlpha before BindTexture)
 			{
@@ -395,7 +396,7 @@ void GUIRenderer::Draw(DrawCalls &Calls, float Z)
 		}
 		else
 		{
-			shader->Uniform("color", cit->m_BackColor);
+			shader->Uniform(str_color, cit->m_BackColor);
 
 			if (cit->m_EnableBlending)
 			{
@@ -425,7 +426,7 @@ void GUIRenderer::Draw(DrawCalls &Calls, float Z)
 
 			if (cit->m_BorderColor != CColor())
 			{
-				shader->Uniform("color", cit->m_BorderColor);
+				shader->Uniform(str_color, cit->m_BorderColor);
 
 				data.clear();
 				ADD(Verts.left + 0.5f, Verts.top + 0.5f, Z + cit->m_DeltaZ);
