@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2014 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -22,16 +22,25 @@ GUI text
 #include "precompiled.h"
 
 #include "GUI.h"
-#include "GUIManager.h"
+#include "graphics/FontMetrics.h"
 #include "ps/CLogger.h"
 #include "ps/Parser.h"
 #include <algorithm>
 
-#include "ps/Font.h"
-
 
 static const wchar_t TagStart = '[';
 static const wchar_t TagEnd   = ']';
+// List of word demlimitor bounds
+// The list contains ranges of word delimitors. The odd indexed chars are the start
+// of a range, the even are the end of a range. The list must be sorted in INCREASING ORDER
+static const int NUM_WORD_DELIMITORS = 4*2;
+static const u16 WordDelimitors[NUM_WORD_DELIMITORS] = {
+	' '   , ' ',    // spaces
+	'-'   , '-',    // hyphens
+	0x3000, 0x31FF, // ideographic symbols
+	0x3400, 0x9FFF
+// TODO add unicode blocks of other languages that don't use spaces
+};
 
 void CGUIString::SFeedback::Reset()
 {
@@ -43,8 +52,39 @@ void CGUIString::SFeedback::Reset()
 	m_NewLine=false;
 }
 
-void CGUIString::GenerateTextCall(SFeedback &Feedback,
-								  const CStrW& DefaultFont,
+// TODO this method should be removed at one point
+// It is a solution for CParser not recognising backslash-escaped quoted
+CStrW UnescapeString(CStrW originalString)
+{
+	// no reason to mess with more memory if there is nothing to unescape
+	if (originalString.Find('&') == -1)
+		return originalString;
+	// clone the string
+	CStrW newString = CStrW();
+	for (size_t i=0; i < originalString.length(); ++i)
+	{
+		if (originalString[i] == '&' && originalString[i+1] == '#')
+		{
+			i += 2;
+			wchar_t c = 0;
+			while (originalString[i] >= '0' && originalString[i] <= '9')
+			{
+				c = c*10 + (originalString[i] - '0');
+				++i;
+			}
+			ENSURE(originalString[i] == ';');
+			if (c >= ' ')
+				newString += c;
+		}
+		else
+			newString += originalString[i];
+	}
+	return newString;
+}
+
+void CGUIString::GenerateTextCall(const CGUI *pGUI,
+								  SFeedback &Feedback,
+								  CStrIntern DefaultFont,
 								  const int &from, const int &to,
 								  const bool FirstLine,
 								  const IGUIObject *pObject) const
@@ -63,7 +103,7 @@ void CGUIString::GenerateTextCall(SFeedback &Feedback,
 		int _from, _to;
 		_from = std::max(from, itTextChunk->m_From);
 		_to = std::min(to, itTextChunk->m_To);
-		
+
 		// If from is larger than to, than they are not overlapping
 		if (_to == _from && itTextChunk->m_From == itTextChunk->m_To)
 		{
@@ -75,8 +115,8 @@ void CGUIString::GenerateTextCall(SFeedback &Feedback,
 			//  in the words-list, it can be counted twice if placed on an
 			//  edge. But there is always only one logical preference that
 			//  we want. This check filters the unwanted.
-			
-			// it's in the end of one word, and the icon 
+
+			// it's in the end of one word, and the icon
 			//  should really belong to the beginning of the next one
 			if (_to == to && to >= 1)
 			{
@@ -88,8 +128,8 @@ void CGUIString::GenerateTextCall(SFeedback &Feedback,
 			// This std::string is just a break
 			if (_from == from && from >= 1)
 			{
-				if (GetRawString()[from] == '\n' && 
-					GetRawString()[from-1] != '\n' && 
+				if (GetRawString()[from] == '\n' &&
+					GetRawString()[from-1] != '\n' &&
 					GetRawString()[from-1] != ' ' &&
 					GetRawString()[from-1] != '-')
 					continue;
@@ -99,7 +139,7 @@ void CGUIString::GenerateTextCall(SFeedback &Feedback,
 			if (itTextChunk->m_Tags[0].m_TagType == CGUIString::TextChunk::Tag::TAG_IMGLEFT)
 			{
 				// Only add the image if the icon exists.
-				if (g_GUI->IconExists(itTextChunk->m_Tags[0].m_TagValue))
+				if (pGUI->IconExists(itTextChunk->m_Tags[0].m_TagValue))
 				{
 					Feedback.m_Images[SFeedback::Left].push_back(itTextChunk->m_Tags[0].m_TagValue);
 				}
@@ -112,7 +152,7 @@ void CGUIString::GenerateTextCall(SFeedback &Feedback,
 			if (itTextChunk->m_Tags[0].m_TagType == CGUIString::TextChunk::Tag::TAG_IMGRIGHT)
 			{
 				// Only add the image if the icon exists.
-				if (g_GUI->IconExists(itTextChunk->m_Tags[0].m_TagValue))
+				if (pGUI->IconExists(itTextChunk->m_Tags[0].m_TagValue))
 				{
 					Feedback.m_Images[SFeedback::Right].push_back(itTextChunk->m_Tags[0].m_TagValue);
 				}
@@ -125,7 +165,7 @@ void CGUIString::GenerateTextCall(SFeedback &Feedback,
 			if (itTextChunk->m_Tags[0].m_TagType == CGUIString::TextChunk::Tag::TAG_ICON)
 			{
 				// Only add the image if the icon exists.
-				if (g_GUI->IconExists(itTextChunk->m_Tags[0].m_TagValue))
+				if (pGUI->IconExists(itTextChunk->m_Tags[0].m_TagValue))
 				{
 					// We'll need to setup a text-call that will point
 					//  to the icon, this is to be able to iterate
@@ -136,8 +176,8 @@ void CGUIString::GenerateTextCall(SFeedback &Feedback,
 					// Also add it to the sprites being rendered.
 					SGUIText::SSpriteCall SpriteCall;
 
-					// Get Icon from icon database in g_GUI
-					SGUIIcon icon = g_GUI->GetIcon(itTextChunk->m_Tags[0].m_TagValue);
+					// Get Icon from icon database in pGUI
+					SGUIIcon icon = pGUI->GetIcon(itTextChunk->m_Tags[0].m_TagValue);
 
 					CSize size = icon.m_Size;
 
@@ -203,8 +243,8 @@ void CGUIString::GenerateTextCall(SFeedback &Feedback,
 			TextCall.m_UseCustomColor = false;
 
 			// Extract substd::string from RawString.
-			TextCall.m_String = GetRawString().substr(_from, _to-_from);
-			
+			TextCall.m_String = UnescapeString(GetRawString().substr(_from, _to-_from));
+
 			// Go through tags and apply changes.
 			std::vector<CGUIString::TextChunk::Tag>::const_iterator it2;
 			for (it2 = itTextChunk->m_Tags.begin(); it2 != itTextChunk->m_Tags.end(); ++it2)
@@ -213,7 +253,7 @@ void CGUIString::GenerateTextCall(SFeedback &Feedback,
 				{
 					// Set custom color
 					TextCall.m_UseCustomColor = true;
-					
+
 					// Try parsing the color std::string
 					if (!GUI<CColor>::ParseString(CStr(it2->m_TagValue).FromUTF8(), TextCall.m_Color))
 					{
@@ -225,14 +265,14 @@ void CGUIString::GenerateTextCall(SFeedback &Feedback,
 				if (it2->m_TagType == CGUIString::TextChunk::Tag::TAG_FONT)
 				{
 					// TODO Gee: (2004-08-15) Check if Font exists?
-					TextCall.m_Font = CStr(it2->m_TagValue).FromUTF8();
+					TextCall.m_Font = CStrIntern(it2->m_TagValue);
 				}
 			}
 
 			// Calculate the size of the font
 			CSize size;
 			int cx, cy;
-			CFont font (TextCall.m_Font);
+			CFontMetrics font (TextCall.m_Font);
 			font.CalculateStringSize(TextCall.m_String.c_str(), cx, cy);
 			// For anything other than the first line, the line spacing
 			// needs to be considered rather than just the height of the text
@@ -265,20 +305,8 @@ void CGUIString::GenerateTextCall(SFeedback &Feedback,
 
 bool CGUIString::TextChunk::Tag::SetTagType(const CStr& tagtype)
 {
-	CStr _tagtype = tagtype.UpperCase();	
+	CStr _tagtype = tagtype.UpperCase();
 
-	if (_tagtype == CStr("B"))
-	{
-		m_TagType = TAG_B;
-		return true;
-	}
-	else
-	if (_tagtype == CStr("I"))
-	{
-		m_TagType = TAG_I;
-		return true;
-	}
-	else
 	if (_tagtype == CStr("COLOR"))
 	{
 		m_TagType = TAG_COLOR;
@@ -412,9 +440,9 @@ void CGUIString::SetValue(const CStrW& str)
 							//Handle arbitrary number of additional parameters
 							size_t argn;
 							for(argn = 2; argn < Line.GetArgCount(); argn += 2)
-							{	
+							{
 								TextChunk::Tag::TagAttribute a;
-								
+
 								Line.GetArgString(argn, a.attrib);
 								Line.GetArgString(argn+1, a.value);
 
@@ -462,7 +490,7 @@ void CGUIString::SetValue(const CStrW& str)
 										break;
 									}
 								}
-		
+
 								// Add!
 								CurrentTextChunk.m_Tags.push_back(tag);
 							}
@@ -493,7 +521,7 @@ void CGUIString::SetValue(const CStrW& str)
 							}
 							from_nonraw = pos_right+1;
 
-							// Search for the tag, if it's not added, then 
+							// Search for the tag, if it's not added, then
 							//  pass it as plain text.
 							std::vector<TextChunk::Tag>::iterator it;
 							for (it = CurrentTextChunk.m_Tags.begin(); it != CurrentTextChunk.m_Tags.end(); ++it)
@@ -526,54 +554,32 @@ void CGUIString::SetValue(const CStrW& str)
 	// Add a delimiter at start and at end, it helps when
 	//  processing later, because we don't have make exceptions for
 	//  those cases.
-	// We'll sort later.
 	m_Words.push_back(0);
+
+	// Add word boundaries in increasing order
+	for (u32 i = 0; i < m_RawString.length(); ++i)
+	{
+		wchar_t c = m_RawString[i];
+		if (c == '\n')
+		{
+			m_Words.push_back((int)i);
+			m_Words.push_back((int)i+1);
+			continue;
+		}
+		for (int n = 0; n < NUM_WORD_DELIMITORS; n += 2)
+		{
+			if (c <= WordDelimitors[n+1])
+			{
+				if (c >= WordDelimitors[n])
+					m_Words.push_back((int)i+1);
+				// assume the WordDelimitors list is stored in increasing order
+				break;
+			}
+		}
+	}
+
 	m_Words.push_back((int)m_RawString.length());
 
-	// Space: ' '
-	for (position=0, curpos=0;;position = curpos+1)
-	{
-		// Find the next word-delimiter.
-		long dl = m_RawString.Find(position, ' ');
-
-		if (dl == -1)
-			break;
-
-		curpos = dl;
-		m_Words.push_back((int)dl+1);
-	}
-
-	// Dash: '-'
-	for (position=0, curpos=0;;position = curpos+1)
-	{
-		// Find the next word-delimiter.
-		long dl = m_RawString.Find(position, '-');
-
-		if (dl == -1)
-			break;
-
-		curpos = dl;
-		m_Words.push_back((int)dl+1);
-	}
-
-	// New Line: '\n'
-	for (position=0, curpos=0;;position = curpos+1)
-	{
-		// Find the next word-delimiter.
-		long dl = m_RawString.Find(position, '\n');
-
-		if (dl == -1)
-			break;
-
-		curpos = dl;
-
-		// Add before and 
-		m_Words.push_back((int)dl);
-		m_Words.push_back((int)dl+1);
-	}
-
-	sort(m_Words.begin(), m_Words.end());
-
 	// Remove duplicates (only if larger than 2)
 	if (m_Words.size() > 2)
 	{
