@@ -22,7 +22,6 @@ IGUIScrollBar
 #include "precompiled.h"
 #include "GUI.h"
 #include "maths/MathUtil.h"
-#include "ps/CLogger.h"
 
 //-------------------------------------------------------------------
 //	IGUIScrollBar
@@ -32,7 +31,6 @@ IGUIScrollBar::IGUIScrollBar() : m_pStyle(NULL), m_pGUI(NULL),
 								 m_ScrollRange(1.f), m_ScrollSpace(0.f), // MaxPos: not 0, due to division.
 								 m_Length(200.f), m_Width(20.f), 
 								 m_BarSize(0.f), m_Pos(0.f),
-								 m_UseEdgeButtons(true),
 								 m_ButtonPlusPressed(false),
 								 m_ButtonMinusPressed(false),
 								 m_ButtonPlusHovered(false),
@@ -48,20 +46,19 @@ IGUIScrollBar::~IGUIScrollBar()
 
 void IGUIScrollBar::SetupBarSize()
 {
+	if (!GetStyle())
+		return;
 	float min = GetStyle()->m_MinimumBarSize;
 	float max = GetStyle()->m_MaximumBarSize;
 	float length = m_Length;
 
 	// Check for edge buttons
-	if (m_UseEdgeButtons)
+	if (GetStyle()->m_UseEdgeButtons)
 		length -= GetStyle()->m_Width * 2.f;
 
 	// Check min and max are valid
 	if (min > length)
-	{
-		LOGWARNING(L"Minimum scrollbar size of %g is larger than the total scrollbar size of %g", min, length);
 		min = 0.f;
-	}
 	if (max < min)
 		max = length;
 
@@ -74,7 +71,7 @@ const SGUIScrollBarStyle *IGUIScrollBar::GetStyle() const
 	if (!m_pHostObject)
 		return NULL;
 
-    return m_pHostObject->GetScrollBarStyle(m_ScrollBarStyle);
+	return m_pHostObject->GetScrollBarStyle(m_ScrollBarStyle);
 }
 
 CGUI *IGUIScrollBar::GetGUI() const 
@@ -99,13 +96,13 @@ void IGUIScrollBar::HandleMessage(SGUIMessage &Message)
 {
 	switch (Message.type)
 	{
-	case GUIM_MOUSE_MOTION:
+		case GUIM_MOUSE_MOTION:
 		{
 			// TODO Gee: Optimizations needed!
 
 			CPos mouse = m_pHostObject->GetMousePos();
 
-			// If bar is being dragged
+				// If bar is being dragged
 			if (m_BarPressed)
 			{
 				SetPosFromMousePos(mouse);
@@ -122,9 +119,10 @@ void IGUIScrollBar::HandleMessage(SGUIMessage &Message)
 
 			if (!m_ButtonPlusHovered)
 				m_ButtonPlusPressed = false;
-		} break;
+			break;
+		}
 
-	case GUIM_MOUSE_PRESS_LEFT:
+		case GUIM_MOUSE_PRESS_LEFT:
 		{
 			if (!m_pHostObject)
 				break;
@@ -165,20 +163,42 @@ void IGUIScrollBar::HandleMessage(SGUIMessage &Message)
 						ScrollMinusPlenty();
 					else
 						ScrollPlusPlenty();
-
-					// Simulate mouse movement to see if bar now is hovered
+						// Simulate mouse movement to see if bar now is hovered
 					SGUIMessage msg(GUIM_MOUSE_MOTION);
 					HandleMessage(msg);
 				}
 			}
-		} break;
+			break;
+		}
 
-	case GUIM_MOUSE_RELEASE_LEFT:
-		m_ButtonMinusPressed = false;
-		m_ButtonPlusPressed = false;
-		break;
+		case GUIM_MOUSE_RELEASE_LEFT:
+		{
+			m_ButtonMinusPressed = false;
+			m_ButtonPlusPressed = false;
+			break;
+		}
 
-	default:
-		break;
+		case GUIM_MOUSE_WHEEL_UP:
+		{
+			ScrollMinus();
+			// Since the scroll was changed, let's simulate a mouse movement
+			//  to check if scrollbar now is hovered
+			SGUIMessage msg(GUIM_MOUSE_MOTION);
+			HandleMessage(msg);
+			break;
+		}
+
+		case GUIM_MOUSE_WHEEL_DOWN:
+		{
+			ScrollPlus();
+			// Since the scroll was changed, let's simulate a mouse movement
+			//  to check if scrollbar now is hovered
+			SGUIMessage msg(GUIM_MOUSE_MOTION);
+			HandleMessage(msg);
+			break;
+		}
+
+		default:
+			break;
 	}
 }
