@@ -28,10 +28,13 @@
 #include "graphics/TerrainTextureEntry.h"
 #include "graphics/TerrainTextureManager.h"
 #include "graphics/TerritoryTexture.h"
+#include "gui/GUI.h"
+#include "gui/GUIManager.h"
 #include "lib/ogl.h"
 #include "lib/external_libraries/libsdl.h"
 #include "lib/bits.h"
 #include "lib/timer.h"
+#include "ps/ConfigDB.h"
 #include "ps/Game.h"
 #include "ps/Profile.h"
 #include "ps/World.h"
@@ -53,12 +56,13 @@ static unsigned int ScaleColor(unsigned int color, float x)
 	unsigned int r = unsigned(float(color & 0xff) * x);
 	unsigned int g = unsigned(float((color>>8) & 0xff) * x);
 	unsigned int b = unsigned(float((color>>16) & 0xff) * x);
-	return (0xff000000 | r | g<<8 | b<<16);
+	return (0xff000000 | b | g<<8 | r<<16);
 }
 
 CMiniMap::CMiniMap() :
 	m_TerrainTexture(0), m_TerrainData(0), m_MapSize(0), m_Terrain(0), m_TerrainDirty(true), m_MapScale(1.f),
-	m_EntitiesDrawn(0), m_IndexArray(GL_STATIC_DRAW), m_VertexArray(GL_DYNAMIC_DRAW)
+	m_EntitiesDrawn(0), m_IndexArray(GL_STATIC_DRAW), m_VertexArray(GL_DYNAMIC_DRAW),
+	m_NextBlinkTime(0.0), m_PingDuration(25.0), m_BlinkState(false)
 {
 	AddSetting(GUIST_CColor,	"fov_wedge_color");
 	AddSetting(GUIST_CStrW,		"tooltip");
@@ -114,6 +118,16 @@ CMiniMap::CMiniMap() :
 		
 	}
 	m_VertexArray.Upload();
+
+	double blinkDuration = 1.0;
+
+	// Tests won't have config initialised
+	if (CConfigDB::IsInitialised())
+	{
+		CFG_GET_VAL("gui.session.minimap.pingduration", Double, m_PingDuration);
+		CFG_GET_VAL("gui.session.minimap.blinkduration", Double, blinkDuration);
+	}
+	m_HalfBlinkDuration = blinkDuration/2;
 }
 
 CMiniMap::~CMiniMap()
@@ -137,18 +151,14 @@ void CMiniMap::HandleMessage(SGUIMessage &Message)
 	case GUIM_MOUSE_RELEASE_LEFT:
 		{
 			if(m_MouseHovering && m_Clicking)
-			{
 				SetCameraPos();
-			}
 			m_Clicking = false;
 			break;
 		}
 	case GUIM_MOUSE_DBLCLICK_LEFT:
 		{
 			if(m_MouseHovering && m_Clicking)
-			{
 				SetCameraPos();
-			}
 			m_Clicking = false;
 			break;
 		}
@@ -176,9 +186,7 @@ void CMiniMap::HandleMessage(SGUIMessage &Message)
 	case GUIM_MOUSE_MOTION:
 		{
 			if (m_MouseHovering && m_Clicking)
-			{
 				SetCameraPos();
-			}
 			break;
 		}
 	case GUIM_MOUSE_WHEEL_DOWN:
@@ -191,6 +199,21 @@ void CMiniMap::HandleMessage(SGUIMessage &Message)
 	}	// switch
 }
 
+bool CMiniMap::MouseOver()
+{
+	// Get the mouse position.
+	CPos mousePos = GetMousePos();
+	// Get the position of the center of the minimap.
+	CPos minimapCenter = CPos(m_CachedActualSize.left + m_CachedActualSize.GetWidth() / 2.0, m_CachedActualSize.bottom - m_CachedActualSize.GetHeight() / 2.0);
+	// Take the magnitude of the difference of the mouse position and minimap center.
+	double distFromCenter = sqrt(pow((mousePos.x - minimapCenter.x), 2) + pow((mousePos.y - minimapCenter.y), 2));
+	// If the distance is less then the radius of the minimap (half the width) the mouse is over the minimap.
+	if (distFromCenter < m_CachedActualSize.GetWidth() / 2.0)
+		return true;
+	else
+		return false;
+}
+
 void CMiniMap::GetMouseWorldCoordinates(float& x, float& z)
 {
 	// Determine X and Z according to proportion of mouse position and minimap
@@ -229,9 +252,9 @@ void CMiniMap::FireWorldClickEvent(int button, int clicks)
 	GetMouseWorldCoordinates(x, z);
 
 	CScriptValRooted coords;
-	g_ScriptingHost.GetScriptInterface().Eval("({})", coords);
-	g_ScriptingHost.GetScriptInterface().SetProperty(coords.get(), "x", x, false);
-	g_ScriptingHost.GetScriptInterface().SetProperty(coords.get(), "z", z, false);
+	g_GUI->GetActiveGUI()->GetScriptInterface()->Eval("({})", coords);
+	g_GUI->GetActiveGUI()->GetScriptInterface()->SetProperty(coords.get(), "x", x, false);
+	g_GUI->GetActiveGUI()->GetScriptInterface()->SetProperty(coords.get(), "z", z, false);
 	ScriptEvent("worldclick", coords);
 
 	UNUSED2(button);
@@ -302,7 +325,25 @@ struct MinimapUnitVertex
 	float x, y;
 };
 
-void CMiniMap::DrawTexture(float coordMax, float angle, float x, float y, float x2, float y2, float z)
+// Adds a vertex to the passed VertexArray
+static void inline addVertex(const MinimapUnitVertex& v,
+					  VertexArrayIterator<u8[4]>& attrColor,
+					  VertexArrayIterator<float[2]>& attrPos)
+{
+	(*attrColor)[0] = v.r;
+	(*attrColor)[1] = v.g;
+	(*attrColor)[2] = v.b;
+	(*attrColor)[3] = v.a;
+	++attrColor;
+
+	(*attrPos)[0] = v.x;
+	(*attrPos)[1] = v.y;
+
+	++attrPos;
+}
+
+
+void CMiniMap::DrawTexture(CShaderProgramPtr shader, float coordMax, float angle, float x, float y, float x2, float y2, float z)
 {
 	// Rotate the texture coordinates (0,0)-(coordMax,coordMax) around their center point (m,m)
 	// Scale square maps to fit in circular minimap area
@@ -310,16 +351,47 @@ void CMiniMap::DrawTexture(float coordMax, float angle, float x, float y, float
 	const float c = cos(angle) * m_MapScale;
 	const float m = coordMax / 2.f;
 
-	glBegin(GL_QUADS);
-	glTexCoord2f(m*(-c + s + 1.f), m*(-c + -s + 1.f));
-	glVertex3f(x, y, z);
-	glTexCoord2f(m*(c + s + 1.f), m*(-c + s + 1.f));
-	glVertex3f(x2, y, z);
-	glTexCoord2f(m*(c + -s + 1.f), m*(c + s + 1.f));
-	glVertex3f(x2, y2, z);
-	glTexCoord2f(m*(-c + -s + 1.f), m*(c + -s + 1.f));
-	glVertex3f(x, y2, z);
-	glEnd();
+	float quadTex[] = {
+		m*(-c + s + 1.f), m*(-c + -s + 1.f),
+		m*(c + s + 1.f), m*(-c + s + 1.f),
+		m*(c + -s + 1.f), m*(c + s + 1.f),
+
+		m*(c + -s + 1.f), m*(c + s + 1.f),
+		m*(-c + -s + 1.f), m*(c + -s + 1.f),
+		m*(-c + s + 1.f), m*(-c + -s + 1.f)
+	};
+	float quadVerts[] = {
+		x, y, z,
+		x2, y, z,
+		x2, y2, z,
+
+		x2, y2, z,
+		x, y2, z,
+		x, y, z
+	};
+
+	if (g_Renderer.GetRenderPath() == CRenderer::RP_SHADER)
+	{
+		shader->TexCoordPointer(GL_TEXTURE0, 2, GL_FLOAT, 0, quadTex);
+		shader->VertexPointer(3, GL_FLOAT, 0, quadVerts);
+		shader->AssertPointersBound();
+	}
+	else
+	{
+		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+		glEnableClientState(GL_VERTEX_ARRAY);
+
+		glTexCoordPointer(2, GL_FLOAT, 0, quadTex);
+		glVertexPointer(3, GL_FLOAT, 0, quadVerts);
+	}
+
+	glDrawArrays(GL_TRIANGLES, 0, 6);
+
+	if (g_Renderer.GetRenderPath() == CRenderer::RP_FIXED)
+	{
+		glDisableClientState(GL_VERTEX_ARRAY);
+		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+	}
 }
 
 // TODO: render the minimap in a framebuffer and just draw the frambuffer texture
@@ -384,8 +456,8 @@ void CMiniMap::Draw()
 	if (g_Renderer.GetRenderPath() == CRenderer::RP_SHADER)
 	{
 		CShaderDefines defines;
-		defines.Add("MINIMAP_BASE", "1");
-		tech = g_Renderer.GetShaderManager().LoadEffect(CStrIntern("minimap"), g_Renderer.GetSystemShaderDefines(), defines);
+		defines.Add(str_MINIMAP_BASE, str_1);
+		tech = g_Renderer.GetShaderManager().LoadEffect(str_minimap, g_Renderer.GetSystemShaderDefines(), defines);
 		tech->BeginPass();
 		shader = tech->GetShader();
 	}
@@ -398,21 +470,21 @@ void CMiniMap::Draw()
 
 	// Draw the main textured quad
 	if (g_Renderer.GetRenderPath() == CRenderer::RP_SHADER)
-		shader->BindTexture("baseTex", m_TerrainTexture);
+		shader->BindTexture(str_baseTex, m_TerrainTexture);
 	else
 		g_Renderer.BindTexture(0, m_TerrainTexture);
 	
 	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-	DrawTexture(texCoordMax, angle, x, y, x2, y2, z);
+	DrawTexture(shader, texCoordMax, angle, x, y, x2, y2, z);
 
 
 	// Draw territory boundaries
 	CTerritoryTexture& territoryTexture = g_Game->GetView()->GetTerritoryTexture();
 	
 	if (g_Renderer.GetRenderPath() == CRenderer::RP_SHADER)
-		shader->BindTexture("baseTex", territoryTexture.GetTexture());
+		shader->BindTexture(str_baseTex, territoryTexture.GetTexture());
 	else
 		territoryTexture.BindTexture(0);
 	
@@ -421,7 +493,7 @@ void CMiniMap::Draw()
 	glLoadMatrixf(territoryTexture.GetMinimapTextureMatrix());
 	glMatrixMode(GL_MODELVIEW);
 
-	DrawTexture(1.0f, angle, x, y, x2, y2, z);
+	DrawTexture(shader, 1.0f, angle, x, y, x2, y2, z);
 
 	glMatrixMode(GL_TEXTURE);
 	glLoadIdentity();
@@ -437,11 +509,11 @@ void CMiniMap::Draw()
 		tech->EndPass();
 
 		CShaderDefines defines;
-		defines.Add("MINIMAP_LOS", "1");
-		tech = g_Renderer.GetShaderManager().LoadEffect(CStrIntern("minimap"), g_Renderer.GetSystemShaderDefines(), defines);
+		defines.Add(str_MINIMAP_LOS, str_1);
+		tech = g_Renderer.GetShaderManager().LoadEffect(str_minimap, g_Renderer.GetSystemShaderDefines(), defines);
 		tech->BeginPass();
 		shader = tech->GetShader();
-		shader->BindTexture("baseTex", losTexture.GetTexture());
+		shader->BindTexture(str_baseTex, losTexture.GetTexture());
 	}
 	else
 	{
@@ -463,7 +535,7 @@ void CMiniMap::Draw()
 	glLoadMatrixf(losTexture.GetMinimapTextureMatrix());
 	glMatrixMode(GL_MODELVIEW);
 
-	DrawTexture(1.0f, angle, x, y, x2, y2, z);
+	DrawTexture(shader, 1.0f, angle, x, y, x2, y2, z);
 
 	glMatrixMode(GL_TEXTURE);
 	glLoadIdentity();
@@ -476,8 +548,8 @@ void CMiniMap::Draw()
 		tech->EndPass();
 
 		CShaderDefines defines;
-		defines.Add("MINIMAP_POINT", "1");
-		tech = g_Renderer.GetShaderManager().LoadEffect(CStrIntern("minimap"), g_Renderer.GetSystemShaderDefines(), defines);
+		defines.Add(str_MINIMAP_POINT, str_1);
+		tech = g_Renderer.GetShaderManager().LoadEffect(str_minimap, g_Renderer.GetSystemShaderDefines(), defines);
 		tech->BeginPass();
 		shader = tech->GetShader();
 	}
@@ -509,6 +581,17 @@ void CMiniMap::Draw()
 
 		m_EntitiesDrawn = 0;
 		MinimapUnitVertex v;
+		std::vector<MinimapUnitVertex> pingingVertices;
+		pingingVertices.reserve(MAX_ENTITIES_DRAWN/2);
+
+		const double time = timer_Time();
+
+		if (time > m_NextBlinkTime)
+		{
+			m_BlinkState = !m_BlinkState;
+			m_NextBlinkTime = time + m_HalfBlinkDuration;
+		}
+
 		entity_pos_t posX, posZ;
 		for (CSimulation2::InterfaceList::const_iterator it = ents.begin(); it != ents.end(); ++it)
 		{
@@ -521,23 +604,32 @@ void CMiniMap::Draw()
 					v.a = 255;
 					v.x = posX.ToFloat()*sx;
 					v.y = -posZ.ToFloat()*sy;
-
 					
-					(*attrColor)[0] = v.r;
-					(*attrColor)[1] = v.g;
-					(*attrColor)[2] = v.b;
-					(*attrColor)[3] = v.a;
-					++attrColor;
+					// Check minimap pinging to indicate something
+					if (m_BlinkState && cmpMinimap->CheckPing(time, m_PingDuration))
+					{
+						v.r = 255; // ping color is white
+						v.g = 255;
+						v.b = 255;
 
-					(*attrPos)[0] = v.x;
-					(*attrPos)[1] = v.y;
-		
-					++attrPos;
-
-					m_EntitiesDrawn++;
+						pingingVertices.push_back(v);
+					}
+					else
+					{
+						addVertex(v, attrColor, attrPos);
+						++m_EntitiesDrawn;
+					}
 				}
 			}
 		}
+
+		// Add the pinged vertices at the end, so they are drawn on top
+		for (size_t v = 0; v < pingingVertices.size(); ++v)
+		{
+			addVertex(pingingVertices[v], attrColor, attrPos);
+			++m_EntitiesDrawn;
+		}
+
 		ENSURE(m_EntitiesDrawn < MAX_ENTITIES_DRAWN);
 		m_VertexArray.Upload();
 	}
@@ -579,14 +671,14 @@ void CMiniMap::Draw()
 	}
 
 	PROFILE_END("minimap units");
-	
+
 	if (g_Renderer.GetRenderPath() == CRenderer::RP_SHADER)
 	{
 		tech->EndPass();
 
 		CShaderDefines defines;
-		defines.Add("MINIMAP_LINE", "1");
-		tech = g_Renderer.GetShaderManager().LoadEffect(CStrIntern("minimap"), g_Renderer.GetSystemShaderDefines(), defines);
+		defines.Add(str_MINIMAP_LINE, str_1);
+		tech = g_Renderer.GetShaderManager().LoadEffect(str_minimap, g_Renderer.GetSystemShaderDefines(), defines);
 		tech->BeginPass();
 		shader = tech->GetShader();
 	}
@@ -632,7 +724,7 @@ void CMiniMap::CreateTextures()
 	u32* texData = new u32[m_TextureSize * m_TextureSize];
 	for (ssize_t i = 0; i < m_TextureSize * m_TextureSize; ++i)
 		texData[i] = 0xFF000000;
-	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, m_TextureSize, m_TextureSize, 0, GL_BGRA_EXT, GL_UNSIGNED_BYTE, texData);
+	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, m_TextureSize, m_TextureSize, 0, GL_RGBA, GL_UNSIGNED_BYTE, texData);
 	delete[] texData;
 
 	m_TerrainData = new u32[(m_MapSize - 1) * (m_MapSize - 1)];
@@ -667,14 +759,15 @@ void CMiniMap::RebuildTerrainTexture()
 					+ m_Terrain->GetVertexGroundLevel((int)i+1, (int)j+1)
 				) / 4.0f;
 
-			if(avgHeight < waterHeight && avgHeight > waterHeight - m_ShallowPassageHeight)
+			if (avgHeight < waterHeight && avgHeight > waterHeight - m_ShallowPassageHeight)
 			{
 				// shallow water
-				*dataPtr++ = 0xff7098c0;
-			} else if (avgHeight < waterHeight)
+				*dataPtr++ = 0xffc09870;
+			}
+			else if (avgHeight < waterHeight)
 			{
 				// Set water as constant color for consistency on different maps
-				*dataPtr++ = 0xff5078a0;
+				*dataPtr++ = 0xffa07850;
 			}
 			else
 			{
@@ -705,7 +798,7 @@ void CMiniMap::RebuildTerrainTexture()
 
 	// Upload the texture
 	g_Renderer.BindTexture(0, m_TerrainTexture);
-	glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, m_MapSize - 1, m_MapSize - 1, GL_BGRA_EXT, GL_UNSIGNED_BYTE, m_TerrainData);
+	glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, m_MapSize - 1, m_MapSize - 1, GL_RGBA, GL_UNSIGNED_BYTE, m_TerrainData);
 }
 
 void CMiniMap::Destroy()
