@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2013 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -39,6 +39,11 @@ protected:
 	 */
 	virtual void HandleMessage(SGUIMessage &Message);
 
+	/**
+	 * @see IGUIObject#MouseOver()
+	 */
+	virtual bool MouseOver();
+
 	// create the minimap textures
 	void CreateTextures();
 
@@ -83,7 +88,7 @@ protected:
 	// maximal water height to allow the passage of a unit (for underwater shallows).
 	float m_ShallowPassageHeight;
 
-	void DrawTexture(float coordMax, float angle, float x, float y, float x2, float y2, float z);
+	void DrawTexture(CShaderProgramPtr shader, float coordMax, float angle, float x, float y, float x2, float y2, float z);
 
 	void DrawViewRect();
 
@@ -97,6 +102,11 @@ protected:
 	VertexArray::Attribute m_AttributeColor;
 
 	size_t m_EntitiesDrawn;
+
+	double m_PingDuration;
+	double m_HalfBlinkDuration;
+	double m_NextBlinkTime;
+	bool m_BlinkState;
 };
 
 #endif
