@@ -1,4 +1,4 @@
-/* Copyright (C) 2010 Wildfire Games.
+/* Copyright (C) 2013 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -32,28 +32,28 @@
 
 JSClass JSI_IGUIObject::JSI_class = {
 	"GUIObject", JSCLASS_HAS_PRIVATE,
-	JS_PropertyStub, JS_PropertyStub,
+	JS_PropertyStub, JS_DeletePropertyStub,
 	JSI_IGUIObject::getProperty, JSI_IGUIObject::setProperty,
 	JS_EnumerateStub, JS_ResolveStub,
-	JS_ConvertStub, JS_FinalizeStub,
+	JS_ConvertStub, NULL,
 	NULL, NULL, NULL, JSI_IGUIObject::construct
 };
 
-JSPropertySpec JSI_IGUIObject::JSI_props[] = 
+JSPropertySpec JSI_IGUIObject::JSI_props[] =
 {
 	{ 0 }
 };
 
-JSFunctionSpec JSI_IGUIObject::JSI_methods[] = 
+JSFunctionSpec JSI_IGUIObject::JSI_methods[] =
 {
-	{ "toString", JSI_IGUIObject::toString, 0, 0 },
-	{ "focus", JSI_IGUIObject::focus, 0, 0 },
-	{ "blur", JSI_IGUIObject::blur, 0, 0 },
-	{ "getComputedSize", JSI_IGUIObject::getComputedSize, 0, 0 },
-	{ 0 }
+	JS_FS("toString", JSI_IGUIObject::toString, 0, 0),
+	JS_FS("focus", JSI_IGUIObject::focus, 0, 0),
+	JS_FS("blur", JSI_IGUIObject::blur, 0, 0),
+	JS_FS("getComputedSize", JSI_IGUIObject::getComputedSize, 0, 0),
+	JS_FS_END
 };
 
-JSBool JSI_IGUIObject::getProperty(JSContext* cx, JSObject* obj, jsid id, jsval* vp)
+JSBool JSI_IGUIObject::getProperty(JSContext* cx, JS::HandleObject obj, JS::HandleId id, JS::MutableHandleValue vp)
 {
 	IGUIObject* e = (IGUIObject*)JS_GetInstancePrivate(cx, obj, &JSI_IGUIObject::JSI_class, NULL);
 	if (!e)
@@ -86,11 +86,11 @@ JSBool JSI_IGUIObject::getProperty(JSContext* cx, JSObject* obj, jsid id, jsval*
 	if (propName.substr(0, 2) == "on")
 	{
 		CStr eventName (CStr(propName.substr(2)).LowerCase());
-		std::map<CStr, JSObject**>::iterator it = e->m_ScriptHandlers.find(eventName);
+		std::map<CStr, CScriptValRooted>::iterator it = e->m_ScriptHandlers.find(eventName);
 		if (it == e->m_ScriptHandlers.end())
-			*vp = JSVAL_NULL;
+			vp.set(JS::NullValue());
 		else
-			*vp = OBJECT_TO_JSVAL(*(it->second));
+			vp.set((*it).second.get());
 		return JS_TRUE;
 	}
 
@@ -102,19 +102,19 @@ JSBool JSI_IGUIObject::getProperty(JSContext* cx, JSObject* obj, jsid id, jsval*
 		if (parent)
 		{
 			// If the object isn't parentless, return a new object
-			*vp = OBJECT_TO_JSVAL(parent->GetJSObject());
+			vp.set(JS::ObjectValue(*parent->GetJSObject()));
 		}
 		else
 		{
 			// Return null if there's no parent
-			*vp = JSVAL_NULL;
+			vp.set(JS::NullValue());
 		}
 		return JS_TRUE;
 	}
 	// Also handle "name" specially
 	else if (propName == "name")
 	{
-		*vp = STRING_TO_JSVAL(JS_NewStringCopyZ(cx, e->GetName().c_str()));
+		vp.set(JS::StringValue(JS_NewStringCopyZ(cx, e->GetName().c_str())));
 		return JS_TRUE;
 	}
 	// Handle all other properties
@@ -135,7 +135,7 @@ JSBool JSI_IGUIObject::getProperty(JSContext* cx, JSObject* obj, jsid id, jsval*
 			{
 				bool value;
 				GUI<bool>::GetSetting(e, propName, value);
-				*vp = value ? JSVAL_TRUE : JSVAL_FALSE;
+				vp.set(JS::BooleanValue(value));
 				break;
 			}
 
@@ -143,7 +143,7 @@ JSBool JSI_IGUIObject::getProperty(JSContext* cx, JSObject* obj, jsid id, jsval*
 			{
 				int value;
 				GUI<int>::GetSetting(e, propName, value);
-				*vp = INT_TO_JSVAL(value);
+				vp.set(JS::Int32Value(value));
 				break;
 			}
 
@@ -152,19 +152,22 @@ JSBool JSI_IGUIObject::getProperty(JSContext* cx, JSObject* obj, jsid id, jsval*
 				float value;
 				GUI<float>::GetSetting(e, propName, value);
 				// Create a garbage-collectable double
-				return JS_NewNumberValue(cx, value, vp);
+				vp.set(JS::NumberValue(value));
+				return !vp.isNull();
 			}
 
 		case GUIST_CColor:
 			{
 				CColor colour;
 				GUI<CColor>::GetSetting(e, propName, colour);
-				JSObject* obj = JS_NewObject(cx, &JSI_GUIColor::JSI_class, NULL, NULL);
-				*vp = OBJECT_TO_JSVAL(obj); // root it
-
-				jsval c;
+				JS::RootedObject obj(cx, JS_NewObject(cx, &JSI_GUIColor::JSI_class, NULL, NULL));
+				vp.set(JS::ObjectValue(*obj));
+				JS::RootedValue c(cx);
 				// Attempt to minimise ugliness through macrosity
-				#define P(x) if (!JS_NewNumberValue(cx, colour.x, &c)) return JS_FALSE; JS_SetProperty(cx, obj, #x, &c)
+				#define P(x) c = JS::NumberValue(colour.x); \
+				if (c.isNull()) \
+					return false; \
+				JS_SetProperty(cx, obj, #x, c.address())
 					P(r);
 					P(g);
 					P(b);
@@ -178,11 +181,12 @@ JSBool JSI_IGUIObject::getProperty(JSContext* cx, JSObject* obj, jsid id, jsval*
 			{
 				CClientArea area;
 				GUI<CClientArea>::GetSetting(e, propName, area);
-				JSObject* obj = JS_NewObject(cx, &JSI_GUISize::JSI_class, NULL, NULL);
-				*vp = OBJECT_TO_JSVAL(obj); // root it
+
+				vp.set(JS::ObjectValue(*JS_NewObject(cx, &JSI_GUISize::JSI_class, NULL, NULL)));
 				try
 				{
-				#define P(x, y, z) g_ScriptingHost.SetObjectProperty_Double(obj, #z, area.x.y)
+					ScriptInterface* pScriptInterface = ScriptInterface::GetScriptInterfaceAndCBData(cx)->pScriptInterface;
+				#define P(x, y, z) pScriptInterface->SetProperty(vp, #z, area.x.y, false, true)
 					P(pixel,	left,	left);
 					P(pixel,	top,	top);
 					P(pixel,	right,	right);
@@ -193,7 +197,7 @@ JSBool JSI_IGUIObject::getProperty(JSContext* cx, JSObject* obj, jsid id, jsval*
 					P(percent,	bottom,	rbottom);
 				#undef P
 				}
-				catch (PSERROR_Scripting_ConversionFailed)
+				catch (PSERROR_Scripting_ConversionFailed&)
 				{
 					debug_warn(L"Error creating size object!");
 					break;
@@ -206,7 +210,7 @@ JSBool JSI_IGUIObject::getProperty(JSContext* cx, JSObject* obj, jsid id, jsval*
 			{
 				CGUIString value;
 				GUI<CGUIString>::GetSetting(e, propName, value);
-				*vp = ScriptInterface::ToJSVal(cx, value.GetOriginalString());
+				ScriptInterface::ToJSVal(cx, *vp.address(), value.GetOriginalString());
 				break;
 			}
 
@@ -214,7 +218,7 @@ JSBool JSI_IGUIObject::getProperty(JSContext* cx, JSObject* obj, jsid id, jsval*
 			{
 				CStr value;
 				GUI<CStr>::GetSetting(e, propName, value);
-				*vp = ScriptInterface::ToJSVal(cx, value);
+				ScriptInterface::ToJSVal(cx, *vp.address(), value);
 				break;
 			}
 
@@ -222,7 +226,7 @@ JSBool JSI_IGUIObject::getProperty(JSContext* cx, JSObject* obj, jsid id, jsval*
 			{
 				CStrW value;
 				GUI<CStrW>::GetSetting(e, propName, value);
-				*vp = ScriptInterface::ToJSVal(cx, value);
+				ScriptInterface::ToJSVal(cx, *vp.address(), value);
 				break;
 			}
 
@@ -230,7 +234,7 @@ JSBool JSI_IGUIObject::getProperty(JSContext* cx, JSObject* obj, jsid id, jsval*
 			{
 				CGUISpriteInstance *value;
 				GUI<CGUISpriteInstance>::GetSettingPointer(e, propName, value);
-				*vp = ScriptInterface::ToJSVal(cx, value->GetName());
+				ScriptInterface::ToJSVal(cx, *vp.address(), value->GetName());
 				break;
 			}
 
@@ -246,7 +250,7 @@ JSBool JSI_IGUIObject::getProperty(JSContext* cx, JSObject* obj, jsid id, jsval*
 				case EAlign_Center: word = "center"; break;
 				default: debug_warn(L"Invalid EAlign!"); word = "error"; break;
 				}
-				*vp = ScriptInterface::ToJSVal(cx, word);
+				ScriptInterface::ToJSVal(cx, *vp.address(), word);
 				break;
 			}
 
@@ -262,7 +266,7 @@ JSBool JSI_IGUIObject::getProperty(JSContext* cx, JSObject* obj, jsid id, jsval*
 				case EVAlign_Center: word = "center"; break;
 				default: debug_warn(L"Invalid EVAlign!"); word = "error"; break;
 				}
-				*vp = ScriptInterface::ToJSVal(cx, word);
+				ScriptInterface::ToJSVal(cx, *vp.address(), word);
 				break;
 			}
 
@@ -271,13 +275,14 @@ JSBool JSI_IGUIObject::getProperty(JSContext* cx, JSObject* obj, jsid id, jsval*
 				CGUIList value;
 				GUI<CGUIList>::GetSetting(e, propName, value);
 
-				JSObject *obj = JS_NewArrayObject(cx, 0, NULL);
-				*vp = OBJECT_TO_JSVAL(obj); // root it
-				
-				for (size_t i = 0; i < value.m_Items.size(); ++i)
+				JS::RootedObject obj(cx, JS_NewArrayObject(cx, 0, NULL));
+				vp.set(JS::ObjectValue(*obj));
+
+				for (u32 i = 0; i < value.m_Items.size(); ++i)
 				{
-					jsval val = ScriptInterface::ToJSVal(cx, value.m_Items[i].GetOriginalString());
-					JS_SetElement(cx, obj, (jsint)i, &val);
+					JS::RootedValue val(cx);
+					ScriptInterface::ToJSVal(cx, val.get(), value.m_Items[i].GetOriginalString());
+					JS_SetElement(cx, obj, i, val.address());
 				}
 
 				break;
@@ -293,7 +298,7 @@ JSBool JSI_IGUIObject::getProperty(JSContext* cx, JSObject* obj, jsid id, jsval*
 	}
 }
 
-JSBool JSI_IGUIObject::setProperty(JSContext* cx, JSObject* obj, jsid id, JSBool UNUSED(strict), jsval* vp)
+JSBool JSI_IGUIObject::setProperty(JSContext* cx, JS::HandleObject obj, JS::HandleId id, JSBool UNUSED(strict), JS::MutableHandleValue vp)
 {
 	IGUIObject* e = (IGUIObject*)JS_GetInstancePrivate(cx, obj, &JSI_IGUIObject::JSI_class, NULL);
 	if (!e)
@@ -310,7 +315,7 @@ JSBool JSI_IGUIObject::setProperty(JSContext* cx, JSObject* obj, jsid id, JSBool
 	if (propName == "name")
 	{
 		std::string value;
-		if (!ScriptInterface::FromJSVal(cx, *vp, value))
+		if (!ScriptInterface::FromJSVal(cx, vp, value))
 			return JS_FALSE;
 		e->SetName(value);
 		return JS_TRUE;
@@ -319,14 +324,14 @@ JSBool JSI_IGUIObject::setProperty(JSContext* cx, JSObject* obj, jsid id, JSBool
 	// Use onWhatever to set event handlers
 	if (propName.substr(0, 2) == "on")
 	{
-		if (!JSVAL_IS_OBJECT(*vp) || !JS_ObjectIsFunction(cx, JSVAL_TO_OBJECT(*vp)))
+		if (vp.isPrimitive() || vp.isNull() || !JS_ObjectIsFunction(cx, &vp.toObject()))
 		{
 			JS_ReportError(cx, "on- event-handlers must be functions");
 			return JS_FALSE;
 		}
 
 		CStr eventName (CStr(propName.substr(2)).LowerCase());
-		e->SetScriptHandler(eventName, JSVAL_TO_OBJECT(*vp));
+		e->SetScriptHandler(eventName, &vp.toObject());
 
 		return JS_TRUE;
 	}
@@ -345,7 +350,7 @@ JSBool JSI_IGUIObject::setProperty(JSContext* cx, JSObject* obj, jsid id, JSBool
 	case GUIST_CStr:
 		{
 			std::string value;
-			if (!ScriptInterface::FromJSVal(cx, *vp, value))
+			if (!ScriptInterface::FromJSVal(cx, vp, value))
 				return JS_FALSE;
 
 			GUI<CStr>::SetSetting(e, propName, value);
@@ -355,8 +360,8 @@ JSBool JSI_IGUIObject::setProperty(JSContext* cx, JSObject* obj, jsid id, JSBool
 	case GUIST_CStrW:
 		{
 			std::wstring value;
-			if (!ScriptInterface::FromJSVal(cx, *vp, value))
-				return JS_FALSE;
+			if (!ScriptInterface::FromJSVal(cx, vp, value))
+				return false;
 
 			GUI<CStrW>::SetSetting(e, propName, value);
 			break;
@@ -365,8 +370,8 @@ JSBool JSI_IGUIObject::setProperty(JSContext* cx, JSObject* obj, jsid id, JSBool
 	case GUIST_CGUISpriteInstance:
 		{
 			std::string value;
-			if (!ScriptInterface::FromJSVal(cx, *vp, value))
-				return JS_FALSE;
+			if (!ScriptInterface::FromJSVal(cx, vp, value))
+				return false;
 
 			GUI<CGUISpriteInstance>::SetSetting(e, propName, CGUISpriteInstance(value));
 			break;
@@ -375,7 +380,7 @@ JSBool JSI_IGUIObject::setProperty(JSContext* cx, JSObject* obj, jsid id, JSBool
 	case GUIST_CGUIString:
 		{
 			std::wstring value;
-			if (!ScriptInterface::FromJSVal(cx, *vp, value))
+			if (!ScriptInterface::FromJSVal(cx, vp, value))
 				return JS_FALSE;
 
 			CGUIString str;
@@ -387,7 +392,7 @@ JSBool JSI_IGUIObject::setProperty(JSContext* cx, JSObject* obj, jsid id, JSBool
 	case GUIST_EAlign:
 		{
 			std::string value;
-			if (!ScriptInterface::FromJSVal(cx, *vp, value))
+			if (!ScriptInterface::FromJSVal(cx, vp, value))
 				return JS_FALSE;
 
 			EAlign a;
@@ -406,7 +411,7 @@ JSBool JSI_IGUIObject::setProperty(JSContext* cx, JSObject* obj, jsid id, JSBool
 	case GUIST_EVAlign:
 		{
 			std::string value;
-			if (!ScriptInterface::FromJSVal(cx, *vp, value))
+			if (!ScriptInterface::FromJSVal(cx, vp, value))
 				return JS_FALSE;
 
 			EVAlign a;
@@ -424,8 +429,8 @@ JSBool JSI_IGUIObject::setProperty(JSContext* cx, JSObject* obj, jsid id, JSBool
 
 	case GUIST_int:
 		{
-			int32 value;
-			if (JS_ValueToInt32(cx, *vp, &value) == JS_TRUE)
+			int value;
+			if (ScriptInterface::FromJSVal(cx, vp, value))
 				GUI<int>::SetSetting(e, propName, value);
 			else
 			{
@@ -437,8 +442,8 @@ JSBool JSI_IGUIObject::setProperty(JSContext* cx, JSObject* obj, jsid id, JSBool
 
 	case GUIST_float:
 		{
-			jsdouble value;
-			if (JS_ValueToNumber(cx, *vp, &value) == JS_TRUE)
+			double value;
+			if (JS_ValueToNumber(cx, vp, &value) == true)
 				GUI<float>::SetSetting(e, propName, (float)value);
 			else
 			{
@@ -451,8 +456,8 @@ JSBool JSI_IGUIObject::setProperty(JSContext* cx, JSObject* obj, jsid id, JSBool
 	case GUIST_bool:
 		{
 			JSBool value;
-			if (JS_ValueToBoolean(cx, *vp, &value) == JS_TRUE)
-				GUI<bool>::SetSetting(e, propName, value == JS_TRUE);
+			if (JS_ValueToBoolean(cx, vp, &value))
+				GUI<bool>::SetSetting(e, propName, value);
 			else
 			{
 				JS_ReportError(cx, "Cannot convert value to bool");
@@ -463,10 +468,10 @@ JSBool JSI_IGUIObject::setProperty(JSContext* cx, JSObject* obj, jsid id, JSBool
 
 	case GUIST_CClientArea:
 		{
-			if (JSVAL_IS_STRING(*vp))
+			if (vp.isString())
 			{
 				std::wstring value;
-				if (!ScriptInterface::FromJSVal(cx, *vp, value))
+				if (!ScriptInterface::FromJSVal(cx, vp, value))
 					return JS_FALSE;
 
 				if (e->SetSetting(propName, value) != PSRETURN_OK)
@@ -475,13 +480,13 @@ JSBool JSI_IGUIObject::setProperty(JSContext* cx, JSObject* obj, jsid id, JSBool
 					return JS_FALSE;
 				}
 			}
-			else if (JSVAL_IS_OBJECT(*vp) && JS_InstanceOf(cx, JSVAL_TO_OBJECT(*vp), &JSI_GUISize::JSI_class, NULL))
+			else if (vp.isObject() && JS_InstanceOf(cx, &vp.toObject(), &JSI_GUISize::JSI_class, NULL))
 			{
 				CClientArea area;
 				GUI<CClientArea>::GetSetting(e, propName, area);
 
-				JSObject* obj = JSVAL_TO_OBJECT(*vp);
-				#define P(x, y, z) area.x.y = (float)g_ScriptingHost.GetObjectProperty_Double(obj, #z)
+				ScriptInterface* pScriptInterface = ScriptInterface::GetScriptInterfaceAndCBData(cx)->pScriptInterface;
+				#define P(x, y, z) pScriptInterface->GetProperty(vp, #z, area.x.y)
 					P(pixel,	left,	left);
 					P(pixel,	top,	top);
 					P(pixel,	right,	right);
@@ -504,10 +509,10 @@ JSBool JSI_IGUIObject::setProperty(JSContext* cx, JSObject* obj, jsid id, JSBool
 
 	case GUIST_CColor:
 		{
-			if (JSVAL_IS_STRING(*vp))
+			if (vp.isString())
 			{
 				std::wstring value;
-				if (!ScriptInterface::FromJSVal(cx, *vp, value))
+				if (!ScriptInterface::FromJSVal(cx, vp, value))
 					return JS_FALSE;
 
 				if (e->SetSetting(propName, value) != PSRETURN_OK)
@@ -516,13 +521,14 @@ JSBool JSI_IGUIObject::setProperty(JSContext* cx, JSObject* obj, jsid id, JSBool
 					return JS_FALSE;
 				}
 			}
-			else if (JSVAL_IS_OBJECT(*vp) && JS_InstanceOf(cx, JSVAL_TO_OBJECT(*vp), &JSI_GUIColor::JSI_class, NULL))
+			else if (vp.isObject() && JS_InstanceOf(cx, &vp.toObject(), &JSI_GUIColor::JSI_class, NULL))
 			{
 				CColor colour;
-				JSObject* obj = JSVAL_TO_OBJECT(*vp);
-				jsval t; double s;
-				#define PROP(x) JS_GetProperty(cx, obj, #x, &t); \
-								JS_ValueToNumber(cx, t, &s); \
+				JS::RootedObject (cx, &vp.toObject());
+				JS::RootedValue t(cx);
+				double s;
+				#define PROP(x) JS_GetProperty(cx, obj, #x, t.address()); \
+								s = t.toDouble(); \
 								colour.x = (float)s
 				PROP(r); PROP(g); PROP(b); PROP(a);
 				#undef PROP
@@ -539,16 +545,16 @@ JSBool JSI_IGUIObject::setProperty(JSContext* cx, JSObject* obj, jsid id, JSBool
 
 	case GUIST_CGUIList:
 		{
-			JSObject* obj = JSVAL_TO_OBJECT(*vp);
-			jsuint length;
-			if (JSVAL_IS_OBJECT(*vp) && JS_GetArrayLength(cx, obj, &length) == JS_TRUE)
+			u32 length;
+			if (vp.isObject() && JS_GetArrayLength(cx, &vp.toObject(), &length) == true)
 			{
 				CGUIList list;
-
-				for (int i=0; i<(int)length; ++i)
+				JS::RootedObject obj(cx, &vp.toObject());
+				
+				for (u32 i=0; i<length; ++i)
 				{
-					jsval element;
-					if (! JS_GetElement(cx, obj, i, &element))
+					JS::RootedValue element(cx);
+					if (! JS_GetElement(cx, obj, i, element.address()))
 					{
 						JS_ReportError(cx, "Failed to get list element");
 						return JS_FALSE;
@@ -560,7 +566,7 @@ JSBool JSI_IGUIObject::setProperty(JSContext* cx, JSObject* obj, jsid id, JSBool
 
 					CGUIString str;
 					str.SetValue(value);
-					
+
 					list.m_Items.push_back(str);
 				}
 
@@ -585,7 +591,7 @@ JSBool JSI_IGUIObject::setProperty(JSContext* cx, JSObject* obj, jsid id, JSBool
 }
 
 
-JSBool JSI_IGUIObject::construct(JSContext* cx, uintN argc, jsval* vp)
+JSBool JSI_IGUIObject::construct(JSContext* cx, uint argc, jsval* vp)
 {
 	if (argc == 0)
 	{
@@ -593,22 +599,22 @@ JSBool JSI_IGUIObject::construct(JSContext* cx, uintN argc, jsval* vp)
 		return JS_FALSE;
 	}
 
-	JSObject* obj = JS_NewObject(cx, &JSI_IGUIObject::JSI_class, NULL, NULL);
+	JS::RootedObject obj(cx, JS_NewObject(cx, &JSI_IGUIObject::JSI_class, NULL, NULL));
 
 	// Store the IGUIObject in the JS object's 'private' area
 	IGUIObject* guiObject = (IGUIObject*)JSVAL_TO_PRIVATE(JS_ARGV(cx, vp)[0]);
-	JS_SetPrivate(cx, obj, guiObject);
+	JS_SetPrivate(obj, guiObject);
 
-	JS_SET_RVAL(cx, vp, OBJECT_TO_JSVAL(obj));
+	JS_SET_RVAL(cx, vp, JS::ObjectValue(*obj));
 	return JS_TRUE;
 }
 
-void JSI_IGUIObject::init()
+void JSI_IGUIObject::init(ScriptInterface& scriptInterface)
 {
-	g_ScriptingHost.DefineCustomObjectType(&JSI_class, construct, 1, JSI_props, JSI_methods, NULL, NULL);
+	scriptInterface.DefineCustomObjectType(&JSI_class, construct, 1, JSI_props, JSI_methods, NULL, NULL);
 }
 
-JSBool JSI_IGUIObject::toString(JSContext* cx, uintN argc, jsval* vp)
+JSBool JSI_IGUIObject::toString(JSContext* cx, uint argc, jsval* vp)
 {
 	UNUSED2(argc);
 
@@ -619,11 +625,11 @@ JSBool JSI_IGUIObject::toString(JSContext* cx, uintN argc, jsval* vp)
 	char buffer[256];
 	snprintf(buffer, 256, "[GUIObject: %s]", e->GetName().c_str());
 	buffer[255] = 0;
-	JS_SET_RVAL(cx, vp, STRING_TO_JSVAL(JS_NewStringCopyZ(cx, buffer)));
+	JS_SET_RVAL(cx, vp, JS::StringValue(JS_NewStringCopyZ(cx, buffer)));
 	return JS_TRUE;
 }
 
-JSBool JSI_IGUIObject::focus(JSContext* cx, uintN argc, jsval* vp)
+JSBool JSI_IGUIObject::focus(JSContext* cx, uint argc, jsval* vp)
 {
 	UNUSED2(argc);
 
@@ -637,7 +643,7 @@ JSBool JSI_IGUIObject::focus(JSContext* cx, uintN argc, jsval* vp)
 	return JS_TRUE;
 }
 
-JSBool JSI_IGUIObject::blur(JSContext* cx, uintN argc, jsval* vp)
+JSBool JSI_IGUIObject::blur(JSContext* cx, uint argc, jsval* vp)
 {
 	UNUSED2(argc);
 
@@ -651,10 +657,9 @@ JSBool JSI_IGUIObject::blur(JSContext* cx, uintN argc, jsval* vp)
 	return JS_TRUE;
 }
 
-JSBool JSI_IGUIObject::getComputedSize(JSContext* cx, uintN argc, jsval* vp)
+JSBool JSI_IGUIObject::getComputedSize(JSContext* cx, uint argc, jsval* vp)
 {
 	UNUSED2(argc);
-
 	IGUIObject* e = (IGUIObject*)JS_GetInstancePrivate(cx, JS_THIS_OBJECT(cx, vp), &JSI_IGUIObject::JSI_class, NULL);
 	if (!e)
 		return JS_FALSE;
@@ -662,20 +667,21 @@ JSBool JSI_IGUIObject::getComputedSize(JSContext* cx, uintN argc, jsval* vp)
 	e->UpdateCachedSize();
 	CRect size = e->m_CachedActualSize;
 
-	JSObject* obj = JS_NewObject(cx, NULL, NULL, NULL);
+	JS::RootedValue objVal(cx, JS::ObjectValue(*JS_NewObject(cx, NULL, NULL, NULL)));
 	try
 	{
-		g_ScriptingHost.SetObjectProperty_Double(obj, "left", size.left);
-		g_ScriptingHost.SetObjectProperty_Double(obj, "right", size.right);
-		g_ScriptingHost.SetObjectProperty_Double(obj, "top", size.top);
-		g_ScriptingHost.SetObjectProperty_Double(obj, "bottom", size.bottom);
+		ScriptInterface* pScriptInterface = ScriptInterface::GetScriptInterfaceAndCBData(cx)->pScriptInterface;
+		pScriptInterface->SetProperty(objVal, "left", size.left, false, true);
+		pScriptInterface->SetProperty(objVal, "right", size.right, false, true);
+		pScriptInterface->SetProperty(objVal, "top", size.top, false, true);
+		pScriptInterface->SetProperty(objVal, "bottom", size.bottom, false, true);
 	}
-	catch (PSERROR_Scripting_ConversionFailed)
+	catch (PSERROR_Scripting_ConversionFailed&)
 	{
 		debug_warn(L"Error creating size object!");
 		return JS_FALSE;
 	}
 
-	JS_SET_RVAL(cx, vp, OBJECT_TO_JSVAL(obj));
+	JS_SET_RVAL(cx, vp, objVal);
 	return JS_TRUE;
 }
