@@ -1,4 +1,4 @@
-/* Copyright (C) 2011 Wildfire Games.
+/* Copyright (C) 2013 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -29,6 +29,15 @@
 #include "ps/CLogger.h"
 #include "scriptinterface/ScriptInterface.h"
 #include "simulation2/Simulation2.h"
+#include "ps/ConfigDB.h"
+
+#if CONFIG2_MINIUPNPC
+// Next four files are for UPnP port forwarding.
+#include <miniupnpc/miniwget.h>
+#include <miniupnpc/miniupnpc.h>
+#include <miniupnpc/upnpcommands.h>
+#include <miniupnpc/upnperrors.h>
+#endif
 
 #define	DEFAULT_SERVER_NAME			L"Unnamed Server"
 #define DEFAULT_WELCOME_MESSAGE		L"Welcome"
@@ -182,9 +191,115 @@ bool CNetServerWorker::SetupConnection()
 	int ret = pthread_create(&m_WorkerThread, NULL, &RunThread, this);
 	ENSURE(ret == 0);
 
+#if CONFIG2_MINIUPNPC
+	// Launch the UPnP thread
+	ret = pthread_create(&m_UPnPThread, NULL, &SetupUPnP, NULL);
+	ENSURE(ret == 0);
+#endif
+
 	return true;
 }
 
+#if CONFIG2_MINIUPNPC
+void* CNetServerWorker::SetupUPnP(void*)
+{
+	// Values we want to set.
+	char psPort[6];
+	sprintf_s(psPort, ARRAY_SIZE(psPort), "%d", PS_DEFAULT_PORT);
+	const char* leaseDuration = "0"; // Indefinite/permanent lease duration.
+	const char* description = "0AD Multiplayer";
+	const char* protocall = "UDP";
+	char internalIPAddress[64];
+	char externalIPAddress[40];
+	// Variables to hold the values that actually get set.
+	char intClient[40];
+	char intPort[6];
+	char duration[16];
+	// Intermediate variables.
+	struct UPNPUrls urls;
+	struct IGDdatas data;
+	struct UPNPDev* devlist = 0;
+
+	// Cached root descriptor URL.
+	std::string rootDescURL;
+	CFG_GET_VAL("network.upnprootdescurl", String, rootDescURL);
+	if (!rootDescURL.empty())
+		LOGMESSAGE(L"Net server: attempting to use cached root descriptor URL: %hs", rootDescURL.c_str());
+
+	// Init the return variable for UPNP_GetValidIGD to 1 so things behave when using cached URLs.
+	int ret = 1;
+
+	// If we have a cached URL, try that first, otherwise try getting a valid UPnP device for 10 seconds. We also get our LAN address here.
+	if (!((!rootDescURL.empty() && UPNP_GetIGDFromUrl(rootDescURL.c_str(), &urls, &data, internalIPAddress, sizeof(internalIPAddress)))
+	  || ((devlist = upnpDiscover(10000, 0, 0, 0, 0, 0)) != NULL && (ret = UPNP_GetValidIGD(devlist, &urls, &data, internalIPAddress, sizeof(internalIPAddress))) != 0)))
+	{
+		LOGMESSAGE(L"Net server: upnpDiscover failed and no working cached URL.");
+		return NULL;
+	}
+
+	switch (ret)
+	{
+	case 1:
+		LOGMESSAGE(L"Net server: found valid IGD = %hs", urls.controlURL);
+		break;
+	case 2:
+		LOGMESSAGE(L"Net server: found a valid, not connected IGD = %hs, will try to continue anyway", urls.controlURL);
+		break;
+	case 3:
+		LOGMESSAGE(L"Net server: found a UPnP device unrecognized as IGD = %hs, will try to continue anyway", urls.controlURL);
+		break;
+	default:
+		debug_warn(L"Unrecognized return value from UPNP_GetValidIGD");
+	}
+
+	// Try getting our external/internet facing IP. TODO: Display this on the game-setup page for conviniance.
+	ret = UPNP_GetExternalIPAddress(urls.controlURL, data.first.servicetype, externalIPAddress);
+	if (ret != UPNPCOMMAND_SUCCESS)
+	{
+		LOGMESSAGE(L"Net server: GetExternalIPAddress failed with code %d (%hs)", ret, strupnperror(ret));
+		return NULL;
+	}
+	LOGMESSAGE(L"Net server: ExternalIPAddress = %hs", externalIPAddress);
+
+	// Try to setup port forwarding.
+	ret = UPNP_AddPortMapping(urls.controlURL, data.first.servicetype, psPort, psPort,
+							internalIPAddress, description, protocall, 0, leaseDuration);
+	if (ret != UPNPCOMMAND_SUCCESS)
+	{
+		LOGMESSAGE(L"Net server: AddPortMapping(%hs, %hs, %hs) failed with code %d (%hs)",
+			   psPort, psPort, internalIPAddress, ret, strupnperror(ret));
+		return NULL;
+	}
+
+	// Check that the port was actually forwarded.
+	ret = UPNP_GetSpecificPortMappingEntry(urls.controlURL,
+									 data.first.servicetype,
+									 psPort, protocall,
+									 intClient, intPort, NULL/*desc*/,
+									 NULL/*enabled*/, duration);
+
+	if (ret != UPNPCOMMAND_SUCCESS)
+	{
+		LOGMESSAGE(L"Net server: GetSpecificPortMappingEntry() failed with code %d (%hs)", ret, strupnperror(ret));
+		return NULL;
+	}
+
+	LOGMESSAGE(L"Net server: External %hs:%hs %hs is redirected to internal %hs:%hs (duration=%hs)",
+				   externalIPAddress, psPort, protocall, intClient, intPort, duration);
+
+	// Cache root descriptor URL to try to avoid discovery next time.
+	g_ConfigDB.SetValueString(CFG_USER, "network.upnprootdescurl", urls.controlURL);
+	g_ConfigDB.WriteFile(CFG_USER);
+	LOGMESSAGE(L"Net server: cached UPnP root descriptor URL as %hs", urls.controlURL);
+
+	// Make sure everything is properly freed.
+	FreeUPNPUrls(&urls);
+	freeUPNPDevlist(devlist);
+
+	return NULL;
+}
+#endif // CONFIG2_MINIUPNPC
+
 bool CNetServerWorker::SendMessage(ENetPeer* peer, const CNetMessage* message)
 {
 	ENSURE(m_Host);
@@ -227,10 +342,12 @@ void* CNetServerWorker::RunThread(void* data)
 
 void CNetServerWorker::Run()
 {
+	// The script runtime uses the profiler and therefore the thread must be registered before the runtime is created
+	g_Profiler2.RegisterCurrentThread("Net server");
+	
 	// To avoid the need for JS_SetContextThread, we create and use and destroy
 	// the script interface entirely within this network thread
 	m_ScriptInterface = new ScriptInterface("Engine", "Net server", ScriptInterface::CreateRuntime());
-
 	while (true)
 	{
 		if (!RunStep())
@@ -259,6 +376,8 @@ bool CNetServerWorker::RunStep()
 
 	std::vector<std::pair<int, CStr> > newAssignPlayer;
 	std::vector<bool> newStartGame;
+	std::vector<std::pair<CStr, int> > newPlayerReady;
+	std::vector<bool> newPlayerResetReady;
 	std::vector<std::string> newGameAttributes;
 	std::vector<u32> newTurnLength;
 
@@ -269,6 +388,8 @@ bool CNetServerWorker::RunStep()
 			return false;
 
 		newStartGame.swap(m_StartGameQueue);
+		newPlayerReady.swap(m_PlayerReadyQueue);
+		newPlayerResetReady.swap(m_PlayerResetReadyQueue);
 		newAssignPlayer.swap(m_AssignPlayerQueue);
 		newGameAttributes.swap(m_GameAttributesQueue);
 		newTurnLength.swap(m_TurnLengthQueue);
@@ -277,6 +398,12 @@ bool CNetServerWorker::RunStep()
 	for (size_t i = 0; i < newAssignPlayer.size(); ++i)
 		AssignPlayer(newAssignPlayer[i].first, newAssignPlayer[i].second);
 
+	for (size_t i = 0; i < newPlayerReady.size(); ++i)
+		SetPlayerReady(newPlayerReady[i].first, newPlayerReady[i].second);
+
+	if (!newPlayerResetReady.empty())
+		ClearAllPlayerReady();
+
 	if (!newGameAttributes.empty())
 		UpdateGameAttributes(GetScriptInterface().ParseJSON(newGameAttributes.back()));
 
@@ -431,6 +558,7 @@ void CNetServerWorker::SetupSession(CNetServerSession* session)
 
 	session->AddTransition(NSS_PREGAME, (uint)NMT_CONNECTION_LOST, NSS_UNCONNECTED, (void*)&OnDisconnect, context);
 	session->AddTransition(NSS_PREGAME, (uint)NMT_CHAT, NSS_PREGAME, (void*)&OnChat, context);
+	session->AddTransition(NSS_PREGAME, (uint)NMT_READY, NSS_PREGAME, (void*)&OnReady, context);
 	session->AddTransition(NSS_PREGAME, (uint)NMT_LOADED_GAME, NSS_INGAME, (void*)&OnLoadedGame, context);
 
 	session->AddTransition(NSS_JOIN_SYNCING, (uint)NMT_LOADED_GAME, NSS_INGAME, (void*)&OnJoinSyncingLoadedGame, context);
@@ -480,10 +608,10 @@ void CNetServerWorker::OnUserLeave(CNetServerSession* session)
 
 void CNetServerWorker::AddPlayer(const CStr& guid, const CStrW& name)
 {
-	// Find all player IDs in active use; we mustn't give them to a second player
+	// Find all player IDs in active use; we mustn't give them to a second player (excluding the unassigned ID: -1)
 	std::set<i32> usedIDs;
 	for (PlayerAssignmentMap::iterator it = m_PlayerAssignments.begin(); it != m_PlayerAssignments.end(); ++it)
-		if (it->second.m_Enabled)
+		if (it->second.m_Enabled && it->second.m_PlayerID != -1)
 			usedIDs.insert(it->second.m_PlayerID);
 
 	// If the player is rejoining after disconnecting, try to give them
@@ -532,6 +660,7 @@ void CNetServerWorker::AddPlayer(const CStr& guid, const CStrW& name)
 	assignment.m_Enabled = true;
 	assignment.m_Name = name;
 	assignment.m_PlayerID = playerID;
+	assignment.m_Status = 0;
 	m_PlayerAssignments[guid] = assignment;
 
 	// Send the new assignments to all currently active players
@@ -546,6 +675,21 @@ void CNetServerWorker::RemovePlayer(const CStr& guid)
 	SendPlayerAssignments();
 }
 
+void CNetServerWorker::SetPlayerReady(const CStr& guid, const int ready)
+{
+	m_PlayerAssignments[guid].m_Status = ready;
+
+	SendPlayerAssignments();
+}
+
+void CNetServerWorker::ClearAllPlayerReady()
+{
+	for (PlayerAssignmentMap::iterator it = m_PlayerAssignments.begin(); it != m_PlayerAssignments.end(); ++it)
+		it->second.m_Status = 0;
+
+	SendPlayerAssignments();
+}
+
 void CNetServerWorker::AssignPlayer(int playerID, const CStr& guid)
 {
 	// Remove anyone who's already assigned to this player
@@ -573,6 +717,7 @@ void CNetServerWorker::ConstructPlayerAssignmentMessage(CPlayerAssignmentMessage
 		h.m_GUID = it->first;
 		h.m_Name = it->second.m_Name;
 		h.m_PlayerID = it->second.m_PlayerID;
+		h.m_Status = it->second.m_Status;
 		message.m_Hosts.push_back(h);
 	}
 }
@@ -743,6 +888,22 @@ bool CNetServerWorker::OnChat(void* context, CFsmEvent* event)
 	return true;
 }
 
+bool CNetServerWorker::OnReady(void* context, CFsmEvent* event)
+{
+	ENSURE(event->GetType() == (uint)NMT_READY);
+
+	CNetServerSession* session = (CNetServerSession*)context;
+	CNetServerWorker& server = session->GetServer();
+
+	CReadyMessage* message = (CReadyMessage*)event->GetParamRef();
+
+	message->m_GUID = session->GetGUID();
+
+	server.Broadcast(message);
+
+	return true;
+}
+
 bool CNetServerWorker::OnLoadedGame(void* context, CFsmEvent* event)
 {
 	ENSURE(event->GetType() == (uint)NMT_LOADED_GAME);
@@ -936,6 +1097,18 @@ void CNetServer::AssignPlayer(int playerID, const CStr& guid)
 	m_Worker->m_AssignPlayerQueue.push_back(std::make_pair(playerID, guid));
 }
 
+void CNetServer::SetPlayerReady(const CStr& guid, int ready)
+{
+	CScopeLock lock(m_Worker->m_WorkerMutex);
+	m_Worker->m_PlayerReadyQueue.push_back(std::make_pair(guid, ready));
+}
+
+void CNetServer::ClearAllPlayerReady()
+{
+	CScopeLock lock(m_Worker->m_WorkerMutex);
+	m_Worker->m_PlayerResetReadyQueue.push_back(false);
+}
+
 void CNetServer::StartGame()
 {
 	CScopeLock lock(m_Worker->m_WorkerMutex);
