@@ -36,6 +36,8 @@ bool g_NoGLVBO = false;
 
 bool g_PauseOnFocusLoss = false;
 
+bool g_RenderActors = true;
+
 bool g_Shadows = false;
 bool g_ShadowPCF = false;
 
@@ -88,6 +90,7 @@ static void LoadGlobals()
 	CFG_GET_VAL("noautomipmap", Bool, g_NoGLAutoMipmap);
 	CFG_GET_VAL("novbo", Bool, g_NoGLVBO);
 	CFG_GET_VAL("pauseonfocusloss", Bool, g_PauseOnFocusLoss);
+	CFG_GET_VAL("renderactors", Bool, g_RenderActors);
 	CFG_GET_VAL("shadows", Bool, g_Shadows);
 	CFG_GET_VAL("shadowpcf", Bool, g_ShadowPCF);
 
@@ -120,17 +123,19 @@ static void LoadGlobals()
 		CFG_GET_VAL("sound.actiongain", Float, actionGain);
 		CFG_GET_VAL("sound.uigain", Float, uiGain);
 
-  	g_SoundManager->SetMasterGain( gain );
-  	g_SoundManager->SetMusicGain( musicGain );
-  	g_SoundManager->SetAmbientGain( ambientGain );
-  	g_SoundManager->SetActionGain( actionGain );
-  	g_SoundManager->SetUIGain( uiGain );
+	g_SoundManager->SetMasterGain(gain);
+	g_SoundManager->SetMusicGain(musicGain);
+	g_SoundManager->SetAmbientGain(ambientGain);
+	g_SoundManager->SetActionGain(actionGain);
+	g_SoundManager->SetUIGain(uiGain);
 	}
 
 
 	CFG_GET_VAL("jsdebugger.enable", Bool, g_JSDebuggerEnabled);
 	CFG_GET_VAL("profiler2.script.enable", Bool, g_ScriptProfilingEnabled);
 
+	if (g_JSDebuggerEnabled)
+		LOGERROR(L"JS debugger temporarily disabled during the SpiderMonkey upgrade (check trac ticket #2348 for details)");
 	// Script Debugging and profiling does not make sense together because of the hooks
 	// that reduce performance a lot - and it wasn't tested if it even works together.
 	if (g_JSDebuggerEnabled && g_ScriptProfilingEnabled)
@@ -143,14 +148,6 @@ static void ProcessCommandLineArgs(const CmdLineArgs& args)
 	// TODO: all these options (and the ones processed elsewhere) should
 	// be documented somewhere for users.
 
-	if (args.Has("buildarchive"))
-	{
-		// note: VFS init is sure to have been completed by now
-		// (since CONFIG_Init reads from file); therefore,
-		// it is safe to call this from here directly.
-//		vfs_opt_rebuild_main_archive("mods/official/official1.zip", "../logs/trace.txt");
-	}
-
 	// Handle "-conf=key:value" (potentially multiple times)
 	std::vector<CStr> conf = args.GetMultiple("conf");
 	for (size_t i = 0; i < conf.size(); ++i)
@@ -160,7 +157,7 @@ static void ProcessCommandLineArgs(const CmdLineArgs& args)
 		{
 			CStr name = name_value.BeforeFirst(":");
 			CStr value = name_value.AfterFirst(":");
-			g_ConfigDB.CreateValue(CFG_COMMAND, name)->m_String = value;
+			g_ConfigDB.SetValueString(CFG_COMMAND, name, value);
 		}
 	}
 
@@ -175,7 +172,7 @@ static void ProcessCommandLineArgs(const CmdLineArgs& args)
 //		trace_enable(true);
 
 	if (args.Has("profile"))
-		g_ConfigDB.CreateValue(CFG_COMMAND, "profile")->m_String = args.Get("profile");
+		g_ConfigDB.SetValueString(CFG_COMMAND, "profile", args.Get("profile"));
 
 	if (args.Has("quickstart"))
 	{
@@ -187,22 +184,22 @@ static void ProcessCommandLineArgs(const CmdLineArgs& args)
 		g_DisableAudio = true;
 
 	if (args.Has("shadows"))
-		g_ConfigDB.CreateValue(CFG_COMMAND, "shadows")->m_String = "true";
+		g_ConfigDB.SetValueString(CFG_COMMAND, "shadows", "true");
 
 	if (args.Has("xres"))
-		g_ConfigDB.CreateValue(CFG_COMMAND, "xres")->m_String = args.Get("xres");
+		g_ConfigDB.SetValueString(CFG_COMMAND, "xres", args.Get("xres"));
 
 	if (args.Has("yres"))
-		g_ConfigDB.CreateValue(CFG_COMMAND, "yres")->m_String = args.Get("yres");
+		g_ConfigDB.SetValueString(CFG_COMMAND, "yres", args.Get("yres"));
 
 	if (args.Has("vsync"))
-		g_ConfigDB.CreateValue(CFG_COMMAND, "vsync")->m_String = "true";
+		g_ConfigDB.SetValueString(CFG_COMMAND, "vsync", "true");
 
 	if (args.Has("ooslog"))
-		g_ConfigDB.CreateValue(CFG_COMMAND, "ooslog")->m_String = "true";
+		g_ConfigDB.SetValueString(CFG_COMMAND, "ooslog", "true");
 
 	if (args.Has("serializationtest"))
-		g_ConfigDB.CreateValue(CFG_COMMAND, "serializationtest")->m_String = "true";
+		g_ConfigDB.SetValueString(CFG_COMMAND, "serializationtest", "true");
 }
 
 
