@@ -44,8 +44,8 @@ extern bool g_NoGLVBO;
 // flag to pause the game on window focus loss
 extern bool g_PauseOnFocusLoss;
 
-// default player name to use in multiplayer
-extern CStr g_PlayerName;
+// flag to switch on actor rendering
+extern bool g_RenderActors;
 
 // flag to switch on shadows
 extern bool g_Shadows;
@@ -85,6 +85,9 @@ extern bool g_VSync;
 extern bool g_Quickstart;
 extern bool g_DisableAudio;
 
+extern bool g_JSDebuggerEnabled;
+extern bool g_ScriptProfilingEnabled;
+
 extern CStrW g_CursorName;
 
 class CmdLineArgs;
