@@ -36,6 +36,7 @@
 #endif
 
 #include "graphics/CinemaTrack.h"
+#include "graphics/FontMetrics.h"
 #include "graphics/GameView.h"
 #include "graphics/LightEnv.h"
 #include "graphics/MapReader.h"
@@ -43,11 +44,8 @@
 #include "graphics/TerrainTextureManager.h"
 #include "gui/GUI.h"
 #include "gui/GUIManager.h"
-#include "gui/scripting/JSInterface_IGUIObject.h"
-#include "gui/scripting/JSInterface_GUITypes.h"
 #include "gui/scripting/ScriptFunctions.h"
 #include "maths/MathUtil.h"
-#include "maths/scripting/JSInterface_Vector3D.h"
 #include "network/NetServer.h"
 #include "network/NetClient.h"
 
@@ -55,7 +53,6 @@
 #include "ps/CLogger.h"
 #include "ps/ConfigDB.h"
 #include "ps/Filesystem.h"
-#include "ps/Font.h"
 #include "ps/Game.h"
 #include "ps/GameSetup/Atlas.h"
 #include "ps/GameSetup/GameSetup.h"
@@ -72,6 +69,7 @@
 #include "ps/ProfileViewer.h"
 #include "ps/Profiler2.h"
 #include "ps/Pyrogenesis.h"	// psSetLogDir
+#include "ps/SavedGame.h"
 #include "ps/scripting/JSInterface_Console.h"
 #include "ps/TouchInput.h"
 #include "ps/UserReport.h"
@@ -82,12 +80,11 @@
 #include "renderer/Renderer.h"
 #include "renderer/VertexBufferManager.h"
 #include "renderer/ModelRenderer.h"
-#include "scripting/ScriptingHost.h"
-#include "scripting/ScriptGlue.h"
 #include "scriptinterface/DebuggingServer.h"
 #include "scriptinterface/ScriptInterface.h"
 #include "scriptinterface/ScriptStats.h"
 #include "simulation2/Simulation2.h"
+#include "lobby/IXmppClient.h"
 #include "soundmanager/scripting/JSInterface_Sound.h"
 #include "soundmanager/ISoundManager.h"
 #include "tools/atlas/GameInterface/GameLoop.h"
@@ -115,8 +112,13 @@ bool g_DoRenderGui = true;
 bool g_DoRenderLogger = true;
 bool g_DoRenderCursor = true;
 
+shared_ptr<ScriptRuntime> g_ScriptRuntime;
+
 static const int SANE_TEX_QUALITY_DEFAULT = 5;	// keep in sync with code
 
+bool g_InDevelopmentCopy;
+bool g_CheckedIfInDevelopmentCopy = false;
+
 static void SetTextureQuality(int quality)
 {
 	int q_flags;
@@ -182,9 +184,9 @@ retry:
 // display progress / description in loading screen
 void GUI_DisplayLoadProgress(int percent, const wchar_t* pending_task)
 {
-	g_ScriptingHost.GetScriptInterface().SetGlobal("g_Progress", percent, true);
-	g_ScriptingHost.GetScriptInterface().SetGlobal("g_LoadDescription", pending_task, true);
-	g_GUI->SendEventToAll("progress");
+	g_GUI->GetActiveGUI()->GetScriptInterface()->SetGlobal("g_Progress", percent, true);
+	g_GUI->GetActiveGUI()->GetScriptInterface()->SetGlobal("g_LoadDescription", pending_task, true);
+	g_GUI->GetActiveGUI()->SendEventToAll("progress");
 }
 
 
@@ -281,8 +283,12 @@ void Render()
 			glLoadMatrixf(&transform._11);
 #endif
 
+#if OS_ANDROID
+#warning TODO: cursors for Android
+#else
 			if (cursor_draw(g_VFS, cursorName.c_str(), g_mouse_x, g_yres-g_mouse_y, forceGL) < 0)
 				LOGWARNING(L"Failed to draw cursor '%ls'", cursorName.c_str());
+#endif
 
 #if CONFIG2_GLES
 #warning TODO: implement cursors for GLES
@@ -316,40 +322,6 @@ void Render()
 }
 
 
-static void RegisterJavascriptInterfaces()
-{
-	// maths
-	JSI_Vector3D::init();
-
-	// graphics
-	CGameView::ScriptingInit();
-
-	// renderer
-	CRenderer::ScriptingInit();
-
-	// ps
-	JSI_Console::init();
-
-	// GUI
-	CGUI::ScriptingInit();
-
-	GuiScriptingInit(g_ScriptingHost.GetScriptInterface());
-	JSI_Sound::RegisterScriptFunctions(g_ScriptingHost.GetScriptInterface());
-}
-
-
-static void InitScripting()
-{
-	TIMER(L"InitScripting");
-
-	// Create the scripting host.  This needs to be done before the GUI is created.
-	// [7ms]
-	new ScriptingHost;
-	
-	RegisterJavascriptInterfaces();
-}
-
-
 static size_t OperatingSystemFootprint()
 {
 #if OS_WIN
@@ -420,15 +392,19 @@ ErrorReactionInternal psDisplayError(const wchar_t* UNUSED(text), size_t UNUSED(
 	return ERI_NOT_IMPLEMENTED;
 }
 
-static std::vector<CStr> GetMods(const CmdLineArgs& args, bool dev)
+static std::vector<CStr> GetMods(const CmdLineArgs& args)
 {
 	std::vector<CStr> mods = args.GetMultiple("mod");
+	// List of the mods, to be used by the Gui
+	g_modsLoaded.clear();
+	for (size_t i = 0; i < mods.size(); ++i)
+		g_modsLoaded.push_back((std::string)mods[i]);
 	// TODO: It would be nice to remove this hard-coding
 	mods.insert(mods.begin(), "public");
 
 	// Add the user mod if not explicitly disabled or we have a dev copy so
 	// that saved files end up in version control and not in the user mod.
-	if (!dev && !args.Has("noUserMod"))
+	if (!InDevelopmentCopy() && !args.Has("noUserMod"))
 		mods.push_back("user");
 
 	return mods;
@@ -462,9 +438,11 @@ static void InitVfs(const CmdLineArgs& args, int flags)
 	// (maps, etc) end up in version control.
 	const OsPath readonlyConfig = paths.RData()/"config"/"";
 	g_VFS->Mount(L"config/", readonlyConfig);
-	bool dev = (g_VFS->GetFileInfo(L"config/dev.cfg", NULL) == INFO::OK);
 
-	const std::vector<CStr> mods = GetMods(args, dev);
+	// Engine localization files.
+	g_VFS->Mount(L"l10n/", paths.RData()/"l10n"/"");
+
+	const std::vector<CStr> mods = GetMods(args);
 
 	OsPath modPath = paths.RData()/"mods";
 	OsPath modUserPath = paths.UserData()/"mods";
@@ -475,7 +453,7 @@ static void InitVfs(const CmdLineArgs& args, int flags)
 		size_t baseFlags = userFlags|VFS_MOUNT_MUST_EXIST;
 		
 		OsPath modName(mods[i]);
-		if (dev)
+		if (InDevelopmentCopy())
 		{
 			// We are running a dev copy, so only mount mods in the user mod path
 			// if the mod does not exist in the data path.
@@ -507,7 +485,7 @@ static void InitVfs(const CmdLineArgs& args, int flags)
 }
 
 
-static void InitPs(bool setup_gui, const CStrW& gui_page, CScriptVal initData)
+static void InitPs(bool setup_gui, const CStrW& gui_page, ScriptInterface* srcScriptInterface, CScriptVal initData)
 {
 	{
 		// console
@@ -516,7 +494,7 @@ static void InitPs(bool setup_gui, const CStrW& gui_page, CScriptVal initData)
 		g_Console->UpdateScreenSize(g_xres, g_yres);
 
 		// Calculate and store the line spacing
-		CFont font(CONSOLE_FONT);
+		CFontMetrics font(CStrIntern(CONSOLE_FONT));
 		g_Console->m_iFontHeight = font.GetLineSpacing();
 		g_Console->m_iFontWidth = font.GetCharacterWidth(L'C');
 		g_Console->m_charsPerPage = (size_t)(g_xres / g_Console->m_iFontWidth);
@@ -538,12 +516,12 @@ static void InitPs(bool setup_gui, const CStrW& gui_page, CScriptVal initData)
 	{
 		// We do actually need *some* kind of GUI loaded, so use the
 		// (currently empty) Atlas one
-		g_GUI->SwitchPage(L"page_atlas.xml", initData);
+		g_GUI->SwitchPage(L"page_atlas.xml", srcScriptInterface, initData);
 		return;
 	}
 
 	// GUI uses VFS, so this must come after VFS init.
-	g_GUI->SwitchPage(gui_page, initData);
+	g_GUI->SwitchPage(gui_page, srcScriptInterface, initData);
 }
 
 
@@ -587,6 +565,10 @@ static void ShutdownPs()
 	SAFE_DELETE(g_GUI);
 
 	SAFE_DELETE(g_Console);
+	
+	// This is needed to ensure that no callbacks from the JSAPI try to use 
+	// the profiler when it's already destructed
+	g_ScriptRuntime.reset();
 
 	// disable the special Windows cursor, or free textures for OGL cursors
 	cursor_draw(g_VFS, 0, g_mouse_x, g_yres-g_mouse_y, false);
@@ -681,8 +663,8 @@ static void ShutdownSDL()
 
 void EndGame()
 {
-	SAFE_DELETE(g_NetServer);
 	SAFE_DELETE(g_NetClient);
+	SAFE_DELETE(g_NetServer);
 	SAFE_DELETE(g_Game);
 
 	ISoundManager::CloseGame();
@@ -693,7 +675,9 @@ void Shutdown(int UNUSED(flags))
 {
 	EndGame();
 
-	ShutdownPs(); // Must delete g_GUI before g_ScriptingHost
+	SAFE_DELETE(g_XmppClient);
+
+	ShutdownPs();
 
 	in_reset_handlers();
 
@@ -707,8 +691,6 @@ void Shutdown(int UNUSED(flags))
 	g_VBMan.Shutdown();
 	TIMER_END(L"shutdown Renderer");
 
-	tex_codec_unregister_all();
-
 	g_Profiler2.ShutdownGPU();
 
 	// Free cursors before shutting down SDL, as they may depend on SDL.
@@ -724,10 +706,10 @@ void Shutdown(int UNUSED(flags))
 	g_UserReporter.Deinitialize();
 	TIMER_END(L"shutdown UserReporter");
 
-	TIMER_BEGIN(L"shutdown ScriptingHost");
-	delete &g_ScriptingHost;
-	delete g_DebuggingServer;
-	TIMER_END(L"shutdown ScriptingHost");
+	// JS debugger temporarily disabled during the SpiderMonkey upgrade (check trac ticket #2348 for details)
+	//TIMER_BEGIN(L"shutdown DebuggingServer (if active)");
+	//delete g_DebuggingServer;
+	//TIMER_END(L"shutdown DebuggingServer (if active)");
 
 	TIMER_BEGIN(L"shutdown ConfigDB");
 	delete &g_ConfigDB;
@@ -881,13 +863,16 @@ void Init(const CmdLineArgs& args, int flags)
 	// This must come after VFS init, which sets the current directory
 	// (required for finding our output log files).
 	g_Logger = new CLogger;
+	
+	// Workaround until Simulation and AI use their own threads and also their own runtimes
+	g_ScriptRuntime = ScriptInterface::CreateRuntime(384 * 1024 * 1024);
 
 	// Special command-line mode to dump the entity schemas instead of running the game.
 	// (This must be done after loading VFS etc, but should be done before wasting time
 	// on anything else.)
 	if (args.Has("dumpSchema"))
 	{
-		CSimulation2 sim(NULL, NULL);
+		CSimulation2 sim(NULL, g_ScriptRuntime, NULL);
 		sim.LoadDefaultScripts();
 		std::ofstream f("entity.rng", std::ios_base::out | std::ios_base::trunc);
 		f << sim.GenerateSchema();
@@ -923,12 +908,9 @@ void Init(const CmdLineArgs& args, int flags)
 	CONFIG_Init(args);
 
 	// before scripting 
-	if (g_JSDebuggerEnabled)
-		g_DebuggingServer = new CDebuggingServer();
-
-	InitScripting();	// before GUI
-
-	g_ConfigDB.RegisterJSConfigDB(); 	// after scripting 
+	// JS debugger temporarily disabled during the SpiderMonkey upgrade (check trac ticket #2348 for details)
+	//if (g_JSDebuggerEnabled)
+	//	g_DebuggingServer = new CDebuggingServer();
 
 	// Optionally start profiler HTTP output automatically
 	// (By default it's only enabled by a hotkey, for security/performance)
@@ -961,8 +943,6 @@ void InitGraphics(const CmdLineArgs& args, int flags)
 
 	RunHardwareDetection();
 
-	tex_codec_register_all();
-
 	const int quality = SANE_TEX_QUALITY_DEFAULT;	// TODO: set value from config file
 	SetTextureQuality(quality);
 
@@ -985,7 +965,7 @@ void InitGraphics(const CmdLineArgs& args, int flags)
 	if(g_DisableAudio)
 		ISoundManager::SetEnabled(false);
 
-	g_GUI = new CGUIManager(g_ScriptingHost.GetScriptInterface());
+	g_GUI = new CGUIManager();
 
 	// (must come after SetVideoMode, since it calls ogl_Init)
 	if (ogl_HaveExtensions(0, "GL_ARB_vertex_program", "GL_ARB_fragment_program", NULL) != 0 // ARB
@@ -1044,11 +1024,11 @@ void InitGraphics(const CmdLineArgs& args, int flags)
 			CScriptValRooted data;
 			if (g_GUI)
 			{
-				ScriptInterface& scriptInterface = g_GUI->GetScriptInterface();
-				scriptInterface.Eval("({})", data);
-				scriptInterface.SetProperty(data.get(), "isStartup", true);
+				shared_ptr<ScriptInterface> scriptInterface = g_GUI->GetScriptInterface();
+				scriptInterface->Eval("({})", data);
+				scriptInterface->SetProperty(data.get(), "isStartup", true);
 			}
-			InitPs(setup_gui, L"page_pregame.xml", data.get());
+			InitPs(setup_gui, L"page_pregame.xml", g_GUI->GetScriptInterface().get(), data.get());
 		}
 	}
 	catch (PSERROR_Game_World_MapLoadFailed& e)
@@ -1056,7 +1036,7 @@ void InitGraphics(const CmdLineArgs& args, int flags)
 		// Map Loading failed
 
 		// Start the engine so we have a GUI
-		InitPs(true, L"page_pregame.xml", JSVAL_VOID);
+		InitPs(true, L"page_pregame.xml", NULL, JSVAL_VOID);
 
 		// Call script function to do the actual work
 		//	(delete game data, switch GUI page, show error, etc.)
@@ -1092,6 +1072,7 @@ bool Autostart(const CmdLineArgs& args)
 	 * -autostart-ip=127.0.0.1			-- multiplayer connect to 127.0.0.1
 	 * -autostart-random=104			-- random map, optional seed value = 104 (default is 0, random is -1)
 	 * -autostart-size=192				-- random map size in tiles = 192 (default is 192)
+	 * -autostart-civ=1:hele			-- set player #1 civ to "hele"
 	 *
 	 * Examples:
 	 * -autostart=Acropolis -autostart-host -autostart-players=2		-- Host game on Acropolis map, 2 players
@@ -1128,7 +1109,7 @@ bool Autostart(const CmdLineArgs& args)
 		CStr seedArg = args.Get("autostart-random");
 
 		// Default seed is 0
-		uint32 seed = 0;
+		u32 seed = 0;
 		if (!seedArg.empty())
 		{
 			if (seedArg.compare("-1") == 0)
@@ -1142,8 +1123,7 @@ bool Autostart(const CmdLineArgs& args)
 		}
 		
 		// Random map definition will be loaded from JSON file, so we need to parse it
-		std::wstring mapPath = L"maps/random/";
-		std::wstring scriptPath = mapPath + autoStartName.FromUTF8() + L".json";
+		std::wstring scriptPath = L"maps/random/" + autoStartName.FromUTF8() + L".json";
 		CScriptValRooted scriptData = scriptInterface.ReadJSONFile(scriptPath);
 		if (!scriptData.undefined() && scriptInterface.GetProperty(scriptData.get(), "settings", settings))
 		{
@@ -1168,7 +1148,6 @@ bool Autostart(const CmdLineArgs& args)
 		}
 
 		scriptInterface.SetProperty(attrs.get(), "map", std::string(autoStartName));
-		scriptInterface.SetProperty(attrs.get(), "mapPath", mapPath);
 		scriptInterface.SetProperty(attrs.get(), "mapType", std::string("random"));
 		scriptInterface.SetProperty(settings.get(), "Seed", seed);									// Random seed
 		scriptInterface.SetProperty(settings.get(), "Size", mapSize);								// Random map size (in patches)
@@ -1195,7 +1174,9 @@ bool Autostart(const CmdLineArgs& args)
 	}
 	else
 	{
-		scriptInterface.SetProperty(attrs.get(), "map", std::string(autoStartName));
+		// TODO: support akirmish maps
+		std::string mapFile = "maps/scenarios/" + autoStartName;
+		scriptInterface.SetProperty(attrs.get(), "map", mapFile);
 		scriptInterface.SetProperty(attrs.get(), "mapType", std::string("scenario"));
 	}
 
@@ -1269,9 +1250,8 @@ bool Autostart(const CmdLineArgs& args)
 	scriptInterface.SetProperty(attrs.get(), "settings", settings);
 
 	CScriptVal mpInitData;
-	g_GUI->GetScriptInterface().Eval("({isNetworked:true, playerAssignments:{}})", mpInitData);
-	g_GUI->GetScriptInterface().SetProperty(mpInitData.get(), "attribs",
-			CScriptVal(g_GUI->GetScriptInterface().CloneValueFromOtherContext(scriptInterface, attrs.get())));
+	scriptInterface.Eval("({isNetworked:true, playerAssignments:{}})", mpInitData);
+	scriptInterface.SetProperty(mpInitData.get(), "attribs", attrs);
 
 	// Get optional playername
 	CStrW userName = L"anonymous";
@@ -1282,7 +1262,7 @@ bool Autostart(const CmdLineArgs& args)
 
 	if (args.Has("autostart-host"))
 	{
-		InitPs(true, L"page_loading.xml", mpInitData.get());
+		InitPs(true, L"page_loading.xml", &scriptInterface, mpInitData.get());
 
 		size_t maxPlayers = 2;
 		if (args.Has("autostart-players"))
@@ -1303,7 +1283,7 @@ bool Autostart(const CmdLineArgs& args)
 	}
 	else if (args.Has("autostart-client"))
 	{
-		InitPs(true, L"page_loading.xml", mpInitData.get());
+		InitPs(true, L"page_loading.xml", &scriptInterface, mpInitData.get());
 
 		g_NetClient = new CNetClient(g_Game);
 		g_NetClient->SetUserName(userName);
@@ -1327,7 +1307,7 @@ bool Autostart(const CmdLineArgs& args)
 		PSRETURN ret = g_Game->ReallyStartGame();
 		ENSURE(ret == PSRETURN_OK);
 
-		InitPs(true, L"page_session.xml", JSVAL_VOID);
+		InitPs(true, L"page_session.xml", NULL, JSVAL_VOID);
 	}
 
 	return true;
@@ -1342,14 +1322,17 @@ void CancelLoad(const CStrW& message)
 	// So all GUI pages that load games should include this script
 	if (g_GUI && g_GUI->HasPages())
 	{
-		JSContext* cx = g_ScriptingHost.getContext();
-		jsval fval, rval;
-		JSBool ok = JS_GetProperty(cx, g_GUI->GetScriptObject(), "cancelOnError", &fval);
-		ENSURE(ok);
-
-		jsval msgval = ToJSVal(message);
-
-		if (ok && !JSVAL_IS_VOID(fval))
-			JS_CallFunctionValue(cx, g_GUI->GetScriptObject(), fval, 1, &msgval, &rval);
+		if (g_GUI->GetActiveGUI()->GetScriptInterface()->HasProperty(g_GUI->GetActiveGUI()->GetGlobalObject(), "cancelOnError" ))
+			g_GUI->GetActiveGUI()->GetScriptInterface()->CallFunctionVoid(g_GUI->GetActiveGUI()->GetGlobalObject(), "cancelOnError", message);
 	}
 }
+
+bool InDevelopmentCopy()
+{
+	if (!g_CheckedIfInDevelopmentCopy)
+	{
+		g_InDevelopmentCopy = (g_VFS->GetFileInfo(L"config/dev.cfg", NULL) == INFO::OK);
+		g_CheckedIfInDevelopmentCopy = true;
+	}
+	return g_InDevelopmentCopy;
+}
