@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2013 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -17,13 +17,13 @@
 
 #include "precompiled.h"
 
-#include "scripting/ScriptingHost.h"
 #include "scriptinterface/ScriptInterface.h"
 
 #include "lib/ogl.h"
 #include "lib/svn_revision.h"
 #include "lib/timer.h"
 #include "lib/utf8.h"
+#include "lib/external_libraries/libsdl.h"
 #include "lib/res/graphics/ogl_tex.h"
 #include "lib/posix/posix_utsname.h"
 #include "lib/sysdep/cpu.h"
@@ -42,6 +42,30 @@
 #include "ps/VideoMode.h"
 #include "ps/GameSetup/Config.h"
 
+#ifdef SDL_VIDEO_DRIVER_X11
+#include <GL/glx.h>
+#include <SDL/SDL_syswm.h>
+
+// Define the GLX_MESA_query_renderer macros if built with
+// an old Mesa (<10.0) that doesn't provide them
+#ifndef GLX_MESA_query_renderer
+#define GLX_MESA_query_renderer 1
+#define GLX_RENDERER_VENDOR_ID_MESA                      0x8183
+#define GLX_RENDERER_DEVICE_ID_MESA                      0x8184
+#define GLX_RENDERER_VERSION_MESA                        0x8185
+#define GLX_RENDERER_ACCELERATED_MESA                    0x8186
+#define GLX_RENDERER_VIDEO_MEMORY_MESA                   0x8187
+#define GLX_RENDERER_UNIFIED_MEMORY_ARCHITECTURE_MESA    0x8188
+#define GLX_RENDERER_PREFERRED_PROFILE_MESA              0x8189
+#define GLX_RENDERER_OPENGL_CORE_PROFILE_VERSION_MESA    0x818A
+#define GLX_RENDERER_OPENGL_COMPATIBILITY_PROFILE_VERSION_MESA    0x818B
+#define GLX_RENDERER_OPENGL_ES_PROFILE_VERSION_MESA      0x818C
+#define GLX_RENDERER_OPENGL_ES2_PROFILE_VERSION_MESA     0x818D
+#define GLX_RENDERER_ID_MESA                             0x818E
+#endif /* GLX_MESA_query_renderer */
+
+#endif
+
 static void ReportGLLimits(ScriptInterface& scriptInterface, CScriptValRooted settings);
 
 #if ARCH_X86_X64
@@ -98,30 +122,30 @@ static bool IsOverridden(const char* setting)
 	return !(ns == CFG_LAST || ns == CFG_DEFAULT);
 }
 
-void SetDisableAudio(void* UNUSED(cbdata), bool disabled)
+void SetDisableAudio(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), bool disabled)
 {
 	g_DisableAudio = disabled;
 }
 
-void SetDisableS3TC(void* UNUSED(cbdata), bool disabled)
+void SetDisableS3TC(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), bool disabled)
 {
 	if (!IsOverridden("nos3tc"))
 		ogl_tex_override(OGL_TEX_S3TC, disabled ? OGL_TEX_DISABLE : OGL_TEX_ENABLE);
 }
 
-void SetDisableShadows(void* UNUSED(cbdata), bool disabled)
+void SetDisableShadows(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), bool disabled)
 {
 	if (!IsOverridden("shadows"))
 		g_Shadows = !disabled;
 }
 
-void SetDisableShadowPCF(void* UNUSED(cbdata), bool disabled)
+void SetDisableShadowPCF(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), bool disabled)
 {
 	if (!IsOverridden("shadowpcf"))
 		g_ShadowPCF = !disabled;
 }
 
-void SetDisableAllWater(void* UNUSED(cbdata), bool disabled)
+void SetDisableAllWater(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), bool disabled)
 {
 	if (!IsOverridden("waternormals"))
 		g_WaterNormal = !disabled;
@@ -138,7 +162,7 @@ void SetDisableAllWater(void* UNUSED(cbdata), bool disabled)
 	if (!IsOverridden("watershadows"))
 		g_WaterShadows = !disabled;
 }
-void SetDisableFancyWater(void* UNUSED(cbdata), bool disabled)
+void SetDisableFancyWater(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), bool disabled)
 {
 	if (!IsOverridden("waterrealdepth"))
 		g_WaterRealDepth = !disabled;
@@ -149,7 +173,7 @@ void SetDisableFancyWater(void* UNUSED(cbdata), bool disabled)
 	if (!IsOverridden("watershadows"))
 		g_WaterShadows = !disabled;
 }
-void SetDisableFBOWater(void* UNUSED(cbdata), bool disabled)
+void SetDisableFBOWater(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), bool disabled)
 {
 	if (!IsOverridden("waterfoam"))
 		g_WaterFoam = !disabled;
@@ -157,7 +181,7 @@ void SetDisableFBOWater(void* UNUSED(cbdata), bool disabled)
 		g_WaterCoastalWaves = !disabled;
 }
 
-void SetRenderPath(void* UNUSED(cbdata), std::string renderpath)
+void SetRenderPath(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), std::string renderpath)
 {
 	g_RenderPath = renderpath;
 }
@@ -166,7 +190,7 @@ void RunHardwareDetection()
 {
 	TIMER(L"RunHardwareDetection");
 
-	ScriptInterface& scriptInterface = g_ScriptingHost.GetScriptInterface();
+	ScriptInterface scriptInterface("Engine", "HWDetect", g_ScriptRuntime);
 
 	scriptInterface.RegisterFunction<void, bool, &SetDisableAudio>("SetDisableAudio");
 	scriptInterface.RegisterFunction<void, bool, &SetDisableS3TC>("SetDisableS3TC");
@@ -225,7 +249,6 @@ void RunHardwareDetection()
 
 	scriptInterface.SetProperty(settings.get(), "gfx_card", gfx::CardName());
 	scriptInterface.SetProperty(settings.get(), "gfx_drv_ver", gfx::DriverInfo());
-	scriptInterface.SetProperty(settings.get(), "gfx_mem", (u32)gfx::MemorySizeMiB());
 
 	scriptInterface.SetProperty(settings.get(), "snd_card", std::wstring(snd_card));
 	scriptInterface.SetProperty(settings.get(), "snd_drv_ver", std::wstring(snd_drv_ver));
@@ -652,4 +675,68 @@ static void ReportGLLimits(ScriptInterface& scriptInterface, CScriptValRooted se
 	INTEGER(MAX_RENDERBUFFER_SIZE);
 
 #endif // CONFIG2_GLES
+
+
+#ifdef SDL_VIDEO_DRIVER_X11
+
+#define GLXQCR_INTEGER(id) do { \
+	unsigned int i = UINT_MAX; \
+	if (pglXQueryCurrentRendererIntegerMESA(id, &i)) \
+		scriptInterface.SetProperty(settings.get(), #id, i); \
+	} while (false)
+
+#define GLXQCR_INTEGER2(id) do { \
+	unsigned int i[2] = { UINT_MAX, UINT_MAX }; \
+	if (pglXQueryCurrentRendererIntegerMESA(id, i)) { \
+		scriptInterface.SetProperty(settings.get(), #id "[0]", i[0]); \
+		scriptInterface.SetProperty(settings.get(), #id "[1]", i[1]); \
+	} \
+	} while (false)
+
+#define GLXQCR_INTEGER3(id) do { \
+	unsigned int i[3] = { UINT_MAX, UINT_MAX, UINT_MAX }; \
+	if (pglXQueryCurrentRendererIntegerMESA(id, i)) { \
+		scriptInterface.SetProperty(settings.get(), #id "[0]", i[0]); \
+		scriptInterface.SetProperty(settings.get(), #id "[1]", i[1]); \
+		scriptInterface.SetProperty(settings.get(), #id "[2]", i[2]); \
+	} \
+	} while (false)
+
+#define GLXQCR_STRING(id) do { \
+	const char* str = pglXQueryCurrentRendererStringMESA(id); \
+	if (str) \
+		scriptInterface.SetProperty(settings.get(), #id ".string", str); \
+	} while (false)
+
+
+	SDL_SysWMinfo wminfo;
+	SDL_VERSION(&wminfo.version);
+	if (SDL_GetWMInfo(&wminfo) && wminfo.subsystem == SDL_SYSWM_X11)
+	{
+		Display* dpy = wminfo.info.x11.gfxdisplay;
+		int scrnum = DefaultScreen(dpy);
+
+		const char* glxexts = glXQueryExtensionsString(dpy, scrnum);
+
+		scriptInterface.SetProperty(settings.get(), "glx_extensions", glxexts);
+
+		if (strstr(glxexts, "GLX_MESA_query_renderer") && pglXQueryCurrentRendererIntegerMESA && pglXQueryCurrentRendererStringMESA)
+		{
+			GLXQCR_INTEGER(GLX_RENDERER_VENDOR_ID_MESA);
+			GLXQCR_INTEGER(GLX_RENDERER_DEVICE_ID_MESA);
+			GLXQCR_INTEGER3(GLX_RENDERER_VERSION_MESA);
+			GLXQCR_INTEGER(GLX_RENDERER_ACCELERATED_MESA);
+			GLXQCR_INTEGER(GLX_RENDERER_VIDEO_MEMORY_MESA);
+			GLXQCR_INTEGER(GLX_RENDERER_UNIFIED_MEMORY_ARCHITECTURE_MESA);
+			GLXQCR_INTEGER(GLX_RENDERER_PREFERRED_PROFILE_MESA);
+			GLXQCR_INTEGER2(GLX_RENDERER_OPENGL_CORE_PROFILE_VERSION_MESA);
+			GLXQCR_INTEGER2(GLX_RENDERER_OPENGL_COMPATIBILITY_PROFILE_VERSION_MESA);
+			GLXQCR_INTEGER2(GLX_RENDERER_OPENGL_ES_PROFILE_VERSION_MESA);
+			GLXQCR_INTEGER2(GLX_RENDERER_OPENGL_ES2_PROFILE_VERSION_MESA);
+			GLXQCR_STRING(GLX_RENDERER_VENDOR_ID_MESA);
+			GLXQCR_STRING(GLX_RENDERER_DEVICE_ID_MESA);
+		}
+	}
+#endif // SDL_VIDEO_DRIVER_X11
+
 }
