@@ -659,14 +659,14 @@ private:
 							if (counter.type == INTEL_PERFQUERIES_TYPE_UNSIGNED_INT)
 							{
 								ENSURE(counter.size == 4);
-								GLuint value;
+								GLuint value = 0;
 								memcpy(&value, buf + counter.offset, counter.size);
 								m_Storage.RecordAttributePrintf("%s: %u", counter.name.c_str(), value);
 							}
 							else if (counter.type == INTEL_PERFQUERIES_TYPE_UNSIGNED_INT64)
 							{
 								ENSURE(counter.size == 8);
-								GLuint64 value;
+								GLuint64 value = 0;
 								memcpy(&value, buf + counter.offset, counter.size);
 								m_Storage.RecordAttributePrintf("%s: %.0f", counter.name.c_str(), (double)value);
 
@@ -676,14 +676,14 @@ private:
 							else if (counter.type == INTEL_PERFQUERIES_TYPE_FLOAT)
 							{
 								ENSURE(counter.size == 4);
-								GLfloat value;
+								GLfloat value = 0;
 								memcpy(&value, buf + counter.offset, counter.size);
 								m_Storage.RecordAttributePrintf("%s: %f", counter.name.c_str(), value);
 							}
 							else if (counter.type == INTEL_PERFQUERIES_TYPE_BOOL)
 							{
 								ENSURE(counter.size == 4);
-								GLuint value;
+								GLuint value = 0;
 								memcpy(&value, buf + counter.offset, counter.size);
 								ENSURE(value == 0 || value == 1);
 								m_Storage.RecordAttributePrintf("%s: %u", counter.name.c_str(), value);
@@ -776,9 +776,9 @@ CProfiler2GPU::CProfiler2GPU(CProfiler2& profiler) :
 	bool enabledARB = false;
 	bool enabledEXT = false;
 	bool enabledINTEL = false;
-	CFG_GET_USER_VAL("profiler2.gpu.arb.enable", Bool, enabledARB);
-	CFG_GET_USER_VAL("profiler2.gpu.ext.enable", Bool, enabledEXT);
-	CFG_GET_USER_VAL("profiler2.gpu.intel.enable", Bool, enabledINTEL);
+	CFG_GET_VAL("profiler2.gpu.arb.enable", Bool, enabledARB);
+	CFG_GET_VAL("profiler2.gpu.ext.enable", Bool, enabledEXT);
+	CFG_GET_VAL("profiler2.gpu.intel.enable", Bool, enabledINTEL);
 
 	// Only enable either ARB or EXT, not both, because they are redundant
 	// (EXT is only needed for compatibility with older systems), and because
