@@ -1,4 +1,4 @@
-/* Copyright (C) 2011 Wildfire Games.
+/* Copyright (C) 2013 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -127,13 +127,13 @@ void CReplayPlayer::Replay()
 	new CProfileManager;
 	g_ScriptStatsTable = new CScriptStatsTable;
 	g_ProfileViewer.AddRootTable(g_ScriptStatsTable);
+	g_ScriptRuntime = ScriptInterface::CreateRuntime(384 * 1024 * 1024);
 
 	CGame game(true);
 	g_Game = &game;
 
 	// Need some stuff for terrain movement costs:
 	// (TODO: this ought to be independent of any graphics code)
-	tex_codec_register_all();
 	new CTerrainTextureManager;
 	g_TexMan.LoadTerrainTextures();
 
@@ -170,7 +170,7 @@ void CReplayPlayer::Replay()
 		}
 		else if (type == "cmd")
 		{
-			u32 player;
+			player_id_t player;
 			*m_Stream >> player;
 
 			std::string line;
@@ -243,9 +243,12 @@ void CReplayPlayer::Replay()
 
 	timer_DisplayClientTotals();
 
+	// Must be explicitly destructed here to avoid callbacks from the JSAPI trying to use g_Profiler2 when
+	// it's already destructed.
+	g_ScriptRuntime.reset();
+	
 	// Clean up
 	delete &g_TexMan;
-	tex_codec_unregister_all();
 
 	delete &g_Profiler;
 	delete &g_ProfileViewer;
