@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2014 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -19,20 +19,26 @@
 
 #include "SavedGame.h"
 
+#include "graphics/GameView.h"
 #include "gui/GUIManager.h"
 #include "lib/allocators/shared_ptr.h"
 #include "lib/file/archive/archive_zip.h"
+#include "i18n/L10n.h"
+#include "lib/utf8.h"
 #include "ps/CLogger.h"
 #include "ps/Filesystem.h"
+#include "ps/Game.h"
 #include "scriptinterface/ScriptInterface.h"
 #include "simulation2/Simulation2.h"
 
 static const int SAVED_GAME_VERSION_MAJOR = 1; // increment on incompatible changes to the format
 static const int SAVED_GAME_VERSION_MINOR = 0; // increment on compatible changes to the format
+std::vector<std::string> g_modsLoaded; // list of mods loaded
+
 // TODO: we ought to check version numbers when loading files
 
 
-Status SavedGames::SavePrefix(const std::wstring& prefix, const std::wstring& description, CSimulation2& simulation, CGUIManager* gui, int playerID)
+Status SavedGames::SavePrefix(const std::wstring& prefix, const std::wstring& description, CSimulation2& simulation, shared_ptr<ScriptInterface::StructuredClone> guiMetadataClone, int playerID)
 {
 	// Determine the filename to save under
 	const VfsPath basenameFormat(L"saves/" + prefix + L"-%04d");
@@ -44,10 +50,10 @@ Status SavedGames::SavePrefix(const std::wstring& prefix, const std::wstring& de
 	size_t nextSaveNumber = 0;
 	vfs::NextNumberedFilename(g_VFS, filenameFormat, nextSaveNumber, filename);
 
-	return Save(filename.Filename().string(), description, simulation, gui, playerID);
+	return Save(filename.Filename().string(), description, simulation, guiMetadataClone, playerID);
 }
 
-Status SavedGames::Save(const std::wstring& name, const std::wstring& description, CSimulation2& simulation, CGUIManager* gui, int playerID)
+Status SavedGames::Save(const std::wstring& name, const std::wstring& description, CSimulation2& simulation, shared_ptr<ScriptInterface::StructuredClone> guiMetadataClone, int playerID)
 {
 	// Determine the filename to save under
 	const VfsPath basenameFormat(L"saves/" + name);
@@ -76,14 +82,25 @@ Status SavedGames::Save(const std::wstring& name, const std::wstring& descriptio
 	simulation.GetScriptInterface().Eval("({})", metadata);
 	simulation.GetScriptInterface().SetProperty(metadata.get(), "version_major", SAVED_GAME_VERSION_MAJOR);
 	simulation.GetScriptInterface().SetProperty(metadata.get(), "version_minor", SAVED_GAME_VERSION_MINOR);
+	simulation.GetScriptInterface().SetProperty(metadata.get(), "mods", g_modsLoaded);
 	simulation.GetScriptInterface().SetProperty(metadata.get(), "time", (double)now);
 	simulation.GetScriptInterface().SetProperty(metadata.get(), "player", playerID);
 	simulation.GetScriptInterface().SetProperty(metadata.get(), "initAttributes", simulation.GetInitAttributes());
-	if (gui)
-	{
-		CScriptVal guiMetadata = simulation.GetScriptInterface().CloneValueFromOtherContext(gui->GetScriptInterface(), gui->GetSavedGameData().get());
-		simulation.GetScriptInterface().SetProperty(metadata.get(), "gui", guiMetadata);
-	}
+
+	CScriptVal guiMetadata = simulation.GetScriptInterface().ReadStructuredClone(guiMetadataClone);
+
+	// get some camera data
+	CScriptVal cameraMetadata;
+	simulation.GetScriptInterface().Eval("({})", cameraMetadata);
+	simulation.GetScriptInterface().SetProperty(cameraMetadata.get(), "PosX", g_Game->GetView()->GetCameraPosX());
+	simulation.GetScriptInterface().SetProperty(cameraMetadata.get(), "PosY", g_Game->GetView()->GetCameraPosY());
+	simulation.GetScriptInterface().SetProperty(cameraMetadata.get(), "PosZ", g_Game->GetView()->GetCameraPosZ());
+	simulation.GetScriptInterface().SetProperty(cameraMetadata.get(), "RotX", g_Game->GetView()->GetCameraRotX());
+	simulation.GetScriptInterface().SetProperty(cameraMetadata.get(), "RotY", g_Game->GetView()->GetCameraRotY());
+	simulation.GetScriptInterface().SetProperty(cameraMetadata.get(), "Zoom", g_Game->GetView()->GetCameraZoom());
+	simulation.GetScriptInterface().SetProperty(guiMetadata.get(), "camera", cameraMetadata);
+	simulation.GetScriptInterface().SetProperty(metadata.get(), "gui", guiMetadata);
+
 	simulation.GetScriptInterface().SetProperty(metadata.get(), "description", description);
 	
 	std::string metadataString = simulation.GetScriptInterface().StringifyJSON(metadata.get(), true);
@@ -98,7 +115,7 @@ Status SavedGames::Save(const std::wstring& name, const std::wstring& descriptio
 	archiveWriter.reset(); // close the file
 
 	WriteBuffer buffer;
-	FileInfo tempSaveFile;
+	CFileInfo tempSaveFile;
 	WARN_RETURN_STATUS_IF_ERR(GetFileInfo(tempSaveFileRealPath, &tempSaveFile));
 	buffer.Reserve(tempSaveFile.Size());
 	WARN_RETURN_STATUS_IF_ERR(io::Load(tempSaveFileRealPath, buffer.Data().get(), buffer.Size()));
@@ -106,7 +123,7 @@ Status SavedGames::Save(const std::wstring& name, const std::wstring& descriptio
 
 	OsPath realPath;
 	WARN_RETURN_STATUS_IF_ERR(g_VFS->GetRealPath(filename, realPath));
-	LOGMESSAGERENDER(L"Saved game to %ls\n", realPath.string().c_str());
+	LOGMESSAGERENDER(wstring_from_utf8(L10n::Instance().Translate("Saved game to '%ls'") + "\n").c_str(), realPath.string().c_str());
 
 	return INFO::OK;
 }
@@ -123,12 +140,12 @@ public:
 	{
 	}
 
-	static void ReadEntryCallback(const VfsPath& pathname, const FileInfo& fileInfo, PIArchiveFile archiveFile, uintptr_t cbData)
+	static void ReadEntryCallback(const VfsPath& pathname, const CFileInfo& fileInfo, PIArchiveFile archiveFile, uintptr_t cbData)
 	{
 		((CGameLoader*)cbData)->ReadEntry(pathname, fileInfo, archiveFile);
 	}
 
-	void ReadEntry(const VfsPath& pathname, const FileInfo& fileInfo, PIArchiveFile archiveFile)
+	void ReadEntry(const VfsPath& pathname, const CFileInfo& fileInfo, PIArchiveFile archiveFile)
 	{
 		if (pathname == L"metadata.json" && m_Metadata)
 		{
@@ -242,3 +259,14 @@ bool SavedGames::DeleteSavedGame(const std::wstring& name)
 	// Successfully deleted file
 	return true;
 }
+
+CScriptValRooted SavedGames::GetEngineInfo(ScriptInterface& scriptInterface) 
+{ 
+	CScriptValRooted metainfo; 
+	scriptInterface.Eval("({})", metainfo); 
+	scriptInterface.SetProperty(metainfo.get(), "version_major", SAVED_GAME_VERSION_MAJOR); 
+	scriptInterface.SetProperty(metainfo.get(), "version_minor", SAVED_GAME_VERSION_MINOR); 
+	scriptInterface.SetProperty(metainfo.get(), "mods"         , g_modsLoaded);
+	return metainfo; 
+}
+
