@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2014 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -33,6 +33,11 @@
 #include "ps/GameSetup/Config.h"
 #include "renderer/Renderer.h"
 
+#if OS_MACOSX
+# include "lib/sysdep/os/osx/osx_sys_version.h"
+#endif
+
+
 static int DEFAULT_WINDOW_W = 1024;
 static int DEFAULT_WINDOW_H = 768;
 
@@ -42,7 +47,7 @@ static int DEFAULT_FULLSCREEN_H = 768;
 CVideoMode g_VideoMode;
 
 CVideoMode::CVideoMode() :
-	m_IsInitialised(false), m_Window(NULL),
+	m_IsFullscreen(false), m_IsInitialised(false), m_Window(NULL),
 	m_PreferredW(0), m_PreferredH(0), m_PreferredBPP(0), m_PreferredFreq(0),
 	m_ConfigW(0), m_ConfigH(0), m_ConfigBPP(0), m_ConfigFullscreen(false), m_ConfigForceS3TCEnable(true),
 	m_WindowedW(DEFAULT_WINDOW_W), m_WindowedH(DEFAULT_WINDOW_H)
@@ -234,12 +239,13 @@ bool CVideoMode::InitSDL()
 	SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 24);
 	SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE, 8);
 	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
-#if !SDL_VERSION_ATLEAST(1, 3, 0)
+#if !SDL_VERSION_ATLEAST(2, 0, 0)
 	SDL_GL_SetAttribute(SDL_GL_SWAP_CONTROL, g_VSync ? 1 : 0);
 #endif
 
-#if CONFIG2_GLES && SDL_VERSION_ATLEAST(1, 3, 0)
+#if CONFIG2_GLES && SDL_VERSION_ATLEAST(2, 0, 0)
 	// Require GLES 2.0
+	SDL_GL_SetAttribute(SDL_GL_CONTEXT_PROFILE_MASK, SDL_GL_CONTEXT_PROFILE_ES);
 	SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, 2);
 	SDL_GL_SetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, 0);
 #endif
@@ -254,7 +260,7 @@ bool CVideoMode::InitSDL()
 			return false;
 	}
 
-#if SDL_VERSION_ATLEAST(1, 3, 0)
+#if SDL_VERSION_ATLEAST(2, 0, 0)
 	SDL_GL_SetSwapInterval(g_VSync ? 1 : 0);
 #endif
 
@@ -277,8 +283,18 @@ bool CVideoMode::InitSDL()
 	if (SDL_SetWindowGammaRamp(m_Window, ramp, ramp, ramp) < 0)
 		LOGWARNING(L"SDL_SetGamma failed");
 #else
+# if OS_MACOSX
+	// Workaround for crash on Mavericks, see http://trac.wildfiregames.com/ticket/2272
+	int major, minor, bugfix;
+	GetSystemVersion(major, minor, bugfix);
+	if (minor < 9)
+	{
+# endif
 	if (SDL_SetGamma(g_Gamma, g_Gamma, g_Gamma) < 0)
 		LOGWARNING(L"SDL_SetGamma failed");
+# if OS_MACOSX
+	}
+# endif
 #endif
 
 	m_IsInitialised = true;
@@ -309,6 +325,7 @@ void CVideoMode::Shutdown()
 {
 	ENSURE(m_IsInitialised);
 
+	m_IsFullscreen = false;
 	m_IsInitialised = false;
 }
 
