@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2014 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -28,20 +28,21 @@
 #include "graphics/ShaderProgram.h"
 #include "lib/res/handle.h"
 #include "ps/Singleton.h"
-#include "scripting/ScriptableObject.h"
 
+#include "graphics/ShaderDefines.h"
 #include "renderer/PostprocManager.h"
 #include "renderer/Scene.h"
 #include "renderer/TimeManager.h"
+#include "scriptinterface/ScriptInterface.h"
 
 // necessary declarations
+class CFontManager;
 class CLightEnv;
 class CMaterial;
 class CMaterialManager;
 class CModel;
 class CParticleManager;
 class CPatch;
-class CShaderDefines;
 class CShaderManager;
 class CSimulation2;
 class CTextureManager;
@@ -71,7 +72,6 @@ struct CRendererInternals;
 
 class CRenderer :
 	public Singleton<CRenderer>,
-	public CJSObject<CRenderer>,
 	private SceneCollector
 {
 public:
@@ -89,8 +89,12 @@ public:
 		OPT_WATERSHADOW,
 		OPT_SHADOWPCF,
 		OPT_PARTICLES,
+		OPT_GENTANGENTS,
+		OPT_PREFERGLSL,
 		OPT_SILHOUETTES,
-		OPT_SHOWSKY
+		OPT_SHOWSKY,
+		OPT_SMOOTHLOS,
+		OPT_POSTPROC,
 	};
 
 	enum RenderPath {
@@ -219,6 +223,9 @@ public:
 	// set the viewport
 	void SetViewport(const SViewPort &);
 
+	// get the last viewport
+	SViewPort GetViewport();
+
 	/**
 	 * Render the given scene immediately.
 	 * @param scene a Scene object describing what should be rendered.
@@ -304,7 +311,9 @@ public:
 
 	CMaterialManager& GetMaterialManager();
 
-	CShaderDefines GetSystemShaderDefines();
+	CFontManager& GetFontManager();
+
+	CShaderDefines GetSystemShaderDefines() { return m_SystemShaderDefines; }
 	
 	CTimeManager& GetTimeManager();
 	
@@ -317,7 +326,7 @@ public:
 	 */
 	const Caps& GetCapabilities() const { return m_Caps; }
 
-	static void ScriptingInit();
+	static void RegisterScriptFunctions(ScriptInterface& scriptInterface);
 
 protected:
 	friend struct CRendererInternals;
@@ -335,24 +344,6 @@ protected:
 	friend class ShaderInstancingModelRenderer;
 	friend class TerrainRenderer;
 
-	// scripting
-	// TODO: Perhaps we could have a version of AddLocalProperty for function-driven
-	// properties? Then we could hide these function in the private implementation class.
-	jsval JSI_GetRenderPath(JSContext*);
-	void JSI_SetRenderPath(JSContext* ctx, jsval newval);
-	jsval JSI_GetDepthTextureBits(JSContext*);
-	void JSI_SetDepthTextureBits(JSContext* ctx, jsval newval);
-	jsval JSI_GetShadows(JSContext*);
-	void JSI_SetShadows(JSContext* ctx, jsval newval);
-	jsval JSI_GetShadowAlphaFix(JSContext*);
-	void JSI_SetShadowAlphaFix(JSContext* ctx, jsval newval);
-	jsval JSI_GetShadowPCF(JSContext*);
-	void JSI_SetShadowPCF(JSContext* ctx, jsval newval);
-	jsval JSI_GetPreferGLSL(JSContext*);
-	void JSI_SetPreferGLSL(JSContext* ctx, jsval newval);
-	jsval JSI_GetSky(JSContext*);
-	void JSI_SetSky(JSContext* ctx, jsval newval);
-
 	//BEGIN: Implementation of SceneCollector
 	void Submit(CPatch* patch);
 	void Submit(SOverlayLine* overlay);
@@ -392,6 +383,7 @@ protected:
 	void SetObliqueFrustumClipping(const CVector4D& clipPlane);
 
 	void ReloadShaders();
+	void RecomputeSystemShaderDefines();
 
 	// hotloading
 	static Status ReloadChangedFileCB(void* param, const VfsPath& path);
@@ -408,6 +400,10 @@ protected:
 	// current model rendering mode
 	ERenderMode m_ModelRenderMode;
 
+	CShaderDefines m_SystemShaderDefines;
+
+	SViewPort m_Viewport;
+
 	/**
 	 * m_ViewCamera: determines the eye position for rendering
 	 *
