@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2013 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -31,15 +31,17 @@
 
 #include "ps/CStr.h"
 #include "ps/CLogger.h"
+#include "ps/Game.h"
 #include "ps/Loader.h"
 #include "ps/Filesystem.h"
+#include "ps/World.h"
 
 #include "renderer/SkyManager.h"
 #include "renderer/Renderer.h"
 
-#include "graphics/Camera.h"
 #include "graphics/LightEnv.h"
 #include "graphics/ShaderManager.h"
+#include "graphics/Terrain.h"
 #include "graphics/TextureManager.h"
 
 
@@ -86,7 +88,7 @@ void SkyManager::LoadSkyTextures()
 		texture->Prefetch();
 		m_SkyTexture[i] = texture;
 	}*/
-	
+
 	///////////////////////////////////////////////////////////////////////////
 	// HACK: THE HORRIBLENESS HERE IS OVER 9000. The following code is a HUGE hack and will be removed completely
 	// as soon as all the hardcoded GL_TEXTURE_2D references are corrected in the TextureManager/OGL/tex libs.
@@ -130,45 +132,46 @@ void SkyManager::LoadSkyTextures()
 		}
 		
 		Tex tex;
-		tex_decode(file, fileSize, &tex);
+		tex.decode(file, fileSize);
 		
-		tex_transform_to(&tex, (tex.flags | TEX_BOTTOM_UP | TEX_ALPHA) & ~(TEX_DXT | TEX_MIPMAPS));
+		tex.transform_to((tex.m_Flags | TEX_BOTTOM_UP | TEX_ALPHA) & ~(TEX_DXT | TEX_MIPMAPS));
 		
-		u8* data = tex_get_data(&tex);
+		u8* data = tex.get_data();
 		
 		if (types[i] == GL_TEXTURE_CUBE_MAP_NEGATIVE_Y || types[i] == GL_TEXTURE_CUBE_MAP_POSITIVE_Y)
 		{
-			std::vector<u8> rotated(tex.dataSize);
+			std::vector<u8> rotated(tex.m_DataSize);
 		
-			for (size_t y = 0; y < tex.h; ++y)
+			for (size_t y = 0; y < tex.m_Height; ++y)
 			{
-				for (size_t x = 0; x < tex.w; ++x)
+				for (size_t x = 0; x < tex.m_Width; ++x)
 				{
-					size_t invx = y, invy = tex.w-x-1;
+					size_t invx = y, invy = tex.m_Width-x-1;
 					
-					rotated[(y*tex.w + x) * 4 + 0] = data[(invy*tex.w + invx) * 4 + 0];
-					rotated[(y*tex.w + x) * 4 + 1] = data[(invy*tex.w + invx) * 4 + 1];
-					rotated[(y*tex.w + x) * 4 + 2] = data[(invy*tex.w + invx) * 4 + 2];
-					rotated[(y*tex.w + x) * 4 + 3] = data[(invy*tex.w + invx) * 4 + 3];
+					rotated[(y*tex.m_Width + x) * 4 + 0] = data[(invy*tex.m_Width + invx) * 4 + 0];
+					rotated[(y*tex.m_Width + x) * 4 + 1] = data[(invy*tex.m_Width + invx) * 4 + 1];
+					rotated[(y*tex.m_Width + x) * 4 + 2] = data[(invy*tex.m_Width + invx) * 4 + 2];
+					rotated[(y*tex.m_Width + x) * 4 + 3] = data[(invy*tex.m_Width + invx) * 4 + 3];
 				}
 			}
 			
-			glTexImage2D(types[i], 0, GL_RGB, tex.w, tex.h, 0, GL_RGBA, GL_UNSIGNED_BYTE, &rotated[0]);
+			glTexImage2D(types[i], 0, GL_RGB, tex.m_Width, tex.m_Height, 0, GL_RGBA, GL_UNSIGNED_BYTE, &rotated[0]);
 		}
 		else
 		{
-			glTexImage2D(types[i], 0, GL_RGB, tex.w, tex.h, 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
+			glTexImage2D(types[i], 0, GL_RGB, tex.m_Width, tex.m_Height, 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
 		}
-		
-		tex_free(&tex);
 	}
 	
 	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
 	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
+#if CONFIG2_GLES
+#warning TODO: fix SkyManager::LoadSkyTextures for GLES
+#else
 	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
-	
+#endif
 	glBindTexture(GL_TEXTURE_2D, 0);
 	///////////////////////////////////////////////////////////////////////////
 }
@@ -223,12 +226,9 @@ void SkyManager::RenderSky()
 #warning TODO: implement SkyManager::RenderSky for GLES
 #else
 
-	// Draw the sky as a small box around the camera position, with depth write enabled.
+	// Draw the sky as a small box around the map, with depth write enabled.
 	// This will be done before anything else is drawn so we'll be overlapped by everything else.
-
-	// Note: The coordinates for this were set up through a rather cumbersome trial-and-error 
-	//       process - there might be a smarter way to do it, but this seems to work.
-
+	
 	// Do nothing unless SetSkySet was called
 	if (m_SkySet.empty())
 		return;
@@ -241,29 +241,28 @@ void SkyManager::RenderSky()
 	glMatrixMode(GL_MODELVIEW);
 	glPushMatrix();
 	
-	// Translate so we are at the camera in the X and Z directions, but
-	// put the horizon at a fixed height regardless of camera Y
-	const CCamera& camera = g_Renderer.GetViewCamera();
-	CVector3D pos = camera.m_Orientation.GetTranslation();
-	
-	glTranslatef( pos.X, m_HorizonHeight, pos.Z );
+	// Translate so we are at the center of the map.
+	ssize_t mapSize = g_Game->GetWorld()->GetTerrain()->GetVerticesPerSide();
+	glTranslatef(mapSize*(TERRAIN_TILE_SIZE/2.0f), m_HorizonHeight, mapSize*(TERRAIN_TILE_SIZE/2.0f) );
 
 	// Rotate so that the "left" face, which contains the brightest part of each
 	// skymap, is in the direction of the sun from our light environment
 	glRotatef( 180.0f /*+ 45.0f*/ + RADTODEG(g_Renderer.GetLightEnv().GetRotation()), 0.0f, 1.0f, 0.0f );
 
 	// Distance to draw the faces at
-	const float D = 2000.0;
+	const float D = 1500.0f; // distance from map center
+	const float H = 500.0f; // height of the ceiling
+	const float FH = -100.0f; // height of the "floor"
 	
 	CShaderProgramPtr shader;
 	CShaderTechniquePtr skytech;
 	
 	if (g_Renderer.GetRenderPath() == CRenderer::RP_SHADER)
 	{
-		skytech = g_Renderer.GetShaderManager().LoadEffect("sky_simple");
+		skytech = g_Renderer.GetShaderManager().LoadEffect(str_sky_simple);
 		skytech->BeginPass();
 		shader = skytech->GetShader();
-		shader->BindTexture("baseTex", m_SkyCubeMap);
+		shader->BindTexture(str_baseTex, m_SkyCubeMap);
 	}
 	else
 	{
@@ -272,53 +271,43 @@ void SkyManager::RenderSky()
 		glBindTexture(GL_TEXTURE_CUBE_MAP, m_SkyCubeMap);
 	}
 
-	// GL_TEXTURE_CUBE_MAP_NEGATIVE_X
 	glBegin(GL_QUADS);
-		glTexCoord3f( +1, +1, +1 );  glVertex3f( -D, -D, -D ); 
-		glTexCoord3f( +1, +1, -1 );  glVertex3f( -D, -D, +D ); 
-		glTexCoord3f( +1, -1, -1 );  glVertex3f( -D, +D, +D ); 
-		glTexCoord3f( +1, -1, +1 );  glVertex3f( -D, +D, -D ); 
-	glEnd();
+
+	// GL_TEXTURE_CUBE_MAP_NEGATIVE_X
+		glTexCoord3f( +1, +1, +1 );  glVertex3f( -D, FH, -D );
+		glTexCoord3f( +1, +1, -1 );  glVertex3f( -D, FH, +D );
+		glTexCoord3f( +1, -1, -1 );  glVertex3f( -D, +H, +D );
+		glTexCoord3f( +1, -1, +1 );  glVertex3f( -D, +H, -D );
 
 	// GL_TEXTURE_CUBE_MAP_POSITIVE_X
-	glBegin(GL_QUADS);
-		glTexCoord3f( -1, +1, -1 );  glVertex3f( +D, -D, +D ); 
-		glTexCoord3f( -1, +1, +1 );  glVertex3f( +D, -D, -D ); 
-		glTexCoord3f( -1, -1, +1 );  glVertex3f( +D, +D, -D ); 
-		glTexCoord3f( -1, -1, -1 );  glVertex3f( +D, +D, +D ); 
-	glEnd();
+		glTexCoord3f( -1, +1, -1 );  glVertex3f( +D, FH, +D );
+		glTexCoord3f( -1, +1, +1 );  glVertex3f( +D, FH, -D );
+		glTexCoord3f( -1, -1, +1 );  glVertex3f( +D, +H, -D );
+		glTexCoord3f( -1, -1, -1 );  glVertex3f( +D, +H, +D );
 	
 	// GL_TEXTURE_CUBE_MAP_NEGATIVE_Y
-	glBegin(GL_QUADS);
-		glTexCoord3f( -1, +1, +1 );  glVertex3f( +D, -D, -D ); 
-		glTexCoord3f( -1, +1, -1 );  glVertex3f( +D, -D, +D ); 
-		glTexCoord3f( +1, +1, -1 );  glVertex3f( -D, -D, +D ); 
-		glTexCoord3f( +1, +1, +1 );  glVertex3f( -D, -D, -D ); 
+		glTexCoord3f( -1, +1, +1 );  glVertex3f( +D, FH, -D );
+		glTexCoord3f( -1, +1, -1 );  glVertex3f( +D, FH, +D );
+		glTexCoord3f( +1, +1, -1 );  glVertex3f( -D, FH, +D );
+		glTexCoord3f( +1, +1, +1 );  glVertex3f( -D, FH, -D );
 		
-	glEnd();
-	
 	// GL_TEXTURE_CUBE_MAP_POSITIVE_Y
-	glBegin(GL_QUADS);
-		glTexCoord3f( +1, -1, +1 );  glVertex3f( -D, +D, -D ); 
-		glTexCoord3f( +1, -1, -1 );  glVertex3f( -D, +D, +D ); 
-		glTexCoord3f( -1, -1, -1 );  glVertex3f( +D, +D, +D ); 
-		glTexCoord3f( -1, -1, +1 );  glVertex3f( +D, +D, -D ); 
-	glEnd();
+		glTexCoord3f( +1, -1, +1 );  glVertex3f( -D, +H, -D );
+		glTexCoord3f( +1, -1, -1 );  glVertex3f( -D, +H, +D );
+		glTexCoord3f( -1, -1, -1 );  glVertex3f( +D, +H, +D );
+		glTexCoord3f( -1, -1, +1 );  glVertex3f( +D, +H, -D );
 	
 	// GL_TEXTURE_CUBE_MAP_NEGATIVE_Z
-	glBegin(GL_QUADS);
-		glTexCoord3f( -1, +1, +1 );  glVertex3f( +D, -D, -D ); 
-		glTexCoord3f( +1, +1, +1 );  glVertex3f( -D, -D, -D ); 
-		glTexCoord3f( +1, -1, +1 );  glVertex3f( -D, +D, -D ); 
-		glTexCoord3f( -1, -1, +1 );  glVertex3f( +D, +D, -D ); 
-	glEnd();
+		glTexCoord3f( -1, +1, +1 );  glVertex3f( +D, FH, -D );
+		glTexCoord3f( +1, +1, +1 );  glVertex3f( -D, FH, -D );
+		glTexCoord3f( +1, -1, +1 );  glVertex3f( -D, +H, -D );
+		glTexCoord3f( -1, -1, +1 );  glVertex3f( +D, +H, -D );
 
 	// GL_TEXTURE_CUBE_MAP_POSITIVE_Z
-	glBegin(GL_QUADS);
-		glTexCoord3f( +1, +1, -1 );  glVertex3f( -D, -D, +D ); 
-		glTexCoord3f( -1, +1, -1 );  glVertex3f( +D, -D, +D ); 
-		glTexCoord3f( -1, -1, -1 );  glVertex3f( +D, +D, +D ); 
-		glTexCoord3f( +1, -1, -1 );  glVertex3f( -D, +D, +D ); 
+		glTexCoord3f( +1, +1, -1 );  glVertex3f( -D, FH, +D );
+		glTexCoord3f( -1, +1, -1 );  glVertex3f( +D, FH, +D );
+		glTexCoord3f( -1, -1, -1 );  glVertex3f( +D, +H, +D );
+		glTexCoord3f( +1, -1, -1 );  glVertex3f( -D, +H, +D );
 	glEnd();
 
 	if (g_Renderer.GetRenderPath() == CRenderer::RP_SHADER)
