@@ -15,7 +15,9 @@
  * along with 0 A.D.  If not, see <http://www.gnu.org/licenses/>.
  */
  
-#include "precompiled.h"
+ // JS debugger temporarily disabled during the SpiderMonkey upgrade (check trac ticket #2348 for details)
+#include "precompiled.h" // needs to be here for Windows builds
+ /*
 
 #include "ThreadDebugger.h"
 #include "lib/utf8.h"
@@ -168,10 +170,13 @@ public:
 	uint m_ID;
 };
 
-ThreadDebugger_impl::ThreadDebugger_impl() :
-	m_NextDbgCmd(DBG_CMD_NONE), m_IsInBreak(false), m_pLastBreakFrame(new JSStackFrame*)
-{
-}
+ThreadDebugger_impl::ThreadDebugger_impl()
+	: m_NextDbgCmd(DBG_CMD_NONE)
+	, m_pScriptInterface(NULL)
+	, m_pDebuggingServer(NULL)
+	, m_pLastBreakFrame(new JSStackFrame*)
+	, m_IsInBreak(false)
+{ }
 
 ThreadDebugger_impl::~ThreadDebugger_impl()
 {
@@ -188,11 +193,10 @@ void CThreadDebugger::ClearTrapsToRemove()
 	{
 		if ((*itr)->m_ToRemove)
 		{
-				ClearTrap((*itr));
-				// Remove the breakpoint
-				delete (*itr);
-				itr = m->m_ActiveBreakPoints.erase(itr);
-
+			ClearTrap((*itr));
+			// Remove the breakpoint
+			delete (*itr);
+			itr = m->m_ActiveBreakPoints.erase(itr);
 		}
 		else
 			++itr;
@@ -287,10 +291,10 @@ CThreadDebugger::~CThreadDebugger()
 	ReturnActiveBreakPoints(NULL);
 
 	// Remove all the hooks because they store a pointer to this object
-	JS_SetExecuteHook(m->m_pScriptInterface->GetRuntime(), NULL, NULL);
-	JS_SetCallHook(m->m_pScriptInterface->GetRuntime(), NULL, NULL);
-	JS_SetNewScriptHook(m->m_pScriptInterface->GetRuntime(), NULL, NULL);
-	JS_SetDestroyScriptHook(m->m_pScriptInterface->GetRuntime(), NULL, NULL);
+	JS_SetExecuteHook(m->m_pScriptInterface->GetJSRuntime(), NULL, NULL);
+	JS_SetCallHook(m->m_pScriptInterface->GetJSRuntime(), NULL, NULL);
+	JS_SetNewScriptHook(m->m_pScriptInterface->GetJSRuntime(), NULL, NULL);
+	JS_SetDestroyScriptHook(m->m_pScriptInterface->GetJSRuntime(), NULL, NULL);
 }
 
 void CThreadDebugger::ReturnActiveBreakPoints(jsbytecode* pBytecode)
@@ -327,16 +331,16 @@ void CThreadDebugger::Initialize(uint id, std::string name, ScriptInterface* pSc
 	m->m_Name = name;
 	m->m_pScriptInterface = pScriptInterface;
 	m->m_pDebuggingServer = pDebuggingServer;
-	JS_SetExecuteHook(m->m_pScriptInterface->GetRuntime(), CallHook_, (void*)this);
-	JS_SetCallHook(m->m_pScriptInterface->GetRuntime(), CallHook_, (void*)this);
-	JS_SetNewScriptHook(m->m_pScriptInterface->GetRuntime(), NewScriptHook_, (void*)this);
-	JS_SetDestroyScriptHook(m->m_pScriptInterface->GetRuntime(), DestroyScriptHook_, (void*)this);
-	JS_SetThrowHook(m->m_pScriptInterface->GetRuntime(), ThrowHandler_, (void*)this);
+	JS_SetExecuteHook(m->m_pScriptInterface->GetJSRuntime(), CallHook_, (void*)this);
+	JS_SetCallHook(m->m_pScriptInterface->GetJSRuntime(), CallHook_, (void*)this);
+	JS_SetNewScriptHook(m->m_pScriptInterface->GetJSRuntime(), NewScriptHook_, (void*)this);
+	JS_SetDestroyScriptHook(m->m_pScriptInterface->GetJSRuntime(), DestroyScriptHook_, (void*)this);
+	JS_SetThrowHook(m->m_pScriptInterface->GetJSRuntime(), ThrowHandler_, (void*)this);
 	
 	if (m->m_pDebuggingServer->GetSettingSimultaneousThreadBreak())
 	{
 		// Setup a handler to check for break-requests from the DebuggingServer regularly
-		JS_SetInterrupt(m->m_pScriptInterface->GetRuntime(), CheckForBreakRequestHandler_, (void*)this);
+		JS_SetInterrupt(m->m_pScriptInterface->GetJSRuntime(), CheckForBreakRequestHandler_, (void*)this);
 	}
 }
 
@@ -454,7 +458,7 @@ JSTrapStatus CThreadDebugger::BreakHandler(JSContext* cx, JSScript* script, jsby
 	
 	if (breakSrc == BREAK_SRC_INTERRUP)
 	{
-		JS_ClearInterrupt(m->m_pScriptInterface->GetRuntime(), NULL, NULL);
+		JS_ClearInterrupt(m->m_pScriptInterface->GetJSRuntime(), NULL, NULL);
 		JS_SetSingleStepMode(cx, script, false);
 	}
 	
@@ -494,17 +498,17 @@ JSTrapStatus CThreadDebugger::BreakHandler(JSContext* cx, JSScript* script, jsby
 			{
 				if (nextDbgCmd == DBG_CMD_SINGLESTEP)
 				{
-					JS_SetInterrupt(m->m_pScriptInterface->GetRuntime(), StepHandler_, this);
+					JS_SetInterrupt(m->m_pScriptInterface->GetJSRuntime(), StepHandler_, this);
 					break;
 				}
 				else if (nextDbgCmd == DBG_CMD_STEPINTO)
 				{
-					JS_SetInterrupt(m->m_pScriptInterface->GetRuntime(), StepIntoHandler_, this);
+					JS_SetInterrupt(m->m_pScriptInterface->GetJSRuntime(), StepIntoHandler_, this);
 					break;
 				}
 				else if (nextDbgCmd == DBG_CMD_STEPOUT)
 				{
-					JS_SetInterrupt(m->m_pScriptInterface->GetRuntime(), StepOutHandler_, this);
+					JS_SetInterrupt(m->m_pScriptInterface->GetJSRuntime(), StepOutHandler_, this);
 					break;
 				}
 			}
@@ -516,7 +520,7 @@ JSTrapStatus CThreadDebugger::BreakHandler(JSContext* cx, JSScript* script, jsby
 			else
 			{
 				// Setup a handler to check for break-requests from the DebuggingServer regularly
-				JS_SetInterrupt(m->m_pScriptInterface->GetRuntime(), CheckForBreakRequestHandler_, this);
+				JS_SetInterrupt(m->m_pScriptInterface->GetJSRuntime(), CheckForBreakRequestHandler_, this);
 			}
 			break;
 		}
@@ -527,7 +531,7 @@ JSTrapStatus CThreadDebugger::BreakHandler(JSContext* cx, JSScript* script, jsby
 	SetAllNewTraps();
 	SetNextDbgCmd(DBG_CMD_NONE);
 	SetIsInBreak(false);
-	SetBreakFileName("");
+	SetBreakFileName(std::string());
 	
 	// All saved stack data becomes invalid
 	{
@@ -542,7 +546,7 @@ void CThreadDebugger::NewScriptHook(JSContext* cx, const char* filename, unsigne
 {
 	uint scriptExtent = JS_GetScriptLineExtent (cx, script);
 	std::string stringFileName(filename);
-	if (stringFileName == "")
+	if (stringFileName.empty())
 		return;
 
 	for (uint line = lineno; line < scriptExtent + lineno; ++line) 
@@ -648,7 +652,6 @@ void CThreadDebugger::SaveCallstack()
 	
 	JSStackFrame *fp;
 	JSStackFrame *iter = 0;
-	std::string functionName;
 	jsint counter = 0;
 	
 	JSObject* jsArray;
@@ -676,7 +679,7 @@ void CThreadDebugger::SaveCallstack()
 		counter++;
 	}
 	
-	m->m_Callstack = "";
+	m->m_Callstack.clear();
 	m->m_Callstack = m->m_pScriptInterface->StringifyJSON(OBJECT_TO_JSVAL(jsArray), false).c_str();
 }
 
@@ -699,7 +702,7 @@ void CThreadDebugger::GetStackFrameData(std::stringstream& response, uint nestin
 	
 	CScopeLock lock(m->m_Mutex);
 	{
-		response.str("");
+		response.str(std::string());
 		response << m->m_StackFrameData[stackInfoKind][nestingLevel];
 	}
 }
@@ -718,7 +721,6 @@ void CThreadDebugger::SaveStackFrameData(STACK_INFO stackInfo, uint nestingLevel
 	ENSURE(GetIsInBreak());
 	
 	CScopeLock lock(m->m_Mutex);
-	JSStackFrame *fp;
 	JSStackFrame *iter = 0;
 	uint counter = 0;
 	jsval val;
@@ -731,7 +733,7 @@ void CThreadDebugger::SaveStackFrameData(STACK_INFO stackInfo, uint nestingLevel
 	}
 	else
 	{
-		fp = JS_FrameIterator(m->m_pScriptInterface->GetContext(), &iter);
+		JSStackFrame *fp = JS_FrameIterator(m->m_pScriptInterface->GetContext(), &iter);
 		while (fp)
 		{
 			if (counter == nestingLevel)
@@ -757,7 +759,7 @@ void CThreadDebugger::SaveStackFrameData(STACK_INFO stackInfo, uint nestingLevel
 		}
 	}
 }
-
+*/
 
 /*
  * TODO: This is very hacky and ugly and should be improved. 
@@ -766,20 +768,21 @@ void CThreadDebugger::SaveStackFrameData(STACK_INFO stackInfo, uint nestingLevel
  * Unfortunately this seems to require writing (or embedding) a new serializer to JSON or something similar.
  * 
  * Some things about the implementation which aren't optimal:
- * 1. It uses globabl variables (they are limited to a namespace though)
+ * 1. It uses global variables (they are limited to a namespace though).
  * 2. It has to work around a bug in Spidermonkey.
  * 3. It copies code from CScriptInterface. I did this to separate it cleanly because the debugger should not affect
  *    the rest of the game and because this part of code should be replaced anyway in the future.
  */
  
- namespace CyclicRefWorkaround
- {
+ /*
+namespace CyclicRefWorkaround
+{
 	std::set<JSObject*> g_ProcessedObjects;
 	jsval g_LastKey;
 	jsval g_LastValue;
 	bool g_RecursionDetectedInPrevReplacer = false;
 	uint g_countSameKeys = 0;
-
+	
 	struct Stringifier
 	{
 		static JSBool callback(const jschar* buf, uint32 len, void* data)
@@ -794,7 +797,7 @@ void CThreadDebugger::SaveStackFrameData(STACK_INFO stackInfo, uint nestingLevel
 
 		std::stringstream stream;
 	};
-	 
+	
 	JSBool replacer(JSContext* cx, uintN UNUSED(argc), jsval* vp)
 	{
 		jsval value = JS_ARGV(cx, vp)[1];
@@ -830,8 +833,8 @@ void CThreadDebugger::SaveStackFrameData(STACK_INFO stackInfo, uint nestingLevel
 		JS_SET_RVAL(cx, vp, JS_ARGV(cx, vp)[1]);
 		return JS_TRUE;
 	}
- }
- 
+}
+
 std::string CThreadDebugger::StringifyCyclicJSON(jsval obj, bool indent)
 {
 	CyclicRefWorkaround::Stringifier str;
@@ -861,14 +864,14 @@ std::string CThreadDebugger::StringifyCyclicJSON(jsval obj, bool indent)
 			
 		}			
 		JS_ClearPendingException(m->m_pScriptInterface->GetContext());
-		return "";
+		return std::string();
 	}
 
 	return str.stream.str();
 }
 
 
-bool CThreadDebugger::CompareScriptInterfacePtr(ScriptInterface* pScriptInterface)
+bool CThreadDebugger::CompareScriptInterfacePtr(ScriptInterface* pScriptInterface) const
 {
 	return (pScriptInterface == m->m_pScriptInterface);
 }
@@ -933,4 +936,4 @@ uint CThreadDebugger::GetID()
 	CScopeLock lock(m->m_Mutex);
 	return m->m_ID;
 }
-
+*/
