@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2013 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -258,6 +258,21 @@ public:
 	entity_angle_t a;
 };
 
+/*Sent whenever the territory type (neutral,own,enemy) differs from the former type*/
+class CMessageTerritoryPositionChanged : public CMessage
+{
+public:
+	DEFAULT_MESSAGE_IMPL(TerritoryPositionChanged)
+
+	CMessageTerritoryPositionChanged(entity_id_t entity, player_id_t newTerritory) : 
+		entity(entity), newTerritory(newTerritory)
+	{
+	}
+
+	entity_id_t entity;
+	player_id_t newTerritory;
+};
+
 /**
  * Sent by CCmpUnitMotion during Update, whenever the motion status has changed
  * since the previous update.
@@ -314,10 +329,7 @@ class CMessageRangeUpdate : public CMessage
 public:
 	DEFAULT_MESSAGE_IMPL(RangeUpdate)
 
-	CMessageRangeUpdate(u32 tag, const std::vector<entity_id_t>& added, const std::vector<entity_id_t>& removed) :
-		tag(tag), added(added), removed(removed)
-	{
-	}
+
 
 	u32 tag;
 	std::vector<entity_id_t> added;
@@ -327,16 +339,21 @@ public:
 	// swap vectors instead of copying (to save on memory allocations),
 	// so add some constructors for it:
 
-	CMessageRangeUpdate(u32 tag) :
-		tag(tag)
+	// don't init tag in empty ctor
+	CMessageRangeUpdate() 
 	{
 	}
-
-	CMessageRangeUpdate(const CMessageRangeUpdate& other) :
-		CMessage(), tag(other.tag), added(other.added), removed(other.removed)
+	CMessageRangeUpdate(u32 tag) : tag(tag)
+	{
+	}
+	CMessageRangeUpdate(u32 tag, const std::vector<entity_id_t>& added, const std::vector<entity_id_t>& removed)
+		: tag(tag), added(added), removed(removed)
+	{
+	}
+	CMessageRangeUpdate(const CMessageRangeUpdate& other) 
+		: CMessage(), tag(other.tag), added(other.added), removed(other.removed)
 	{
 	}
-
 	CMessageRangeUpdate& operator=(const CMessageRangeUpdate& other)
 	{
 		tag = other.tag;
@@ -364,20 +381,43 @@ public:
 };
 
 /**
- * Sent by technology manager when a technology is researched that modifies a component.
+ * Sent by aura manager when a value of a certain entity's component is changed
  */
-class CMessageTechnologyModification : public CMessage
+class CMessageValueModification : public CMessage
 {
 public:
-	DEFAULT_MESSAGE_IMPL(TechnologyModification)
+	DEFAULT_MESSAGE_IMPL(ValueModification)
 
-	CMessageTechnologyModification(std::wstring component, player_id_t player) :
-		component(component), player(player)
+	CMessageValueModification(const std::vector<entity_id_t>& entities, std::wstring component, const std::vector<std::wstring>& valueNames) :
+	entities(entities),
+	component(component),
+	valueNames(valueNames)
 	{
 	}
-
+	
+	std::vector<entity_id_t> entities;
 	std::wstring component;
+	std::vector<std::wstring> valueNames;
+};
+
+/**
+ * Sent by aura and tech managers when a value of a certain template's component is changed
+ */
+class CMessageTemplateModification : public CMessage
+{
+public:
+	DEFAULT_MESSAGE_IMPL(TemplateModification)
+	
+	CMessageTemplateModification(player_id_t player, std::wstring component, const std::vector<std::wstring>& valueNames) :
+	player(player),
+	component(component),
+	valueNames(valueNames)
+	{
+	}
+	
 	player_id_t player;
+	std::wstring component;
+	std::vector<std::wstring> valueNames;
 };
 
 /**
@@ -398,4 +438,17 @@ public:
 	entity_pos_t newRange;
 };
 
+/**
+ * Sent when an entity pings the minimap
+ */
+class CMessageMinimapPing : public CMessage
+{
+public:
+	DEFAULT_MESSAGE_IMPL(MinimapPing)
+
+	CMessageMinimapPing() 
+	{
+	}
+};
+
 #endif // INCLUDED_MESSAGETYPES
