@@ -19,6 +19,8 @@
 
 #include "Simulation2.h"
 
+#include "scriptinterface/ScriptInterface.h"
+
 #include "simulation2/MessageTypes.h"
 #include "simulation2/system/ComponentManager.h"
 #include "simulation2/system/ParamNode.h"
@@ -59,8 +61,8 @@ static std::string Hexify(const std::string& s) // TODO: shouldn't duplicate thi
 class CSimulation2Impl
 {
 public:
-	CSimulation2Impl(CUnitManager* unitManager, CTerrain* terrain) :
-		m_SimContext(), m_ComponentManager(m_SimContext),
+	CSimulation2Impl(CUnitManager* unitManager, shared_ptr<ScriptRuntime> rt, CTerrain* terrain) :
+		m_SimContext(), m_ComponentManager(m_SimContext, rt),
 		m_EnableOOSLog(false), m_EnableSerializationTest(false)
 	{
 		m_SimContext.m_UnitManager = unitManager;
@@ -97,19 +99,22 @@ public:
 		CParamNode noParam;
 		CComponentManager::ComponentTypeId cid;
 
-		// Add native system components:
-		componentManager.AddComponent(SYSTEM_ENTITY, CID_TemplateManager, noParam);
+		componentManager.InitSystemEntity();
+		CEntityHandle systemEntity = componentManager.GetSystemEntity();
 
-		componentManager.AddComponent(SYSTEM_ENTITY, CID_CommandQueue, noParam);
-		componentManager.AddComponent(SYSTEM_ENTITY, CID_ObstructionManager, noParam);
-		componentManager.AddComponent(SYSTEM_ENTITY, CID_ParticleManager, noParam);
-		componentManager.AddComponent(SYSTEM_ENTITY, CID_Pathfinder, noParam);
-		componentManager.AddComponent(SYSTEM_ENTITY, CID_ProjectileManager, noParam);
-		componentManager.AddComponent(SYSTEM_ENTITY, CID_RangeManager, noParam);
-		componentManager.AddComponent(SYSTEM_ENTITY, CID_SoundManager, noParam);
-		componentManager.AddComponent(SYSTEM_ENTITY, CID_Terrain, noParam);
-		componentManager.AddComponent(SYSTEM_ENTITY, CID_TerritoryManager, noParam);
-		componentManager.AddComponent(SYSTEM_ENTITY, CID_WaterManager, noParam);
+		// Add native system components:
+		componentManager.AddComponent(systemEntity, CID_TemplateManager, noParam);
+
+		componentManager.AddComponent(systemEntity, CID_CommandQueue, noParam);
+		componentManager.AddComponent(systemEntity, CID_ObstructionManager, noParam);
+		componentManager.AddComponent(systemEntity, CID_ParticleManager, noParam);
+		componentManager.AddComponent(systemEntity, CID_Pathfinder, noParam);
+		componentManager.AddComponent(systemEntity, CID_ProjectileManager, noParam);
+		componentManager.AddComponent(systemEntity, CID_RangeManager, noParam);
+		componentManager.AddComponent(systemEntity, CID_SoundManager, noParam);
+		componentManager.AddComponent(systemEntity, CID_Terrain, noParam);
+		componentManager.AddComponent(systemEntity, CID_TerritoryManager, noParam);
+		componentManager.AddComponent(systemEntity, CID_WaterManager, noParam);
 
 		// Add scripted system components:
 		if (!skipScriptedComponents)
@@ -120,21 +125,23 @@ public:
 			cid = componentManager.LookupCID(name); \
 			if (cid == CID__Invalid) \
 				LOGERROR(L"Can't find component type " L##name); \
-			componentManager.AddComponent(SYSTEM_ENTITY, cid, noParam)
+			componentManager.AddComponent(systemEntity, cid, noParam)
 
 			LOAD_SCRIPTED_COMPONENT("AIInterface");
+			LOAD_SCRIPTED_COMPONENT("AuraManager");
 			LOAD_SCRIPTED_COMPONENT("Barter");
 			LOAD_SCRIPTED_COMPONENT("EndGameManager");
 			LOAD_SCRIPTED_COMPONENT("GuiInterface");
 			LOAD_SCRIPTED_COMPONENT("PlayerManager");
 			LOAD_SCRIPTED_COMPONENT("TechnologyTemplateManager");
 			LOAD_SCRIPTED_COMPONENT("Timer");
+			LOAD_SCRIPTED_COMPONENT("ValueModificationManager");
 
 #undef LOAD_SCRIPTED_COMPONENT
 
 			if (!skipAI)
 			{
-				componentManager.AddComponent(SYSTEM_ENTITY, CID_AIManager, noParam);
+				componentManager.AddComponent(systemEntity, CID_AIManager, noParam);
 			}
 		
 		}
@@ -371,7 +378,7 @@ void CSimulation2Impl::Update(int turnLength, const std::vector<SimulationComman
 		CTerrain secondaryTerrain;
 		CSimContext secondaryContext;
 		secondaryContext.m_Terrain = &secondaryTerrain;
-		CComponentManager secondaryComponentManager(secondaryContext);
+		CComponentManager secondaryComponentManager(secondaryContext, m_ComponentManager.GetScriptInterface().GetRuntime());
 		secondaryComponentManager.LoadComponentTypes();
 		ENSURE(LoadDefaultScripts(secondaryComponentManager, NULL));
 		ResetComponentState(secondaryComponentManager, false, false);
@@ -384,22 +391,20 @@ void CSimulation2Impl::Update(int turnLength, const std::vector<SimulationComman
 		// TODO: this duplicates CWorld::RegisterInit and could probably be cleaned up a bit
 		std::string mapType;
 		m_ComponentManager.GetScriptInterface().GetProperty(m_InitAttributes.get(), "mapType", mapType);
-		if (mapType == "scenario")
-		{
-			// Load scenario attributes
-			std::wstring mapFile;
-			m_ComponentManager.GetScriptInterface().GetProperty(m_InitAttributes.get(), "map", mapFile);
-
-			VfsPath mapfilename(VfsPath("maps/scenarios") / (mapFile + L".pmp"));
-			mapReader->LoadMap(mapfilename, &secondaryTerrain, NULL, NULL, NULL, NULL, NULL, NULL,
-				NULL, NULL, &secondaryContext, INVALID_PLAYER, true); // throws exception on failure
-		}
-		else
+		if (mapType == "random")
 		{
 			// TODO: support random map scripts
 			debug_warn(L"Serialization test mode only supports scenarios");
 		}
+		else
+		{
+			std::wstring mapFile;
+			m_ComponentManager.GetScriptInterface().GetProperty(m_InitAttributes.get(), "map", mapFile);
 
+			VfsPath mapfilename = VfsPath(mapFile).ChangeExtension(L".pmp");
+			mapReader->LoadMap(mapfilename, CScriptValRooted(), &secondaryTerrain, NULL, NULL, NULL, NULL, NULL, NULL,
+				NULL, NULL, &secondaryContext, INVALID_PLAYER, true); // throws exception on failure
+		}
 		LDR_EndRegistering();
 		ENSURE(LDR_NonprogressiveLoad() == INFO::OK);
 
@@ -448,8 +453,8 @@ void CSimulation2Impl::Update(int turnLength, const std::vector<SimulationComman
 	// Run the GC occasionally
 	// (TODO: we ought to schedule this for a frame where we're not
 	// running the sim update, to spread the load)
-	if (m_TurnNumber % 10 == 0)
-		m_ComponentManager.GetScriptInterface().MaybeGC();
+	if (m_TurnNumber % 1 == 0)
+		m_ComponentManager.GetScriptInterface().MaybeIncrementalRuntimeGC();
 
 	if (m_EnableOOSLog)
 		DumpState();
@@ -562,8 +567,8 @@ void CSimulation2Impl::DumpState()
 
 ////////////////////////////////////////////////////////////////
 
-CSimulation2::CSimulation2(CUnitManager* unitManager, CTerrain* terrain) :
-	m(new CSimulation2Impl(unitManager, terrain))
+CSimulation2::CSimulation2(CUnitManager* unitManager, shared_ptr<ScriptRuntime> rt, CTerrain* terrain) :
+	m(new CSimulation2Impl(unitManager, rt, terrain))
 {
 }
 
@@ -644,6 +649,11 @@ ScriptInterface& CSimulation2::GetScriptInterface() const
 	return m->m_ComponentManager.GetScriptInterface();
 }
 
+void CSimulation2::ReplaceSkirmishGlobals()
+{
+	GetScriptInterface().CallFunctionVoid(GetScriptInterface().GetGlobalObject(), "ReplaceSkirmishGlobals");
+}
+
 void CSimulation2::InitGame(const CScriptVal& data)
 {
 	GetScriptInterface().CallFunctionVoid(GetScriptInterface().GetGlobalObject(), "InitGame", data);
