@@ -78,22 +78,9 @@ private:
 		NONCOPYABLE(CAIPlayer);
 	public:
 		CAIPlayer(CAIWorker& worker, const std::wstring& aiName, player_id_t player, uint8_t difficulty,
-				const shared_ptr<ScriptRuntime>& runtime, boost::rand48& rng) :
-			m_Worker(worker), m_AIName(aiName), m_Player(player), m_Difficulty(difficulty), m_ScriptInterface("Engine", "AI", runtime)
+				shared_ptr<ScriptInterface> scriptInterface) :
+			m_Worker(worker), m_AIName(aiName), m_Player(player), m_Difficulty(difficulty), m_ScriptInterface(scriptInterface)
 		{
-			m_ScriptInterface.SetCallbackData(static_cast<void*> (this));
-
-			m_ScriptInterface.ReplaceNondeterministicRNG(rng);
-			m_ScriptInterface.LoadGlobalScripts();
-
-			m_ScriptInterface.RegisterFunction<void, std::wstring, CAIPlayer::IncludeModule>("IncludeModule");
-			m_ScriptInterface.RegisterFunction<void, CAIPlayer::DumpHeap>("DumpHeap");
-			m_ScriptInterface.RegisterFunction<void, CAIPlayer::ForceGC>("ForceGC");
-			m_ScriptInterface.RegisterFunction<void, CScriptValRooted, CAIPlayer::PostCommand>("PostCommand");
-
-			m_ScriptInterface.RegisterFunction<void, std::wstring, std::vector<u32>, u32, u32, u32, CAIPlayer::DumpImage>("DumpImage");
-
-			m_ScriptInterface.RegisterFunction<void, std::wstring, CScriptVal, CAIPlayer::RegisterSerializablePrototype>("RegisterSerializablePrototype");
 		}
 
 		~CAIPlayer()
@@ -103,112 +90,10 @@ private:
 			m_Commands.clear();
 		}
 
-		static void IncludeModule(void* cbdata, std::wstring name)
-		{
-			CAIPlayer* self = static_cast<CAIPlayer*> (cbdata);
-
-			self->LoadScripts(name);
-		}
-		static void DumpHeap(void* cbdata)
-		{
-			CAIPlayer* self = static_cast<CAIPlayer*> (cbdata);
-			
-			//std::cout << JS_GetGCParameter(self->m_ScriptInterface.GetRuntime(), JSGC_BYTES) << std::endl;
-			self->m_ScriptInterface.DumpHeap();
-		}
-		static void ForceGC(void* cbdata)
-		{
-			CAIPlayer* self = static_cast<CAIPlayer*> (cbdata);
-			
-			JS_GC(self->m_ScriptInterface.GetContext());
-		}		
-		static void PostCommand(void* cbdata, CScriptValRooted cmd)
-		{
-			CAIPlayer* self = static_cast<CAIPlayer*> (cbdata);
-
-			self->m_Commands.push_back(self->m_ScriptInterface.WriteStructuredClone(cmd.get()));
-		}
-
-		/**
-		 * Debug function for AI scripts to dump 2D array data (e.g. terrain tile weights).
-		 * TODO: check if this needs to be here too.
-		 */
-		static void DumpImage(void* UNUSED(cbdata), std::wstring name, std::vector<u32> data, u32 w, u32 h, u32 max)
-		{
-			// TODO: this is totally not threadsafe.
-
-			VfsPath filename = L"screenshots/aidump/" + name;
-
-			if (data.size() != w*h)
-			{
-				debug_warn(L"DumpImage: data size doesn't match w*h");
-				return;
-			}
-
-			if (max == 0)
-			{
-				debug_warn(L"DumpImage: max must not be 0");
-				return;
-			}
-
-			const size_t bpp = 8;
-			int flags = TEX_BOTTOM_UP|TEX_GREY;
-
-			const size_t img_size = w * h * bpp/8;
-			const size_t hdr_size = tex_hdr_size(filename);
-			shared_ptr<u8> buf;
-			AllocateAligned(buf, hdr_size+img_size, maxSectorSize);
-			Tex t;
-			if (tex_wrap(w, h, bpp, flags, buf, hdr_size, &t) < 0)
-				return;
-
-			u8* img = buf.get() + hdr_size;
-			for (size_t i = 0; i < data.size(); ++i)
-				img[i] = (u8)((data[i] * 255) / max);
-
-			tex_write(&t, filename);
-			tex_free(&t);
-		}
-
-		static void RegisterSerializablePrototype(void* cbdata, std::wstring name, CScriptVal proto)
-		{
-			CAIPlayer* self = static_cast<CAIPlayer*> (cbdata);
-			// Add our player number to avoid name conflicts with other prototypes
-			// TODO: it would be better if serializable prototypes were stored in ScriptInterfaces
-			//	and then each serializer would access those matching its own context, but that's
-			//	not possible with AIs sharing data across contexts
-			std::wstringstream protoID;
-			protoID << self->m_Player << L"." << name.c_str();
-			self->m_Worker.RegisterSerializablePrototype(protoID.str(), proto);
-		}
-
-		bool LoadScripts(const std::wstring& moduleName)
-		{
-			// Ignore modules that are already loaded
-			if (m_LoadedModules.find(moduleName) != m_LoadedModules.end())
-				return true;
-
-			// Mark this as loaded, to prevent it recursively loading itself
-			m_LoadedModules.insert(moduleName);
-
-			// Load and execute *.js
-			VfsPaths pathnames;
-			vfs::GetPathnames(g_VFS, L"simulation/ai/" + moduleName + L"/", L"*.js", pathnames);
-			for (VfsPaths::iterator it = pathnames.begin(); it != pathnames.end(); ++it)
-			{
-				if (!m_ScriptInterface.LoadGlobalScriptFile(*it))
-				{
-					LOGERROR(L"Failed to load script %ls", it->string().c_str());
-					return false;
-				}
-			}
-
-			return true;
-		}
-
 		bool Initialise(bool callConstructor)
 		{
-			if (!LoadScripts(m_AIName))
+			// LoadScripts will only load each script once even though we call it for each player
+			if (!m_Worker.LoadScripts(m_AIName))
 				return false;
 
 			OsPath path = L"simulation/ai/" + m_AIName + L"/data.json";
@@ -220,23 +105,40 @@ private:
 			}
 
 			// Get the constructor name from the metadata
+			// If the AI doesn't use modules, we look for the constructor in the global object
+			// TODO: All AIs should use modules. Remove the condition if this requirement is met.
+			std::string moduleName;
 			std::string constructor;
-			if (!m_ScriptInterface.GetProperty(metadata.get(), "constructor", constructor))
+			CScriptVal objectWithConstructor; // object that should contain the constructor function
+			CScriptVal ctor;
+			if (!m_ScriptInterface->HasProperty(metadata.get(), "moduleName"))
+			{
+				objectWithConstructor = m_ScriptInterface->GetGlobalObject();
+			}
+			else
+			{
+				m_ScriptInterface->GetProperty(metadata.get(), "moduleName", moduleName);
+				if(!m_ScriptInterface->GetProperty(m_ScriptInterface->GetGlobalObject(), moduleName.c_str(), objectWithConstructor) || objectWithConstructor.undefined())
+				{
+					LOGERROR(L"Failed to create AI player: %ls: can't find the module that should contain the constructor: '%hs'", path.string().c_str(), moduleName.c_str());
+					return false;
+				}
+			}
+			if (!m_ScriptInterface->GetProperty(metadata.get(), "constructor", constructor))
 			{
 				LOGERROR(L"Failed to create AI player: %ls: missing 'constructor'", path.string().c_str());
 				return false;
 			}
 
 			// Get the constructor function from the loaded scripts
-			CScriptVal ctor;
-			if (!m_ScriptInterface.GetProperty(m_ScriptInterface.GetGlobalObject(), constructor.c_str(), ctor)
+			if (!m_ScriptInterface->GetProperty(objectWithConstructor.get(), constructor.c_str(), ctor)
 				|| ctor.undefined())
 			{
 				LOGERROR(L"Failed to create AI player: %ls: can't find constructor '%hs'", path.string().c_str(), constructor.c_str());
 				return false;
 			}
 
-			m_ScriptInterface.GetProperty(metadata.get(), "useShared", m_UseSharedComponent);
+			m_ScriptInterface->GetProperty(metadata.get(), "useShared", m_UseSharedComponent);
 			
 			CScriptVal obj;
 
@@ -244,20 +146,22 @@ private:
 			{
 				// Set up the data to pass as the constructor argument
 				CScriptVal settings;
-				m_ScriptInterface.Eval(L"({})", settings);
-				m_ScriptInterface.SetProperty(settings.get(), "player", m_Player, false);
-				m_ScriptInterface.SetProperty(settings.get(), "difficulty", m_Difficulty, false);
+				m_ScriptInterface->Eval(L"({})", settings);
+				m_ScriptInterface->SetProperty(settings.get(), "player", m_Player, false);
+				m_ScriptInterface->SetProperty(settings.get(), "difficulty", m_Difficulty, false);
 				ENSURE(m_Worker.m_HasLoadedEntityTemplates);
-				m_ScriptInterface.SetProperty(settings.get(), "templates", m_Worker.m_EntityTemplates, false);
+				m_ScriptInterface->SetProperty(settings.get(), "templates", m_Worker.m_EntityTemplates, false);
 
-				obj = m_ScriptInterface.CallConstructor(ctor.get(), settings.get());
+				JS::AutoValueVector argv(m_ScriptInterface->GetContext());
+				argv.append(settings.get());
+				obj = m_ScriptInterface->CallConstructor(ctor.get(), argv.length(), argv.handleAt(0));
 			}
 			else
 			{
 				// For deserialization, we want to create the object with the correct prototype
 				// but don't want to actually run the constructor again
 				// XXX: actually we don't currently use this path for deserialization - maybe delete it?
-				obj = m_ScriptInterface.NewObjectFromConstructor(ctor.get());
+				obj = m_ScriptInterface->NewObjectFromConstructor(ctor.get());
 			}
 
 			if (obj.undefined())
@@ -266,26 +170,26 @@ private:
 				return false;
 			}
 
-			m_Obj = CScriptValRooted(m_ScriptInterface.GetContext(), obj);
+			m_Obj = CScriptValRooted(m_ScriptInterface->GetContext(), obj);
 			return true;
 		}
 
-		void Run(CScriptVal state)
+		void Run(CScriptVal state, int playerID)
 		{
 			m_Commands.clear();
-			m_ScriptInterface.CallFunctionVoid(m_Obj.get(), "HandleMessage", state);
+			m_ScriptInterface->CallFunctionVoid(m_Obj.get(), "HandleMessage", state, playerID);
 		}
 		// overloaded with a sharedAI part.
 		// javascript can handle both natively on the same function.
-		void Run(CScriptVal state, CScriptValRooted SharedAI)
+		void Run(CScriptVal state, int playerID, CScriptValRooted SharedAI)
 		{
 			m_Commands.clear();
-			m_ScriptInterface.CallFunctionVoid(m_Obj.get(), "HandleMessage", state, SharedAI);
+			m_ScriptInterface->CallFunctionVoid(m_Obj.get(), "HandleMessage", state, playerID, SharedAI);
 		}
-		void InitWithSharedScript(CScriptVal state, CScriptValRooted SharedAI)
+		void InitAI(CScriptVal state, CScriptValRooted SharedAI)
 		{
 			m_Commands.clear();
-			m_ScriptInterface.CallFunctionVoid(m_Obj.get(), "InitWithSharedScript", state, SharedAI);
+			m_ScriptInterface->CallFunctionVoid(m_Obj.get(), "Init", state, m_Player, SharedAI);
 		}
 
 		CAIWorker& m_Worker;
@@ -294,10 +198,9 @@ private:
 		uint8_t m_Difficulty;
 		bool m_UseSharedComponent;
 		
-		ScriptInterface m_ScriptInterface;
+		shared_ptr<ScriptInterface> m_ScriptInterface;
 		CScriptValRooted m_Obj;
 		std::vector<shared_ptr<ScriptInterface::StructuredClone> > m_Commands;
-		std::set<std::wstring> m_LoadedModules;
 	};
 
 public:
@@ -308,12 +211,7 @@ public:
 	};
 
 	CAIWorker() :
-		// TODO: Passing a 32 MB argument to CreateRuntime() is a temporary fix
-		// to prevent frequent AI out-of-memory crashes. The argument should be
-		// removed as soon whenever the new pathfinder is committed
-		// And the AIs can stop relying on their own little hands.
-		m_ScriptRuntime(ScriptInterface::CreateRuntime(33554432)),
-		m_ScriptInterface("Engine", "AI", m_ScriptRuntime),
+		m_ScriptInterface(new ScriptInterface("Engine", "AI", g_ScriptRuntime)),
 		m_TurnNum(0),
 		m_CommandsComputed(true),
 		m_HasLoadedEntityTemplates(false),
@@ -321,16 +219,17 @@ public:
 	{
 
 		// TODO: ought to seed the RNG (in a network-synchronised way) before we use it
-		m_ScriptInterface.ReplaceNondeterministicRNG(m_RNG);
-		m_ScriptInterface.LoadGlobalScripts();
+		m_ScriptInterface->ReplaceNondeterministicRNG(m_RNG);
+		m_ScriptInterface->LoadGlobalScripts();
 
-		m_ScriptInterface.SetCallbackData(NULL);
+		m_ScriptInterface->SetCallbackData(static_cast<void*> (this));
 
-		m_ScriptInterface.RegisterFunction<void, CScriptValRooted, CAIWorker::PostCommand>("PostCommand");
-		m_ScriptInterface.RegisterFunction<void, CAIWorker::DumpHeap>("DumpHeap");
-		m_ScriptInterface.RegisterFunction<void, CAIWorker::ForceGC>("ForceGC");
+		m_ScriptInterface->RegisterFunction<void, int, CScriptValRooted, CAIWorker::PostCommand>("PostCommand");
+		m_ScriptInterface->RegisterFunction<void, std::wstring, CAIWorker::IncludeModule>("IncludeModule");
+		m_ScriptInterface->RegisterFunction<void, CAIWorker::DumpHeap>("DumpHeap");
+		m_ScriptInterface->RegisterFunction<void, CAIWorker::ForceGC>("ForceGC");
 		
-		m_ScriptInterface.RegisterFunction<void, std::wstring, std::vector<u32>, u32, u32, u32, CAIWorker::DumpImage>("DumpImage");
+		m_ScriptInterface->RegisterFunction<void, std::wstring, std::vector<u32>, u32, u32, u32, CAIWorker::DumpImage>("DumpImage");
 	}
 
 	~CAIWorker()
@@ -343,44 +242,73 @@ public:
 		m_PassabilityMapVal = CScriptValRooted();
 		m_TerritoryMapVal = CScriptValRooted();
 	}
-
-	// This is called by AIs if they use the v3 API.
-	// If the AIs originate the call, cbdata is not NULL.
-	// If the shared component does, it is, so it must not be taken into account.
-	static void PostCommand(void* cbdata, CScriptValRooted cmd)
+	
+	bool LoadScripts(const std::wstring& moduleName)
 	{
-		if (cbdata == NULL) {
-			debug_warn(L"Warning: the shared component has tried to push an engine command. Ignoring.");
-			return;
+		// Ignore modules that are already loaded
+		if (m_LoadedModules.find(moduleName) != m_LoadedModules.end())
+			return true;
+
+		// Mark this as loaded, to prevent it recursively loading itself
+		m_LoadedModules.insert(moduleName);
+
+		// Load and execute *.js
+		VfsPaths pathnames;
+		vfs::GetPathnames(g_VFS, L"simulation/ai/" + moduleName + L"/", L"*.js", pathnames);
+		for (VfsPaths::iterator it = pathnames.begin(); it != pathnames.end(); ++it)
+		{
+			if (!m_ScriptInterface->LoadGlobalScriptFile(*it))
+			{
+				LOGERROR(L"Failed to load script %ls", it->string().c_str());
+				return false;
+			}
 		}
-		CAIPlayer* self = static_cast<CAIPlayer*> (cbdata);
-		self->m_Commands.push_back(self->m_ScriptInterface.WriteStructuredClone(cmd.get()));
+
+		return true;
+	}
+	
+	static void IncludeModule(ScriptInterface::CxPrivate* pCxPrivate, std::wstring name)
+	{
+		ENSURE(pCxPrivate->pCBData);
+		CAIWorker* self = static_cast<CAIWorker*> (pCxPrivate->pCBData);
+		self->LoadScripts(name);
+	}
+
+	static void PostCommand(ScriptInterface::CxPrivate* pCxPrivate, int playerid, CScriptValRooted cmd)
+	{
+		ENSURE(pCxPrivate->pCBData);
+		CAIWorker* self = static_cast<CAIWorker*> (pCxPrivate->pCBData);
+		self->PostCommand(playerid, cmd);
+	}
+	
+	void PostCommand(int playerid, CScriptValRooted cmd)
+	{
+		for (size_t i=0; i<m_Players.size(); i++)
+		{
+			if (m_Players[i]->m_Player == playerid)	
+			{
+				m_Players[i]->m_Commands.push_back(m_ScriptInterface->WriteStructuredClone(cmd.get()));
+				return;
+			}
+		}
+		
+		LOGERROR(L"Invalid playerid in PostCommand!");	
 	}
 	// The next two ought to be implmeneted someday but for now as it returns "null" it can't
-	static void DumpHeap(void* cbdata)
+	static void DumpHeap(ScriptInterface::CxPrivate* pCxPrivate)
 	{
-		if (cbdata == NULL) {
-			debug_warn(L"Warning: the shared component has asked for DumpHeap. Ignoring.");
-			return;
-		}
-		CAIWorker* self = static_cast<CAIWorker*> (cbdata);
-		self->m_ScriptInterface.DumpHeap();
+		pCxPrivate->pScriptInterface->DumpHeap();
 	}
-	static void ForceGC(void* cbdata)
+	static void ForceGC(ScriptInterface::CxPrivate* pCxPrivate)
 	{
-		if (cbdata == NULL) {
-			debug_warn(L"Warning: the shared component has asked for ForceGC. Ignoring.");
-			return;
-		}
-		CAIWorker* self = static_cast<CAIWorker*> (cbdata);
 		PROFILE3("AI compute GC");
-		JS_GC(self->m_ScriptInterface.GetContext());
+		JS_GC(pCxPrivate->pScriptInterface->GetJSRuntime());
 	}
 	
 	/**
 	 * Debug function for AI scripts to dump 2D array data (e.g. terrain tile weights).
 	 */
-	static void DumpImage(void* UNUSED(cbdata), std::wstring name, std::vector<u32> data, u32 w, u32 h, u32 max)
+	static void DumpImage(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), std::wstring name, std::vector<u32> data, u32 w, u32 h, u32 max)
 	{
 		// TODO: this is totally not threadsafe.
 		VfsPath filename = L"screenshots/aidump/" + name;
@@ -405,7 +333,7 @@ public:
 		shared_ptr<u8> buf;
 		AllocateAligned(buf, hdr_size+img_size, maxSectorSize);
 		Tex t;
-		if (tex_wrap(w, h, bpp, flags, buf, hdr_size, &t) < 0)
+		if (t.wrap(w, h, bpp, flags, buf, hdr_size) < 0)
 			return;
 		
 		u8* img = buf.get() + hdr_size;
@@ -413,7 +341,6 @@ public:
 			img[i] = (u8)((data[i] * 255) / max);
 		
 		tex_write(&t, filename);
-		tex_free(&t);
 	}
 
 	bool TryLoadSharedComponent(bool hasTechs)
@@ -424,82 +351,66 @@ public:
 		
 		// reset the value so it can be used to determine if we actually initialized it.
 		m_HasSharedComponent = false;
-				
-		VfsPaths sharedPathnames;
-		// Check for "shared" module.
-		vfs::GetPathnames(g_VFS, L"simulation/ai/common-api-v3/", L"*.js", sharedPathnames);
-		for (VfsPaths::iterator it = sharedPathnames.begin(); it != sharedPathnames.end(); ++it)
-		{
-			if (!m_ScriptInterface.LoadGlobalScriptFile(*it))
-			{
-				LOGERROR(L"Failed to load shared script %ls", it->string().c_str());
-				return false;
-			}
-			m_HasSharedComponent = true;
-		}
-		if (!m_HasSharedComponent)
-			return false;
 		
+		if (LoadScripts(L"common-api"))
+			m_HasSharedComponent = true;
+		else
+			return false;
+
 		// mainly here for the error messages
 		OsPath path = L"simulation/ai/common-api-v2/";
 		
-		// Constructor name is SharedScript
+		// Constructor name is SharedScript, it's in the module API3
+		// TODO: Hardcoding this is bad, we need a smarter way. 
+		CScriptVal AIModule;
 		CScriptVal ctor;
-		if (!m_ScriptInterface.GetProperty(m_ScriptInterface.GetGlobalObject(), "SharedScript", ctor)
+		if (!m_ScriptInterface->GetProperty(m_ScriptInterface->GetGlobalObject(), "API3", AIModule) || AIModule.undefined())
+		{
+			LOGERROR(L"Failed to create shared AI component: %ls: can't find module '%hs'", path.string().c_str(), "API3");
+			return false;
+		}
+		
+		if (!m_ScriptInterface->GetProperty(AIModule.get(), "SharedScript", ctor)
 			|| ctor.undefined())
 		{
 			LOGERROR(L"Failed to create shared AI component: %ls: can't find constructor '%hs'", path.string().c_str(), "SharedScript");
 			return false;
 		}
 		
+		// Set up the data to pass as the constructor argument
+		CScriptVal settings;
+		m_ScriptInterface->Eval(L"({})", settings);
+		CScriptVal playersID;
+		m_ScriptInterface->Eval(L"({})", playersID);
+		
+		for (size_t i = 0; i < m_Players.size(); ++i)
+		{
+			JS::Value val;
+			m_ScriptInterface->ToJSVal(m_ScriptInterface->GetContext(), val, m_Players[i]->m_Player);
+			m_ScriptInterface->SetPropertyInt(playersID.get(), i, CScriptVal(val), true);
+		}
+		
+		m_ScriptInterface->SetProperty(settings.get(), "players", playersID);
+		ENSURE(m_HasLoadedEntityTemplates);
+		m_ScriptInterface->SetProperty(settings.get(), "templates", m_EntityTemplates, false);
+		
 		if (hasTechs)
 		{
-			// Set up the data to pass as the constructor argument
-			CScriptVal settings;
-			m_ScriptInterface.Eval(L"({})", settings);
-			CScriptVal playersID;
-			m_ScriptInterface.Eval(L"({})", playersID);
-			
-			for (size_t i = 0; i < m_Players.size(); ++i)
-			{
-				jsval val = m_ScriptInterface.ToJSVal(m_ScriptInterface.GetContext(), m_Players[i]->m_Player);
-				m_ScriptInterface.SetPropertyInt(playersID.get(), i, CScriptVal(val), true);
-			}
-			
-			m_ScriptInterface.SetProperty(settings.get(), "players", playersID);
-			
-			ENSURE(m_HasLoadedEntityTemplates);
-			m_ScriptInterface.SetProperty(settings.get(), "templates", m_EntityTemplates, false);
-		
-			m_ScriptInterface.SetProperty(settings.get(), "techTemplates", m_TechTemplates, false);
-
-			m_SharedAIObj = CScriptValRooted(m_ScriptInterface.GetContext(),m_ScriptInterface.CallConstructor(ctor.get(), settings.get()));
+			m_ScriptInterface->SetProperty(settings.get(), "techTemplates", m_TechTemplates, false);
 		}
 		else
 		{
 			// won't get the tech templates directly.
-			// Set up the data to pass as the constructor argument
-			CScriptVal settings;
-			m_ScriptInterface.Eval(L"({})", settings);
-			CScriptVal playersID;
-			m_ScriptInterface.Eval(L"({})", playersID);
-			for (size_t i = 0; i < m_Players.size(); ++i)
-			{
-				jsval val = m_ScriptInterface.ToJSVal(m_ScriptInterface.GetContext(), m_Players[i]->m_Player);
-				m_ScriptInterface.SetPropertyInt(playersID.get(), i, CScriptVal(val), true);
-			}
-			
-			m_ScriptInterface.SetProperty(settings.get(), "players", playersID);
-			
-			CScriptVal m_fakeTech;
-			m_ScriptInterface.Eval("({})", m_fakeTech);
-			m_ScriptInterface.SetProperty(settings.get(), "techTemplates", m_fakeTech, false);
-
-			ENSURE(m_HasLoadedEntityTemplates);
-			m_ScriptInterface.SetProperty(settings.get(), "templates", m_EntityTemplates, false);
-			m_SharedAIObj = CScriptValRooted(m_ScriptInterface.GetContext(),m_ScriptInterface.CallConstructor(ctor.get(), settings.get()));
+			CScriptVal fakeTech;
+			m_ScriptInterface->Eval("({})", fakeTech);
+			m_ScriptInterface->SetProperty(settings.get(), "techTemplates", fakeTech, false);
 		}
 		
+		JS::AutoValueVector argv(m_ScriptInterface->GetContext());
+		argv.append(settings.get());
+		m_SharedAIObj = CScriptValRooted(m_ScriptInterface->GetContext(),m_ScriptInterface->CallConstructor(ctor.get(), argv.length(), argv.handleAt(0)));
+	
+		
 		if (m_SharedAIObj.undefined())
 		{
 			LOGERROR(L"Failed to create shared AI component: %ls: error calling constructor '%hs'", path.string().c_str(), "SharedScript");
@@ -511,7 +422,7 @@ public:
 
 	bool AddPlayer(const std::wstring& aiName, player_id_t player, uint8_t difficulty, bool callConstructor)
 	{
-		shared_ptr<CAIPlayer> ai(new CAIPlayer(*this, aiName, player, difficulty, m_ScriptRuntime, m_RNG));
+		shared_ptr<CAIPlayer> ai(new CAIPlayer(*this, aiName, player, difficulty, m_ScriptInterface));
 		if (!ai->Initialise(callConstructor))
 			return false;
 		
@@ -519,8 +430,6 @@ public:
 		if (!m_HasSharedComponent)
 			m_HasSharedComponent = ai->m_UseSharedComponent;
 
-		m_ScriptInterface.MaybeGC();
-
 		m_Players.push_back(ai);
 
 		return true;
@@ -531,23 +440,25 @@ public:
 		// this will be run last by InitGame.Js, passing the full game representation.
 		// For now it will run for the shared Component.
 		// This is NOT run during deserialization.
-		CScriptVal state = m_ScriptInterface.ReadStructuredClone(gameState);
-		JSContext* cx = m_ScriptInterface.GetContext();
+		CScriptVal state = m_ScriptInterface->ReadStructuredClone(gameState);
+		JSContext* cx = m_ScriptInterface->GetContext();
 
-		m_PassabilityMapVal = CScriptValRooted(cx, ScriptInterface::ToJSVal(cx, passabilityMap));
-		m_TerritoryMapVal = CScriptValRooted(cx, ScriptInterface::ToJSVal(cx, territoryMap));
+		JS::RootedValue tmpVal(cx);
+		ScriptInterface::ToJSVal(cx, tmpVal.get(), passabilityMap);
+		m_PassabilityMapVal = CScriptValRooted(cx, tmpVal.get());
+		ScriptInterface::ToJSVal(cx, tmpVal.get(), territoryMap);
+		m_TerritoryMapVal = CScriptValRooted(cx, tmpVal);
 		if (m_HasSharedComponent)
 		{
-			m_ScriptInterface.SetProperty(state.get(), "passabilityMap", m_PassabilityMapVal, true);
-			m_ScriptInterface.SetProperty(state.get(), "territoryMap", m_TerritoryMapVal, true);
+			m_ScriptInterface->SetProperty(state.get(), "passabilityMap", m_PassabilityMapVal, true);
+			m_ScriptInterface->SetProperty(state.get(), "territoryMap", m_TerritoryMapVal, true);
 
-			m_ScriptInterface.CallFunctionVoid(m_SharedAIObj.get(), "initWithState", state);
-			m_ScriptInterface.MaybeGC();
+			m_ScriptInterface->CallFunctionVoid(m_SharedAIObj.get(), "init", state);
 			
 			for (size_t i = 0; i < m_Players.size(); ++i)
 			{
 				if (m_HasSharedComponent && m_Players[i]->m_UseSharedComponent)
-					m_Players[i]->InitWithSharedScript(state,m_SharedAIObj);
+					m_Players[i]->InitAI(state,m_SharedAIObj);
 			}
 		}
 		
@@ -558,21 +469,24 @@ public:
 		ENSURE(m_CommandsComputed);
 
 		m_GameState = gameState;
+		
+		JSContext* cx = m_ScriptInterface->GetContext();
+		JSAutoRequest rq(cx);
 
 		if (passabilityMap.m_DirtyID != m_PassabilityMap.m_DirtyID)
 		{
 			m_PassabilityMap = passabilityMap;
-
-			JSContext* cx = m_ScriptInterface.GetContext();
-			m_PassabilityMapVal = CScriptValRooted(cx, ScriptInterface::ToJSVal(cx, m_PassabilityMap));
+			JS::RootedValue tmpVal(cx);
+			ScriptInterface::ToJSVal(cx, tmpVal.get(), m_PassabilityMap);
+			m_PassabilityMapVal = CScriptValRooted(cx, tmpVal);
 		}
 
 		if (territoryMapDirty)
 		{
 			m_TerritoryMap = territoryMap;
-
-			JSContext* cx = m_ScriptInterface.GetContext();
-			m_TerritoryMapVal = CScriptValRooted(cx, ScriptInterface::ToJSVal(cx, m_TerritoryMap));
+			JS::RootedValue tmpVal(cx);
+			ScriptInterface::ToJSVal(cx, tmpVal.get(), m_TerritoryMap);
+			m_TerritoryMapVal = CScriptValRooted(cx, tmpVal);
 		}
 
 		m_CommandsComputed = false;
@@ -601,25 +515,25 @@ public:
 	}
 
 	void RegisterTechTemplates(const shared_ptr<ScriptInterface::StructuredClone>& techTemplates) {
-		JSContext* cx = m_ScriptInterface.GetContext();
-		m_TechTemplates = CScriptValRooted(cx, m_ScriptInterface.ReadStructuredClone(techTemplates));
+		JSContext* cx = m_ScriptInterface->GetContext();
+		m_TechTemplates = CScriptValRooted(cx, m_ScriptInterface->ReadStructuredClone(techTemplates));
 	}
 	
 	void LoadEntityTemplates(const std::vector<std::pair<std::string, const CParamNode*> >& templates)
 	{
 		m_HasLoadedEntityTemplates = true;
 
-		m_ScriptInterface.Eval("({})", m_EntityTemplates);
+		m_ScriptInterface->Eval("({})", m_EntityTemplates);
 
 		for (size_t i = 0; i < templates.size(); ++i)
 		{
-			jsval val = templates[i].second->ToJSVal(m_ScriptInterface.GetContext(), false);
-			m_ScriptInterface.SetProperty(m_EntityTemplates.get(), templates[i].first.c_str(), CScriptVal(val), true);
+			jsval val = templates[i].second->ToJSVal(m_ScriptInterface->GetContext(), false);
+			m_ScriptInterface->SetProperty(m_EntityTemplates.get(), templates[i].first.c_str(), CScriptVal(val), true);
 		}
 
 		// Since the template data is shared between AI players, freeze it
 		// to stop any of them changing it and confusing the other players
-		m_ScriptInterface.FreezeObject(m_EntityTemplates.get(), true);
+		m_ScriptInterface->FreezeObject(m_EntityTemplates.get(), true);
 	}
 
 	void Serialize(std::ostream& stream, bool isDebug)
@@ -628,13 +542,13 @@ public:
 
 		if (isDebug)
 		{
-			CDebugSerializer serializer(m_ScriptInterface, stream);
+			CDebugSerializer serializer(*m_ScriptInterface, stream);
 			serializer.Indent(4);
 			SerializeState(serializer);
 		}
 		else
 		{
-			CStdSerializer serializer(m_ScriptInterface, stream);
+			CStdSerializer serializer(*m_ScriptInterface, stream);
 			// TODO: see comment in Deserialize()
 			serializer.SetSerializablePrototypes(m_SerializablePrototypes);
 			SerializeState(serializer);
@@ -655,7 +569,7 @@ public:
 		if (m_HasSharedComponent)
 		{
 			CScriptVal sharedData;
-			if (!m_ScriptInterface.CallFunction(m_SharedAIObj.get(), "Serialize", sharedData))
+			if (!m_ScriptInterface->CallFunction(m_SharedAIObj.get(), "Serialize", sharedData))
 				LOGERROR(L"AI shared script Serialize call failed");
 			serializer.ScriptVal("sharedData", sharedData);
 		}
@@ -668,15 +582,15 @@ public:
 			serializer.NumberU32_Unbounded("num commands", (u32)m_Players[i]->m_Commands.size());
 			for (size_t j = 0; j < m_Players[i]->m_Commands.size(); ++j)
 			{
-				CScriptVal val = m_ScriptInterface.ReadStructuredClone(m_Players[i]->m_Commands[j]);
+				CScriptVal val = m_ScriptInterface->ReadStructuredClone(m_Players[i]->m_Commands[j]);
 				serializer.ScriptVal("command", val);
 			}
 
-			bool hasCustomSerialize = m_ScriptInterface.HasProperty(m_Players[i]->m_Obj.get(), "Serialize");
+			bool hasCustomSerialize = m_ScriptInterface->HasProperty(m_Players[i]->m_Obj.get(), "Serialize");
 			if (hasCustomSerialize)
 			{
 				CScriptVal scriptData;
-				if (!m_ScriptInterface.CallFunction(m_Players[i]->m_Obj.get(), "Serialize", scriptData))
+				if (!m_ScriptInterface->CallFunction(m_Players[i]->m_Obj.get(), "Serialize", scriptData))
 					LOGERROR(L"AI script Serialize call failed");
 				serializer.ScriptVal("data", scriptData);
 			}
@@ -691,7 +605,7 @@ public:
 	{
 		ENSURE(m_CommandsComputed); // deserializing while we're still actively computing would be bad
 
-		CStdDeserializer deserializer(m_ScriptInterface, stream);
+		CStdDeserializer deserializer(*m_ScriptInterface, stream);
 
 		m_PlayerMetadata.clear();
 		m_Players.clear();
@@ -713,7 +627,7 @@ public:
 		{
 			CScriptVal sharedData;
 			deserializer.ScriptVal("sharedData", sharedData);
-			if (!m_ScriptInterface.CallFunctionVoid(m_SharedAIObj.get(), "Deserialize", sharedData))
+			if (!m_ScriptInterface->CallFunctionVoid(m_SharedAIObj.get(), "Deserialize", sharedData))
 				LOGERROR(L"AI shared script Deserialize call failed");
 		}
 
@@ -735,7 +649,7 @@ public:
 			{
 				CScriptVal val;
 				deserializer.ScriptVal("command", val);
-				m_Players.back()->m_Commands.push_back(m_ScriptInterface.WriteStructuredClone(val.get()));
+				m_Players.back()->m_Commands.push_back(m_ScriptInterface->WriteStructuredClone(val.get()));
 			}
 			
 			// TODO: this is yucky but necessary while the AIs are sharing data between contexts;
@@ -744,17 +658,17 @@ public:
 			//	prototypes could be stored in their ScriptInterface
 			deserializer.SetSerializablePrototypes(m_DeserializablePrototypes);
 
-			bool hasCustomDeserialize = m_ScriptInterface.HasProperty(m_Players.back()->m_Obj.get(), "Deserialize");
+			bool hasCustomDeserialize = m_ScriptInterface->HasProperty(m_Players.back()->m_Obj.get(), "Deserialize");
 			if (hasCustomDeserialize)
 			{
 				CScriptVal scriptData;
 				deserializer.ScriptVal("data", scriptData);
 				if (m_Players[i]->m_UseSharedComponent)
 				{
-					if (!m_ScriptInterface.CallFunctionVoid(m_Players.back()->m_Obj.get(), "Deserialize", scriptData, m_SharedAIObj))
+					if (!m_ScriptInterface->CallFunctionVoid(m_Players.back()->m_Obj.get(), "Deserialize", scriptData, m_SharedAIObj))
 						LOGERROR(L"AI script Deserialize call failed");
 				}
-				else if (!m_ScriptInterface.CallFunctionVoid(m_Players.back()->m_Obj.get(), "Deserialize", scriptData))
+				else if (!m_ScriptInterface->CallFunctionVoid(m_Players.back()->m_Obj.get(), "Deserialize", scriptData))
 				{
 					LOGERROR(L"AI script deserialize() call failed");
 				}
@@ -788,38 +702,25 @@ private:
 		if (m_PlayerMetadata.find(path) == m_PlayerMetadata.end())
 		{
 			// Load and cache the AI player metadata
-			m_PlayerMetadata[path] = m_ScriptInterface.ReadJSONFile(path);
+			m_PlayerMetadata[path] = m_ScriptInterface->ReadJSONFile(path);
 		}
 
 		return m_PlayerMetadata[path];
 	}
 
 	void PerformComputation()
-	{
-		if (m_Players.size() == 0)
-		{
-			// Run the GC every so often.
-			// (This isn't particularly necessary, but it makes profiling clearer
-			// since it avoids random GC delays while running other scripts)
-			if (m_TurnNum++ % 50 == 0)
-			{
-				PROFILE3("AI compute GC");
-				m_ScriptInterface.MaybeGC();
-			}
-			return;
-		}
-				
+	{			
 		// Deserialize the game state, to pass to the AI's HandleMessage
 		CScriptVal state;
 		{
 			PROFILE3("AI compute read state");
-			state = m_ScriptInterface.ReadStructuredClone(m_GameState);
-			m_ScriptInterface.SetProperty(state.get(), "passabilityMap", m_PassabilityMapVal, true);
-			m_ScriptInterface.SetProperty(state.get(), "territoryMap", m_TerritoryMapVal, true);
+			state = m_ScriptInterface->ReadStructuredClone(m_GameState);
+			m_ScriptInterface->SetProperty(state.get(), "passabilityMap", m_PassabilityMapVal, true);
+			m_ScriptInterface->SetProperty(state.get(), "territoryMap", m_TerritoryMapVal, true);
 		}
 
 		// It would be nice to do
-		//   m_ScriptInterface.FreezeObject(state.get(), true);
+		//   m_ScriptInterface->FreezeObject(state.get(), true);
 		// to prevent AI scripts accidentally modifying the state and
 		// affecting other AI scripts they share it with. But the performance
 		// cost is far too high, so we won't do that.
@@ -828,7 +729,7 @@ private:
 		if (m_HasSharedComponent)
 		{
 			PROFILE3("AI run shared component");
-			m_ScriptInterface.CallFunctionVoid(m_SharedAIObj.get(), "onUpdate", state);
+			m_ScriptInterface->CallFunctionVoid(m_SharedAIObj.get(), "onUpdate", state);
 		}
 		
 		for (size_t i = 0; i < m_Players.size(); ++i)
@@ -836,32 +737,16 @@ private:
 			PROFILE3("AI script");
 			PROFILE2_ATTR("player: %d", m_Players[i]->m_Player);
 			PROFILE2_ATTR("script: %ls", m_Players[i]->m_AIName.c_str());
+
 			if (m_HasSharedComponent && m_Players[i]->m_UseSharedComponent)
-				m_Players[i]->Run(state,m_SharedAIObj);
+				m_Players[i]->Run(state, m_Players[i]->m_Player, m_SharedAIObj);
 			else
-				m_Players[i]->Run(state);
+				m_Players[i]->Run(state, m_Players[i]->m_Player);
 		}
-
-		// Run GC if we are about to overflow
-		if (JS_GetGCParameter(m_ScriptInterface.GetRuntime(), JSGC_BYTES) > 33000000)
-		{
-			PROFILE3("AI compute GC");
-
-			JS_GC(m_ScriptInterface.GetContext());
-		}
-		
-		// Run the GC every so often.
-		// (This isn't particularly necessary, but it makes profiling clearer
-		// since it avoids random GC delays while running other scripts)
-		/*if (m_TurnNum++ % 20 == 0)
-		{
-			PROFILE3("AI compute GC");
-			m_ScriptInterface.MaybeGC();
-		}*/
 	}
 
 	shared_ptr<ScriptRuntime> m_ScriptRuntime;
-	ScriptInterface m_ScriptInterface;
+	shared_ptr<ScriptInterface> m_ScriptInterface;
 	boost::rand48 m_RNG;
 	u32 m_TurnNum;
 
@@ -876,6 +761,8 @@ private:
 	CScriptValRooted m_SharedAIObj;
 	std::vector<SCommandSets> m_Commands;
 	
+	std::set<std::wstring> m_LoadedModules;
+	
 	shared_ptr<ScriptInterface::StructuredClone> m_GameState;
 	Grid<u16> m_PassabilityMap;
 	CScriptValRooted m_PassabilityMapVal;
@@ -910,6 +797,7 @@ public:
 	virtual void Init(const CParamNode& UNUSED(paramNode))
 	{
 		m_TerritoriesDirtyID = 0;
+		m_JustDeserialized = false;
 
 		StartLoadEntityTemplates();
 	}
@@ -935,6 +823,8 @@ public:
 		ForceLoadEntityTemplates();
 
 		m_Worker.Deserialize(deserialize.GetStream());
+		
+		m_JustDeserialized = true;
 	}
 
 	virtual void HandleMessage(const CMessage& msg, bool UNUSED(global))
@@ -967,7 +857,7 @@ public:
 		// AI players can cheat and see through FoW/SoD, since that greatly simplifies
 		// their implementation.
 		// (TODO: maybe cleverer AIs should be able to optionally retain FoW/SoD)
-		CmpPtr<ICmpRangeManager> cmpRangeManager(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpRangeManager> cmpRangeManager(GetSystemEntity());
 		if (cmpRangeManager)
 			cmpRangeManager->SetLosRevealAll(player, true);
 	}
@@ -976,7 +866,7 @@ public:
 	{
 		ScriptInterface& scriptInterface = GetSimContext().GetScriptInterface();
 		// load the technology templates
-		CmpPtr<ICmpTechnologyTemplateManager> cmpTechTemplateManager(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpTechnologyTemplateManager> cmpTechTemplateManager(GetSystemEntity());
 		ENSURE(cmpTechTemplateManager);
 		
 		// Get the game state from AIInterface
@@ -990,16 +880,17 @@ public:
 	{
 		ScriptInterface& scriptInterface = GetSimContext().GetScriptInterface();
 		
-		CmpPtr<ICmpAIInterface> cmpAIInterface(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpAIInterface> cmpAIInterface(GetSystemEntity());
 		ENSURE(cmpAIInterface);
 		
 		// Get the game state from AIInterface
-		CScriptVal state = cmpAIInterface->GetFullRepresentation();
+		// We flush events from the initialization so we get a clean state now.
+		CScriptVal state = cmpAIInterface->GetFullRepresentation(true);
 
 		// Get the passability data
 		Grid<u16> dummyGrid;
 		const Grid<u16>* passabilityMap = &dummyGrid;
-		CmpPtr<ICmpPathfinder> cmpPathfinder(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpPathfinder> cmpPathfinder(GetSystemEntity());
 		if (cmpPathfinder)
 			passabilityMap = &cmpPathfinder->GetPassabilityGrid();
 		
@@ -1007,7 +898,7 @@ public:
 		//	Since getting the territory grid can trigger a recalculation, we check NeedUpdate first
 		Grid<u8> dummyGrid2;
 		const Grid<u8>* territoryMap = &dummyGrid2;
-		CmpPtr<ICmpTerritoryManager> cmpTerritoryManager(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpTerritoryManager> cmpTerritoryManager(GetSystemEntity());
 		if (cmpTerritoryManager && cmpTerritoryManager->NeedUpdate(&m_TerritoriesDirtyID))
 		{
 			territoryMap = &cmpTerritoryManager->GetTerritoryGrid();
@@ -1029,16 +920,20 @@ public:
 		if (m_Worker.getPlayerSize() == 0)
 			return;
 		
-		CmpPtr<ICmpAIInterface> cmpAIInterface(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpAIInterface> cmpAIInterface(GetSystemEntity());
 		ENSURE(cmpAIInterface);
 
 		// Get the game state from AIInterface
-		CScriptVal state = cmpAIInterface->GetRepresentation();
+		CScriptVal state;
+		if (m_JustDeserialized)
+			state = cmpAIInterface->GetFullRepresentation(true);
+		else
+			state = cmpAIInterface->GetRepresentation();
 
 		// Get the passability data
 		Grid<u16> dummyGrid;
 		const Grid<u16>* passabilityMap = &dummyGrid;
-		CmpPtr<ICmpPathfinder> cmpPathfinder(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpPathfinder> cmpPathfinder(GetSystemEntity());
 		if (cmpPathfinder)
 			passabilityMap = &cmpPathfinder->GetPassabilityGrid();
 
@@ -1047,7 +942,7 @@ public:
 		bool territoryMapDirty = false;
 		Grid<u8> dummyGrid2;
 		const Grid<u8>* territoryMap = &dummyGrid2;
-		CmpPtr<ICmpTerritoryManager> cmpTerritoryManager(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpTerritoryManager> cmpTerritoryManager(GetSystemEntity());
 		if (cmpTerritoryManager && cmpTerritoryManager->NeedUpdate(&m_TerritoriesDirtyID))
 		{
 			territoryMap = &cmpTerritoryManager->GetTerritoryGrid();
@@ -1057,6 +952,8 @@ public:
 		LoadPathfinderClasses(state);
 
 		m_Worker.StartComputation(scriptInterface.WriteStructuredClone(state.get()), *passabilityMap, *territoryMap, territoryMapDirty);
+		
+		m_JustDeserialized = false;
 	}
 
 	virtual void PushCommands()
@@ -1066,7 +963,7 @@ public:
 		std::vector<CAIWorker::SCommandSets> commands;
 		m_Worker.GetCommands(commands);
 
-		CmpPtr<ICmpCommandQueue> cmpCommandQueue(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpCommandQueue> cmpCommandQueue(GetSystemEntity());
 		if (!cmpCommandQueue)
 			return;
 
@@ -1086,9 +983,11 @@ private:
 	std::vector<std::pair<std::string, const CParamNode*> > m_Templates;
 	size_t m_TerritoriesDirtyID;
 
+	bool m_JustDeserialized;
+
 	void StartLoadEntityTemplates()
 	{
-		CmpPtr<ICmpTemplateManager> cmpTemplateManager(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpTemplateManager> cmpTemplateManager(GetSystemEntity());
 		ENSURE(cmpTemplateManager);
 
 		m_TemplateNames = cmpTemplateManager->FindAllTemplates(false);
@@ -1102,7 +1001,7 @@ private:
 		if (m_TemplateLoadedIdx >= m_TemplateNames.size())
 			return false;
 
-		CmpPtr<ICmpTemplateManager> cmpTemplateManager(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpTemplateManager> cmpTemplateManager(GetSystemEntity());
 		ENSURE(cmpTemplateManager);
 
 		const CParamNode* node = cmpTemplateManager->GetTemplateWithoutValidation(m_TemplateNames[m_TemplateLoadedIdx]);
@@ -1127,7 +1026,7 @@ private:
 
 	void LoadPathfinderClasses(CScriptVal state)
 	{
-		CmpPtr<ICmpPathfinder> cmpPathfinder(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpPathfinder> cmpPathfinder(GetSystemEntity());
 		if (!cmpPathfinder)
 			return;
 
