@@ -292,7 +292,7 @@ public:
 			if (!data.inWorld && !m_Tag.valid())
 				break; // nothing needs to change
 
-			CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSimContext(), SYSTEM_ENTITY);
+			CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSystemEntity());
 			if (!cmpObstructionManager)
 				break; // error
 
@@ -336,7 +336,7 @@ public:
 		{
 			if (m_Tag.valid())
 			{
-				CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSimContext(), SYSTEM_ENTITY);
+				CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSystemEntity());
 				if (!cmpObstructionManager)
 					break; // error
 
@@ -358,11 +358,11 @@ public:
 
 			// Construct the obstruction shape
 
-			CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSimContext(), SYSTEM_ENTITY);
+			CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSystemEntity());
 			if (!cmpObstructionManager)
 				return; // error
 
-			CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), GetEntityId());
+			CmpPtr<ICmpPosition> cmpPosition(GetEntityHandle());
 			if (!cmpPosition)
 				return; // error
 
@@ -389,7 +389,7 @@ public:
 			// TODO: code duplication from message handlers
 			if (m_Tag.valid())
 			{
-				CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSimContext(), SYSTEM_ENTITY);
+				CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSystemEntity());
 				if (!cmpObstructionManager)
 					return; // error
 
@@ -442,20 +442,34 @@ public:
 		return m_Tag;
 	}
 
+	virtual bool GetPreviousObstructionSquare(ICmpObstructionManager::ObstructionSquare& out)
+	{
+		return GetObstructionSquare(out, true);
+	}
+
 	virtual bool GetObstructionSquare(ICmpObstructionManager::ObstructionSquare& out)
 	{
-		CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), GetEntityId());
+		return GetObstructionSquare(out, false);
+	}
+
+	virtual bool GetObstructionSquare(ICmpObstructionManager::ObstructionSquare& out, bool previousPosition)
+	{
+		CmpPtr<ICmpPosition> cmpPosition(GetEntityHandle());
 		if (!cmpPosition)
 			return false; // error
 
-		CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSystemEntity());
 		if (!cmpObstructionManager)
 			return false; // error
 
 		if (!cmpPosition->IsInWorld())
 			return false; // no obstruction square
 
-		CFixedVector2D pos = cmpPosition->GetPosition2D();
+		CFixedVector2D pos;
+		if (previousPosition)
+			pos = cmpPosition->GetPreviousPosition2D();
+		else
+			pos = cmpPosition->GetPosition2D();
 		if (m_Type == STATIC)
 			out = cmpObstructionManager->GetStaticShapeObstruction(pos.X, pos.Y, cmpPosition->GetRotation().Y, m_Size0, m_Size1);
 		else if (m_Type == UNIT)
@@ -485,7 +499,7 @@ public:
 
 	virtual EFoundationCheck CheckFoundation(std::string className, bool onlyCenterPoint)
 	{
-		CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), GetEntityId());
+		CmpPtr<ICmpPosition> cmpPosition(GetEntityHandle());
 		if (!cmpPosition)
 			return FOUNDATION_CHECK_FAIL_ERROR; // error
 
@@ -494,7 +508,7 @@ public:
 
 		CFixedVector2D pos = cmpPosition->GetPosition2D();
 
-		CmpPtr<ICmpPathfinder> cmpPathfinder(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpPathfinder> cmpPathfinder(GetSystemEntity());
 		if (!cmpPathfinder)
 			return FOUNDATION_CHECK_FAIL_ERROR; // error
 
@@ -522,7 +536,7 @@ public:
 
 	virtual bool CheckDuplicateFoundation()
 	{
-		CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), GetEntityId());
+		CmpPtr<ICmpPosition> cmpPosition(GetEntityHandle());
 		if (!cmpPosition)
 			return false; // error
 
@@ -531,7 +545,7 @@ public:
 
 		CFixedVector2D pos = cmpPosition->GetPosition2D();
 
-		CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSystemEntity());
 		if (!cmpObstructionManager)
 			return false; // error
 
@@ -556,7 +570,7 @@ public:
 	{
 		std::vector<entity_id_t> ret;
 
-		CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), GetEntityId());
+		CmpPtr<ICmpPosition> cmpPosition(GetEntityHandle());
 		if (!cmpPosition)
 			return ret; // error
 
@@ -565,7 +579,7 @@ public:
 
 		CFixedVector2D pos = cmpPosition->GetPosition2D();
 
-		CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSystemEntity());
 		if (!cmpObstructionManager)
 			return ret; // error
 
@@ -620,7 +634,7 @@ public:
 
 		if (m_Tag.valid() && m_Type == UNIT)
 		{
-			CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSimContext(), SYSTEM_ENTITY);
+			CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSystemEntity());
 			if (cmpObstructionManager)
 				cmpObstructionManager->SetUnitMovingFlag(m_Tag, m_Moving);
 		}
@@ -652,7 +666,7 @@ public:
 	{
 		if (m_Tag.valid())
 		{
-			CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSimContext(), SYSTEM_ENTITY);
+			CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSystemEntity());
 			if (cmpObstructionManager)
 			{
 				if (m_Type == UNIT)
@@ -680,11 +694,11 @@ public:
 		if (m_Type == UNIT)
 			return;
 
-		CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSystemEntity());
 		if (!cmpObstructionManager)
 			return;
 
-		CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), GetEntityId());
+		CmpPtr<ICmpPosition> cmpPosition(GetEntityHandle());
 		if (!cmpPosition)
 			return; // error
 
@@ -750,7 +764,7 @@ protected:
 
 	inline void AddClusterShapes(entity_pos_t x, entity_pos_t z, entity_angle_t a)
 	{
-		CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSystemEntity());
 		if (!cmpObstructionManager)
 			return; // error
 
@@ -776,7 +790,7 @@ protected:
 
 	inline void RemoveClusterShapes()
 	{
-		CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSystemEntity());
 		if (!cmpObstructionManager)
 			return; // error
 
