@@ -21,6 +21,7 @@
 #include "ICmpRangeManager.h"
 
 #include "ICmpTerrain.h"
+#include "simulation2/system/EntityMap.h"
 #include "simulation2/MessageTypes.h"
 #include "simulation2/components/ICmpPosition.h"
 #include "simulation2/components/ICmpTerritoryManager.h"
@@ -32,11 +33,12 @@
 #include "graphics/Overlay.h"
 #include "graphics/Terrain.h"
 #include "lib/timer.h"
-#include "maths/FixedVector2D.h"
 #include "ps/CLogger.h"
 #include "ps/Overlay.h"
 #include "ps/Profile.h"
 #include "renderer/Scene.h"
+#include "lib/ps_stl.h"
+
 
 #define DEBUG_RANGE_MANAGER_BOUNDS 0
 
@@ -47,7 +49,7 @@ struct Query
 {
 	bool enabled;
 	bool parabolic;
-	entity_id_t source;
+	CEntityHandle source; // TODO: this could crash if an entity is destroyed while a Query is still referencing it
 	entity_pos_t minRange;
 	entity_pos_t maxRange;
 	entity_pos_t elevationBonus;
@@ -61,7 +63,7 @@ struct Query
  * Convert an owner ID (-1 = unowned, 0 = gaia, 1..30 = players)
  * into a 32-bit mask for quick set-membership tests.
  */
-static u32 CalcOwnerMask(player_id_t owner)
+static inline u32 CalcOwnerMask(player_id_t owner)
 {
 	if (owner >= -1 && owner < 31)
 		return 1 << (1+owner);
@@ -72,7 +74,7 @@ static u32 CalcOwnerMask(player_id_t owner)
 /**
  * Returns LOS mask for given player.
  */
-static u32 CalcPlayerLosMask(player_id_t player)
+static inline u32 CalcPlayerLosMask(player_id_t player)
 {
 	if (player > 0 && player <= 16)
 		return ICmpRangeManager::LOS_MASK << (2*(player-1));
@@ -95,23 +97,22 @@ static u32 CalcSharedLosMask(std::vector<player_id_t> players)
  * Checks whether v is in a parabolic range of (0,0,0)
  * The highest point of the paraboloid is (0,range/2,0)
  * and the circle of distance 'range' around (0,0,0) on height y=0 is part of the paraboloid
- * 
+ *
  * Avoids sqrting and overflowing.
  */
-static bool InParabolicRange(CFixedVector3D v, fixed range) 
+static bool InParabolicRange(CFixedVector3D v, fixed range)
 {
-	i64 x = (i64)v.X.GetInternalValue(); // abs(x) <= 2^31
-	i64 z = (i64)v.Z.GetInternalValue();
-	i64 xx = (x * x); // xx <= 2^62
-	i64 zz = (z * z);
+	i32 x = v.X.GetInternalValue(); // abs(x) <= 2^31
+	i32 z = v.Z.GetInternalValue();
+	u64 xx = (u64)FIXED_MUL_I64_I32_I32(x, x); // xx <= 2^62
+	u64 zz = (u64)FIXED_MUL_I64_I32_I32(z, z);
 	i64 d2 = (xx + zz) >> 1; // d2 <= 2^62 (no overflow)
-	
-	i64 y = (i64)v.Y.GetInternalValue();
 
-	i64 c = (i64)range.GetInternalValue();
-	i64 c_2 = c >> 1; 
+	i32 y = v.Y.GetInternalValue();
+	i32 c = range.GetInternalValue();
+	i32 c_2 = c >> 1;
 
-	i64 c2 = (c_2-y)*c;
+	i64 c2 = FIXED_MUL_I64_I32_I32(c_2 - y, c);
 
 	if (d2 <= c2)
 		return true;
@@ -145,18 +146,16 @@ struct EntityData
 
 cassert(sizeof(EntityData) == 16);
 
-
 /**
  * Serialization helper template for Query
  */
 struct SerializeQuery
 {
 	template<typename S>
-	void operator()(S& serialize, const char* UNUSED(name), Query& value)
+	void Common(S& serialize, const char* UNUSED(name), Query& value)
 	{
 		serialize.Bool("enabled", value.enabled);
 		serialize.Bool("parabolic",value.parabolic);
-		serialize.NumberU32_Unbounded("source", value.source);
 		serialize.NumberFixed_Unbounded("min range", value.minRange);
 		serialize.NumberFixed_Unbounded("max range", value.maxRange);
 		serialize.NumberFixed_Unbounded("elevation bonus", value.elevationBonus);
@@ -165,6 +164,25 @@ struct SerializeQuery
 		SerializeVector<SerializeU32_Unbounded>()(serialize, "last match", value.lastMatch);
 		serialize.NumberU8_Unbounded("flagsMask", value.flagsMask);
 	}
+
+	void operator()(ISerializer& serialize, const char* name, Query& value, const CSimContext& UNUSED(context))
+	{
+		Common(serialize, name, value);
+
+		uint32_t id = value.source.GetId();
+		serialize.NumberU32_Unbounded("source", id);
+	}
+
+	void operator()(IDeserializer& deserialize, const char* name, Query& value, const CSimContext& context)
+	{
+		Common(deserialize, name, value);
+
+		uint32_t id;
+		deserialize.NumberU32_Unbounded("source", id);
+		value.source = context.GetComponentManager().LookupEntityHandle(id, true);
+			// the referenced entity might not have been deserialized yet,
+			// so tell LookupEntityHandle to allocate the handle if necessary
+	}
 };
 
 /**
@@ -193,7 +211,7 @@ struct SerializeEntityData
  */
 struct EntityDistanceOrdering
 {
-	EntityDistanceOrdering(const std::map<entity_id_t, EntityData>& entities, const CFixedVector2D& source) :
+	EntityDistanceOrdering(const EntityMap<EntityData>& entities, const CFixedVector2D& source) :
 		m_EntityData(entities), m_Source(source)
 	{
 	}
@@ -207,7 +225,7 @@ struct EntityDistanceOrdering
 		return (vecA.CompareLength(vecB) < 0);
 	}
 
-	const std::map<entity_id_t, EntityData>& m_EntityData;
+	const EntityMap<EntityData>& m_EntityData;
 	CFixedVector2D m_Source;
 
 private:
@@ -254,12 +272,14 @@ public:
 	// Range query state:
 	tag_t m_QueryNext; // next allocated id
 	std::map<tag_t, Query> m_Queries;
-	std::map<entity_id_t, EntityData> m_EntityData;
-	SpatialSubdivision<entity_id_t> m_Subdivision; // spatial index of m_EntityData
+	EntityMap<EntityData> m_EntityData;
+
+	SpatialSubdivision m_Subdivision; // spatial index of m_EntityData
 
 	// LOS state:
+	static const player_id_t MAX_LOS_PLAYER_ID = 16;
 
-	std::map<player_id_t, bool> m_LosRevealAll;
+	std::vector<bool> m_LosRevealAll;
 	bool m_LosCircular;
 	i32 m_TerrainVerticesPerSide;
 	size_t m_TerritoriesDirtyID;
@@ -273,14 +293,13 @@ public:
 
 	// 2-bit ELosState per player, starting with player 1 (not 0!) up to player MAX_LOS_PLAYER_ID (inclusive)
 	std::vector<u32> m_LosState;
-	static const player_id_t MAX_LOS_PLAYER_ID = 16;
 
 	// Special static visibility data for the "reveal whole map" mode
 	// (TODO: this is usually a waste of memory)
 	std::vector<u32> m_LosStateRevealed;
 
 	// Shared LOS masks, one per player.
-	std::map<player_id_t, u32> m_SharedLosMasks;
+	std::vector<u32> m_SharedLosMasks;
 
 	// Cache explored vertices per player (not serialized)
 	u32 m_TotalInworldVertices;
@@ -302,15 +321,12 @@ public:
 
 		// Initialise with bogus values (these will get replaced when
 		// SetBounds is called)
-		ResetSubdivisions(entity_pos_t::FromInt(1), entity_pos_t::FromInt(1));
+		ResetSubdivisions(entity_pos_t::FromInt(1024), entity_pos_t::FromInt(1024));
 
 		// The whole map should be visible to Gaia by default, else e.g. animals
 		// will get confused when trying to run from enemies
-		m_LosRevealAll[0] = true;
-
-		// This is not really an error condition, an entity recently created or destroyed
-		//	might have an owner of INVALID_PLAYER
-		m_SharedLosMasks[INVALID_PLAYER] = 0;
+		m_LosRevealAll.resize(MAX_LOS_PLAYER_ID+2,false);
+		m_SharedLosMasks.resize(MAX_LOS_PLAYER_ID+2,0);
 
 		m_LosCircular = false;
 		m_TerrainVerticesPerSide = 0;
@@ -331,10 +347,10 @@ public:
 		serialize.NumberFixed_Unbounded("world z1", m_WorldZ1);
 
 		serialize.NumberU32_Unbounded("query next", m_QueryNext);
-		SerializeMap<SerializeU32_Unbounded, SerializeQuery>()(serialize, "queries", m_Queries);
-		SerializeMap<SerializeU32_Unbounded, SerializeEntityData>()(serialize, "entity data", m_EntityData);
+		SerializeMap<SerializeU32_Unbounded, SerializeQuery>()(serialize, "queries", m_Queries, GetSimContext());
+		SerializeEntityMap<SerializeEntityData>()(serialize, "entity data", m_EntityData);
 
-		SerializeMap<SerializeI32_Unbounded, SerializeBool>()(serialize, "los reveal all", m_LosRevealAll);
+		SerializeVector<SerializeBool>()(serialize, "los reveal all", m_LosRevealAll);
 		serialize.Bool("los circular", m_LosCircular);
 		serialize.NumberI32_Unbounded("terrain verts per side", m_TerrainVerticesPerSide);
 
@@ -343,8 +359,7 @@ public:
 		// m_LosState must be serialized since it depends on the history of exploration
 
 		SerializeVector<SerializeU32_Unbounded>()(serialize, "los state", m_LosState);
-
-		SerializeMap<SerializeI32_Unbounded, SerializeU32_Unbounded>()(serialize, "shared los masks", m_SharedLosMasks);
+		SerializeVector<SerializeU32_Unbounded>()(serialize, "shared los masks", m_SharedLosMasks);
 	}
 
 	virtual void Serialize(ISerializer& serialize)
@@ -394,8 +409,7 @@ public:
 			}
 
 			// Remember this entity
-			m_EntityData.insert(std::make_pair(ent, entdata));
-
+			m_EntityData.insert(ent, entdata);
 			break;
 		}
 		case MT_PositionChanged:
@@ -403,7 +417,7 @@ public:
 			const CMessagePositionChanged& msgData = static_cast<const CMessagePositionChanged&> (msg);
 			entity_id_t ent = msgData.entity;
 
-			std::map<entity_id_t, EntityData>::iterator it = m_EntityData.find(ent);
+			EntityMap<EntityData>::iterator it = m_EntityData.find(ent);
 
 			// Ignore if we're not already tracking this entity
 			if (it == m_EntityData.end())
@@ -450,7 +464,7 @@ public:
 			const CMessageOwnershipChanged& msgData = static_cast<const CMessageOwnershipChanged&> (msg);
 			entity_id_t ent = msgData.entity;
 
-			std::map<entity_id_t, EntityData>::iterator it = m_EntityData.find(ent);
+			EntityMap<EntityData>::iterator it = m_EntityData.find(ent);
 
 			// Ignore if we're not already tracking this entity
 			if (it == m_EntityData.end())
@@ -473,7 +487,7 @@ public:
 			const CMessageDestroy& msgData = static_cast<const CMessageDestroy&> (msg);
 			entity_id_t ent = msgData.entity;
 
-			std::map<entity_id_t, EntityData>::iterator it = m_EntityData.find(ent);
+			EntityMap<EntityData>::iterator it = m_EntityData.find(ent);
 
 			// Ignore if we're not already tracking this entity
 			if (it == m_EntityData.end())
@@ -495,7 +509,7 @@ public:
 			const CMessageVisionRangeChanged& msgData = static_cast<const CMessageVisionRangeChanged&> (msg);
 			entity_id_t ent = msgData.entity;
 
-			std::map<entity_id_t, EntityData>::iterator it = m_EntityData.find(ent);
+			EntityMap<EntityData>::iterator it = m_EntityData.find(ent);
 
 			// Ignore if we're not already tracking this entity
 			if (it == m_EntityData.end())
@@ -559,7 +573,7 @@ public:
 
 		std::vector<std::vector<u16> > oldPlayerCounts = m_LosPlayerCounts;
 		std::vector<u32> oldStateRevealed = m_LosStateRevealed;
-		SpatialSubdivision<entity_id_t> oldSubdivision = m_Subdivision;
+		SpatialSubdivision oldSubdivision = m_Subdivision;
 
 		ResetDerivedData(true);
 
@@ -587,6 +601,11 @@ public:
 			debug_warn(L"inconsistent subdivs");
 	}
 
+	SpatialSubdivision* GetSubdivision()
+	{
+		return & m_Subdivision;
+	}
+
 	// Reinitialise subdivisions and LOS data, based on entity data
 	void ResetDerivedData(bool skipLosState)
 	{
@@ -620,7 +639,7 @@ public:
 		m_LosStateRevealed.clear();
 		m_LosStateRevealed.resize(m_TerrainVerticesPerSide*m_TerrainVerticesPerSide);
 
-		for (std::map<entity_id_t, EntityData>::const_iterator it = m_EntityData.begin(); it != m_EntityData.end(); ++it)
+		for (EntityMap<EntityData>::const_iterator it = m_EntityData.begin(); it != m_EntityData.end(); ++it)
 		{
 			if (it->second.inWorld)
 				LosAdd(it->second.owner, it->second.visionRange, CFixedVector2D(it->second.x, it->second.z));
@@ -646,7 +665,7 @@ public:
 		// (TODO: find the optimal number instead of blindly guessing)
 		m_Subdivision.Reset(x1, z1, entity_pos_t::FromInt(8*TERRAIN_TILE_SIZE));
 
-		for (std::map<entity_id_t, EntityData>::const_iterator it = m_EntityData.begin(); it != m_EntityData.end(); ++it)
+		for (EntityMap<EntityData>::const_iterator it = m_EntityData.begin(); it != m_EntityData.end(); ++it)
 		{
 			if (it->second.inWorld)
 				m_Subdivision.Add(it->first, CFixedVector2D(it->second.x, it->second.z));
@@ -710,6 +729,20 @@ public:
 		q.enabled = false;
 	}
 
+	virtual std::vector<entity_id_t> ExecuteQueryAroundPos(CFixedVector2D pos,
+		entity_pos_t minRange, entity_pos_t maxRange,
+		std::vector<int> owners, int requiredInterface)
+	{
+		Query q = ConstructQuery(INVALID_ENTITY, minRange, maxRange, owners, requiredInterface, GetEntityFlagMask("normal"));
+		std::vector<entity_id_t> r;
+		PerformQuery(q, r, pos);
+
+		// Return the list sorted by distance from the entity
+		std::stable_sort(r.begin(), r.end(), EntityDistanceOrdering(m_EntityData, pos));
+
+		return r;
+	}
+
 	virtual std::vector<entity_id_t> ExecuteQuery(entity_id_t source,
 		entity_pos_t minRange, entity_pos_t maxRange,
 		std::vector<int> owners, int requiredInterface)
@@ -720,17 +753,17 @@ public:
 
 		std::vector<entity_id_t> r;
 
-		CmpPtr<ICmpPosition> cmpSourcePosition(GetSimContext(), q.source);
+		CmpPtr<ICmpPosition> cmpSourcePosition(q.source);
 		if (!cmpSourcePosition || !cmpSourcePosition->IsInWorld())
 		{
 			// If the source doesn't have a position, then the result is just the empty list
 			return r;
 		}
 
-		PerformQuery(q, r);
+		CFixedVector2D pos = cmpSourcePosition->GetPosition2D();
+		PerformQuery(q, r, pos);
 
 		// Return the list sorted by distance from the entity
-		CFixedVector2D pos = cmpSourcePosition->GetPosition2D();
 		std::stable_sort(r.begin(), r.end(), EntityDistanceOrdering(m_EntityData, pos));
 
 		return r;
@@ -752,7 +785,7 @@ public:
 		Query& q = it->second;
 		q.enabled = true;
 
-		CmpPtr<ICmpPosition> cmpSourcePosition(GetSimContext(), q.source);
+		CmpPtr<ICmpPosition> cmpSourcePosition(q.source);
 		if (!cmpSourcePosition || !cmpSourcePosition->IsInWorld())
 		{
 			// If the source doesn't have a position, then the result is just the empty list
@@ -760,12 +793,12 @@ public:
 			return r;
 		}
 
-		PerformQuery(q, r);
+		CFixedVector2D pos = cmpSourcePosition->GetPosition2D();
+		PerformQuery(q, r, pos);
 
 		q.lastMatch = r;
 
 		// Return the list sorted by distance from the entity
-		CFixedVector2D pos = cmpSourcePosition->GetPosition2D();
 		std::stable_sort(r.begin(), r.end(), EntityDistanceOrdering(m_EntityData, pos));
 
 		return r;
@@ -777,7 +810,7 @@ public:
 
 		u32 ownerMask = CalcOwnerMask(player);
 
-		for (std::map<entity_id_t, EntityData>::const_iterator it = m_EntityData.begin(); it != m_EntityData.end(); ++it)
+		for (EntityMap<EntityData>::const_iterator it = m_EntityData.begin(); it != m_EntityData.end(); ++it)
 		{
 			// Check owner and add to list if it matches
 			if (CalcOwnerMask(it->second.owner) & ownerMask)
@@ -805,46 +838,52 @@ public:
 		// Store a queue of all messages before sending any, so we can assume
 		// no entities will move until we've finished checking all the ranges
 		std::vector<std::pair<entity_id_t, CMessageRangeUpdate> > messages;
+		std::vector<entity_id_t> results;
+		std::vector<entity_id_t> added;
+		std::vector<entity_id_t> removed;
 
 		for (std::map<tag_t, Query>::iterator it = m_Queries.begin(); it != m_Queries.end(); ++it)
 		{
-			Query& q = it->second;
+			Query& query = it->second;
 
-			if (!q.enabled)
+			if (!query.enabled)
 				continue;
 
-			CmpPtr<ICmpPosition> cmpSourcePosition(GetSimContext(), q.source);
+			CmpPtr<ICmpPosition> cmpSourcePosition(query.source);
 			if (!cmpSourcePosition || !cmpSourcePosition->IsInWorld())
 				continue;
 
-			std::vector<entity_id_t> r;
-			r.reserve(q.lastMatch.size());
-
-			PerformQuery(q, r);
+			results.clear();
+			results.reserve(query.lastMatch.size());
+			CFixedVector2D pos = cmpSourcePosition->GetPosition2D();
+			PerformQuery(query, results, pos);
 
 			// Compute the changes vs the last match
-			std::vector<entity_id_t> added;
-			std::vector<entity_id_t> removed;
-			std::set_difference(r.begin(), r.end(), q.lastMatch.begin(), q.lastMatch.end(), std::back_inserter(added));
-			std::set_difference(q.lastMatch.begin(), q.lastMatch.end(), r.begin(), r.end(), std::back_inserter(removed));
-
+			added.clear();
+			removed.clear();
+			// Return the 'added' list sorted by distance from the entity
+			// (Don't bother sorting 'removed' because they might not even have positions or exist any more)
+			std::set_difference(results.begin(), results.end(), query.lastMatch.begin(), query.lastMatch.end(),
+				std::back_inserter(added));
+			std::set_difference(query.lastMatch.begin(), query.lastMatch.end(), results.begin(), results.end(),
+				std::back_inserter(removed));
 			if (added.empty() && removed.empty())
 				continue;
 
-			// Return the 'added' list sorted by distance from the entity
-			// (Don't bother sorting 'removed' because they might not even have positions or exist any more)
-			CFixedVector2D pos = cmpSourcePosition->GetPosition2D();
-			std::stable_sort(added.begin(), added.end(), EntityDistanceOrdering(m_EntityData, pos));
+			std::stable_sort(added.begin(), added.end(), EntityDistanceOrdering(m_EntityData, cmpSourcePosition->GetPosition2D()));
 
-			messages.push_back(std::make_pair(q.source, CMessageRangeUpdate(it->first)));
-			messages.back().second.added.swap(added);
-			messages.back().second.removed.swap(removed);
-
-			it->second.lastMatch.swap(r);
+			messages.resize(messages.size() + 1);
+			std::pair<entity_id_t, CMessageRangeUpdate>& back = messages.back();
+			back.first = query.source.GetId();
+			back.second.tag = it->first;
+			back.second.added.swap(added);
+			back.second.removed.swap(removed);
+			it->second.lastMatch.swap(results);
 		}
 
+		CComponentManager& cmpMgr = GetSimContext().GetComponentManager();
 		for (size_t i = 0; i < messages.size(); ++i)
-			GetSimContext().GetComponentManager().PostMessage(messages[i].first, messages[i].second);
+			cmpMgr.PostMessage(messages[i].first, messages[i].second);
 	}
 
 	/**
@@ -865,7 +904,7 @@ public:
 			return false;
 
 		// Ignore self
-		if (id == q.source)
+		if (id == q.source.GetId())
 			return false;
 
 		// Ignore if it's missing the required interface
@@ -878,17 +917,13 @@ public:
 	/**
 	 * Returns a list of distinct entity IDs that match the given query, sorted by ID.
 	 */
-	void PerformQuery(const Query& q, std::vector<entity_id_t>& r)
+	void PerformQuery(const Query& q, std::vector<entity_id_t>& r, CFixedVector2D pos)
 	{
-		CmpPtr<ICmpPosition> cmpSourcePosition(GetSimContext(), q.source);
-		if (!cmpSourcePosition || !cmpSourcePosition->IsInWorld())
-			return;
-		CFixedVector2D pos = cmpSourcePosition->GetPosition2D();
 
 		// Special case: range -1.0 means check all entities ignoring distance
 		if (q.maxRange == entity_pos_t::FromInt(-1))
 		{
-			for (std::map<entity_id_t, EntityData>::const_iterator it = m_EntityData.begin(); it != m_EntityData.end(); ++it)
+			for (EntityMap<EntityData>::const_iterator it = m_EntityData.begin(); it != m_EntityData.end(); ++it)
 			{
 				if (!TestEntityQuery(q, it->first, it->second))
 					continue;
@@ -897,22 +932,24 @@ public:
 			}
 		}
 		// Not the entire world, so check a parabolic range, or a regular range
-		else if (q.parabolic) 
+		else if (q.parabolic)
 		{
 			// elevationBonus is part of the 3D position, as the source is really that much heigher
+			CmpPtr<ICmpPosition> cmpSourcePosition(q.source);
 			CFixedVector3D pos3d = cmpSourcePosition->GetPosition()+
 			    CFixedVector3D(entity_pos_t::Zero(), q.elevationBonus, entity_pos_t::Zero()) ;
 			// Get a quick list of entities that are potentially in range, with a cutoff of 2*maxRange
-			std::vector<entity_id_t> ents = m_Subdivision.GetNear(pos, q.maxRange*2);
+			SpatialQueryArray ents;
+			m_Subdivision.GetNear(ents, pos, q.maxRange*2);
 
-			for (size_t i = 0; i < ents.size(); ++i)
+			for (int i = 0; i < ents.size(); ++i)
 			{
-				std::map<entity_id_t, EntityData>::const_iterator it = m_EntityData.find(ents[i]);
+				EntityMap<EntityData>::const_iterator it = m_EntityData.find(ents[i]);
 				ENSURE(it != m_EntityData.end());
 
 				if (!TestEntityQuery(q, it->first, it->second))
 					continue;
-				
+
 				CmpPtr<ICmpPosition> cmpSecondPosition(GetSimContext(), ents[i]);
 				if (!cmpSecondPosition || !cmpSecondPosition->IsInWorld())
 					continue;
@@ -920,8 +957,8 @@ public:
 
 				// Restrict based on precise distance
 				if (!InParabolicRange(
-						CFixedVector3D(it->second.x, secondPosition.Y, it->second.z) 
-							- pos3d, 
+						CFixedVector3D(it->second.x, secondPosition.Y, it->second.z)
+							- pos3d,
 						q.maxRange))
 					continue;
 
@@ -933,18 +970,18 @@ public:
 				}
 
 				r.push_back(it->first);
-
 			}
 		}
 		// check a regular range (i.e. not the entire world, and not parabolic)
-		else 
+		else
 		{
 			// Get a quick list of entities that are potentially in range
-			std::vector<entity_id_t> ents = m_Subdivision.GetNear(pos, q.maxRange);
+			SpatialQueryArray ents;
+			m_Subdivision.GetNear(ents, pos, q.maxRange);
 
-			for (size_t i = 0; i < ents.size(); ++i)
+			for (int i = 0; i < ents.size(); ++i)
 			{
-				std::map<entity_id_t, EntityData>::const_iterator it = m_EntityData.find(ents[i]);
+				EntityMap<EntityData>::const_iterator it = m_EntityData.find(ents[i]);
 				ENSURE(it != m_EntityData.end());
 
 				if (!TestEntityQuery(q, it->first, it->second))
@@ -963,16 +1000,14 @@ public:
 				}
 
 				r.push_back(it->first);
-
 			}
 		}
 	}
 
-
 	virtual entity_pos_t GetElevationAdaptedRange(CFixedVector3D pos, CFixedVector3D rot, entity_pos_t range, entity_pos_t elevationBonus, entity_pos_t angle)
 	{
 		entity_pos_t r = entity_pos_t::Zero() ;
-		
+
 		pos.Y += elevationBonus;
 		entity_pos_t orientation = rot.Y;
 
@@ -992,26 +1027,26 @@ public:
 		{
 			r = r + CFixedVector2D(coords[2*i],coords[2*i+1]).Length() / part;
 		}
-		
+
 		return r;
-		
+
 	}
 
-	virtual std::vector<entity_pos_t> getParabolicRangeForm(CFixedVector3D pos, entity_pos_t maxRange, entity_pos_t cutoff, entity_pos_t minAngle, entity_pos_t maxAngle, int numberOfSteps) 
+	virtual std::vector<entity_pos_t> getParabolicRangeForm(CFixedVector3D pos, entity_pos_t maxRange, entity_pos_t cutoff, entity_pos_t minAngle, entity_pos_t maxAngle, int numberOfSteps)
 	{
-		
+
 		// angle = 0 goes in the positive Z direction
 		entity_pos_t precision = entity_pos_t::FromInt((int)TERRAIN_TILE_SIZE)/8;
 
 		std::vector<entity_pos_t> r;
-		
 
-		CmpPtr<ICmpTerrain> cmpTerrain(GetSimContext(), SYSTEM_ENTITY);
-		CmpPtr<ICmpWaterManager> cmpWaterManager(GetSimContext(), SYSTEM_ENTITY);
+
+		CmpPtr<ICmpTerrain> cmpTerrain(GetSystemEntity());
+		CmpPtr<ICmpWaterManager> cmpWaterManager(GetSystemEntity());
 		entity_pos_t waterLevel = cmpWaterManager->GetWaterLevel(pos.X,pos.Z);
 		entity_pos_t thisHeight = pos.Y > waterLevel ? pos.Y : waterLevel;
 
-		if (cmpTerrain) 
+		if (cmpTerrain)
 		{
 			for (int i = 0; i < numberOfSteps; i++)
 			{
@@ -1034,7 +1069,7 @@ public:
 					r.push_back(maxVector.Y);
 					continue;
 				}
-				
+
 				// Loop until vectors come close enough
 				while ((maxVector - minVector).CompareLength(precision) > 0)
 				{
@@ -1053,26 +1088,26 @@ public:
 						minVector = newVector;
 						minDistance = newDistance;
 					}
-					else 
+					else
 					{
 						// new vector is out parabolic range, so this is a new maxVector
 						maxVector = newVector;
 						maxDistance = newDistance;
 					}
-					
+
 				}
 				r.push_back(maxVector.X);
 				r.push_back(maxVector.Y);
-				
+
 			}
 			r.push_back(r[0]);
-			r.push_back(r[1]); 
+			r.push_back(r[1]);
 
 		}
 		return r;
 
 	}
-	
+
 	Query ConstructQuery(entity_id_t source,
 		entity_pos_t minRange, entity_pos_t maxRange,
 		const std::vector<int>& owners, int requiredInterface, u8 flagsMask)
@@ -1088,7 +1123,7 @@ public:
 		Query q;
 		q.enabled = false;
 		q.parabolic = false;
-		q.source = source;
+		q.source = GetSimContext().GetComponentManager().LookupEntityHandle(source);
 		q.minRange = minRange;
 		q.maxRange = maxRange;
 		q.elevationBonus = entity_pos_t::Zero();
@@ -1118,9 +1153,10 @@ public:
 	{
 		if (!m_DebugOverlayEnabled)
 			return;
-		CColor enabledRingColour(0, 1, 0, 1);
-		CColor disabledRingColour(1, 0, 0, 1);
-		CColor rayColour(1, 1, 0, 0.2f);
+		static CColor disabledRingColour(1, 0, 0, 1);	// red
+		static CColor enabledRingColour(0, 1, 0, 1);	// green
+		static CColor subdivColour(0, 0, 1, 1);			// blue
+		static CColor rayColour(1, 1, 0, 0.2f);
 
 		if (m_DebugOverlayDirty)
 		{
@@ -1130,7 +1166,7 @@ public:
 			{
 				Query& q = it->second;
 
-				CmpPtr<ICmpPosition> cmpSourcePosition(GetSimContext(), q.source);
+				CmpPtr<ICmpPosition> cmpSourcePosition(q.source);
 				if (!cmpSourcePosition || !cmpSourcePosition->IsInWorld())
 					continue;
 				CFixedVector2D pos = cmpSourcePosition->GetPosition2D();
@@ -1142,52 +1178,52 @@ public:
 					m_DebugOverlayLines.back().m_Color = (q.enabled ? enabledRingColour : disabledRingColour);
 					SimRender::ConstructCircleOnGround(GetSimContext(), pos.X.ToFloat(), pos.Y.ToFloat(), q.maxRange.ToFloat(), m_DebugOverlayLines.back(), true);
 				}
-				else 
-				{ 
+				else
+				{
 					// elevation bonus is part of the 3D position. As if the unit is really that much higher
-					CFixedVector3D pos = cmpSourcePosition->GetPosition(); 
+					CFixedVector3D pos = cmpSourcePosition->GetPosition();
 					pos.Y += q.elevationBonus;
 
 					std::vector<entity_pos_t> coords;
-					
+
 					// Get the outline from cache if possible
-					if (ParabolicRangesOutlines.find(q.source) != ParabolicRangesOutlines.end())
+					if (ParabolicRangesOutlines.find(q.source.GetId()) != ParabolicRangesOutlines.end())
 					{
-						EntityParabolicRangeOutline e = ParabolicRangesOutlines[q.source];
-						if (e.position == pos && e.range == q.maxRange) 
+						EntityParabolicRangeOutline e = ParabolicRangesOutlines[q.source.GetId()];
+						if (e.position == pos && e.range == q.maxRange)
 						{
 							// outline is cached correctly, use it
-							coords = e.outline;	
+							coords = e.outline;
 						}
 						else
 						{
-							// outline was cached, but important parameters changed 
+							// outline was cached, but important parameters changed
 							// (position, elevation, range)
 							// update it
 							coords = getParabolicRangeForm(pos,q.maxRange,q.maxRange*2, entity_pos_t::Zero(), entity_pos_t::FromFloat(2.0f*3.14f),70);
 							e.outline = coords;
 							e.range = q.maxRange;
 							e.position = pos;
-							ParabolicRangesOutlines[q.source] = e;
+							ParabolicRangesOutlines[q.source.GetId()] = e;
 						}
 					}
-					else 
+					else
 					{
-						// outline wasn't cached (first time you enable the range overlay 
+						// outline wasn't cached (first time you enable the range overlay
 						// or you created a new entiy)
 						// cache a new outline
 						coords = getParabolicRangeForm(pos,q.maxRange,q.maxRange*2, entity_pos_t::Zero(), entity_pos_t::FromFloat(2.0f*3.14f),70);
 						EntityParabolicRangeOutline e;
-						e.source = q.source;
+						e.source = q.source.GetId();
 						e.range = q.maxRange;
 						e.position = pos;
 						e.outline = coords;
-						ParabolicRangesOutlines[q.source] = e;
+						ParabolicRangesOutlines[q.source.GetId()] = e;
 					}
-					
+
 					CColor thiscolor = q.enabled ? enabledRingColour : disabledRingColour;
-					
-					// draw the outline (piece by piece)	
+
+					// draw the outline (piece by piece)
 					for (size_t i = 3; i < coords.size(); i += 2)
 					{
 						std::vector<float> c;
@@ -1227,6 +1263,24 @@ public:
 				}
 			}
 
+			// render subdivision grid
+			float divSize = m_Subdivision.GetDivisionSize().ToFloat();
+			int width = m_Subdivision.GetWidth();
+			int height = m_Subdivision.GetHeight();
+			for (int x = 0; x < width; ++x)
+			{
+				for (int y = 0; y < height; ++y)
+				{
+					m_DebugOverlayLines.push_back(SOverlayLine());
+					m_DebugOverlayLines.back().m_Color = subdivColour;
+
+					float xpos = x*divSize + divSize/2;
+					float zpos = y*divSize + divSize/2;
+					SimRender::ConstructSquareOnGround(GetSimContext(), xpos, zpos, divSize, divSize, 0.0f,
+						m_DebugOverlayLines.back(), false, 1.0f);
+				}
+			}
+
 			m_DebugOverlayDirty = false;
 		}
 
@@ -1247,7 +1301,7 @@ public:
 
 	virtual void SetEntityFlag(entity_id_t ent, std::string identifier, bool value)
 	{
-		std::map<entity_id_t, EntityData>::iterator it = m_EntityData.find(ent);
+		EntityMap<EntityData>::iterator it = m_EntityData.find(ent);
 
 		// We don't have this entity
 		if (it == m_EntityData.end())
@@ -1280,12 +1334,14 @@ public:
 			return CLosQuerier(GetSharedLosMask(player), m_LosState, m_TerrainVerticesPerSide);
 	}
 
-	virtual ELosVisibility GetLosVisibility(entity_id_t ent, player_id_t player, bool forceRetainInFog)
+	virtual ELosVisibility GetLosVisibility(CEntityHandle ent, player_id_t player, bool forceRetainInFog)
 	{
 		// (We can't use m_EntityData since this needs to handle LOCAL entities too)
 
 		// Entities not with positions in the world are never visible
-		CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), ent);
+		if (ent.GetId() == INVALID_ENTITY)
+			return VIS_HIDDEN;
+		CmpPtr<ICmpPosition> cmpPosition(ent);
 		if (!cmpPosition || !cmpPosition->IsInWorld())
 			return VIS_HIDDEN;
 
@@ -1312,7 +1368,7 @@ public:
 		// Fogged if the 'retain in fog' flag is set, and in a non-visible explored region
 		if (los.IsExplored(i, j))
 		{
-			CmpPtr<ICmpVision> cmpVision(GetSimContext(), ent);
+			CmpPtr<ICmpVision> cmpVision(ent);
 			if (forceRetainInFog || (cmpVision && cmpVision->GetRetainInFog()))
 				return VIS_FOGGED;
 		}
@@ -1321,23 +1377,32 @@ public:
 		return VIS_HIDDEN;
 	}
 
+	virtual ELosVisibility GetLosVisibility(entity_id_t ent, player_id_t player, bool forceRetainInFog)
+	{
+		CEntityHandle handle = GetSimContext().GetComponentManager().LookupEntityHandle(ent);
+		return GetLosVisibility(handle, player, forceRetainInFog);
+	}
+
+
 	virtual void SetLosRevealAll(player_id_t player, bool enabled)
 	{
-		m_LosRevealAll[player] = enabled;
+		if (player == -1)
+			m_LosRevealAll[MAX_LOS_PLAYER_ID+1] = enabled;
+		else
+		{
+			ENSURE(player >= 0 && player <= MAX_LOS_PLAYER_ID);
+			m_LosRevealAll[player] = enabled;
+		}
 	}
 
 	virtual bool GetLosRevealAll(player_id_t player)
 	{
-		std::map<player_id_t, bool>::const_iterator it;
-
 		// Special player value can force reveal-all for every player
-		it = m_LosRevealAll.find(-1);
-		if (it != m_LosRevealAll.end() && it->second)
+		if (m_LosRevealAll[MAX_LOS_PLAYER_ID+1] || player == -1)
 			return true;
-
+		ENSURE(player >= 0 && player <= MAX_LOS_PLAYER_ID+1);
 		// Otherwise check the player-specific flag
-		it = m_LosRevealAll.find(player);
-		if (it != m_LosRevealAll.end() && it->second)
+		if (m_LosRevealAll[player])
 			return true;
 
 		return false;
@@ -1362,15 +1427,27 @@ public:
 
 	virtual u32 GetSharedLosMask(player_id_t player)
 	{
-		std::map<player_id_t, u32>::const_iterator it = m_SharedLosMasks.find(player);
-		ENSURE(it != m_SharedLosMasks.end());
-
 		return m_SharedLosMasks[player];
 	}
 
+	void ExploreAllTiles(player_id_t p)
+	{
+		for (u16 j = 0; j < m_TerrainVerticesPerSide; ++j)
+		{
+			for (u16 i = 0; i < m_TerrainVerticesPerSide; ++i)
+			{
+				if (LosIsOffWorld(i,j))
+					continue;
+				u32 &explored = m_ExploredVertices.at(p);
+				explored += !(m_LosState[i + j*m_TerrainVerticesPerSide] & (LOS_EXPLORED << (2*(p-1))));
+				m_LosState[i + j*m_TerrainVerticesPerSide] |= (LOS_EXPLORED << (2*(p-1)));
+			}
+		}
+	}
+
 	void UpdateTerritoriesLos()
 	{
-		CmpPtr<ICmpTerritoryManager> cmpTerritoryManager(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpTerritoryManager> cmpTerritoryManager(GetSystemEntity());
 		if (!cmpTerritoryManager || !cmpTerritoryManager->NeedUpdate(&m_TerritoriesDirtyID))
 			return;
 
