@@ -417,7 +417,7 @@ void CCmpTemplateManager::ConstructTemplateActor(const std::string& actorName, C
 	std::wstring actorNameW = wstring_from_utf8(actorName);
 	std::string name = utf8_from_wstring(CParamNode::EscapeXMLString(actorNameW));
 	std::string xml = "<Entity>"
-	                      "<VisualActor><Actor>" + name + "</Actor></VisualActor>"
+	                      "<VisualActor><Actor>" + name + "</Actor><ActorOnly/></VisualActor>"
 						  // arbitrary-sized Footprint definition to make actors' selection outlines show up in Atlas
 						  "<Footprint><Circle radius='2.0'/><Height>1.0</Height></Footprint>"
 	                      "<Selectable>"
@@ -429,7 +429,7 @@ void CCmpTemplateManager::ConstructTemplateActor(const std::string& actorName, C
 	CParamNode::LoadXMLString(out, xml.c_str(), actorNameW.c_str());
 }
 
-static Status AddToTemplates(const VfsPath& pathname, const FileInfo& UNUSED(fileInfo), const uintptr_t cbData)
+static Status AddToTemplates(const VfsPath& pathname, const CFileInfo& UNUSED(fileInfo), const uintptr_t cbData)
 {
 	std::vector<std::string>& templates = *(std::vector<std::string>*)cbData;
 
@@ -446,7 +446,7 @@ static Status AddToTemplates(const VfsPath& pathname, const FileInfo& UNUSED(fil
 	return INFO::OK;
 }
 
-static Status AddActorToTemplates(const VfsPath& pathname, const FileInfo& UNUSED(fileInfo), const uintptr_t cbData)
+static Status AddActorToTemplates(const VfsPath& pathname, const CFileInfo& UNUSED(fileInfo), const uintptr_t cbData)
 {
 	std::vector<std::string>& templates = *(std::vector<std::string>*)cbData;
 
@@ -525,11 +525,12 @@ void CCmpTemplateManager::CopyPreviewSubset(CParamNode& out, const CParamNode& i
 	if (out.GetChild("Entity").GetChild("Obstruction").IsOk())
 		CParamNode::LoadXMLString(out, "<Entity><Obstruction><Active>false</Active></Obstruction></Entity>");
 
-	if (out.GetChild("Entity").GetChild("VisualActor").IsOk())
-		CParamNode::LoadXMLString(out, "<Entity><VisualActor><DisableShadows/></VisualActor></Entity>");
-
 	if (!corpse)
 	{
+		// Previews should not cast shadows
+		if (out.GetChild("Entity").GetChild("VisualActor").IsOk())
+			CParamNode::LoadXMLString(out, "<Entity><VisualActor><DisableShadows/></VisualActor></Entity>");
+
 		// Previews should always be visible in fog-of-war/etc
 		CParamNode::LoadXMLString(out, "<Entity><Vision><Range>0</Range><RetainInFog>false</RetainInFog><AlwaysVisible>true</AlwaysVisible></Vision></Entity>");
 	}
