@@ -66,7 +66,7 @@ public:
 	{
 		componentManager.SubscribeGloballyToMessageType(MT_OwnershipChanged);
 		componentManager.SubscribeGloballyToMessageType(MT_PositionChanged);
-		componentManager.SubscribeGloballyToMessageType(MT_TechnologyModification);
+		componentManager.SubscribeGloballyToMessageType(MT_ValueModification);
 		componentManager.SubscribeToMessageType(MT_TerrainChanged);
 		componentManager.SubscribeToMessageType(MT_Update);
 		componentManager.SubscribeToMessageType(MT_Interpolate);
@@ -165,9 +165,9 @@ public:
 			MakeDirtyIfRelevantEntity(msgData.entity);
 			break;
 		}
-		case MT_TechnologyModification:
+		case MT_ValueModification:
 		{
-			const CMessageTechnologyModification& msgData = static_cast<const CMessageTechnologyModification&> (msg);
+			const CMessageValueModification& msgData = static_cast<const CMessageValueModification&> (msg);
 			if (msgData.component == L"TerritoryInfluence")
 				MakeDirty();
 			break;
@@ -336,7 +336,7 @@ void CCmpTerritoryManager::CalculateTerritories()
 
 	PROFILE("CalculateTerritories");
 
-	CmpPtr<ICmpTerrain> cmpTerrain(GetSimContext(), SYSTEM_ENTITY);
+	CmpPtr<ICmpTerrain> cmpTerrain(GetSystemEntity());
 
 	// If the terrain hasn't been loaded (e.g. this is called during map initialisation),
 	// abort the computation (and assume callers can cope with m_Territories == NULL)
@@ -351,7 +351,7 @@ void CCmpTerritoryManager::CalculateTerritories()
 	// Compute terrain-passability-dependent costs per tile
 	Grid<u8> influenceGrid(tilesW, tilesH);
 
-	CmpPtr<ICmpPathfinder> cmpPathfinder(GetSimContext(), SYSTEM_ENTITY);
+	CmpPtr<ICmpPathfinder> cmpPathfinder(GetSystemEntity());
 	ICmpPathfinder::pass_class_t passClassDefault = cmpPathfinder->GetPassabilityClass("default");
 	ICmpPathfinder::pass_class_t passClassUnrestricted = cmpPathfinder->GetPassabilityClass("unrestricted");
 
@@ -451,9 +451,7 @@ void CCmpTerritoryManager::CalculateTerritories()
 			FloodFill(entityGrid, influenceGrid, openTiles, falloff);
 
 			// TODO: we should do a sparse grid and only add the non-zero regions, for performance
-			for (u16 j = 0; j < entityGrid.m_H; ++j)
-				for (u16 i = 0; i < entityGrid.m_W; ++i)
-					playerGrid.set(i, j, playerGrid.get(i, j) + entityGrid.get(i, j));
+			playerGrid.add(entityGrid);
 		}
 
 		playerGrids.push_back(std::make_pair(it->first, playerGrid));
@@ -628,7 +626,7 @@ void CCmpTerritoryManager::UpdateBoundaryLines()
 	texturePropsMask.SetMaxAnisotropy(2.f);
 	CTexturePtr textureMask = g_Renderer.GetTextureManager().CreateTexture(texturePropsMask);
 
-	CmpPtr<ICmpPlayerManager> cmpPlayerManager(GetSimContext(), SYSTEM_ENTITY);
+	CmpPtr<ICmpPlayerManager> cmpPlayerManager(GetSystemEntity());
 	if (!cmpPlayerManager)
 		return;
 
