@@ -26,6 +26,7 @@
 #include "simulation2/components/ICmpPosition.h"
 #include "simulation2/components/ICmpPathfinder.h"
 #include "simulation2/components/ICmpRangeManager.h"
+#include "simulation2/components/ICmpValueModificationManager.h"
 #include "simulation2/helpers/Geometry.h"
 #include "simulation2/helpers/Render.h"
 #include "simulation2/MessageTypes.h"
@@ -95,6 +96,14 @@ static const entity_pos_t CHECK_TARGET_MOVEMENT_MIN_DELTA_FORMATION = entity_pos
  */
 static const entity_pos_t CHECK_TARGET_MOVEMENT_AT_MAX_DIST = entity_pos_t::FromInt(TERRAIN_TILE_SIZE*16);
 
+/**
+ * If we're following something and the angle between the (straight-line) directions to its previous target
+ * position and its present target position is greater than a given angle, recompute the path even far away
+ * (i.e. even if CHECK_TARGET_MOVEMENT_AT_MAX_DIST condition is not fulfilled). The actual check is done
+ * on the cosine of this angle, with a PI/6 angle.
+ */
+static const fixed CHECK_TARGET_MOVEMENT_MIN_COS = fixed::FromInt(866)/1000;
+
 static const CColor OVERLAY_COLOUR_LONG_PATH(1, 1, 1, 1);
 static const CColor OVERLAY_COLOUR_SHORT_PATH(1, 0, 0, 1);
 
@@ -109,6 +118,7 @@ public:
 		componentManager.SubscribeToMessageType(MT_Update_MotionUnit);
 		componentManager.SubscribeToMessageType(MT_RenderSubmit); // for debug overlays
 		componentManager.SubscribeToMessageType(MT_PathResult);
+		componentManager.SubscribeToMessageType(MT_ValueModification);
 	}
 
 	DEFAULT_COMPONENT_ALLOCATOR(UnitMotion)
@@ -120,8 +130,8 @@ public:
 	// Template state:
 
 	bool m_FormationController;
-	fixed m_WalkSpeed; // in metres per second
-	fixed m_RunSpeed;
+	fixed m_WalkSpeed, m_OriginalWalkSpeed; // in metres per second
+	fixed m_RunSpeed, m_OriginalRunSpeed;
 	ICmpPathfinder::pass_class_t m_PassClass;
 	ICmpPathfinder::cost_class_t m_CostClass;
 
@@ -129,6 +139,7 @@ public:
 
 	entity_pos_t m_Radius;
 	bool m_Moving;
+	bool m_FacePointAfterMove;
 
 	enum State
 	{
@@ -281,27 +292,25 @@ public:
 		m_FormationController = paramNode.GetChild("FormationController").ToBool();
 
 		m_Moving = false;
-		m_WalkSpeed = paramNode.GetChild("WalkSpeed").ToFixed();
+		m_FacePointAfterMove = true;
+
+		m_WalkSpeed = m_OriginalWalkSpeed = paramNode.GetChild("WalkSpeed").ToFixed();
 		m_Speed = m_WalkSpeed;
 		m_CurSpeed = fixed::Zero();
 
 		if (paramNode.GetChild("Run").IsOk())
-		{
-			m_RunSpeed = paramNode.GetChild("Run").GetChild("Speed").ToFixed();
-		}
+			m_RunSpeed = m_OriginalRunSpeed = paramNode.GetChild("Run").GetChild("Speed").ToFixed();
 		else
-		{
-			m_RunSpeed = m_WalkSpeed;
-		}
+			m_RunSpeed = m_OriginalRunSpeed = m_WalkSpeed;
 
-		CmpPtr<ICmpPathfinder> cmpPathfinder(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpPathfinder> cmpPathfinder(GetSystemEntity());
 		if (cmpPathfinder)
 		{
 			m_PassClass = cmpPathfinder->GetPassabilityClass(paramNode.GetChild("PassabilityClass").ToUTF8());
 			m_CostClass = cmpPathfinder->GetCostClass(paramNode.GetChild("CostClass").ToUTF8());
 		}
 
-		CmpPtr<ICmpObstruction> cmpObstruction(GetSimContext(), GetEntityId());
+		CmpPtr<ICmpObstruction> cmpObstruction(GetEntityHandle());
 		if (cmpObstruction)
 			m_Radius = cmpObstruction->GetUnitRadius();
 
@@ -342,6 +351,7 @@ public:
 		serialize.NumberFixed_Unbounded("speed", m_Speed);
 
 		serialize.Bool("moving", m_Moving);
+		serialize.Bool("facePointAfterMove", m_FacePointAfterMove);
 
 		SerializeVector<SerializeWaypoint>()(serialize, "long path", m_LongPath.m_Waypoints);
 		SerializeVector<SerializeWaypoint>()(serialize, "short path", m_ShortPath.m_Waypoints);
@@ -395,6 +405,26 @@ public:
 			PathResult(msgData.ticket, msgData.path);
 			break;
 		}
+		case MT_ValueModification:
+		{
+			const CMessageValueModification& msgData = static_cast<const CMessageValueModification&> (msg); 
+			if (msgData.component != L"UnitMotion")
+				break;
+
+			CmpPtr<ICmpValueModificationManager> cmpValueModificationManager(GetSimContext(), SYSTEM_ENTITY);
+
+			fixed newWalkSpeed = cmpValueModificationManager->ApplyModifications(L"UnitMotion/WalkSpeed", m_OriginalWalkSpeed, GetEntityId());
+			fixed newRunSpeed = cmpValueModificationManager->ApplyModifications(L"UnitMotion/Run/Speed", m_OriginalRunSpeed, GetEntityId());
+
+			// update m_Speed (the actual speed) if set to one of the variables
+			if (m_Speed == m_WalkSpeed)
+				m_Speed = newWalkSpeed;
+			else if (m_Speed == m_RunSpeed)
+				m_Speed = newRunSpeed;
+
+			m_WalkSpeed = newWalkSpeed;
+			m_RunSpeed = newRunSpeed;
+		}
 		}
 	}
 
@@ -428,6 +458,11 @@ public:
 		m_Speed = speed;
 	}
 
+	virtual void SetFacePointAfterMove(bool facePointAfterMove)
+	{
+		m_FacePointAfterMove = facePointAfterMove;
+	}
+
 	virtual void SetDebugOverlay(bool enabled)
 	{
 		m_DebugOverlayEnabled = enabled;
@@ -472,7 +507,7 @@ private:
 		StopMoving();
 		m_State = STATE_IDLE; // don't go through the STOPPING state since we never even started
 
-		CmpPtr<ICmpObstruction> cmpObstruction(GetSimContext(), GetEntityId());
+		CmpPtr<ICmpObstruction> cmpObstruction(GetEntityHandle());
 		if (cmpObstruction)
 			cmpObstruction->SetMovingFlag(false);
 
@@ -484,7 +519,7 @@ private:
 	{
 		StopMoving();
 
-		CmpPtr<ICmpObstruction> cmpObstruction(GetSimContext(), GetEntityId());
+		CmpPtr<ICmpObstruction> cmpObstruction(GetEntityHandle());
 		if (cmpObstruction)
 			cmpObstruction->SetMovingFlag(false);
 
@@ -502,7 +537,7 @@ private:
 	{
 		m_Moving = false;
 
-		CmpPtr<ICmpObstruction> cmpObstruction(GetSimContext(), GetEntityId());
+		CmpPtr<ICmpObstruction> cmpObstruction(GetEntityHandle());
 		if (cmpObstruction)
 			cmpObstruction->SetMovingFlag(false);
 
@@ -513,6 +548,8 @@ private:
 		GetSimContext().GetComponentManager().PostMessage(GetEntityId(), msg);
 	}
 
+	bool MoveToPointRange(entity_pos_t x, entity_pos_t z, entity_pos_t minRange, entity_pos_t maxRange, entity_id_t target);
+
 	/**
 	 * Handle the result of an asynchronous path query.
 	 */
@@ -620,7 +657,7 @@ void CCmpUnitMotion::PathResult(u32 ticket, const ICmpPathfinder::Path& path)
 			m_LongPath.m_Waypoints.push_back(wp);
 		}
 
-		CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), GetEntityId());
+		CmpPtr<ICmpPosition> cmpPosition(GetEntityHandle());
 		if (!cmpPosition || !cmpPosition->IsInWorld())
 		{
 			StartFailed();
@@ -658,7 +695,7 @@ void CCmpUnitMotion::PathResult(u32 ticket, const ICmpPathfinder::Path& path)
 			}
 		}
 
-		CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), GetEntityId());
+		CmpPtr<ICmpPosition> cmpPosition(GetEntityHandle());
 		if (!cmpPosition || !cmpPosition->IsInWorld())
 		{
 			StartFailed();
@@ -686,7 +723,7 @@ void CCmpUnitMotion::PathResult(u32 ticket, const ICmpPathfinder::Path& path)
 			m_LongPath.m_Waypoints.push_back(wp);
 		}
 
-		CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), GetEntityId());
+		CmpPtr<ICmpPosition> cmpPosition(GetEntityHandle());
 		if (!cmpPosition || !cmpPosition->IsInWorld())
 		{
 			StopMoving();
@@ -765,9 +802,7 @@ void CCmpUnitMotion::Move(fixed dt)
 	}
 
 	if (m_State == STATE_IDLE)
-	{
 		return;
-	}
 
 	switch (m_PathState)
 	{
@@ -794,11 +829,11 @@ void CCmpUnitMotion::Move(fixed dt)
 		// Maybe we should split the updates into multiple phases to minimise
 		// that problem.
 
-		CmpPtr<ICmpPathfinder> cmpPathfinder (GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpPathfinder> cmpPathfinder(GetSystemEntity());
 		if (!cmpPathfinder)
 			return;
 
-		CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), GetEntityId());
+		CmpPtr<ICmpPosition> cmpPosition(GetEntityHandle());
 		if (!cmpPosition || !cmpPosition->IsInWorld())
 			return;
 
@@ -833,8 +868,9 @@ void CCmpUnitMotion::Move(fixed dt)
 		// We want to move (at most) maxSpeed*dt units from pos towards the next waypoint
 
 		fixed timeLeft = dt;
-
-		while (timeLeft > fixed::Zero())
+		fixed zero = fixed::Zero();
+		
+		while (timeLeft > zero)
 		{
 			// If we ran out of short path, we have to stop
 			if (m_ShortPath.m_Waypoints.empty())
@@ -843,13 +879,6 @@ void CCmpUnitMotion::Move(fixed dt)
 			CFixedVector2D target(m_ShortPath.m_Waypoints.back().x, m_ShortPath.m_Waypoints.back().z);
 			CFixedVector2D offset = target - pos;
 
-			// Face towards the target
-			if (!offset.IsZero())
-			{
-				entity_angle_t angle = atan2_approx(offset.X, offset.Y);
-				cmpPosition->TurnTo(angle);
-			}
-
 			// Work out how far we can travel in timeLeft
 			fixed maxdist = maxSpeed.Multiply(timeLeft);
 
@@ -896,18 +925,25 @@ void CCmpUnitMotion::Move(fixed dt)
 
 		// Update the Position component after our movement (if we actually moved anywhere)
 		if (pos != initialPos)
-			cmpPosition->MoveTo(pos.X, pos.Y);
-
-		// Calculate the mean speed over this past turn.
-		m_CurSpeed = cmpPosition->GetDistanceTravelled() / dt;
+		{
+			CFixedVector2D offset = pos - initialPos;
+			
+			// Face towards the target
+			entity_angle_t angle = atan2_approx(offset.X, offset.Y);
+			cmpPosition->MoveAndTurnTo(pos.X,pos.Y, angle);
 
+			// Calculate the mean speed over this past turn.
+			m_CurSpeed = cmpPosition->GetDistanceTravelled() / dt;
+		}
+		
 		if (wasObstructed)
 		{
 			// Oops, we hit something (very likely another unit).
 			// Stop, and recompute the whole path.
 			// TODO: if the target has UnitMotion and is higher priority,
 			// we should wait a little bit.
-
+			
+			m_CurSpeed = zero;
 			RequestLongPath(pos, m_FinalGoal);
 			m_PathState = PATHSTATE_WAITING_REQUESTING_LONG;
 
@@ -959,11 +995,22 @@ void CCmpUnitMotion::Move(fixed dt)
 						}
 						else
 						{
+							// check if target was reached in case of a moving target
+							CmpPtr<ICmpUnitMotion> cmpUnitMotion(GetSimContext(), m_TargetEntity);
+							if 
+							(
+								cmpUnitMotion && cmpUnitMotion->IsMoving() &&
+								MoveToTargetRange(m_TargetEntity, m_TargetMinRange, m_TargetMaxRange)
+							)
+								return;
+
 							// Not in formation, so just finish moving
-
 							StopMoving();
+							m_State = STATE_IDLE;
+							MoveSucceeded();
 
-							FaceTowardsPointFromPos(pos, m_FinalGoal.x, m_FinalGoal.z);
+							if (m_FacePointAfterMove)
+								FaceTowardsPointFromPos(pos, m_FinalGoal.x, m_FinalGoal.z);
 							// TODO: if the goal was a square building, we ought to point towards the
 							// nearest point on the square, not towards its center
 						}
@@ -1020,7 +1067,7 @@ bool CCmpUnitMotion::TryGoingStraightToTargetEntity(CFixedVector2D from)
 	if ((targetPos - from).CompareLength(DIRECT_PATH_RANGE) > 0)
 		return false;
 
-	CmpPtr<ICmpPathfinder> cmpPathfinder (GetSimContext(), SYSTEM_ENTITY);
+	CmpPtr<ICmpPathfinder> cmpPathfinder(GetSystemEntity());
 	if (!cmpPathfinder)
 		return false;
 
@@ -1059,17 +1106,27 @@ bool CCmpUnitMotion::CheckTargetMovement(CFixedVector2D from, entity_pos_t minDe
 	if ((targetPos - oldTargetPos).CompareLength(minDelta) < 0)
 		return false;
 
+	CmpPtr<ICmpPosition> cmpPosition(GetEntityHandle());
+	if (!cmpPosition || !cmpPosition->IsInWorld())
+		return false;
+	CFixedVector2D pos = cmpPosition->GetPosition2D();
+	CFixedVector2D oldDir = (oldTargetPos - pos);
+	CFixedVector2D newDir = (targetPos - pos);
+	oldDir.Normalize();
+	newDir.Normalize();
+
 	// Fail unless we're close enough to the target to care about its movement
-	if (!PathIsShort(m_LongPath, from, CHECK_TARGET_MOVEMENT_AT_MAX_DIST))
+	// and the angle between the (straight-line) directions of the previous and new target positions is small
+	if (oldDir.Dot(newDir) > CHECK_TARGET_MOVEMENT_MIN_COS && !PathIsShort(m_LongPath, from, CHECK_TARGET_MOVEMENT_AT_MAX_DIST))
 		return false;
 
 	// Fail if the target is no longer visible to this entity's owner
 	// (in which case we'll continue moving to its last known location,
 	// unless it comes back into view before we reach that location)
-	CmpPtr<ICmpOwnership> cmpOwnership(GetSimContext(), GetEntityId());
+	CmpPtr<ICmpOwnership> cmpOwnership(GetEntityHandle());
 	if (cmpOwnership)
 	{
-		CmpPtr<ICmpRangeManager> cmpRangeManager(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpRangeManager> cmpRangeManager(GetSystemEntity());
 		if (cmpRangeManager)
 		{
 			if (cmpRangeManager->GetLosVisibility(m_TargetEntity, cmpOwnership->GetOwner()) == ICmpRangeManager::VIS_HIDDEN)
@@ -1089,20 +1146,19 @@ bool CCmpUnitMotion::CheckTargetMovement(CFixedVector2D from, entity_pos_t minDe
 
 bool CCmpUnitMotion::PathIsShort(const ICmpPathfinder::Path& path, CFixedVector2D from, entity_pos_t minDistance)
 {
-	CFixedVector2D pos = from;
 	entity_pos_t distLeft = minDistance;
 
 	for (ssize_t i = (ssize_t)path.m_Waypoints.size()-1; i >= 0; --i)
 	{
 		// Check if the next path segment is longer than the requested minimum
 		CFixedVector2D waypoint(path.m_Waypoints[i].x, path.m_Waypoints[i].z);
-		CFixedVector2D delta = waypoint - pos;
+		CFixedVector2D delta = waypoint - from;
 		if (delta.CompareLength(distLeft) > 0)
 			return false;
 
 		// Still short enough - prepare to check the next segment
 		distLeft -= delta.Length();
-		pos = waypoint;
+		from = waypoint;
 	}
 
 	// Reached the end of the path before exceeding minDistance
@@ -1111,7 +1167,7 @@ bool CCmpUnitMotion::PathIsShort(const ICmpPathfinder::Path& path, CFixedVector2
 
 void CCmpUnitMotion::FaceTowardsPoint(entity_pos_t x, entity_pos_t z)
 {
-	CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), GetEntityId());
+	CmpPtr<ICmpPosition> cmpPosition(GetEntityHandle());
 	if (!cmpPosition || !cmpPosition->IsInWorld())
 		return;
 
@@ -1127,7 +1183,7 @@ void CCmpUnitMotion::FaceTowardsPointFromPos(CFixedVector2D pos, entity_pos_t x,
 	{
 		entity_angle_t angle = atan2_approx(offset.X, offset.Y);
 
-		CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), GetEntityId());
+		CmpPtr<ICmpPosition> cmpPosition(GetEntityHandle());
 		if (!cmpPosition)
 			return;
 		cmpPosition->TurnTo(angle);
@@ -1156,7 +1212,7 @@ void CCmpUnitMotion::BeginPathing(CFixedVector2D from, const ICmpPathfinder::Goa
 	m_Moving = true;
 
 	// Set our 'moving' flag, so other units pathfinding now will ignore us
-	CmpPtr<ICmpObstruction> cmpObstruction(GetSimContext(), GetEntityId());
+	CmpPtr<ICmpObstruction> cmpObstruction(GetEntityHandle());
 	if (cmpObstruction)
 		cmpObstruction->SetMovingFlag(true);
 
@@ -1183,7 +1239,7 @@ void CCmpUnitMotion::BeginPathing(CFixedVector2D from, const ICmpPathfinder::Goa
 
 void CCmpUnitMotion::RequestLongPath(CFixedVector2D from, const ICmpPathfinder::Goal& goal)
 {
-	CmpPtr<ICmpPathfinder> cmpPathfinder(GetSimContext(), SYSTEM_ENTITY);
+	CmpPtr<ICmpPathfinder> cmpPathfinder(GetSystemEntity());
 	if (!cmpPathfinder)
 		return;
 
@@ -1194,7 +1250,7 @@ void CCmpUnitMotion::RequestLongPath(CFixedVector2D from, const ICmpPathfinder::
 
 void CCmpUnitMotion::RequestShortPath(CFixedVector2D from, const ICmpPathfinder::Goal& goal, bool avoidMovingUnits)
 {
-	CmpPtr<ICmpPathfinder> cmpPathfinder(GetSimContext(), SYSTEM_ENTITY);
+	CmpPtr<ICmpPathfinder> cmpPathfinder(GetSystemEntity());
 	if (!cmpPathfinder)
 		return;
 
@@ -1242,7 +1298,7 @@ bool CCmpUnitMotion::PickNextLongWaypoint(const CFixedVector2D& pos, bool avoidM
 		goal.z = targetZ;
 	}
 
-	CmpPtr<ICmpPathfinder> cmpPathfinder(GetSimContext(), SYSTEM_ENTITY);
+	CmpPtr<ICmpPathfinder> cmpPathfinder(GetSystemEntity());
 	if (!cmpPathfinder)
 		return false;
 
@@ -1251,12 +1307,16 @@ bool CCmpUnitMotion::PickNextLongWaypoint(const CFixedVector2D& pos, bool avoidM
 	return true;
 }
 
-
 bool CCmpUnitMotion::MoveToPointRange(entity_pos_t x, entity_pos_t z, entity_pos_t minRange, entity_pos_t maxRange)
+{
+	return MoveToPointRange(x, z, minRange, maxRange, INVALID_ENTITY);
+}
+
+bool CCmpUnitMotion::MoveToPointRange(entity_pos_t x, entity_pos_t z, entity_pos_t minRange, entity_pos_t maxRange, entity_id_t target)
 {
 	PROFILE("MoveToPointRange");
 
-	CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), GetEntityId());
+	CmpPtr<ICmpPosition> cmpPosition(GetEntityHandle());
 	if (!cmpPosition || !cmpPosition->IsInWorld())
 		return false;
 
@@ -1270,7 +1330,7 @@ bool CCmpUnitMotion::MoveToPointRange(entity_pos_t x, entity_pos_t z, entity_pos
 
 		// Check whether this point is in an obstruction
 
-		CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSimContext(), SYSTEM_ENTITY);
+		CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSystemEntity());
 		if (!cmpObstructionManager)
 			return false;
 
@@ -1312,7 +1372,8 @@ bool CCmpUnitMotion::MoveToPointRange(entity_pos_t x, entity_pos_t z, entity_pos
 		else
 		{
 			// We're already in range - no need to move anywhere
-			FaceTowardsPointFromPos(pos, x, z);
+			if (m_FacePointAfterMove)
+				FaceTowardsPointFromPos(pos, x, z);
 			return false;
 		}
 
@@ -1327,7 +1388,7 @@ bool CCmpUnitMotion::MoveToPointRange(entity_pos_t x, entity_pos_t z, entity_pos
 	}
 
 	m_State = STATE_INDIVIDUAL_PATH;
-	m_TargetEntity = INVALID_ENTITY;
+	m_TargetEntity = target;
 	m_TargetOffset = CFixedVector2D();
 	m_TargetMinRange = minRange;
 	m_TargetMaxRange = maxRange;
@@ -1340,14 +1401,14 @@ bool CCmpUnitMotion::MoveToPointRange(entity_pos_t x, entity_pos_t z, entity_pos
 
 bool CCmpUnitMotion::IsInPointRange(entity_pos_t x, entity_pos_t z, entity_pos_t minRange, entity_pos_t maxRange)
 {
-	CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), GetEntityId());
+	CmpPtr<ICmpPosition> cmpPosition(GetEntityHandle());
 	if (!cmpPosition || !cmpPosition->IsInWorld())
 		return false;
 
 	CFixedVector2D pos = cmpPosition->GetPosition2D();
 
 	bool hasObstruction = false;
-	CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSimContext(), SYSTEM_ENTITY);
+	CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSystemEntity());
 	ICmpObstructionManager::ObstructionSquare obstruction;
 	if (cmpObstructionManager)
 		hasObstruction = cmpObstructionManager->FindMostImportantObstruction(GetObstructionFilter(true), x, z, m_Radius, obstruction);
@@ -1406,13 +1467,13 @@ bool CCmpUnitMotion::MoveToTargetRange(entity_id_t target, entity_pos_t minRange
 {
 	PROFILE("MoveToTargetRange");
 
-	CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), GetEntityId());
+	CmpPtr<ICmpPosition> cmpPosition(GetEntityHandle());
 	if (!cmpPosition || !cmpPosition->IsInWorld())
 		return false;
 
 	CFixedVector2D pos = cmpPosition->GetPosition2D();
 
-	CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSimContext(), SYSTEM_ENTITY);
+	CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSystemEntity());
 	if (!cmpObstructionManager)
 		return false;
 
@@ -1456,7 +1517,12 @@ bool CCmpUnitMotion::MoveToTargetRange(entity_id_t target, entity_pos_t minRange
 
 		entity_pos_t distance = Geometry::DistanceToSquare(pos - CFixedVector2D(obstruction.x, obstruction.z), obstruction.u, obstruction.v, halfSize);
 
-		if (distance < minRange)
+		// compare with previous obstruction
+		ICmpObstructionManager::ObstructionSquare previousObstruction;
+		cmpObstruction->GetPreviousObstructionSquare(previousObstruction);
+		entity_pos_t previousDistance = Geometry::DistanceToSquare(pos - CFixedVector2D(previousObstruction.x, previousObstruction.z), obstruction.u, obstruction.v, halfSize);
+
+		if (distance < minRange && previousDistance < minRange)
 		{
 			// Too close to the square - need to move away
 
@@ -1471,10 +1537,11 @@ bool CCmpUnitMotion::MoveToTargetRange(entity_id_t target, entity_pos_t minRange
 			goal.hw = obstruction.hw + delta;
 			goal.hh = obstruction.hh + delta;
 		}
-		else if (maxRange < entity_pos_t::Zero() || distance < maxRange)
+		else if (maxRange < entity_pos_t::Zero() || distance < maxRange || previousDistance < maxRange)
 		{
 			// We're already in range - no need to move anywhere
-			FaceTowardsPointFromPos(pos, goal.x, goal.z);
+			if (m_FacePointAfterMove)
+				FaceTowardsPointFromPos(pos, goal.x, goal.z);
 			return false;
 		}
 		else
@@ -1496,10 +1563,22 @@ bool CCmpUnitMotion::MoveToTargetRange(entity_id_t target, entity_pos_t minRange
 				if (circleDistance < maxRange)
 				{
 					// We're already in range - no need to move anywhere
-					FaceTowardsPointFromPos(pos, goal.x, goal.z);
+					if (m_FacePointAfterMove)
+						FaceTowardsPointFromPos(pos, goal.x, goal.z);
 					return false;
 				}
 
+				entity_pos_t previousCircleDistance = (pos - CFixedVector2D(previousObstruction.x, previousObstruction.z)).Length() - circleRadius;
+
+				if (previousCircleDistance < maxRange)
+				{
+					// We're already in range - no need to move anywhere
+					if (m_FacePointAfterMove)
+						FaceTowardsPointFromPos(pos, goal.x, goal.z);
+					return false;
+				}
+
+
 				entity_pos_t goalDistance = maxRange - g_GoalDelta;
 
 				goal.type = ICmpPathfinder::Goal::CIRCLE;
@@ -1542,7 +1621,7 @@ bool CCmpUnitMotion::MoveToTargetRange(entity_id_t target, entity_pos_t minRange
 
 		CFixedVector2D targetPos = cmpTargetPosition->GetPosition2D();
 
-		return MoveToPointRange(targetPos.X, targetPos.Y, minRange, maxRange);
+		return MoveToPointRange(targetPos.X, targetPos.Y, minRange, maxRange, target);
 	}
 }
 
@@ -1551,13 +1630,13 @@ bool CCmpUnitMotion::IsInTargetRange(entity_id_t target, entity_pos_t minRange,
 	// This function closely mirrors MoveToTargetRange - it needs to return true
 	// after that Move has completed
 
-	CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), GetEntityId());
+	CmpPtr<ICmpPosition> cmpPosition(GetEntityHandle());
 	if (!cmpPosition || !cmpPosition->IsInWorld())
 		return false;
 
 	CFixedVector2D pos = cmpPosition->GetPosition2D();
 
-	CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSimContext(), SYSTEM_ENTITY);
+	CmpPtr<ICmpObstructionManager> cmpObstructionManager(GetSystemEntity());
 	if (!cmpObstructionManager)
 		return false;
 
@@ -1572,12 +1651,17 @@ bool CCmpUnitMotion::IsInTargetRange(entity_id_t target, entity_pos_t minRange,
 		CFixedVector2D halfSize(obstruction.hw, obstruction.hh);
 		entity_pos_t distance = Geometry::DistanceToSquare(pos - CFixedVector2D(obstruction.x, obstruction.z), obstruction.u, obstruction.v, halfSize);
 
+		// compare with previous obstruction
+		ICmpObstructionManager::ObstructionSquare previousObstruction;
+		cmpObstruction->GetPreviousObstructionSquare(previousObstruction);
+		entity_pos_t previousDistance = Geometry::DistanceToSquare(pos - CFixedVector2D(previousObstruction.x, previousObstruction.z), obstruction.u, obstruction.v, halfSize);
+		
 		// See if we're too close to the target square
-		if (distance < minRange)
+		if (distance < minRange && previousDistance < minRange)
 			return false;
 
 		// See if we're close enough to the target square
-		if (maxRange < entity_pos_t::Zero() || distance <= maxRange)
+		if (maxRange < entity_pos_t::Zero() || distance <= maxRange || previousDistance <= maxRange)
 			return true;
 
 		entity_pos_t circleRadius = halfSize.Length();
@@ -1589,6 +1673,11 @@ bool CCmpUnitMotion::IsInTargetRange(entity_id_t target, entity_pos_t minRange,
 
 			entity_pos_t circleDistance = (pos - CFixedVector2D(obstruction.x, obstruction.z)).Length() - circleRadius;
 
+			if (circleDistance <= maxRange)
+				return true;
+			// also check circle around previous position
+			circleDistance = (pos - CFixedVector2D(previousObstruction.x, previousObstruction.z)).Length() - circleRadius;
+
 			if (circleDistance <= maxRange)
 				return true;
 		}
@@ -1601,14 +1690,12 @@ bool CCmpUnitMotion::IsInTargetRange(entity_id_t target, entity_pos_t minRange,
 		if (!cmpTargetPosition || !cmpTargetPosition->IsInWorld())
 			return false;
 
-		CFixedVector2D targetPos = cmpTargetPosition->GetPosition2D();
+		CFixedVector2D targetPos = cmpTargetPosition->GetPreviousPosition2D();
 
 		entity_pos_t distance = (pos - targetPos).Length();
 
-		if (minRange <= distance && (maxRange < entity_pos_t::Zero() || distance <= maxRange))
-			return true;
-
-		return false;
+		return minRange <= distance && 
+			(maxRange < entity_pos_t::Zero() || distance <= maxRange);
 	}
 }
 
@@ -1642,7 +1729,7 @@ void CCmpUnitMotion::MoveToFormationOffset(entity_id_t target, entity_pos_t x, e
 void CCmpUnitMotion::RenderPath(const ICmpPathfinder::Path& path, std::vector<SOverlayLine>& lines, CColor color)
 {
 	bool floating = false;
-	CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), GetEntityId());
+	CmpPtr<ICmpPosition> cmpPosition(GetEntityHandle());
 	if (cmpPosition)
 		floating = cmpPosition->IsFloating();
 
