@@ -30,6 +30,7 @@
 #include "ICmpTemplateManager.h"
 #include "ICmpTerrain.h"
 #include "ICmpUnitMotion.h"
+#include "ICmpValueModificationManager.h"
 #include "ICmpVision.h"
 
 #include "graphics/Decal.h"
@@ -43,6 +44,7 @@
 #include "maths/Matrix3D.h"
 #include "maths/Vector3D.h"
 #include "ps/CLogger.h"
+#include "ps/GameSetup/Config.h"
 #include "renderer/Scene.h"
 
 #include "tools/atlas/GameInterface/GameLoop.h"
@@ -56,13 +58,15 @@ public:
 		componentManager.SubscribeToMessageType(MT_Interpolate);
 		componentManager.SubscribeToMessageType(MT_RenderSubmit);
 		componentManager.SubscribeToMessageType(MT_OwnershipChanged);
+		componentManager.SubscribeToMessageType(MT_ValueModification);
 		componentManager.SubscribeGloballyToMessageType(MT_TerrainChanged);
 	}
 
 	DEFAULT_COMPONENT_ALLOCATOR(VisualActor)
 
 private:
-	std::wstring m_ActorName;
+	std::wstring m_BaseActorName, m_ActorName;
+	bool m_IsFoundationActor;
 	CUnit* m_Unit;
 
 	fixed m_R, m_G, m_B; // shading colour
@@ -86,6 +90,7 @@ private:
 	fixed m_ConstructionProgress;
 
 	bool m_VisibleInAtlasOnly;
+	bool m_IsActorOnly;	// an in-world entity should not have this or it might not be rendered.
 
 	/// Whether the visual actor has been rendered at least once.
 	/// Necessary because the visibility update runs on simulation update,
@@ -127,6 +132,11 @@ public:
 					"<empty/>"
 				"</element>"
 			"</optional>"
+			"<optional>"
+				"<element name='ActorOnly' a:help='Used internally; if present, the unit will only be rendered if the user has high enough graphical settings.'>"
+					"<empty/>"
+				"</element>"
+			"</optional>"
 			"<element name='SilhouetteDisplay'>"
 				"<data type='boolean'/>"
 			"</element>"
@@ -181,13 +191,15 @@ public:
 
 		m_Seed = GetEntityId();
 
-		if (paramNode.GetChild("Foundation").IsOk() && paramNode.GetChild("FoundationActor").IsOk())
-			m_ActorName = paramNode.GetChild("FoundationActor").ToString();
+		m_IsFoundationActor = paramNode.GetChild("Foundation").IsOk() && paramNode.GetChild("FoundationActor").IsOk();
+		if (m_IsFoundationActor)
+			m_BaseActorName = m_ActorName = paramNode.GetChild("FoundationActor").ToString();
 		else
-			m_ActorName = paramNode.GetChild("Actor").ToString();
+			m_BaseActorName = m_ActorName = paramNode.GetChild("Actor").ToString();
 
 		m_VisibleInAtlasOnly = paramNode.GetChild("VisibleInAtlasOnly").ToBool();
-
+		m_IsActorOnly = paramNode.GetChild("ActorOnly").IsOk();
+		
 		InitModel(paramNode);
 
 		// We need to select animation even if graphics are disabled, as this modifies serialized state
@@ -220,6 +232,7 @@ public:
 
 		serialize.NumberU32_Unbounded("seed", m_Seed);
 		// TODO: variation/selection strings
+		serialize.String("actor", m_ActorName, 0, 256);
 
 		serialize.NumberFixed_Unbounded("constructionprogress", m_ConstructionProgress);
 
@@ -232,7 +245,7 @@ public:
 
 		if (serialize.IsDebug())
 		{
-			serialize.String("actor", m_ActorName, 0, 256);
+			serialize.String("base actor", m_BaseActorName, 0, 256);
 		}
 
 		SerializeCommon(serialize);
@@ -246,8 +259,8 @@ public:
 
 		SerializeCommon(deserialize);
 
-		// If we serialized a different seed, reload actor
-		if (oldSeed != GetActorSeed())
+		// If we serialized a different seed or different actor, reload actor
+		if (oldSeed != GetActorSeed() || m_BaseActorName != m_ActorName)
 			ReloadActor();
 
 		fixed repeattime = m_AnimSyncRepeatTime; // save because SelectAnimation overwrites it
@@ -261,7 +274,7 @@ public:
 
 		if (m_Unit)
 		{
-			CmpPtr<ICmpOwnership> cmpOwnership(GetSimContext(), GetEntityId());
+			CmpPtr<ICmpOwnership> cmpOwnership(GetEntityHandle());
 			if (cmpOwnership)
 				m_Unit->GetModel().SetPlayerID(cmpOwnership->GetOwner());
 		}
@@ -305,6 +318,24 @@ public:
 			m_Unit->GetModel().SetTerrainDirty(msgData.i0, msgData.j0, msgData.i1, msgData.j1);
 			break;
 		}
+		case MT_ValueModification:
+		{
+			const CMessageValueModification& msgData = static_cast<const CMessageValueModification&> (msg);
+			if (msgData.component != L"VisualActor")
+				break;
+			CmpPtr<ICmpValueModificationManager> cmpValueModificationManager(GetSystemEntity());
+			std::wstring newActorName;
+			if (m_IsFoundationActor)
+				newActorName = cmpValueModificationManager->ApplyModifications(L"VisualActor/FoundationActor", m_BaseActorName, GetEntityId());
+			else
+				newActorName = cmpValueModificationManager->ApplyModifications(L"VisualActor/Actor", m_BaseActorName, GetEntityId());
+			if (newActorName != m_ActorName)
+			{
+				m_ActorName = newActorName;
+				ReloadActor();
+			}
+			break;
+		}
 		}
 	}
 
@@ -511,7 +542,10 @@ void CCmpVisualActor::InitModel(const CParamNode& paramNode)
 	if (GetSimContext().HasUnitManager())
 	{
 		std::set<CStr> selections;
-		m_Unit = GetSimContext().GetUnitManager().CreateUnit(m_ActorName, GetActorSeed(), selections);
+		std::wstring actorName = m_ActorName;
+		if (actorName.find(L".xml") == std::wstring::npos)
+			actorName += L".xml";
+		m_Unit = GetSimContext().GetUnitManager().CreateUnit(actorName, GetActorSeed(), selections);
 		if (m_Unit)
 		{
 			CModelAbstract& model = m_Unit->GetModel();
@@ -525,7 +559,7 @@ void CCmpVisualActor::InitModel(const CParamNode& paramNode)
 				if (paramNode.GetChild("SilhouetteOccluder").ToBool())
 					modelFlags |= MODELFLAG_SILHOUETTE_OCCLUDER;
 
-				CmpPtr<ICmpVision> cmpVision(GetSimContext(), GetEntityId());
+				CmpPtr<ICmpVision> cmpVision(GetEntityHandle());
 				if (cmpVision && cmpVision->GetAlwaysVisible())
 					modelFlags |= MODELFLAG_IGNORE_LOS;
 
@@ -566,7 +600,7 @@ void CCmpVisualActor::InitSelectionShapeDescriptor(const CParamNode& paramNode)
 		}
 		else if (shapeNode.GetChild("Footprint").IsOk())
 		{
-			CmpPtr<ICmpFootprint> cmpFootprint(GetSimContext(), GetEntityId());
+			CmpPtr<ICmpFootprint> cmpFootprint(GetEntityHandle());
 			if (cmpFootprint)
 			{
 				ICmpFootprint::EShape fpShape;				// fp stands for "footprint"
@@ -629,7 +663,10 @@ void CCmpVisualActor::ReloadActor()
 		return;
 
 	std::set<CStr> selections;
-	CUnit* newUnit = GetSimContext().GetUnitManager().CreateUnit(m_ActorName, GetActorSeed(), selections);
+	std::wstring actorName = m_ActorName;
+	if (actorName.find(L".xml") == std::wstring::npos)
+		actorName += L".xml";
+	CUnit* newUnit = GetSimContext().GetUnitManager().CreateUnit(actorName, GetActorSeed(), selections);
 
 	if (!newUnit)
 		return;
@@ -643,7 +680,7 @@ void CCmpVisualActor::ReloadActor()
 
 	// HACK: selection shape needs template data, but rather than storing all that data
 	//	in the component, we load the template here and pass it into a helper function
-	CmpPtr<ICmpTemplateManager> cmpTemplateManager(GetSimContext(), SYSTEM_ENTITY);
+	CmpPtr<ICmpTemplateManager> cmpTemplateManager(GetSystemEntity());
 	const CParamNode* node = cmpTemplateManager->LoadLatestTemplate(GetEntityId());
 	ENSURE(node && node->GetChild("VisualActor").IsOk());
 
@@ -673,11 +710,11 @@ void CCmpVisualActor::Update(fixed UNUSED(turnLength))
 	// If we're in the special movement mode, select an appropriate animation
 	if (!m_AnimRunThreshold.IsZero())
 	{
-		CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), GetEntityId());
+		CmpPtr<ICmpPosition> cmpPosition(GetEntityHandle());
 		if (!cmpPosition || !cmpPosition->IsInWorld())
 			return;
 
-		CmpPtr<ICmpUnitMotion> cmpUnitMotion(GetSimContext(), GetEntityId());
+		CmpPtr<ICmpUnitMotion> cmpUnitMotion(GetEntityHandle());
 		if (!cmpUnitMotion)
 			return;
 
@@ -712,23 +749,23 @@ void CCmpVisualActor::Update(fixed UNUSED(turnLength))
 void CCmpVisualActor::UpdateVisibility()
 {
 	ICmpRangeManager::ELosVisibility oldVisibility = m_Visibility;
-	CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), GetEntityId());
+	CmpPtr<ICmpPosition> cmpPosition(GetEntityHandle());
 	if (cmpPosition && cmpPosition->IsInWorld())
 	{
 		// The 'always visible' flag means we should always render the unit
 		// (regardless of whether the LOS system thinks it's visible)
-		CmpPtr<ICmpVision> cmpVision(GetSimContext(), GetEntityId());
+		CmpPtr<ICmpVision> cmpVision(GetEntityHandle());
 		if (cmpVision && cmpVision->GetAlwaysVisible())
 			m_Visibility = ICmpRangeManager::VIS_VISIBLE;
 		else
 		{
-			CmpPtr<ICmpRangeManager> cmpRangeManager(GetSimContext(), SYSTEM_ENTITY);
+			CmpPtr<ICmpRangeManager> cmpRangeManager(GetSystemEntity());
 			// Uncomment the following lines to prevent the models from popping into existence
 			// near the LOS boundary. Is rather resource intensive.
 			//if (cmpVision->GetRetainInFog())
 			//	m_Visibility = ICmpRangeManager::VIS_VISIBLE;
 			//else
-				m_Visibility = cmpRangeManager->GetLosVisibility(GetEntityId(), 	
+				m_Visibility = cmpRangeManager->GetLosVisibility(GetEntityHandle(),
 					GetSimContext().GetCurrentDisplayedPlayer());
 		}
 	}
@@ -738,7 +775,7 @@ void CCmpVisualActor::UpdateVisibility()
 	if (m_Visibility != oldVisibility)
 	{
 		// Change the visibility of the visual actor's selectable if it has one.
-		CmpPtr<ICmpSelectable> cmpSelectable(GetSimContext(), GetEntityId());
+		CmpPtr<ICmpSelectable> cmpSelectable(GetEntityHandle());
 		if (cmpSelectable)
 			cmpSelectable->SetVisibility(m_Visibility == ICmpRangeManager::VIS_HIDDEN ? false : true);
 	}
@@ -750,7 +787,7 @@ void CCmpVisualActor::Interpolate(float frameTime, float frameOffset)
 		return;
 
 	// Disable rendering of the unit if it has no position
-	CmpPtr<ICmpPosition> cmpPosition(GetSimContext(), GetEntityId());
+	CmpPtr<ICmpPosition> cmpPosition(GetEntityHandle());
 	if (!cmpPosition || !cmpPosition->IsInWorld())
 		return;
 	else if (!m_PreviouslyRendered)
@@ -778,7 +815,7 @@ void CCmpVisualActor::Interpolate(float frameTime, float frameOffset)
 			// If this is a floating unit, we want it to start all the way under the terrain,
 			// so find the difference between its current position and the terrain
 
-			CmpPtr<ICmpTerrain> cmpTerrain(GetSimContext(), SYSTEM_ENTITY);
+			CmpPtr<ICmpTerrain> cmpTerrain(GetSystemEntity());
 			if (floating && cmpTerrain)
 			{
 				CVector3D pos = transform.GetTranslation();
@@ -812,6 +849,9 @@ void CCmpVisualActor::RenderSubmit(SceneCollector& collector, const CFrustum& fr
 		return;
 
 	CModelAbstract& model = m_Unit->GetModel();
+	
+	if (!g_AtlasGameLoop->running && !g_RenderActors && m_IsActorOnly)
+		return;
 
 	if (culling && !frustum.IsBoxVisible(CVector3D(0, 0, 0), model.GetWorldBoundsRec()))
 		return;
