@@ -26,13 +26,14 @@
 #include "renderer/WaterManager.h"
 #include "simulation2/MessageTypes.h"
 
+#include "tools/atlas/GameInterface/GameLoop.h"
+
 class CCmpWaterManager : public ICmpWaterManager
 {
 public:
 	static void ClassInit(CComponentManager& componentManager)
 	{
 		componentManager.SubscribeToMessageType(MT_Interpolate);
-		componentManager.SubscribeToMessageType(MT_RenderSubmit);
 		componentManager.SubscribeToMessageType(MT_TerrainChanged);
 	}
 
@@ -72,34 +73,45 @@ public:
 	{
 		switch (msg.GetType())
 		{
-		case MT_Interpolate:
-		{
-			const CMessageInterpolate& msgData = static_cast<const CMessageInterpolate&> (msg);
-			if (CRenderer::IsInitialised())
-				g_Renderer.GetWaterManager()->m_WaterTexTimer += msgData.deltaSimTime;
-			break;
-		}
-		case MT_RenderSubmit:
-		{
-			// Don't actually do rendering here, but tell the renderer how to draw water
-			if (CRenderer::IsInitialised())
-				g_Renderer.GetWaterManager()->m_WaterHeight = m_WaterHeight.ToFloat();
-			break;
-		}
+			case MT_Interpolate:
+			{
+				const CMessageInterpolate& msgData = static_cast<const CMessageInterpolate&> (msg);
+				if (CRenderer::IsInitialised())
+					g_Renderer.GetWaterManager()->m_WaterTexTimer += msgData.deltaSimTime;
+				break;
+			}
 			case MT_TerrainChanged:
 			{
-				// Tell the renderer to redraw the map.
+				// Tell the renderer to redraw part of the map.
 				if (CRenderer::IsInitialised())
 				{
-					g_Renderer.GetWaterManager()->m_NeedsFullReloading = true;
-					g_Renderer.GetWaterManager()->m_NeedsReloading = true;
-					g_Renderer.GetWaterManager()->m_TerrainChangeThisTurn = true;
+					const CMessageTerrainChanged& msgData = static_cast<const CMessageTerrainChanged&> (msg);
+					g_Renderer.GetWaterManager()->m_NeedInfoUpdate = true;
+					g_Renderer.GetWaterManager()->m_updatei0 = msgData.i0;
+					g_Renderer.GetWaterManager()->m_updatej0 = msgData.j0;
+					g_Renderer.GetWaterManager()->m_updatei1 = msgData.i1;
+					g_Renderer.GetWaterManager()->m_updatej1 = msgData.j1;
+					
+					GetSimContext().GetTerrain().MakeDirty(msgData.i0,msgData.j0,msgData.i1,msgData.j1,RENDERDATA_UPDATE_VERTICES);
 				}
 				break;
 			}
 		}
 	}
 
+	virtual void RecomputeWaterData()
+	{
+		ssize_t mapSize = GetSimContext().GetTerrain().GetVerticesPerSide();
+		g_Renderer.GetWaterManager()->m_NeedInfoUpdate = true;
+		g_Renderer.GetWaterManager()->m_updatei0 = 0;
+		g_Renderer.GetWaterManager()->m_updatej0 = 0;
+		g_Renderer.GetWaterManager()->m_updatei1 = mapSize-1;
+		g_Renderer.GetWaterManager()->m_updatej1 = mapSize-1;
+
+		// Tell the terrain it'll need to recompute its cached render data
+		GetSimContext().GetTerrain().MakeDirty(RENDERDATA_UPDATE_VERTICES);
+	}
+
 	virtual void SetWaterLevel(entity_pos_t h)
 	{
 		m_WaterHeight = h;
