@@ -47,6 +47,11 @@ public:
 	 */
 	virtual bool GetObstructionSquare(ICmpObstructionManager::ObstructionSquare& out) = 0;
 
+	/**
+	 * Same as the method above, but returns an obstruction shape for the previous turn
+	 */
+	virtual bool GetPreviousObstructionSquare(ICmpObstructionManager::ObstructionSquare& out) = 0;
+
 	virtual entity_pos_t GetUnitRadius() = 0;
 
 	virtual bool IsControlPersistent() = 0;
@@ -59,12 +64,13 @@ public:
 	 *	value describing the type of failure.
 	 */
 	virtual EFoundationCheck CheckFoundation(std::string className) = 0;
+	virtual EFoundationCheck CheckFoundation(std::string className, bool onlyCenterPoint) = 0;
 
 	/**
 	 * CheckFoundation wrapper for script calls, to return friendly strings instead of an EFoundationCheck.
 	 * @return "success" if check passes, else a string describing the type of failure.
 	 */
-	virtual std::string CheckFoundation_wrapper(std::string className);
+	virtual std::string CheckFoundation_wrapper(std::string className, bool onlyCenterPoint);
 
 	/**
 	 * Test whether this entity is colliding with any obstructions that share its
