@@ -19,10 +19,12 @@
 #define INCLUDED_ICMPRANGEMANAGER
 
 #include "maths/FixedVector3D.h"
+#include "maths/FixedVector2D.h"
 
 #include "simulation2/system/Interface.h"
 #include "simulation2/helpers/Position.h"
 #include "simulation2/helpers/Player.h"
+#include "simulation2/helpers/Spatial.h"
 
 #include "graphics/Terrain.h" // for TERRAIN_TILE_SIZE
 
@@ -71,6 +73,12 @@ public:
 	 */
 	typedef u32 tag_t;
 
+	/**
+	 * Access the spatial subdivision kept by the range manager.
+	 * @return pointer to spatial subdivision structure.
+	 */
+	virtual SpatialSubdivision* GetSubdivision() = 0;
+
 	/**
 	 * Set the bounds of the world.
 	 * Entities should not be outside the bounds (else efficiency will suffer).
@@ -91,6 +99,18 @@ public:
 	virtual std::vector<entity_id_t> ExecuteQuery(entity_id_t source,
 		entity_pos_t minRange, entity_pos_t maxRange, std::vector<int> owners, int requiredInterface) = 0;
 
+	/**
+	 * Execute a passive query.
+	 * @param pos the position around which the range will be computed.
+	 * @param minRange non-negative minimum distance in metres (inclusive).
+	 * @param maxRange non-negative maximum distance in metres (inclusive); or -1.0 to ignore distance.
+	 * @param owners list of player IDs that matching entities may have; -1 matches entities with no owner.
+	 * @param requiredInterface if non-zero, an interface ID that matching entities must implement.
+	 * @return list of entities matching the query, ordered by increasing distance from the source entity.
+	 */
+	virtual std::vector<entity_id_t> ExecuteQueryAroundPos(CFixedVector2D pos,
+		entity_pos_t minRange, entity_pos_t maxRange, std::vector<int> owners, int requiredInterface) = 0;
+
 	/**
 	 * Construct an active query. The query will be disabled by default.
 	 * @param source the entity around which the range will be computed.
@@ -105,12 +125,12 @@ public:
 		entity_pos_t minRange, entity_pos_t maxRange, std::vector<int> owners, int requiredInterface, u8 flags) = 0;
 
     /**
-	 * Construct an active query of a paraboloic form around the unit. 
+	 * Construct an active query of a paraboloic form around the unit.
 	 * The query will be disabled by default.
 	 * @param source the entity around which the range will be computed.
 	 * @param minRange non-negative minimum horizontal distance in metres (inclusive). MinRange doesn't do parabolic checks.
-	 * @param maxRange non-negative maximum distance in metres (inclusive) for units on the same elevation; 
-	 *      or -1.0 to ignore distance. 
+	 * @param maxRange non-negative maximum distance in metres (inclusive) for units on the same elevation;
+	 *      or -1.0 to ignore distance.
 	 *      For units on a different elevation, a physical correct paraboloid with height=maxRange/2 above the unit is used to query them
 	 * @param elevationBonus extra bonus so the source can be placed higher and shoot further
 	 * @param owners list of player IDs that matching entities may have; -1 matches entities with no owner.
@@ -303,6 +323,7 @@ public:
 	 * TODO: This is a hack to allow preview entities in FoW to return fogged instead of hidden,
 	 *	see http://trac.wildfiregames.com/ticket/958
 	 */
+	virtual ELosVisibility GetLosVisibility(CEntityHandle ent, player_id_t player, bool forceRetainInFog = false) = 0;
 	virtual ELosVisibility GetLosVisibility(entity_id_t ent, player_id_t player, bool forceRetainInFog = false) = 0;
 
 	/**
@@ -311,6 +332,11 @@ public:
 	 */
 	std::string GetLosVisibility_wrapper(entity_id_t ent, player_id_t player, bool forceRetainInFog);
 
+	/**
+	 * Explore all tiles (but leave them in the FoW) for player p
+	 */
+	virtual void ExploreAllTiles(player_id_t p) = 0;
+
 	/**
 	 * Set whether the whole map should be made visible to the given player.
 	 * If player is -1, the map will be made visible to all players.
