@@ -41,7 +41,6 @@ public:
 
 		// Need some stuff for terrain movement costs:
 		// (TODO: this ought to be independent of any graphics code)
-		tex_codec_register_all();
 		new CTerrainTextureManager;
 		g_TexMan.LoadTerrainTextures();
 	}
@@ -49,7 +48,6 @@ public:
 	void tearDown()
 	{
 		delete &g_TexMan;
-		tex_codec_unregister_all();
 
 		g_VFS.reset();
 
@@ -61,14 +59,14 @@ public:
 	{
 		CTerrain terrain;
 
-		CSimulation2 sim2(NULL, &terrain);
+		CSimulation2 sim2(NULL, g_ScriptRuntime, &terrain);
 		sim2.LoadDefaultScripts();
 		sim2.ResetState();
 
 		CMapReader* mapReader = new CMapReader(); // it'll call "delete this" itself
 
 		LDR_BeginRegistering();
-		mapReader->LoadMap(L"maps/scenarios/Median Oasis 01.pmp", &terrain, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+		mapReader->LoadMap(L"maps/scenarios/Median Oasis 01.pmp", CScriptValRooted(), &terrain, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
 			&sim2, &sim2.GetSimContext(), -1, false);
 		LDR_EndRegistering();
 		TS_ASSERT_OK(LDR_NonprogressiveLoad());
@@ -114,7 +112,7 @@ public:
 		CTerrain terrain;
 		terrain.Initialize(5, NULL);
 
-		CSimulation2 sim2(NULL, &terrain);
+		CSimulation2 sim2(NULL, g_ScriptRuntime, &terrain);
 		sim2.LoadDefaultScripts();
 		sim2.ResetState();
 
