@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2013 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -21,32 +21,40 @@
 #include "scriptinterface/ScriptInterface.h"
 #include "simulation2/MessageTypes.h"
 
-#include "js/jsapi.h"
-
 #define TOJSVAL_SETUP() \
-	JSObject* obj = JS_NewObject(scriptInterface.GetContext(), NULL, NULL, NULL); \
-	if (! obj) \
-		return JSVAL_VOID;
+	JSObject* obj; \
+	{\
+	JSAutoRequest rq(scriptInterface.GetContext()); \
+	obj = JS_NewObject(scriptInterface.GetContext(), NULL, NULL, NULL); \
+	if (!obj) \
+		return JSVAL_VOID; \
+	}
 
 #define SET_MSG_PROPERTY(name) \
 	do { \
-		jsval prop = ScriptInterface::ToJSVal(scriptInterface.GetContext(), this->name); \
-		if (! JS_SetProperty(scriptInterface.GetContext(), obj, #name, &prop)) \
+		JSAutoRequest rq(scriptInterface.GetContext()); \
+		JSContext* cx = scriptInterface.GetContext(); \
+		JS::RootedValue prop(cx);\
+		ScriptInterface::ToJSVal(cx, prop.get(), this->name); \
+		if (! JS_SetProperty(cx, obj, #name, prop.address())) \
 			return JSVAL_VOID; \
 	} while (0);
 
 #define FROMJSVAL_SETUP() \
-	if (! JSVAL_IS_OBJECT(val)) \
+	if ( JSVAL_IS_PRIMITIVE(val)) \
 		return NULL; \
 	JSObject* obj = JSVAL_TO_OBJECT(val); \
-	jsval prop;
+	JS::RootedValue prop(scriptInterface.GetContext());
 
 #define GET_MSG_PROPERTY(type, name) \
-	if (! JS_GetProperty(scriptInterface.GetContext(), obj, #name, &prop)) \
-		return NULL; \
 	type name; \
-	if (! ScriptInterface::FromJSVal(scriptInterface.GetContext(), prop, name)) \
-		return NULL;
+	{ \
+	JSAutoRequest rq(scriptInterface.GetContext()); \
+	if (! JS_GetProperty(scriptInterface.GetContext(), obj, #name, prop.address())) \
+		return NULL; \
+	if (! ScriptInterface::FromJSVal(scriptInterface.GetContext(), prop.get(), name)) \
+		return NULL; \
+	}
 
 jsval CMessage::ToJSValCached(ScriptInterface& scriptInterface) const
 {
@@ -216,6 +224,24 @@ CMessage* CMessagePositionChanged::FromJSVal(ScriptInterface& scriptInterface, j
 
 ////////////////////////////////
 
+jsval CMessageTerritoryPositionChanged::ToJSVal(ScriptInterface& scriptInterface) const
+{
+	TOJSVAL_SETUP();
+	SET_MSG_PROPERTY(entity);
+	SET_MSG_PROPERTY(newTerritory);
+	return OBJECT_TO_JSVAL(obj);
+}
+
+CMessage* CMessageTerritoryPositionChanged::FromJSVal(ScriptInterface& scriptInterface, jsval val)
+{
+	FROMJSVAL_SETUP();
+	GET_MSG_PROPERTY(entity_id_t, entity);
+	GET_MSG_PROPERTY(player_id_t, newTerritory);
+	return new CMessageTerritoryPositionChanged(entity, newTerritory);
+}
+
+////////////////////////////////
+
 jsval CMessageMotionChanged::ToJSVal(ScriptInterface& scriptInterface) const
 {
 	TOJSVAL_SETUP();
@@ -300,20 +326,42 @@ CMessage* CMessagePathResult::FromJSVal(ScriptInterface& UNUSED(scriptInterface)
 
 ////////////////////////////////
 
-jsval CMessageTechnologyModification::ToJSVal(ScriptInterface& scriptInterface) const
+jsval CMessageValueModification::ToJSVal(ScriptInterface& scriptInterface) const
 {
 	TOJSVAL_SETUP();
+	SET_MSG_PROPERTY(entities);
 	SET_MSG_PROPERTY(component);
-	SET_MSG_PROPERTY(player);
+	SET_MSG_PROPERTY(valueNames);
 	return OBJECT_TO_JSVAL(obj);
 }
 
-CMessage* CMessageTechnologyModification::FromJSVal(ScriptInterface& scriptInterface, jsval val)
+CMessage* CMessageValueModification::FromJSVal(ScriptInterface& scriptInterface, jsval val)
 {
 	FROMJSVAL_SETUP();
+	GET_MSG_PROPERTY(std::vector<entity_id_t>, entities);
 	GET_MSG_PROPERTY(std::wstring, component);
+	GET_MSG_PROPERTY(std::vector<std::wstring>, valueNames);
+	return new CMessageValueModification(entities, component, valueNames);
+}
+
+////////////////////////////////
+
+jsval CMessageTemplateModification::ToJSVal(ScriptInterface& scriptInterface) const
+{
+	TOJSVAL_SETUP();
+	SET_MSG_PROPERTY(player);
+	SET_MSG_PROPERTY(component);
+	SET_MSG_PROPERTY(valueNames);
+	return OBJECT_TO_JSVAL(obj);
+}
+
+CMessage* CMessageTemplateModification::FromJSVal(ScriptInterface& scriptInterface, jsval val)
+{
+	FROMJSVAL_SETUP();
 	GET_MSG_PROPERTY(player_id_t, player);
-	return new CMessageTechnologyModification(component, player);
+	GET_MSG_PROPERTY(std::wstring, component);
+	GET_MSG_PROPERTY(std::vector<std::wstring>, valueNames);
+	return new CMessageTemplateModification(player, component, valueNames);
 }
 
 ////////////////////////////////
@@ -336,6 +384,19 @@ CMessage* CMessageVisionRangeChanged::FromJSVal(ScriptInterface& scriptInterface
 	return new CMessageVisionRangeChanged(entity, oldRange, newRange);
 }
 
+////////////////////////////////
+
+jsval CMessageMinimapPing::ToJSVal(ScriptInterface& scriptInterface) const
+{
+	TOJSVAL_SETUP();
+	return OBJECT_TO_JSVAL(obj);
+}
+
+CMessage* CMessageMinimapPing::FromJSVal(ScriptInterface& UNUSED(scriptInterface), jsval UNUSED(val))
+{
+	return new CMessageMinimapPing();
+}
+
 ////////////////////////////////////////////////////////////////
 
 CMessage* CMessageFromJSVal(int mtid, ScriptInterface& scriptingInterface, jsval val)
