@@ -52,9 +52,9 @@ public:
 	CScriptValRooted msg;
 };
 
-CComponentManager::CComponentManager(CSimContext& context, bool skipScriptFunctions) :
+CComponentManager::CComponentManager(CSimContext& context, shared_ptr<ScriptRuntime> rt, bool skipScriptFunctions) :
 	m_NextScriptComponentTypeId(CID__LastNative),
-	m_ScriptInterface("Engine", "Simulation", ScriptInterface::CreateRuntime(128*MiB)),
+	m_ScriptInterface("Engine", "Simulation", rt),
 	m_SimContext(context), m_CurrentlyHotloading(false)
 {
 	context.SetComponentManager(this);
@@ -143,9 +143,11 @@ bool CComponentManager::LoadScript(const VfsPath& filename, bool hotload)
 	return ok;
 }
 
-void CComponentManager::Script_RegisterComponentType(void* cbdata, int iid, std::string cname, CScriptVal ctor)
+void CComponentManager::Script_RegisterComponentType(ScriptInterface::CxPrivate* pCxPrivate, int iid, std::string cname, CScriptVal ctor)
 {
-	CComponentManager* componentManager = static_cast<CComponentManager*> (cbdata);
+	CComponentManager* componentManager = static_cast<CComponentManager*> (pCxPrivate->pCBData);
+	JSContext* cx = componentManager->m_ScriptInterface.GetContext();
+	JSAutoRequest rq(cx);
 
 	// Find the C++ component that wraps the interface
 	int cidWrapper = componentManager->GetScriptWrapper(iid);
@@ -234,7 +236,7 @@ void CComponentManager::Script_RegisterComponentType(void* cbdata, int iid, std:
 		ctWrapper.dealloc,
 		cname,
 		schema,
-		CScriptValRooted(componentManager->m_ScriptInterface.GetContext(), ctor)
+		CScriptValRooted(cx, ctor)
 	};
 	componentManager->m_ComponentTypesById[cid] = ct;
 
@@ -242,13 +244,18 @@ void CComponentManager::Script_RegisterComponentType(void* cbdata, int iid, std:
 
 
 	// Find all the ctor prototype's On* methods, and subscribe to the appropriate messages:
-
-	CScriptVal proto;
-	if (!componentManager->m_ScriptInterface.GetProperty(ctor.get(), "prototype", proto))
+	JS::RootedValue protoVal(cx);
+	if (!componentManager->m_ScriptInterface.GetPropertyJS(ctor.get(), "prototype", &protoVal))
 		return; // error
 
 	std::vector<std::string> methods;
-	if (!componentManager->m_ScriptInterface.EnumeratePropertyNamesWithPrefix(proto.get(), "On", methods))
+	JS::RootedObject proto(cx);
+	if (!protoVal.isObjectOrNull())
+		return; // error
+
+	proto = protoVal.toObjectOrNull();
+	
+	if (!componentManager->m_ScriptInterface.EnumeratePropertyNamesWithPrefix(protoVal, "On", methods))
 		return; // error
 
 	for (std::vector<std::string>::const_iterator it = methods.begin(); it != methods.end(); ++it)
@@ -287,16 +294,18 @@ void CComponentManager::Script_RegisterComponentType(void* cbdata, int iid, std:
 		std::map<entity_id_t, IComponent*>::const_iterator eit = comps.begin();
 		for (; eit != comps.end(); ++eit)
 		{
-			jsval instance = eit->second->GetJSInstance();
-			if (!JSVAL_IS_NULL(instance))
-				componentManager->m_ScriptInterface.SetPrototype(instance, proto.get());
+			JS::RootedValue instance(cx, eit->second->GetJSInstance());
+			if (!instance.isNull())
+			{
+				componentManager->m_ScriptInterface.SetPrototype(instance, protoVal);
+			}
 		}
 	}
 }
 
-void CComponentManager::Script_RegisterInterface(void* cbdata, std::string name)
+void CComponentManager::Script_RegisterInterface(ScriptInterface::CxPrivate* pCxPrivate, std::string name)
 {
-	CComponentManager* componentManager = static_cast<CComponentManager*> (cbdata);
+	CComponentManager* componentManager = static_cast<CComponentManager*> (pCxPrivate->pCBData);
 
 	std::map<std::string, InterfaceId>::iterator it = componentManager->m_InterfaceIdsByName.find(name);
 	if (it != componentManager->m_InterfaceIdsByName.end())
@@ -315,9 +324,9 @@ void CComponentManager::Script_RegisterInterface(void* cbdata, std::string name)
 	componentManager->m_ScriptInterface.SetGlobal(("IID_" + name).c_str(), (int)id);
 }
 
-void CComponentManager::Script_RegisterMessageType(void* cbdata, std::string name)
+void CComponentManager::Script_RegisterMessageType(ScriptInterface::CxPrivate* pCxPrivate, std::string name)
 {
-	CComponentManager* componentManager = static_cast<CComponentManager*> (cbdata);
+	CComponentManager* componentManager = static_cast<CComponentManager*> (pCxPrivate->pCBData);
 
 	std::map<std::string, MessageTypeId>::iterator it = componentManager->m_MessageTypeIdsByName.find(name);
 	if (it != componentManager->m_MessageTypeIdsByName.end())
@@ -335,37 +344,38 @@ void CComponentManager::Script_RegisterMessageType(void* cbdata, std::string nam
 	componentManager->m_ScriptInterface.SetGlobal(("MT_" + name).c_str(), (int)id);
 }
 
-void CComponentManager::Script_RegisterGlobal(void* cbdata, std::string name, CScriptVal value)
+void CComponentManager::Script_RegisterGlobal(ScriptInterface::CxPrivate* pCxPrivate, std::string name, CScriptVal value)
 {
-	CComponentManager* componentManager = static_cast<CComponentManager*> (cbdata);
+	CComponentManager* componentManager = static_cast<CComponentManager*> (pCxPrivate->pCBData);
 
 	// Set the value, and accept duplicates only if hotloading (otherwise it's an error,
 	// in order to detect accidental duplicate definitions of globals)
 	componentManager->m_ScriptInterface.SetGlobal(name.c_str(), value, componentManager->m_CurrentlyHotloading);
 }
 
-IComponent* CComponentManager::Script_QueryInterface(void* cbdata, int ent, int iid)
+IComponent* CComponentManager::Script_QueryInterface(ScriptInterface::CxPrivate* pCxPrivate, int ent, int iid)
 {
-	CComponentManager* componentManager = static_cast<CComponentManager*> (cbdata);
+	CComponentManager* componentManager = static_cast<CComponentManager*> (pCxPrivate->pCBData);
 	IComponent* component = componentManager->QueryInterface((entity_id_t)ent, iid);
 	return component;
 }
 
-std::vector<int> CComponentManager::Script_GetEntitiesWithInterface(void* cbdata, int iid)
+std::vector<int> CComponentManager::Script_GetEntitiesWithInterface(ScriptInterface::CxPrivate* pCxPrivate, int iid)
 {
-	CComponentManager* componentManager = static_cast<CComponentManager*> (cbdata);
+	CComponentManager* componentManager = static_cast<CComponentManager*> (pCxPrivate->pCBData);
 
 	std::vector<int> ret;
 	const InterfaceListUnordered& ents = componentManager->GetEntitiesWithInterfaceUnordered(iid);
 	for (InterfaceListUnordered::const_iterator it = ents.begin(); it != ents.end(); ++it)
-		ret.push_back(it->first); // TODO: maybe we should exclude local entities
+		if (!ENTITY_IS_LOCAL(it->first))
+			ret.push_back(it->first);
 	std::sort(ret.begin(), ret.end());
 	return ret;
 }
 
-std::vector<IComponent*> CComponentManager::Script_GetComponentsWithInterface(void* cbdata, int iid)
+std::vector<IComponent*> CComponentManager::Script_GetComponentsWithInterface(ScriptInterface::CxPrivate* pCxPrivate, int iid)
 {
-	CComponentManager* componentManager = static_cast<CComponentManager*> (cbdata);
+	CComponentManager* componentManager = static_cast<CComponentManager*> (pCxPrivate->pCBData);
 
 	std::vector<IComponent*> ret;
 	InterfaceList ents = componentManager->GetEntitiesWithInterface(iid);
@@ -390,9 +400,9 @@ CMessage* CComponentManager::ConstructMessage(int mtid, CScriptVal data)
 	}
 }
 
-void CComponentManager::Script_PostMessage(void* cbdata, int ent, int mtid, CScriptVal data)
+void CComponentManager::Script_PostMessage(ScriptInterface::CxPrivate* pCxPrivate, int ent, int mtid, CScriptVal data)
 {
-	CComponentManager* componentManager = static_cast<CComponentManager*> (cbdata);
+	CComponentManager* componentManager = static_cast<CComponentManager*> (pCxPrivate->pCBData);
 
 	CMessage* msg = componentManager->ConstructMessage(mtid, data);
 	if (!msg)
@@ -403,9 +413,9 @@ void CComponentManager::Script_PostMessage(void* cbdata, int ent, int mtid, CScr
 	delete msg;
 }
 
-void CComponentManager::Script_BroadcastMessage(void* cbdata, int mtid, CScriptVal data)
+void CComponentManager::Script_BroadcastMessage(ScriptInterface::CxPrivate* pCxPrivate, int mtid, CScriptVal data)
 {
-	CComponentManager* componentManager = static_cast<CComponentManager*> (cbdata);
+	CComponentManager* componentManager = static_cast<CComponentManager*> (pCxPrivate->pCBData);
 
 	CMessage* msg = componentManager->ConstructMessage(mtid, data);
 	if (!msg)
@@ -416,9 +426,9 @@ void CComponentManager::Script_BroadcastMessage(void* cbdata, int mtid, CScriptV
 	delete msg;
 }
 
-int CComponentManager::Script_AddEntity(void* cbdata, std::string templateName)
+int CComponentManager::Script_AddEntity(ScriptInterface::CxPrivate* pCxPrivate, std::string templateName)
 {
-	CComponentManager* componentManager = static_cast<CComponentManager*> (cbdata);
+	CComponentManager* componentManager = static_cast<CComponentManager*> (pCxPrivate->pCBData);
 
 	std::wstring name(templateName.begin(), templateName.end());
 	// TODO: should validate the string to make sure it doesn't contain scary characters
@@ -428,9 +438,9 @@ int CComponentManager::Script_AddEntity(void* cbdata, std::string templateName)
 	return (int)ent;
 }
 
-int CComponentManager::Script_AddLocalEntity(void* cbdata, std::string templateName)
+int CComponentManager::Script_AddLocalEntity(ScriptInterface::CxPrivate* pCxPrivate, std::string templateName)
 {
-	CComponentManager* componentManager = static_cast<CComponentManager*> (cbdata);
+	CComponentManager* componentManager = static_cast<CComponentManager*> (pCxPrivate->pCBData);
 
 	std::wstring name(templateName.begin(), templateName.end());
 	// TODO: should validate the string to make sure it doesn't contain scary characters
@@ -440,9 +450,9 @@ int CComponentManager::Script_AddLocalEntity(void* cbdata, std::string templateN
 	return (int)ent;
 }
 
-void CComponentManager::Script_DestroyEntity(void* cbdata, int ent)
+void CComponentManager::Script_DestroyEntity(ScriptInterface::CxPrivate* pCxPrivate, int ent)
 {
-	CComponentManager* componentManager = static_cast<CComponentManager*> (cbdata);
+	CComponentManager* componentManager = static_cast<CComponentManager*> (pCxPrivate->pCBData);
 
 	componentManager->DestroyComponentsSoon(ent);
 }
@@ -467,6 +477,13 @@ void CComponentManager::ResetState()
 
 	m_ComponentsByTypeId.clear();
 
+	// Delete all SEntityComponentCaches
+	std::map<entity_id_t, SEntityComponentCache*>::iterator ccit = m_ComponentCaches.begin();
+	for (; ccit != m_ComponentCaches.end(); ++ccit)
+		free(ccit->second);
+	m_ComponentCaches.clear();
+	m_SystemEntity = CEntityHandle();
+
 	m_DestructionQueue.clear();
 
 	// Reset IDs
@@ -572,7 +589,7 @@ entity_id_t CComponentManager::AllocateNewEntity(entity_id_t preferredId)
 	return id;
 }
 
-bool CComponentManager::AddComponent(entity_id_t ent, ComponentTypeId cid, const CParamNode& paramNode)
+bool CComponentManager::AddComponent(CEntityHandle ent, ComponentTypeId cid, const CParamNode& paramNode)
 {
 	IComponent* component = ConstructComponent(ent, cid);
 	if (!component)
@@ -582,7 +599,7 @@ bool CComponentManager::AddComponent(entity_id_t ent, ComponentTypeId cid, const
 	return true;
 }
 
-IComponent* CComponentManager::ConstructComponent(entity_id_t ent, ComponentTypeId cid)
+IComponent* CComponentManager::ConstructComponent(CEntityHandle ent, ComponentTypeId cid)
 {
 	std::map<ComponentTypeId, ComponentType>::const_iterator it = m_ComponentTypesById.find(cid);
 	if (it == m_ComponentTypesById.end())
@@ -596,7 +613,7 @@ IComponent* CComponentManager::ConstructComponent(entity_id_t ent, ComponentType
 	ENSURE((size_t)ct.iid < m_ComponentsByInterface.size());
 
 	boost::unordered_map<entity_id_t, IComponent*>& emap1 = m_ComponentsByInterface[ct.iid];
-	if (emap1.find(ent) != emap1.end())
+	if (emap1.find(ent.GetId()) != emap1.end())
 	{
 		LOGERROR(L"Multiple components for interface %d", ct.iid);
 		return NULL;
@@ -608,7 +625,7 @@ IComponent* CComponentManager::ConstructComponent(entity_id_t ent, ComponentType
 	jsval obj = JSVAL_NULL;
 	if (ct.type == CT_Script)
 	{
-		obj = m_ScriptInterface.CallConstructor(ct.ctor.get(), JSVAL_VOID);
+		obj = m_ScriptInterface.CallConstructor(ct.ctor.get(), 0, JSVAL_VOID);
 		if (JSVAL_IS_VOID(obj))
 		{
 			LOGERROR(L"Script component constructor failed");
@@ -620,30 +637,75 @@ IComponent* CComponentManager::ConstructComponent(entity_id_t ent, ComponentType
 	IComponent* component = ct.alloc(m_ScriptInterface, obj);
 	ENSURE(component);
 
-	component->SetEntityId(ent);
+	component->SetEntityHandle(ent);
 	component->SetSimContext(m_SimContext);
 
 	// Store a reference to the new component
-	emap1.insert(std::make_pair(ent, component));
-	emap2.insert(std::make_pair(ent, component));
+	emap1.insert(std::make_pair(ent.GetId(), component));
+	emap2.insert(std::make_pair(ent.GetId(), component));
 	// TODO: We need to more careful about this - if an entity is constructed by a component
 	// while we're iterating over all components, this will invalidate the iterators and everything
 	// will break.
 	// We probably need some kind of delayed addition, so they get pushed onto a queue and then
 	// inserted into the world later on. (Be careful about immediation deletion in that case, too.)
 
+	SEntityComponentCache* cache = ent.GetComponentCache();
+	ENSURE(cache != NULL && ct.iid < (int)cache->numInterfaces && cache->interfaces[ct.iid] == NULL);
+	cache->interfaces[ct.iid] = component;
+
 	return component;
 }
 
-void CComponentManager::AddMockComponent(entity_id_t ent, InterfaceId iid, IComponent& component)
+void CComponentManager::AddMockComponent(CEntityHandle ent, InterfaceId iid, IComponent& component)
 {
 	// Just add it into the by-interface map, not the by-component-type map,
 	// so it won't be considered for messages or deletion etc
 
 	boost::unordered_map<entity_id_t, IComponent*>& emap1 = m_ComponentsByInterface.at(iid);
-	if (emap1.find(ent) != emap1.end())
+	if (emap1.find(ent.GetId()) != emap1.end())
 		debug_warn(L"Multiple components for interface");
-	emap1.insert(std::make_pair(ent, &component));
+	emap1.insert(std::make_pair(ent.GetId(), &component));
+
+	SEntityComponentCache* cache = ent.GetComponentCache();
+	ENSURE(cache != NULL && iid < (int)cache->numInterfaces && cache->interfaces[iid] == NULL);
+	cache->interfaces[iid] = &component;
+}
+
+CEntityHandle CComponentManager::AllocateEntityHandle(entity_id_t ent)
+{
+	// Interface IDs start at 1, and SEntityComponentCache is defined with a 1-sized array,
+	// so we need space for an extra m_InterfaceIdsByName.size() items
+	SEntityComponentCache* cache = (SEntityComponentCache*)calloc(1,
+		sizeof(SEntityComponentCache) + sizeof(IComponent*) * m_InterfaceIdsByName.size());
+	ENSURE(cache != NULL);
+	cache->numInterfaces = m_InterfaceIdsByName.size() + 1;
+
+	ENSURE(m_ComponentCaches.find(ent) == m_ComponentCaches.end());
+	m_ComponentCaches[ent] = cache;
+
+	return CEntityHandle(ent, cache);
+}
+
+CEntityHandle CComponentManager::LookupEntityHandle(entity_id_t ent, bool allowCreate)
+{
+	std::map<entity_id_t, SEntityComponentCache*>::iterator it;
+	it = m_ComponentCaches.find(ent);
+	if (it == m_ComponentCaches.end())
+	{
+		if (allowCreate)
+			return AllocateEntityHandle(ent);
+		else
+			return CEntityHandle(ent, NULL);
+	}
+	else
+		return CEntityHandle(ent, it->second);
+}
+
+void CComponentManager::InitSystemEntity()
+{
+	ENSURE(m_SystemEntity.GetId() == INVALID_ENTITY);
+	m_SystemEntity = AllocateEntityHandle(SYSTEM_ENTITY);
+	m_SimContext.SetSystemEntity(m_SystemEntity);
 }
 
 entity_id_t CComponentManager::AddEntity(const std::wstring& templateName, entity_id_t ent)
@@ -661,6 +723,8 @@ entity_id_t CComponentManager::AddEntity(const std::wstring& templateName, entit
 	if (!tmpl)
 		return INVALID_ENTITY; // LoadTemplate will have reported the error
 
+	CEntityHandle handle = AllocateEntityHandle(ent);
+
 	// Construct a component for each child of the root element
 	const CParamNode::ChildrenMap& tmplChilds = tmpl->GetChildren();
 	for (CParamNode::ChildrenMap::const_iterator it = tmplChilds.begin(); it != tmplChilds.end(); ++it)
@@ -676,12 +740,11 @@ entity_id_t CComponentManager::AddEntity(const std::wstring& templateName, entit
 			return INVALID_ENTITY;
 		}
 
-		if (!AddComponent(ent, cid, it->second))
+		if (!AddComponent(handle, cid, it->second))
 		{
 			LOGERROR(L"Failed to construct component type name '%hs' in entity template '%ls'", it->first.c_str(), templateName.c_str());
 			return INVALID_ENTITY;
 		}
-
 		// TODO: maybe we should delete already-constructed components if one of them fails?
 	}
 
@@ -698,7 +761,7 @@ void CComponentManager::DestroyComponentsSoon(entity_id_t ent)
 
 void CComponentManager::FlushDestroyedComponents()
 {
-	while(!m_DestructionQueue.empty())
+	while (!m_DestructionQueue.empty())
 	{
 		// Make a copy of the destruction queue, so that the iterators won't be invalidated if the
 		// CMessageDestroy handlers try to destroy more entities themselves
@@ -708,6 +771,7 @@ void CComponentManager::FlushDestroyedComponents()
 		for (std::vector<entity_id_t>::iterator it = queue.begin(); it != queue.end(); ++it)
 		{
 			entity_id_t ent = *it;
+			CEntityHandle handle = LookupEntityHandle(ent);
 
 			CMessageDestroy msg(ent);
 			PostMessage(ent, msg);
@@ -722,9 +786,13 @@ void CComponentManager::FlushDestroyedComponents()
 					eit->second->Deinit();
 					m_ComponentTypesById[iit->first].dealloc(eit->second);
 					iit->second.erase(ent);
+					handle.GetComponentCache()->interfaces[m_ComponentTypesById[iit->first].iid] = NULL;
 				}
 			}
 
+			free(handle.GetComponentCache());
+			m_ComponentCaches.erase(ent);
+
 			// Remove from m_ComponentsByInterface
 			std::vector<boost::unordered_map<entity_id_t, IComponent*> >::iterator ifcit = m_ComponentsByInterface.begin();
 			for (; ifcit != m_ComponentsByInterface.end(); ++ifcit)
@@ -942,26 +1010,26 @@ std::string CComponentManager::GenerateSchema()
 	return schema;
 }
 
-CScriptVal CComponentManager::Script_ReadJSONFile(void* cbdata, std::wstring fileName)
+CScriptVal CComponentManager::Script_ReadJSONFile(ScriptInterface::CxPrivate* pCxPrivate, std::wstring fileName)
 {
-	return ReadJSONFile(cbdata, L"simulation/data", fileName);
+	return ReadJSONFile(pCxPrivate, L"simulation/data", fileName);
 }
 
-CScriptVal CComponentManager::Script_ReadCivJSONFile(void* cbdata, std::wstring fileName)
+CScriptVal CComponentManager::Script_ReadCivJSONFile(ScriptInterface::CxPrivate* pCxPrivate, std::wstring fileName)
 {
-	return ReadJSONFile(cbdata, L"civs", fileName);
+	return ReadJSONFile(pCxPrivate, L"civs", fileName);
 }
 
-CScriptVal CComponentManager::ReadJSONFile(void* cbdata, std::wstring filePath, std::wstring fileName)
+CScriptVal CComponentManager::ReadJSONFile(ScriptInterface::CxPrivate* pCxPrivate, std::wstring filePath, std::wstring fileName)
 {
-	CComponentManager* componentManager = static_cast<CComponentManager*> (cbdata);
+	CComponentManager* componentManager = static_cast<CComponentManager*> (pCxPrivate->pCBData);
 
 	VfsPath path = VfsPath(filePath) / fileName;
 
 	return componentManager->GetScriptInterface().ReadJSONFile(path).get();
 }
 	
-Status CComponentManager::FindJSONFilesCallback(const VfsPath& pathname, const FileInfo& UNUSED(fileInfo), const uintptr_t cbData)
+Status CComponentManager::FindJSONFilesCallback(const VfsPath& pathname, const CFileInfo& UNUSED(fileInfo), const uintptr_t cbData)
 {
 	FindJSONFilesCallbackData* data = (FindJSONFilesCallbackData*)cbData;
 	
@@ -974,7 +1042,7 @@ Status CComponentManager::FindJSONFilesCallback(const VfsPath& pathname, const F
 	return INFO::OK;
 }
 
-std::vector<std::string> CComponentManager::Script_FindJSONFiles(void* UNUSED(cbdata), std::wstring subPath, bool recursive)
+std::vector<std::string> CComponentManager::Script_FindJSONFiles(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), std::wstring subPath, bool recursive)
 {
 	FindJSONFilesCallbackData cbData;
 	cbData.path = VfsPath(L"simulation/data/" + subPath + L"/");
