@@ -52,13 +52,14 @@ public:
 	void test_LoadTemplate()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 
 		entity_id_t ent1 = 1, ent2 = 2;
+		CEntityHandle hnd1 = man.LookupEntityHandle(ent1, true);
 		CParamNode noParam;
 
-		TS_ASSERT(man.AddComponent(ent1, CID_TemplateManager, noParam));
+		TS_ASSERT(man.AddComponent(hnd1, CID_TemplateManager, noParam));
 
 		ICmpTemplateManager* tempMan = static_cast<ICmpTemplateManager*> (man.QueryInterface(ent1, IID_TemplateManager));
 		TS_ASSERT(tempMan != NULL);
@@ -79,7 +80,7 @@ public:
 		TS_ASSERT(actor != NULL);
 		TS_ASSERT_WSTR_EQUALS(actor->ToXML(),
 				L"<Footprint><Circle radius=\"2.0\"></Circle><Height>1.0</Height></Footprint><Selectable><EditorOnly></EditorOnly><Overlay><Texture><MainTexture>actor.png</MainTexture><MainTextureMask>actor_mask.png</MainTextureMask></Texture></Overlay></Selectable>"
-				L"<VisualActor><Actor>example1</Actor><SilhouetteDisplay>false</SilhouetteDisplay><SilhouetteOccluder>false</SilhouetteOccluder><VisibleInAtlasOnly>false</VisibleInAtlasOnly></VisualActor>");
+				L"<VisualActor><Actor>example1</Actor><ActorOnly></ActorOnly><SilhouetteDisplay>false</SilhouetteDisplay><SilhouetteOccluder>false</SilhouetteOccluder><VisibleInAtlasOnly>false</VisibleInAtlasOnly></VisualActor>");
 
 		const CParamNode* preview = tempMan->LoadTemplate(ent2, "preview|unit", -1);
 		TS_ASSERT(preview != NULL);
@@ -107,56 +108,59 @@ public:
 		TS_ASSERT_WSTR_EQUALS(previewactor->ToXML(),
 				// the actor's <Selectable> element is not part of the preview element subset, hence not included
 				L"<Footprint><Circle radius=\"2.0\"></Circle><Height>1.0</Height></Footprint><Vision><AlwaysVisible>true</AlwaysVisible><Range>0</Range><RetainInFog>false</RetainInFog></Vision>"
-				L"<VisualActor><Actor>example2</Actor><DisableShadows></DisableShadows><SilhouetteDisplay>false</SilhouetteDisplay><SilhouetteOccluder>false</SilhouetteOccluder><VisibleInAtlasOnly>false</VisibleInAtlasOnly></VisualActor>");
+				L"<VisualActor><Actor>example2</Actor><ActorOnly></ActorOnly><DisableShadows></DisableShadows><SilhouetteDisplay>false</SilhouetteDisplay><SilhouetteOccluder>false</SilhouetteOccluder><VisibleInAtlasOnly>false</VisibleInAtlasOnly></VisualActor>");
 	}
 
 	void test_LoadTemplate_scriptcache()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 
 		entity_id_t ent1 = 1, ent2 = 2;
+		CEntityHandle hnd1 = man.LookupEntityHandle(ent1, true);
 		CParamNode noParam;
 
-		TS_ASSERT(man.AddComponent(ent1, CID_TemplateManager, noParam));
+		TS_ASSERT(man.AddComponent(hnd1, CID_TemplateManager, noParam));
 
 		ICmpTemplateManager* tempMan = static_cast<ICmpTemplateManager*> (man.QueryInterface(ent1, IID_TemplateManager));
 		TS_ASSERT(tempMan != NULL);
 		tempMan->DisableValidation();
 
-		CScriptValRooted val;
 		JSContext* cx = man.GetScriptInterface().GetContext();
+		JSAutoRequest rq(cx);
 
 		// This is testing some bugs in the template JS object caching
 
 		const CParamNode* inherit1 = tempMan->LoadTemplate(ent2, "inherit1", -1);
-		val = CScriptValRooted(cx, ScriptInterface::ToJSVal(cx, inherit1));
+		JS::RootedValue val(cx);
+		ScriptInterface::ToJSVal(cx, val.get(), inherit1);
 		TS_ASSERT_WSTR_EQUALS(man.GetScriptInterface().ToString(val.get()), L"({Test1A:{'@a':\"a1\", '@b':\"b1\", '@c':\"c1\", d:\"d1\", e:\"e1\", f:\"f1\"}})");
 
 		const CParamNode* inherit2 = tempMan->LoadTemplate(ent2, "inherit2", -1);
-		val = CScriptValRooted(cx, ScriptInterface::ToJSVal(cx, inherit2));
+		ScriptInterface::ToJSVal(cx, val.get(), inherit2);
 		TS_ASSERT_WSTR_EQUALS(man.GetScriptInterface().ToString(val.get()), L"({'@parent':\"inherit1\", Test1A:{'@a':\"a2\", '@b':\"b1\", '@c':\"c1\", d:\"d2\", e:\"e1\", f:\"f1\", g:\"g2\"}})");
 
 		const CParamNode* actor = tempMan->LoadTemplate(ent2, "actor|example1", -1);
-		val = CScriptValRooted(cx, ScriptInterface::ToJSVal(cx, &actor->GetChild("VisualActor")));
-		TS_ASSERT_WSTR_EQUALS(man.GetScriptInterface().ToString(val.get()), L"({Actor:\"example1\", SilhouetteDisplay:\"false\", SilhouetteOccluder:\"false\", VisibleInAtlasOnly:\"false\"})");
+		ScriptInterface::ToJSVal(cx, val.get(), &actor->GetChild("VisualActor"));
+		TS_ASSERT_WSTR_EQUALS(man.GetScriptInterface().ToString(val.get()), L"({Actor:\"example1\", ActorOnly:(void 0), SilhouetteDisplay:\"false\", SilhouetteOccluder:\"false\", VisibleInAtlasOnly:\"false\"})");
 
 		const CParamNode* foundation = tempMan->LoadTemplate(ent2, "foundation|actor|example1", -1);
-		val = CScriptValRooted(cx, ScriptInterface::ToJSVal(cx, &foundation->GetChild("VisualActor")));
-		TS_ASSERT_WSTR_EQUALS(man.GetScriptInterface().ToString(val.get()), L"({Actor:\"example1\", Foundation:(void 0), SilhouetteDisplay:\"false\", SilhouetteOccluder:\"false\", VisibleInAtlasOnly:\"false\"})");
+		ScriptInterface::ToJSVal(cx, val.get(), &foundation->GetChild("VisualActor"));
+		TS_ASSERT_WSTR_EQUALS(man.GetScriptInterface().ToString(val.get()), L"({Actor:\"example1\", ActorOnly:(void 0), Foundation:(void 0), SilhouetteDisplay:\"false\", SilhouetteOccluder:\"false\", VisibleInAtlasOnly:\"false\"})");
 	}
 
 	void test_LoadTemplate_errors()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 
 		entity_id_t ent1 = 1, ent2 = 2;
+		CEntityHandle hnd1 = man.LookupEntityHandle(ent1, true);
 		CParamNode noParam;
 
-		TS_ASSERT(man.AddComponent(ent1, CID_TemplateManager, noParam));
+		TS_ASSERT(man.AddComponent(hnd1, CID_TemplateManager, noParam));
 
 		ICmpTemplateManager* tempMan = static_cast<ICmpTemplateManager*> (man.QueryInterface(ent1, IID_TemplateManager));
 		TS_ASSERT(tempMan != NULL);
@@ -181,13 +185,14 @@ public:
 	void test_LoadTemplate_multiple()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 
 		entity_id_t ent1 = 1, ent2 = 2;
+		CEntityHandle hnd1 = man.LookupEntityHandle(ent1, true);
 		CParamNode noParam;
 
-		TS_ASSERT(man.AddComponent(ent1, CID_TemplateManager, noParam));
+		TS_ASSERT(man.AddComponent(hnd1, CID_TemplateManager, noParam));
 
 		ICmpTemplateManager* tempMan = static_cast<ICmpTemplateManager*> (man.QueryInterface(ent1, IID_TemplateManager));
 		TS_ASSERT(tempMan != NULL);
@@ -239,7 +244,7 @@ public:
 	void test_load_all_DISABLED() // disabled since it's a bit slow and noisy
 	{
 		CTerrain dummy;
-		CSimulation2 sim(NULL, &dummy);
+		CSimulation2 sim(NULL, g_ScriptRuntime, &dummy);
 		sim.LoadDefaultScripts();
 		sim.ResetState();
 
