@@ -59,14 +59,14 @@ public:
 	void test_Load()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 	}
 
 	void test_LookupCID()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 
 		TS_ASSERT_EQUALS(man.LookupCID("Test1A"), (int)CID_Test1A);
@@ -76,7 +76,7 @@ public:
 	void test_AllocateNewEntity()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 
 		TS_ASSERT_EQUALS(man.AllocateNewEntity(), (u32)2);
 		TS_ASSERT_EQUALS(man.AllocateNewEntity(), (u32)3);
@@ -99,21 +99,22 @@ public:
 	void test_AddComponent_errors()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
+		CEntityHandle hnd1 = man.AllocateEntityHandle(1);
 
 		CParamNode noParam;
-		TS_ASSERT(man.AddComponent(1, CID_Test1A, noParam));
+		TS_ASSERT(man.AddComponent(hnd1, CID_Test1A, noParam));
 
 		{
 			TestLogger log;
-			TS_ASSERT(! man.AddComponent(1, 12345, noParam));
+			TS_ASSERT(! man.AddComponent(hnd1, 12345, noParam));
 			TS_ASSERT_WSTR_CONTAINS(log.GetOutput(), L"ERROR: Invalid component id 12345");
 		}
 
 		{
 			TestLogger log;
-			TS_ASSERT(! man.AddComponent(1, CID_Test1B, noParam));
+			TS_ASSERT(! man.AddComponent(hnd1, CID_Test1B, noParam));
 			TS_ASSERT_WSTR_CONTAINS(log.GetOutput(), L"ERROR: Multiple components for interface ");
 		}
 	}
@@ -121,22 +122,24 @@ public:
 	void test_QueryInterface()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 
 		entity_id_t ent1 = 1, ent2 = 2;
+		CEntityHandle hnd1 = man.AllocateEntityHandle(ent1);
+		CEntityHandle hnd2 = man.AllocateEntityHandle(ent2);
 		CParamNode noParam;
 
-		man.AddComponent(ent1, CID_Test1A, noParam);
+		man.AddComponent(hnd1, CID_Test1A, noParam);
 		TS_ASSERT(man.QueryInterface(ent1, IID_Test1) != NULL);
 		TS_ASSERT(man.QueryInterface(ent1, IID_Test2) == NULL);
 		TS_ASSERT(man.QueryInterface(ent2, IID_Test1) == NULL);
 		TS_ASSERT(man.QueryInterface(ent2, IID_Test2) == NULL);
 
-		man.AddComponent(ent2, CID_Test1B, noParam);
+		man.AddComponent(hnd2, CID_Test1B, noParam);
 		TS_ASSERT(man.QueryInterface(ent2, IID_Test1) != NULL);
 		TS_ASSERT(man.QueryInterface(ent2, IID_Test2) == NULL);
-		man.AddComponent(ent2, CID_Test2A, noParam);
+		man.AddComponent(hnd2, CID_Test2A, noParam);
 		TS_ASSERT(man.QueryInterface(ent2, IID_Test1) != NULL);
 		TS_ASSERT(man.QueryInterface(ent2, IID_Test2) != NULL);
 	}
@@ -144,17 +147,21 @@ public:
 	void test_SendMessage()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 
 		entity_id_t ent1 = 1, ent2 = 2, ent3 = 3, ent4 = 4;
+		CEntityHandle hnd1 = man.AllocateEntityHandle(ent1);
+		CEntityHandle hnd2 = man.AllocateEntityHandle(ent2);
+		CEntityHandle hnd3 = man.AllocateEntityHandle(ent3);
+		CEntityHandle hnd4 = man.AllocateEntityHandle(ent4);
 		CParamNode noParam;
 
-		man.AddComponent(ent1, CID_Test1A, noParam);
-		man.AddComponent(ent2, CID_Test1B, noParam);
-		man.AddComponent(ent3, CID_Test2A, noParam);
-		man.AddComponent(ent4, CID_Test1A, noParam);
-		man.AddComponent(ent4, CID_Test2A, noParam);
+		man.AddComponent(hnd1, CID_Test1A, noParam);
+		man.AddComponent(hnd2, CID_Test1B, noParam);
+		man.AddComponent(hnd3, CID_Test2A, noParam);
+		man.AddComponent(hnd4, CID_Test1A, noParam);
+		man.AddComponent(hnd4, CID_Test2A, noParam);
 
 		CMessageTurnStart msg1;
 		CMessageUpdate msg2(fixed::FromInt(100));
@@ -214,17 +221,19 @@ public:
 	void test_ParamNode()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 
 		entity_id_t ent1 = 1, ent2 = 2;
+		CEntityHandle hnd1 = man.AllocateEntityHandle(ent1);
+		CEntityHandle hnd2 = man.AllocateEntityHandle(ent2);
 		CParamNode noParam;
 
 		CParamNode testParam;
 		TS_ASSERT_EQUALS(CParamNode::LoadXMLString(testParam, "<x>1234</x>"), PSRETURN_OK);
 
-		man.AddComponent(ent1, CID_Test1A, noParam);
-		man.AddComponent(ent2, CID_Test1A, testParam);
+		man.AddComponent(hnd1, CID_Test1A, noParam);
+		man.AddComponent(hnd2, CID_Test1A, testParam);
 
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent1, IID_Test1))->GetX(), 11000);
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent2, IID_Test1))->GetX(), 1234);
@@ -233,7 +242,7 @@ public:
 	void test_script_basic()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test.js"));
 
@@ -241,12 +250,15 @@ public:
 		TS_ASSERT_EQUALS(man.LookupCID("TestScript1B"), (int)CID__LastNative+1);
 
 		entity_id_t ent1 = 1, ent2 = 2, ent3 = 3;
+		CEntityHandle hnd1 = man.AllocateEntityHandle(ent1);
+		CEntityHandle hnd2 = man.AllocateEntityHandle(ent2);
+		CEntityHandle hnd3 = man.AllocateEntityHandle(ent3);
 		CParamNode noParam;
 
-		man.AddComponent(ent1, CID_Test1A, noParam);
-		man.AddComponent(ent2, man.LookupCID("TestScript1A"), noParam);
-		man.AddComponent(ent3, man.LookupCID("TestScript1B"), noParam);
-		man.AddComponent(ent3, man.LookupCID("TestScript2A"), noParam);
+		man.AddComponent(hnd1, CID_Test1A, noParam);
+		man.AddComponent(hnd2, man.LookupCID("TestScript1A"), noParam);
+		man.AddComponent(hnd3, man.LookupCID("TestScript1B"), noParam);
+		man.AddComponent(hnd3, man.LookupCID("TestScript2A"), noParam);
 
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent1, IID_Test1))->GetX(), 11000);
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent2, IID_Test1))->GetX(), 101000);
@@ -274,15 +286,16 @@ public:
 	void test_script_helper_basic()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-helper.js"));
 		TS_ASSERT(man.LoadScript(L"simulation/helpers/test-helper.js"));
 
 		entity_id_t ent1 = 1;
+		CEntityHandle hnd1 = man.AllocateEntityHandle(ent1);
 		CParamNode noParam;
 
-		man.AddComponent(ent1, man.LookupCID("TestScript1_Helper"), noParam);
+		man.AddComponent(hnd1, man.LookupCID("TestScript1_Helper"), noParam);
 
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent1, IID_Test1))->GetX(), 3);
 	}
@@ -290,14 +303,15 @@ public:
 	void test_script_global_helper()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-global-helper.js"));
 
 		entity_id_t ent1 = 1;
+		CEntityHandle hnd1 = man.AllocateEntityHandle(ent1);
 		CParamNode noParam;
 
-		man.AddComponent(ent1, man.LookupCID("TestScript1_GlobalHelper"), noParam);
+		man.AddComponent(hnd1, man.LookupCID("TestScript1_GlobalHelper"), noParam);
 
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent1, IID_Test1))->GetX(), 2);
 	}
@@ -305,16 +319,17 @@ public:
 	void test_script_interface()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/interfaces/test-interface.js"));
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-interface.js"));
 
 		entity_id_t ent1 = 1;
+		CEntityHandle hnd1 = man.AllocateEntityHandle(ent1);
 		CParamNode noParam;
 
-		man.AddComponent(ent1, man.LookupCID("TestScript1_Interface"), noParam);
-		man.AddComponent(ent1, man.LookupCID("TestScript2_Interface"), noParam);
+		man.AddComponent(hnd1, man.LookupCID("TestScript1_Interface"), noParam);
+		man.AddComponent(hnd1, man.LookupCID("TestScript2_Interface"), noParam);
 
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent1, IID_Test1))->GetX(), 1000 + IID__LastNative);
 	}
@@ -322,7 +337,7 @@ public:
 	void test_script_errors()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		ScriptTestSetup(man.m_ScriptInterface);
 		man.LoadComponentTypes();
 
@@ -339,16 +354,18 @@ public:
 	void test_script_entityID()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		ScriptTestSetup(man.m_ScriptInterface);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-entityid.js"));
 
 		entity_id_t ent1 = 1, ent2 = 234;
+		CEntityHandle hnd1 = man.AllocateEntityHandle(ent1);
+		CEntityHandle hnd2 = man.AllocateEntityHandle(ent2);
 		CParamNode noParam;
 
-		man.AddComponent(ent1, man.LookupCID("TestScript1A"), noParam);
-		man.AddComponent(ent2, man.LookupCID("TestScript1A"), noParam);
+		man.AddComponent(hnd1, man.LookupCID("TestScript1A"), noParam);
+		man.AddComponent(hnd2, man.LookupCID("TestScript1A"), noParam);
 
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent1, IID_Test1))->GetX(), (int)ent1);
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent2, IID_Test1))->GetX(), (int)ent2);
@@ -357,17 +374,19 @@ public:
 	void test_script_QueryInterface()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-query.js"));
 
 		entity_id_t ent1 = 1, ent2 = 2;
+		CEntityHandle hnd1 = man.AllocateEntityHandle(ent1);
+		CEntityHandle hnd2 = man.AllocateEntityHandle(ent2);
 		CParamNode noParam;
 
-		man.AddComponent(ent1, man.LookupCID("TestScript1A"), noParam);
-		man.AddComponent(ent1, man.LookupCID("TestScript2A"), noParam);
-		man.AddComponent(ent2, man.LookupCID("TestScript1A"), noParam);
-		man.AddComponent(ent2, CID_Test2A, noParam);
+		man.AddComponent(hnd1, man.LookupCID("TestScript1A"), noParam);
+		man.AddComponent(hnd1, man.LookupCID("TestScript2A"), noParam);
+		man.AddComponent(hnd2, man.LookupCID("TestScript1A"), noParam);
+		man.AddComponent(hnd2, CID_Test2A, noParam);
 
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent1, IID_Test1))->GetX(), 400);
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent2, IID_Test1))->GetX(), 21000);
@@ -376,18 +395,20 @@ public:
 	void test_script_AddEntity()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-addentity.js"));
 		TS_ASSERT(man.LoadScript(L"simulation/components/addentity/test-addentity.js"));
+		man.InitSystemEntity();
 
 		entity_id_t ent1 = man.AllocateNewEntity();
 		entity_id_t ent2 = ent1 + 2;
+		CEntityHandle hnd1 = man.AllocateEntityHandle(ent1);
 		CParamNode noParam;
 
-		TS_ASSERT(man.AddComponent(SYSTEM_ENTITY, CID_TemplateManager, noParam));
+		TS_ASSERT(man.AddComponent(man.GetSystemEntity(), CID_TemplateManager, noParam));
 
-		TS_ASSERT(man.AddComponent(ent1, man.LookupCID("TestScript1_AddEntity"), noParam));
+		TS_ASSERT(man.AddComponent(hnd1, man.LookupCID("TestScript1_AddEntity"), noParam));
 
 		TS_ASSERT(man.QueryInterface(ent2, IID_Test1) == NULL);
 		TS_ASSERT(man.QueryInterface(ent2, IID_Test2) == NULL);
@@ -407,18 +428,20 @@ public:
 	void test_script_AddLocalEntity()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-addentity.js"));
 		TS_ASSERT(man.LoadScript(L"simulation/components/addentity/test-addentity.js"));
+		man.InitSystemEntity();
 
-		entity_id_t ent1 = 1;
+		entity_id_t ent1 = man.AllocateNewEntity();
 		entity_id_t ent2 = man.AllocateNewLocalEntity() + 2;
+		CEntityHandle hnd1 = man.AllocateEntityHandle(ent1);
 		CParamNode noParam;
 
-		TS_ASSERT(man.AddComponent(SYSTEM_ENTITY, CID_TemplateManager, noParam));
+		TS_ASSERT(man.AddComponent(man.GetSystemEntity(), CID_TemplateManager, noParam));
 
-		TS_ASSERT(man.AddComponent(ent1, man.LookupCID("TestScript1_AddLocalEntity"), noParam));
+		TS_ASSERT(man.AddComponent(hnd1, man.LookupCID("TestScript1_AddLocalEntity"), noParam));
 
 		TS_ASSERT(man.QueryInterface(ent2, IID_Test1) == NULL);
 		TS_ASSERT(man.QueryInterface(ent2, IID_Test2) == NULL);
@@ -438,14 +461,15 @@ public:
 	void test_script_DestroyEntity()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-destroyentity.js"));
 
 		entity_id_t ent1 = 10;
+		CEntityHandle hnd1 = man.AllocateEntityHandle(ent1);
 		CParamNode noParam;
 
-		TS_ASSERT(man.AddComponent(ent1, man.LookupCID("TestScript1_DestroyEntity"), noParam));
+		TS_ASSERT(man.AddComponent(hnd1, man.LookupCID("TestScript1_DestroyEntity"), noParam));
 
 		TS_ASSERT(man.QueryInterface(ent1, IID_Test1) != NULL);
 		static_cast<ICmpTest1*> (man.QueryInterface(ent1, IID_Test1))->GetX();
@@ -457,18 +481,21 @@ public:
 	void test_script_messages()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-msg.js"));
 
 		entity_id_t ent1 = 1, ent2 = 2, ent3 = 3;
+		CEntityHandle hnd1 = man.AllocateEntityHandle(ent1);
+		CEntityHandle hnd2 = man.AllocateEntityHandle(ent2);
+		CEntityHandle hnd3 = man.AllocateEntityHandle(ent3);
 		CParamNode noParam;
 
-		man.AddComponent(ent1, man.LookupCID("TestScript1A"), noParam);
-		man.AddComponent(ent1, man.LookupCID("TestScript2A"), noParam);
-		man.AddComponent(ent2, man.LookupCID("TestScript1A"), noParam);
-		man.AddComponent(ent2, CID_Test2A, noParam);
-		man.AddComponent(ent3, man.LookupCID("TestScript1B"), noParam);
+		man.AddComponent(hnd1, man.LookupCID("TestScript1A"), noParam);
+		man.AddComponent(hnd1, man.LookupCID("TestScript2A"), noParam);
+		man.AddComponent(hnd2, man.LookupCID("TestScript1A"), noParam);
+		man.AddComponent(hnd2, CID_Test2A, noParam);
+		man.AddComponent(hnd3, man.LookupCID("TestScript1B"), noParam);
 
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent1, IID_Test1))->GetX(), 100);
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent2, IID_Test1))->GetX(), 100);
@@ -487,18 +514,20 @@ public:
 	void test_script_template()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-param.js"));
 
 		entity_id_t ent1 = 1, ent2 = 2;
+		CEntityHandle hnd1 = man.AllocateEntityHandle(ent1);
+		CEntityHandle hnd2 = man.AllocateEntityHandle(ent2);
 		CParamNode noParam;
 
 		CParamNode testParam;
 		TS_ASSERT_EQUALS(CParamNode::LoadXMLString(testParam, "<node><x>1</x><y>1<z w='100'><a>1000</a></z>0</y></node>"), PSRETURN_OK);
 
-		man.AddComponent(ent1, man.LookupCID("TestScript1_Init"), noParam);
-		man.AddComponent(ent2, man.LookupCID("TestScript1_Init"), testParam.GetChild("node"));
+		man.AddComponent(hnd1, man.LookupCID("TestScript1_Init"), noParam);
+		man.AddComponent(hnd2, man.LookupCID("TestScript1_Init"), testParam.GetChild("node"));
 
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent1, IID_Test1))->GetX(), 100);
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent2, IID_Test1))->GetX(), 1+10+100+1000);
@@ -507,18 +536,20 @@ public:
 	void test_script_template_readonly()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-param.js"));
 
 		entity_id_t ent1 = 1, ent2 = 2;
+		CEntityHandle hnd1 = man.AllocateEntityHandle(ent1);
+		CEntityHandle hnd2 = man.AllocateEntityHandle(ent2);
 		CParamNode noParam;
 
 		CParamNode testParam;
 		TS_ASSERT_EQUALS(CParamNode::LoadXMLString(testParam, "<x>100</x>"), PSRETURN_OK);
 
-		man.AddComponent(ent1, man.LookupCID("TestScript1_readonly"), testParam);
-		man.AddComponent(ent2, man.LookupCID("TestScript1_readonly"), testParam);
+		man.AddComponent(hnd1, man.LookupCID("TestScript1_readonly"), testParam);
+		man.AddComponent(hnd2, man.LookupCID("TestScript1_readonly"), testParam);
 
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent1, IID_Test1))->GetX(), 102);
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent2, IID_Test1))->GetX(), 102);
@@ -527,19 +558,23 @@ public:
 	void test_script_hotload()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-hotload1.js"));
 
 		entity_id_t ent1 = 1, ent2 = 2, ent3 = 3, ent4 = 4;
+		CEntityHandle hnd1 = man.AllocateEntityHandle(ent1);
+		CEntityHandle hnd2 = man.AllocateEntityHandle(ent2);
+		CEntityHandle hnd3 = man.AllocateEntityHandle(ent3);
+		CEntityHandle hnd4 = man.AllocateEntityHandle(ent4);
 
 		CParamNode testParam;
 		TS_ASSERT_EQUALS(CParamNode::LoadXMLString(testParam, "<x>100</x>"), PSRETURN_OK);
 
-		man.AddComponent(ent1, man.LookupCID("HotloadA"), testParam);
-		man.AddComponent(ent2, man.LookupCID("HotloadB"), testParam);
-		man.AddComponent(ent2, man.LookupCID("HotloadC"), testParam);
+		man.AddComponent(hnd1, man.LookupCID("HotloadA"), testParam);
+		man.AddComponent(hnd2, man.LookupCID("HotloadB"), testParam);
+		man.AddComponent(hnd2, man.LookupCID("HotloadC"), testParam);
 
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent1, IID_Test1))->GetX(), 100);
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent2, IID_Test1))->GetX(), 200);
@@ -549,8 +584,8 @@ public:
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent1, IID_Test1))->GetX(), 1000);
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent2, IID_Test1))->GetX(), 200);
 
-		man.AddComponent(ent3, man.LookupCID("HotloadA"), testParam);
-		man.AddComponent(ent4, man.LookupCID("HotloadB"), testParam);
+		man.AddComponent(hnd3, man.LookupCID("HotloadA"), testParam);
+		man.AddComponent(hnd4, man.LookupCID("HotloadB"), testParam);
 
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent3, IID_Test1))->GetX(), 1000);
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent4, IID_Test1))->GetX(), 200);
@@ -559,19 +594,22 @@ public:
 	void test_serialization()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 
 		entity_id_t ent1 = 1, ent2 = 2, ent3 = FIRST_LOCAL_ENTITY;
+		CEntityHandle hnd1 = man.AllocateEntityHandle(ent1);
+		CEntityHandle hnd2 = man.AllocateEntityHandle(ent2);
+		CEntityHandle hnd3 = man.AllocateEntityHandle(ent3);
 		CParamNode noParam;
 
 		CParamNode testParam;
 		TS_ASSERT_EQUALS(CParamNode::LoadXMLString(testParam, "<x>1234</x>"), PSRETURN_OK);
 
-		man.AddComponent(ent1, CID_Test1A, noParam);
-		man.AddComponent(ent1, CID_Test2A, noParam);
-		man.AddComponent(ent2, CID_Test1A, testParam);
-		man.AddComponent(ent3, CID_Test2A, noParam);
+		man.AddComponent(hnd1, CID_Test1A, noParam);
+		man.AddComponent(hnd1, CID_Test2A, noParam);
+		man.AddComponent(hnd2, CID_Test1A, testParam);
+		man.AddComponent(hnd3, CID_Test2A, noParam);
 
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent1, IID_Test1))->GetX(), 11000);
 		TS_ASSERT_EQUALS(static_cast<ICmpTest2*> (man.QueryInterface(ent1, IID_Test2))->GetX(), 21000);
@@ -626,7 +664,7 @@ public:
 		);
 
 		CSimContext context2;
-		CComponentManager man2(context2);
+		CComponentManager man2(context2, g_ScriptRuntime);
 		man2.LoadComponentTypes();
 
 		TS_ASSERT(man2.QueryInterface(ent1, IID_Test1) == NULL);
@@ -645,21 +683,31 @@ public:
 	void test_script_serialization()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+
+		CComponentManager man(context, g_ScriptRuntime);
 		ScriptTestSetup(man.m_ScriptInterface);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-serialize.js"));
 
 		entity_id_t ent1 = 1, ent2 = 2, ent3 = 3, ent4 = 4;
+		CEntityHandle hnd1 = man.AllocateEntityHandle(ent1);
+		CEntityHandle hnd2 = man.AllocateEntityHandle(ent2);
+		CEntityHandle hnd3 = man.AllocateEntityHandle(ent3);
+		CEntityHandle hnd4 = man.AllocateEntityHandle(ent4);
 		CParamNode noParam;
 
 		CParamNode testParam;
 		TS_ASSERT_EQUALS(CParamNode::LoadXMLString(testParam, "<x>1234</x>"), PSRETURN_OK);
 
-		man.AddComponent(ent1, man.LookupCID("TestScript1_values"), testParam);
-		man.AddComponent(ent2, man.LookupCID("TestScript1_entity"), testParam);
-		man.AddComponent(ent3, man.LookupCID("TestScript1_nontree"), testParam);
-		man.AddComponent(ent4, man.LookupCID("TestScript1_custom"), testParam);
+		man.AddComponent(hnd1, man.LookupCID("TestScript1_values"), testParam);
+		man.AddComponent(hnd2, man.LookupCID("TestScript1_entity"), testParam);
+
+		// TODO: Since the upgrade to SpiderMonkey v24 this test won't be able to correctly represent
+		// non-tree structures because sharp variables were removed (bug 566700).
+		// This also affects the debug serializer and it could make sense to implement correct serialization again.
+		man.AddComponent(hnd3, man.LookupCID("TestScript1_nontree"), testParam);
+
+		man.AddComponent(hnd4, man.LookupCID("TestScript1_custom"), testParam);
 
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent1, IID_Test1))->GetX(), 1234);
 		{
@@ -671,48 +719,48 @@ public:
 		std::stringstream debugStream;
 		TS_ASSERT(man.DumpDebugState(debugStream, true));
 		TS_ASSERT_STR_EQUALS(debugStream.str(),
-				"rng: \"78606\"\n"
-				"entities:\n"
-				"- id: 1\n"
-				"  TestScript1_values:\n"
-				"    object: {\n"
-				"  \"x\": 1234,\n"
-				"  \"str\": \"this is a string\",\n"
-				"  \"things\": {\n"
-				"    \"a\": 1,\n"
-				"    \"b\": \"2\",\n"
-				"    \"c\": [\n"
-				"      3,\n"
-				"      \"4\",\n"
-				"      [\n"
-				"        5,\n"
-				"        []\n"
-				"      ]\n"
-				"    ]\n"
-				"  }\n"
-				"}\n"
-				"\n"
-				"- id: 2\n"
-				"  TestScript1_entity:\n"
-				"    object: {}\n"
-				"\n"
-				"- id: 3\n"
-				"  TestScript1_nontree:\n"
-				"    object: ({x:#2=[#1=[2], #1#, #2#, {y:#1#}]})\n"
-				"\n"
-				"- id: 4\n"
-				"  TestScript1_custom:\n"
-				"    object: {\n"
-				"  \"c\": 1\n"
-				"}\n"
-				"\n"
+				"rng: \"78606\"\n\
+entities:\n\
+- id: 1\n\
+  TestScript1_values:\n\
+    object: {\n\
+  \"x\": 1234,\n\
+  \"str\": \"this is a string\",\n\
+  \"things\": {\n\
+    \"a\": 1,\n\
+    \"b\": \"2\",\n\
+    \"c\": [\n\
+      3,\n\
+      \"4\",\n\
+      [\n\
+        5,\n\
+        []\n\
+      ]\n\
+    ]\n\
+  }\n\
+}\n\
+\n\
+- id: 2\n\
+  TestScript1_entity:\n\
+    object: {}\n\
+\n\
+- id: 3\n\
+  TestScript1_nontree:\n\
+    object: ({x:[[2], [2], [], {y:[2]}]})\n\
+\n\
+- id: 4\n\
+  TestScript1_custom:\n\
+    object: {\n\
+  \"c\": 1\n\
+}\n\
+\n"
 		);
 
 		std::stringstream stateStream;
 		TS_ASSERT(man.SerializeState(stateStream));
 
 		CSimContext context2;
-		CComponentManager man2(context2);
+		CComponentManager man2(context2, g_ScriptRuntime);
 		man2.LoadComponentTypes();
 		TS_ASSERT(man2.LoadScript(L"simulation/components/test-serialize.js"));
 
@@ -729,14 +777,15 @@ public:
 	void test_script_serialization_errors()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-serialize.js"));
 
 		entity_id_t ent1 = 1;
+		CEntityHandle hnd1 = man.AllocateEntityHandle(ent1);
 		CParamNode noParam;
 
-		man.AddComponent(ent1, man.LookupCID("TestScript1_getter"), noParam);
+		man.AddComponent(hnd1, man.LookupCID("TestScript1_getter"), noParam);
 
 		std::stringstream stateStream;
 		TS_ASSERT(man.SerializeState(stateStream));
@@ -746,21 +795,24 @@ public:
 	void test_script_serialization_template()
 	{
 		CSimContext context;
-		CComponentManager man(context);
+
+		CComponentManager man(context, g_ScriptRuntime);
 		man.LoadComponentTypes();
 		TS_ASSERT(man.LoadScript(L"simulation/components/test-serialize.js"));
+		man.InitSystemEntity();
 
 		entity_id_t ent2 = 2;
+		CEntityHandle hnd2 = man.AllocateEntityHandle(ent2);
 		CParamNode noParam;
 
 		// The template manager takes care of reloading templates on deserialization,
 		// so we need to use it here
-		TS_ASSERT(man.AddComponent(SYSTEM_ENTITY, CID_TemplateManager, noParam));
+		TS_ASSERT(man.AddComponent(man.GetSystemEntity(), CID_TemplateManager, noParam));
 		ICmpTemplateManager* tempMan = static_cast<ICmpTemplateManager*> (man.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager));
 
 		const CParamNode* testParam = tempMan->LoadTemplate(ent2, "template-serialize", -1);
 
-		man.AddComponent(ent2, man.LookupCID("TestScript1_consts"), testParam->GetChild("TestScript1_consts"));
+		man.AddComponent(hnd2, man.LookupCID("TestScript1_consts"), testParam->GetChild("TestScript1_consts"));
 
 		TS_ASSERT_EQUALS(static_cast<ICmpTest1*> (man.QueryInterface(ent2, IID_Test1))->GetX(), 12347);
 
@@ -768,7 +820,7 @@ public:
 		TS_ASSERT(man.SerializeState(stateStream));
 
 		CSimContext context2;
-		CComponentManager man2(context2);
+		CComponentManager man2(context2, g_ScriptRuntime);
 		man2.LoadComponentTypes();
 		TS_ASSERT(man2.LoadScript(L"simulation/components/test-serialize.js"));
 
