@@ -25,6 +25,7 @@
 
 #include "graphics/MapReader.h"
 #include "graphics/Terrain.h"
+#include "graphics/TerrainTextureManager.h"
 #include "lib/timer.h"
 #include "ps/CLogger.h"
 #include "ps/Filesystem.h"
@@ -73,7 +74,7 @@ public:
 
 	void test_Debug_basic()
 	{
-		ScriptInterface script("Test", "Test", ScriptInterface::CreateRuntime());
+		ScriptInterface script("Test", "Test", g_ScriptRuntime);
 		std::stringstream stream;
 		CDebugSerializer serialize(script, stream);
 		serialize.NumberI32_Unbounded("x", -123);
@@ -84,7 +85,7 @@ public:
 
 	void test_Debug_floats()
 	{
-		ScriptInterface script("Test", "Test", ScriptInterface::CreateRuntime());
+		ScriptInterface script("Test", "Test", g_ScriptRuntime);
 		std::stringstream stream;
 		CDebugSerializer serialize(script, stream);
 		serialize.NumberFloat_Unbounded("x", 1e4f);
@@ -115,7 +116,7 @@ public:
 
 	void test_Debug_types()
 	{
-		ScriptInterface script("Test", "Test", ScriptInterface::CreateRuntime());
+		ScriptInterface script("Test", "Test", g_ScriptRuntime);
 		std::stringstream stream;
 		CDebugSerializer serialize(script, stream);
 
@@ -146,7 +147,7 @@ public:
 
 	void test_Std_basic()
 	{
-		ScriptInterface script("Test", "Test", ScriptInterface::CreateRuntime());
+		ScriptInterface script("Test", "Test", g_ScriptRuntime);
 		std::stringstream stream;
 		CStdSerializer serialize(script, stream);
 
@@ -172,7 +173,7 @@ public:
 
 	void test_Std_types()
 	{
-		ScriptInterface script("Test", "Test", ScriptInterface::CreateRuntime());
+		ScriptInterface script("Test", "Test", g_ScriptRuntime);
 		std::stringstream stream;
 		CStdSerializer serialize(script, stream);
 
@@ -238,7 +239,7 @@ public:
 
 	void test_Hash_basic()
 	{
-		ScriptInterface script("Test", "Test", ScriptInterface::CreateRuntime());
+		ScriptInterface script("Test", "Test", g_ScriptRuntime);
 		CHashSerializer serialize(script);
 
 		serialize.NumberI32_Unbounded("x", -123);
@@ -252,7 +253,7 @@ public:
 
 	void test_bounds()
 	{
-		ScriptInterface script("Test", "Test", ScriptInterface::CreateRuntime());
+		ScriptInterface script("Test", "Test", g_ScriptRuntime);
 		std::stringstream stream;
 		CDebugSerializer serialize(script, stream);
 		serialize.NumberI32("x", 16, -16, 16);
@@ -265,7 +266,7 @@ public:
 
 	void test_script_basic()
 	{
-		ScriptInterface script("Test", "Test", ScriptInterface::CreateRuntime());
+		ScriptInterface script("Test", "Test", g_ScriptRuntime);
 		CScriptVal obj;
 		TS_ASSERT(script.Eval("({'x': 123, 'y': [1, 1.5, '2', 'test', undefined, null, true, false]})", obj));
 
@@ -338,7 +339,7 @@ public:
 
 	void helper_script_roundtrip(const char* msg, const char* input, const char* expected, size_t expstreamlen = 0, const char* expstream = NULL)
 	{
-		ScriptInterface script("Test", "Test", ScriptInterface::CreateRuntime());
+		ScriptInterface script("Test", "Test", g_ScriptRuntime);
 		CScriptVal obj;
 		TSM_ASSERT(msg, script.Eval(input, obj));
 
@@ -400,7 +401,7 @@ public:
 	{
 		helper_script_roundtrip("Int8Array",
 			"var arr=new Int8Array(8);"
-			"for(i=0; i<arr.length; ++i)"
+			"for(var i=0; i<arr.length; ++i)"
 			"  arr[i]=(i+1)*32;"
 			"arr",
 		/* expected: */
@@ -409,7 +410,7 @@ public:
 
 		helper_script_roundtrip("Uint8Array",
 			"var arr=new Uint8Array(8);"
-			"for(i=0; i<arr.length; ++i)"
+			"for(var i=0; i<arr.length; ++i)"
 			"  arr[i]=(i+1)*32;"
 			"arr",
 		/* expected: */
@@ -418,7 +419,7 @@ public:
 
 		helper_script_roundtrip("Int16Array",
 			"var arr=new Int16Array(8);"
-			"for(i=0; i<arr.length; ++i)"
+			"for(var i=0; i<arr.length; ++i)"
 			"  arr[i]=(i+1)*8192;"
 			"arr",
 		/* expected: */
@@ -427,7 +428,7 @@ public:
 
 		helper_script_roundtrip("Uint16Array",
 			"var arr=new Uint16Array(8);"
-			"for(i=0; i<arr.length; ++i)"
+			"for(var i=0; i<arr.length; ++i)"
 			"  arr[i]=(i+1)*8192;"
 			"arr",
 		/* expected: */
@@ -436,7 +437,7 @@ public:
 
 		helper_script_roundtrip("Int32Array",
 			"var arr=new Int32Array(8);"
-			"for(i=0; i<arr.length; ++i)"
+			"for(var i=0; i<arr.length; ++i)"
 			"  arr[i]=(i+1)*536870912;"
 			"arr",
 		/* expected: */
@@ -445,7 +446,7 @@ public:
 
 		helper_script_roundtrip("Uint32Array",
 			"var arr=new Uint32Array(8);"
-			"for(i=0; i<arr.length; ++i)"
+			"for(var i=0; i<arr.length; ++i)"
 			"  arr[i]=(i+1)*536870912;"
 			"arr", 
 		/* expected: */
@@ -472,7 +473,7 @@ public:
 
 		helper_script_roundtrip("Uint8ClampedArray",
 			"var arr=new Uint8ClampedArray(8);"
-			"for(i=0; i<arr.length; ++i)"
+			"for(var i=0; i<arr.length; ++i)"
 			"  arr[i]=(i-2)*64;"
 			"arr",
 		/* expected: */
@@ -485,7 +486,7 @@ public:
 		helper_script_roundtrip("ArrayBuffer with Int16Array",
 			"var buf=new ArrayBuffer(16);"
 			"var int16=Int16Array(buf);"
-			"for(i=0; i<int16.length; ++i)"
+			"for(var i=0; i<int16.length; ++i)"
 			"  int16[i]=(i+1)*8192;"
 			"int16",
 		/* expected: */
@@ -495,7 +496,7 @@ public:
 		helper_script_roundtrip("ArrayBuffer with Int16Array and Uint32Array",
 			"var buf = new ArrayBuffer(16);"
 			"var int16 = Int16Array(buf);"
-			"for(i=0; i < int16.length; ++i)"
+			"for(var i=0; i < int16.length; ++i)"
 			"  int16[i] = (i+1)*32768;"
 			"var uint32 = new Uint32Array(buf);"
 			"uint32[0] = 4294967295;"
@@ -511,11 +512,11 @@ public:
 			"var chunk1=Int8Array(buf, 0, 4);" // 4 bytes
 			"var chunk2=Uint16Array(buf, 4, 2);" // 4 bytes
 			"var chunk3=Int32Array(buf, 8, 2);" // 8 bytes
-			"for(i=0; i<chunk1.length; ++i)"
+			"for(var i=0; i<chunk1.length; ++i)"
 			"  chunk1[i]=255;"
-			"for(i=0; i<chunk2.length; ++i)"
+			"for(var i=0; i<chunk2.length; ++i)"
 			"  chunk2[i]=65535;"
-			"for(i=0; i<chunk3.length; ++i)"
+			"for(var i=0; i<chunk3.length; ++i)"
 			"  chunk3[i]=4294967295;"
 			"var bytes = Uint8Array(buf);"
 			"({'struct':[chunk1, chunk2, chunk3], 'bytes':bytes})",
@@ -565,7 +566,7 @@ public:
 
 	void test_script_exceptions()
 	{
-		ScriptInterface script("Test", "Test", ScriptInterface::CreateRuntime());
+		ScriptInterface script("Test", "Test", g_ScriptRuntime);
 		CScriptVal obj;
 
 		std::stringstream stream;
@@ -573,9 +574,6 @@ public:
 
 		TestLogger logger;
 
-		TS_ASSERT(script.Eval("({x:1, y:<x/>})", obj));
-		TS_ASSERT_THROWS(serialize.ScriptVal("script", obj), PSERROR_Serialize_InvalidScriptValue);
-
 		TS_ASSERT(script.Eval("([1, 2, function () { }])", obj));
 		TS_ASSERT_THROWS(serialize.ScriptVal("script", obj), PSERROR_Serialize_InvalidScriptValue);
 	}
@@ -600,7 +598,7 @@ public:
 	{
 		const char* input = "var x = {}; for (var i=0;i<256;++i) x[i]=Math.pow(i, 2); x";
 
-		ScriptInterface script("Test", "Test", ScriptInterface::CreateRuntime());
+		ScriptInterface script("Test", "Test", g_ScriptRuntime);
 		CScriptVal obj;
 		TS_ASSERT(script.Eval(input, obj));
 
@@ -635,16 +633,21 @@ public:
 		TS_ASSERT_OK(g_VFS->Mount(L"", DataDir()/"mods"/"public", VFS_MOUNT_MUST_EXIST));
 		TS_ASSERT_OK(g_VFS->Mount(L"cache/", DataDir()/"cache"));
 
+		// Need some stuff for terrain movement costs:
+		// (TODO: this ought to be independent of any graphics code)
+		new CTerrainTextureManager;
+		g_TexMan.LoadTerrainTextures();
+
 		CTerrain terrain;
 
-		CSimulation2 sim2(NULL, &terrain);
+		CSimulation2 sim2(NULL, g_ScriptRuntime, &terrain);
 		sim2.LoadDefaultScripts();
 		sim2.ResetState();
 
 		CMapReader* mapReader = new CMapReader(); // it'll call "delete this" itself
 
 		LDR_BeginRegistering();
-		mapReader->LoadMap(L"maps/scenarios/Latium.pmp", &terrain, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+		mapReader->LoadMap(L"maps/scenarios/Acropolis 01.pmp", CScriptValRooted(), &terrain, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
 			&sim2, &sim2.GetSimContext(), -1, false);
 		LDR_EndRegistering();
 		TS_ASSERT_OK(LDR_NonprogressiveLoad());
@@ -677,6 +680,7 @@ public:
 		debug_printf(L"# time = %f (%f/%d)\n", t/reps, t, (int)reps);
 
 		// Shut down the world
+		delete &g_TexMan;
 		g_VFS.reset();
 		CXeromyces::Terminate();
 	}
