@@ -1,4 +1,4 @@
-	/* Copyright (C) 2013 Wildfire Games.
+	/* Copyright (C) 2014 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -20,6 +20,8 @@
 #include "ISoundManager.h"
 #include "SoundManager.h"
 
+#include "ps/Filesystem.h"
+
 #include "soundmanager/data/SoundData.h"
 #include "soundmanager/items/CSoundItem.h"
 #include "soundmanager/items/CBufferItem.h"
@@ -208,7 +210,8 @@ private:
 
 void ISoundManager::CreateSoundManager()
 {
-	g_SoundManager = new CSoundManager();
+	if ( !g_SoundManager )
+		g_SoundManager = new CSoundManager();
 }
 
 void ISoundManager::SetEnabled(bool doEnable)
@@ -217,7 +220,7 @@ void ISoundManager::SetEnabled(bool doEnable)
 	{
 		SAFE_DELETE(g_SoundManager);
 	}
-	else if ( ! g_SoundManager && doEnable ) 
+	else if ( !g_SoundManager && doEnable ) 
 	{
 		ISoundManager::CreateSoundManager();
 	}
@@ -240,6 +243,18 @@ void CSoundManager::al_check(const char* caller, int line)
 		al_ReportError(err, caller, line);
 }
 
+Status CSoundManager::ReloadChangedFiles(const VfsPath& UNUSED(path))
+{
+//	LOGERROR(L"GUI file '%ls' changed - reloading page", path.string().c_str());
+
+	return INFO::OK;
+}
+
+/*static*/ Status CSoundManager::ReloadChangedFileCB(void* param, const VfsPath& path)
+{
+	return static_cast<CSoundManager*>(param)->ReloadChangedFiles(path);
+}
+
 CSoundManager::CSoundManager()
 {
 	m_CurrentEnvirons	= 0;
@@ -282,10 +297,14 @@ CSoundManager::CSoundManager()
 		m_Worker = new CSoundManagerWorker();
 		m_Worker->SetEnabled( true );
 	}
+
+	RegisterFileReloadFunc(ReloadChangedFileCB, this);
 }
 
 CSoundManager::~CSoundManager()
 {
+	UnregisterFileReloadFunc(ReloadChangedFileCB, this);
+
 	if (m_Worker )
 	{
 		AL_CHECK
@@ -365,7 +384,7 @@ Status CSoundManager::AlcInit()
 		debug_printf(L"Sound: AlcInit success, using %hs\n", dev_name);
 	else
 	{
-		LOGERROR(L"Sound: AlcInit failed, m_Device=%p m_Context=%p dev_name=%hs err=%d\n", m_Device, m_Context, dev_name, err);
+		LOGERROR(L"Sound: AlcInit failed, m_Device=%p m_Context=%p dev_name=%hs err=%x\n", m_Device, m_Context, dev_name, err);
 
 
 
@@ -453,41 +472,43 @@ long CSoundManager::GetBufferSize()
 	return m_BufferSize;
 }
 
-void CSoundManager::AddPlayListItem( const VfsPath* itemPath)
+void CSoundManager::AddPlayListItem(const VfsPath& itemPath)
 {
-  m_PlayListItems->push_back( *itemPath );
+	if (m_Enabled)
+		m_PlayListItems->push_back(itemPath);
 }
 
 void CSoundManager::ClearPlayListItems()
 {
-  if ( m_PlayingPlaylist )
-    SetMusicItem( NULL );
+	if (m_Enabled)
+	{
+		if ( m_PlayingPlaylist )
+			SetMusicItem( NULL );
 
-  m_PlayingPlaylist = false;
-  m_LoopingPlaylist = false;
-  m_RunningPlaylist = false;
+		m_PlayingPlaylist = false;
+		m_LoopingPlaylist = false;
+		m_RunningPlaylist = false;
 
-  m_PlayListItems->clear();
+		m_PlayListItems->clear();
+	}
 }
 
 void CSoundManager::StartPlayList( bool doLoop )
 {
-	if ( m_MusicEnabled )
+	if (m_Enabled && m_MusicEnabled)
 	{
-	  if ( m_PlayListItems->size() > 0 )
-	  {
-	    m_PlayingPlaylist = true;
-	    m_LoopingPlaylist = doLoop;
-	    m_RunningPlaylist = false;
+		if (m_PlayListItems->size() > 0)
+		{
+			m_PlayingPlaylist = true;
+			m_LoopingPlaylist = doLoop;
+			m_RunningPlaylist = false;
 	    
-	    ISoundItem* aSnd = LoadItem( (m_PlayListItems->at( 0 )) );
-	    if ( aSnd )
-	      SetMusicItem( aSnd );
-	    else
-	    {
-	      SetMusicItem( NULL );
-	    }
-	  }
+			ISoundItem* aSnd = LoadItem( (m_PlayListItems->at( 0 )) );
+			if (aSnd)
+				SetMusicItem(aSnd);
+			else
+				SetMusicItem(NULL);
+		}
 	}
 }
 
@@ -685,36 +706,45 @@ void CSoundManager::PlayAsGroup(const VfsPath& groupPath, CVector3D sourcePos, e
 
 void CSoundManager::PlayAsMusic( const VfsPath& itemPath, bool looping )
 {
-		UNUSED2( looping );
+	if (m_Enabled)
+	{
+		UNUSED2(looping);
 
-    ISoundItem* aSnd = LoadItem(itemPath);
-    if (aSnd != NULL)
-      SetMusicItem( aSnd );
+		ISoundItem* aSnd = LoadItem(itemPath);
+		if (aSnd != NULL)
+			SetMusicItem(aSnd);
+	}
 }
 
 void CSoundManager::PlayAsAmbient( const VfsPath& itemPath, bool looping )
 {
+	if (m_Enabled)
+	{
 		UNUSED2( looping );
-    ISoundItem* aSnd = LoadItem(itemPath);
-    if (aSnd != NULL)
-      SetAmbientItem( aSnd );
+		ISoundItem* aSnd = LoadItem(itemPath);
+		if (aSnd != NULL)
+			SetAmbientItem( aSnd );
+	}
 }
 
 
 void CSoundManager::PlayAsUI(const VfsPath& itemPath, bool looping)
 {
-	IdleTask();
-	
-  if ( ISoundItem* anItem = LoadItem(itemPath) )
+	if (m_Enabled)
 	{
-		if (m_Enabled && (m_UIGain > 0))
+		IdleTask();
+	
+		if (ISoundItem* anItem = LoadItem(itemPath))
 		{
-			anItem->SetGain(m_UIGain);
-			anItem->SetLooping( looping );
-			anItem->PlayAndDelete();
+			if (m_UIGain > 0)
+			{
+				anItem->SetGain(m_UIGain);
+				anItem->SetLooping( looping );
+				anItem->PlayAndDelete();
+			}
 		}
+		AL_CHECK
 	}
-	AL_CHECK
 }
 
 
@@ -757,60 +787,67 @@ void CSoundManager::PauseAction (bool pauseIt)
 
 void CSoundManager::SetMusicItem(ISoundItem* anItem)
 {
-	AL_CHECK
-	if (m_CurrentTune)
+	if (m_Enabled)
 	{
-		m_CurrentTune->FadeAndDelete(2.00);
-		m_CurrentTune = 0L;
-	}
-
-	IdleTask();
-
-	if (anItem)
-	{
-		if (m_MusicEnabled && m_Enabled)
+		AL_CHECK
+		if (m_CurrentTune)
 		{
-			m_CurrentTune = anItem;
-			m_CurrentTune->SetGain(0);
+			m_CurrentTune->FadeAndDelete(2.00);
+			m_CurrentTune = 0L;
+		}
 
-			if ( m_PlayingPlaylist )
+		IdleTask();
+
+		if (anItem)
+		{
+			if (m_MusicEnabled)
 			{
-				m_RunningPlaylist = true;
-				m_CurrentTune->Play();
+				m_CurrentTune = anItem;
+				m_CurrentTune->SetGain(0);
+
+				if (m_PlayingPlaylist)
+				{
+					m_RunningPlaylist = true;
+					m_CurrentTune->Play();
+				}
+				else
+					m_CurrentTune->PlayLoop();
+
+				m_MusicPaused = false;
+				m_CurrentTune->FadeToIn( m_MusicGain, 1.00);
 			}
 			else
-				m_CurrentTune->PlayLoop();
-
-			m_CurrentTune->FadeToIn( m_MusicGain, 1.00);
-		}
-		else
-		{
-			anItem->StopAndDelete();
+			{
+				anItem->StopAndDelete();
+			}
 		}
+		AL_CHECK
 	}
-	AL_CHECK
 }
 
 void CSoundManager::SetAmbientItem(ISoundItem* anItem)
 {
-	if (m_CurrentEnvirons)
+	if (m_Enabled)
 	{
-		m_CurrentEnvirons->FadeAndDelete(3.00);
-		m_CurrentEnvirons = 0L;
-	}
-	IdleTask();
-	
-	if (anItem)
-	{
-		if (m_Enabled && (m_AmbientGain > 0))
+		if (m_CurrentEnvirons)
 		{
-			m_CurrentEnvirons = anItem;
-			m_CurrentEnvirons->SetGain(0);
-			m_CurrentEnvirons->PlayLoop();
-			m_CurrentEnvirons->FadeToIn( m_AmbientGain, 2.00);
+			m_CurrentEnvirons->FadeAndDelete(3.00);
+			m_CurrentEnvirons = 0L;
 		}
+		IdleTask();
+	
+		if (anItem)
+		{
+			if (m_AmbientGain > 0)
+			{
+				m_CurrentEnvirons = anItem;
+				m_CurrentEnvirons->SetGain(0);
+				m_CurrentEnvirons->PlayLoop();
+				m_CurrentEnvirons->FadeToIn( m_AmbientGain, 2.00);
+			}
+		}
+		AL_CHECK
 	}
-	AL_CHECK
 }
 #else // CONFIG2_AUDIO
 
