@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Wildfire Games.
+/* Copyright (C) 2013 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -31,71 +31,105 @@ namespace JSI_Sound
 {
   #if CONFIG2_AUDIO
 
-  void StartMusic(void* UNUSED(cbdata))
+  void StartMusic(ScriptInterface::CxPrivate* UNUSED(pCxPrivate))
   {
     if ( CSoundManager* sndManager = (CSoundManager*)g_SoundManager )
       sndManager->SetMusicEnabled(true);
   }
 
-  void StopMusic(void* UNUSED(cbdata))
+  void StopMusic(ScriptInterface::CxPrivate* UNUSED(pCxPrivate))
   {
     if ( CSoundManager* sndManager = (CSoundManager*)g_SoundManager )
       sndManager->SetMusicEnabled(false);
   }
 
-  void ClearPlaylist(void* UNUSED(cbdata))
+  void ClearPlaylist(ScriptInterface::CxPrivate* UNUSED(pCxPrivate))
   {
     if ( CSoundManager* sndManager = (CSoundManager*)g_SoundManager )
       sndManager->ClearPlayListItems();
   }
 
-  void AddPlaylistItem(void* UNUSED(cbdata), std::wstring filename)
+  void AddPlaylistItem(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), std::wstring filename)
   {  
     if ( CSoundManager* sndManager = (CSoundManager*)g_SoundManager )
-      sndManager->AddPlayListItem( new VfsPath( filename ) );
+      sndManager->AddPlayListItem(VfsPath(filename));
   }
 
-  void StartPlaylist(void* UNUSED(cbdata), bool looping)
+  void StartPlaylist(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), bool looping)
   {
     if ( CSoundManager* sndManager = (CSoundManager*)g_SoundManager )
       sndManager->StartPlayList( looping );
   }
 
-  void PlayMusic(void* UNUSED(cbdata), std::wstring filename, bool looping)
+  void PlayMusic(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), std::wstring filename, bool looping)
   {
     if ( CSoundManager* sndManager = (CSoundManager*)g_SoundManager )
       sndManager->PlayAsMusic( filename, looping);
   }
 
-  void PlayUISound(void* UNUSED(cbdata), std::wstring filename, bool looping)
+  void PlayUISound(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), std::wstring filename, bool looping)
   {
     if ( CSoundManager* sndManager = (CSoundManager*)g_SoundManager )
       sndManager->PlayAsUI( filename, looping);
   }
 
-  void PlayAmbientSound(void* UNUSED(cbdata), std::wstring filename, bool looping)
+  void PlayAmbientSound(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), std::wstring filename, bool looping)
   {
     if ( CSoundManager* sndManager = (CSoundManager*)g_SoundManager )
       sndManager->PlayAsAmbient( filename, looping);
   }
 
-  bool MusicPlaying(void* UNUSED(cbdata))
+  bool MusicPlaying(ScriptInterface::CxPrivate* UNUSED(pCxPrivate))
   {
     return true;
   }
 
+  void SetMasterGain(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), float gain)
+  {
+    if ( CSoundManager* sndManager = (CSoundManager*)g_SoundManager )
+      sndManager->SetMasterGain(gain);
+  }
+
+  void SetMusicGain(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), float gain)
+  {
+    if ( CSoundManager* sndManager = (CSoundManager*)g_SoundManager )
+      sndManager->SetMusicGain(gain);
+  }
+
+  void SetAmbientGain(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), float gain)
+  {
+    if ( CSoundManager* sndManager = (CSoundManager*)g_SoundManager )
+      sndManager->SetAmbientGain(gain);
+  }
+
+  void SetActionGain(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), float gain)
+  {
+    if ( CSoundManager* sndManager = (CSoundManager*)g_SoundManager )
+      sndManager->SetActionGain(gain);
+  }
+
+  void SetUIGain(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), float gain)
+  {
+    if ( CSoundManager* sndManager = (CSoundManager*)g_SoundManager )
+      sndManager->SetUIGain(gain);
+  }
 
 
   #else
-    bool MusicPlaying(void* UNUSED(cbdata) ){ return false; }
-    void PlayAmbientSound(void* UNUSED(cbdata), std::wstring UNUSED(filename), bool UNUSED(looping) ){}
-    void PlayUISound(void* UNUSED(cbdata), std::wstring UNUSED(filename), bool UNUSED(looping) ) {}
-    void PlayMusic(void* UNUSED(cbdata), std::wstring UNUSED(filename), bool UNUSED(looping) ) {}
-    void StartPlaylist(void* UNUSED(cbdata), bool UNUSED(looping) ){}
-    void AddPlaylistItem(void* UNUSED(cbdata), std::wstring UNUSED(filename) ){}
-    void ClearPlaylist(void* UNUSED(cbdata) ){}
-    void StopMusic(void* UNUSED(cbdata) ){}
-    void StartMusic(void* UNUSED(cbdata) ){}
+	bool MusicPlaying(ScriptInterface::CxPrivate* UNUSED(pCxPrivate) ){ return false; }
+	void PlayAmbientSound(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), std::wstring UNUSED(filename), bool UNUSED(looping) ){}
+	void PlayUISound(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), std::wstring UNUSED(filename), bool UNUSED(looping) ) {}
+	void PlayMusic(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), std::wstring UNUSED(filename), bool UNUSED(looping) ) {}
+	void StartPlaylist(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), bool UNUSED(looping) ){}
+	void AddPlaylistItem(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), std::wstring UNUSED(filename) ){}
+	void ClearPlaylist(ScriptInterface::CxPrivate* UNUSED(pCxPrivate) ){}
+	void StopMusic(ScriptInterface::CxPrivate* UNUSED(pCxPrivate) ){}
+	void StartMusic(ScriptInterface::CxPrivate* UNUSED(pCxPrivate) ){}
+	void SetMasterGain(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), float gain){}
+	void SetMusicGain(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), float gain){}
+	void SetAmbientGain(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), float gain){}
+	void SetActionGain(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), float gain){}
+	void SetUIGain(ScriptInterface::CxPrivate* UNUSED(pCxPrivate), float gain){}
 
   #endif
 
@@ -111,6 +145,11 @@ namespace JSI_Sound
     scriptInterface.RegisterFunction<void, std::wstring, bool, &PlayUISound>("PlayUISound");
     scriptInterface.RegisterFunction<void, std::wstring, bool, &PlayAmbientSound>("PlayAmbientSound");
     scriptInterface.RegisterFunction<bool, &MusicPlaying>("MusicPlaying");
+    scriptInterface.RegisterFunction<void, float, &SetMasterGain>("SetMasterGain");
+    scriptInterface.RegisterFunction<void, float, &SetMusicGain>("SetMusicGain");
+    scriptInterface.RegisterFunction<void, float, &SetAmbientGain>("SetAmbientGain");
+    scriptInterface.RegisterFunction<void, float, &SetActionGain>("SetActionGain");
+    scriptInterface.RegisterFunction<void, float, &SetUIGain>("SetUIGain");
   }
 }
 
