@@ -39,7 +39,6 @@
 #include "ps/Profile.h"
 #include "ps/GameSetup/Paths.h"
 #include "renderer/Renderer.h"
-#include "scripting/ScriptingHost.h"
 
 using namespace AtlasMessage;
 
@@ -219,7 +218,7 @@ static void* RunEngine(void* data)
 
 		// Pump SDL events (e.g. hotkeys)
 		SDL_Event_ ev;
-		while (SDL_PollEvent(&ev.ev))
+		while (in_poll_event(&ev))
 			in_dispatch_event(&ev);
 
 		if (g_GUI)
@@ -329,7 +328,6 @@ bool BeginAtlas(const CmdLineArgs& args, const DllLoader& dll)
 
 	// Clean up
 	AtlasView::DestroyViews();
-	ScriptingHost::FinalShutdown();
 	AtlasMessage::g_MessagePasser = NULL;
 
 	return true;
