@@ -116,7 +116,7 @@ QUERYHANDLER(GenerateMap)
 		CScriptValRooted atts;
 		si.Eval("({})", atts);
 		si.SetProperty(atts.get(), "mapType", std::string("scenario"));
-		si.SetProperty(atts.get(), "map", std::wstring(L"_default"));
+		si.SetProperty(atts.get(), "map", std::wstring(L"maps/scenarios/_default"));
 		StartGame(atts);
 
 		msg->status = -1;
@@ -172,22 +172,21 @@ MESSAGEHANDLER(ImportHeightmap)
 
 	// decode to a raw pixel format
 	Tex tex;
-	if (tex_decode(fileData, fileSize, &tex) < 0)
+	if (tex.decode(fileData, fileSize) < 0)
 	{
 		LOGERROR(L"Failed to decode heightmap.");
 		return;
 	}
 
 	// Convert to uncompressed BGRA with no mipmaps
-	if (tex_transform_to(&tex, (tex.flags | TEX_BGR | TEX_ALPHA) & ~(TEX_DXT | TEX_MIPMAPS)) < 0)
+	if (tex.transform_to((tex.m_Flags | TEX_BGR | TEX_ALPHA) & ~(TEX_DXT | TEX_MIPMAPS)) < 0)
 	{
 		LOGERROR(L"Failed to transform heightmap.");
-		tex_free(&tex);
 		return;
 	}
 
 	// pick smallest side of texture; truncate if not divisible by PATCH_SIZE
-	ssize_t terrainSize = std::min(tex.w, tex.h);
+	ssize_t terrainSize = std::min(tex.m_Width, tex.m_Height);
 	terrainSize -= terrainSize % PATCH_SIZE;
 	
 	// resize terrain to heightmap size
@@ -198,9 +197,9 @@ MESSAGEHANDLER(ImportHeightmap)
 	u16* heightmap = g_Game->GetWorld()->GetTerrain()->GetHeightMap();
 	ssize_t hmSize = g_Game->GetWorld()->GetTerrain()->GetVerticesPerSide();
 	
-	u8* mapdata = tex_get_data(&tex);
-	ssize_t bytesPP = tex.bpp / 8;
-	ssize_t mapLineSkip = tex.w * bytesPP;
+	u8* mapdata = tex.get_data();
+	ssize_t bytesPP = tex.m_Bpp / 8;
+	ssize_t mapLineSkip = tex.m_Width * bytesPP;
 	
 	for (ssize_t y = 0; y < terrainSize; ++y)
 	{
@@ -215,8 +214,6 @@ MESSAGEHANDLER(ImportHeightmap)
 		}
 	}
 	
-	tex_free(&tex);
-	
 	// update simulation
 	CmpPtr<ICmpTerrain> cmpTerrain(*g_Game->GetSimulation2(), SYSTEM_ENTITY);
 	if (cmpTerrain) cmpTerrain->ReloadTerrain();
@@ -226,7 +223,7 @@ MESSAGEHANDLER(ImportHeightmap)
 MESSAGEHANDLER(SaveMap)
 {
 	CMapWriter writer;
-	const VfsPath pathname = VfsPath("maps/scenarios") / *msg->filename;
+	VfsPath pathname = VfsPath(*msg->filename).ChangeExtension(L".pmp");
 	writer.SaveMap(pathname,
 		g_Game->GetWorld()->GetTerrain(),
 		g_Renderer.GetWaterManager(), g_Renderer.GetSkyManager(),
@@ -352,4 +349,22 @@ QUERYHANDLER(VFSFileExists)
 	msg->exists = VfsFileExists(*msg->path);
 }
 
+static Status AddToFilenames(const VfsPath& pathname, const CFileInfo& UNUSED(fileInfo), const uintptr_t cbData)
+{
+	std::vector<std::wstring>& filenames = *(std::vector<std::wstring>*)cbData;
+	filenames.push_back(pathname.string().c_str());
+	return INFO::OK;
+}
+
+QUERYHANDLER(GetMapList)
+{
+	std::vector<std::wstring> scenarioFilenames;
+	vfs::ForEachFile(g_VFS, L"maps/scenarios/", AddToFilenames, (uintptr_t)&scenarioFilenames, L"*.xml", vfs::DIR_RECURSIVE);
+	msg->scenarioFilenames = scenarioFilenames;
+	
+	std::vector<std::wstring> skirmishFilenames;
+	vfs::ForEachFile(g_VFS, L"maps/skirmishes/", AddToFilenames, (uintptr_t)&skirmishFilenames, L"*.xml", vfs::DIR_RECURSIVE);
+	msg->skirmishFilenames = skirmishFilenames;
+}
+
 }
