@@ -159,6 +159,12 @@ MESSAGE(SaveMap,
 		((std::wstring, filename))
 		);
 
+QUERY(GetMapList,
+		,
+		((std::vector<std::wstring>, scenarioFilenames))
+		((std::vector<std::wstring>, skirmishFilenames))
+		);
+
 QUERY(GetMapSettings,
 		,
 		((std::string, settings))
@@ -416,7 +422,6 @@ MESSAGE(SetView,
 struct sEnvironmentSettings
 {
 	Shareable<float> waterheight; // range 0..1 corresponds to min..max terrain height; out-of-bounds values allowed
-	Shareable<float> watershininess; // range ???
 	Shareable<float> waterwaviness; // range ???
 	Shareable<float> watermurkiness; // range ???
 	
@@ -465,6 +470,8 @@ COMMAND(SetEnvironmentSettings, MERGE,	// merge lots of small changes into one u
 		((sEnvironmentSettings, settings))
 		);
 
+COMMAND(RecalculateWaterData, NOMERGE, ((float,unused)));
+
 QUERY(GetSkySets,
 	  // no inputs
 	  ,
