@@ -7,7 +7,7 @@ use JSON;
 
 use Entity;
 
-use constant CHECK_SCENARIOS_XML => 0;
+use constant CHECK_MAPS_XML => 0;
 use constant ROOT_ACTORS => 1;
 
 my @files;
@@ -48,6 +48,15 @@ sub find_files
     return @files;
 }
 
+sub parse_json_file
+{
+    my ($vfspath) = @_;
+    open my $fh, vfs_to_physical($vfspath) or die "Failed to open '$vfspath': $!";
+    # decode_json expects a UTF-8 string and doesn't handle BOMs, so we strip those
+    # (see http://trac.wildfiregames.com/ticket/1556)
+    return decode_json(do { local $/; my $file = <$fh>; $file =~ s/^\xEF\xBB\xBF//; $file });
+}
+
 sub add_entities
 {
     print "Loading entities...\n";
@@ -162,10 +171,11 @@ sub add_particles
     }
 }
 
-sub add_scenarios_xml
+sub add_maps_xml
 {
-    print "Loading scenario XML...\n";
+    print "Loading maps XML...\n";
     my @mapfiles = find_files('maps/scenarios', 'xml');
+	push @mapfiles, find_files('maps/skirmishes', 'xml');
     for my $f (sort @mapfiles)
     {
         print "  $f\n";
@@ -206,20 +216,26 @@ sub add_scenarios_xml
     }
 }
 
-sub add_scenarios_pmp
+sub add_maps_pmp
 {
-    print "Loading scenario PMP...\n";
+    print "Loading maps PMP...\n";
 
     # Need to generate terrain texture filename=>path lookup first
     my %terrains;
-    for my $f (find_files('art/textures/terrain/types', 'dds|png|jpg|tga'))
+    for my $f (find_files('art/terrains', 'xml'))
     {
-        $f =~ /([^\/]+)\.(dds|png|jpg|tga)/ or die;
-        warn "Duplicate terrain name '$1' (from '$terrains{$1}' and '$f')\n" if $terrains{$1};
-        $terrains{$1} = $f;
+        $f =~ /([^\/]+)\.xml/ or die;
+
+        # ignore terrains.xml
+        if ($f !~ /terrains.xml$/)
+        {
+            warn "Duplicate terrain name '$1' (from '$terrains{$1}' and '$f')\n" if $terrains{$1};
+            $terrains{$1} = $f;
+        }
     }
 
     my @mapfiles = find_files('maps/scenarios', 'pmp');
+	push @mapfiles, find_files('maps/skirmishes', 'pmp');
     for my $f (sort @mapfiles)
     {
         push @files, $f;
@@ -256,7 +272,7 @@ sub add_scenarios_pmp
             my $str;
             read $fh, $str, $len;
 
-            push @deps, [ $f, $terrains{$str} || "art/textures/terrain/types/(unknown)/$str" ];
+            push @deps, [ $f, $terrains{$str} || "art/terrains/(unknown)/$str" ];
         }
 
         # ignore patches data
@@ -369,10 +385,7 @@ sub add_civs
 
         push @roots, $f;
 
-        open my $fh, vfs_to_physical($f) or die "Failed to open '$f': $!";
-        # decode_json expects a UTF-8 string and doesn't handle BOMs, so we strip those
-        # (see http://trac.wildfiregames.com/ticket/1556)
-        my $civ = decode_json(do { local $/; my $file = <$fh>; $file =~ s/^\xEF\xBB\xBF//; $file });
+        my $civ = parse_json_file($f);
 
         push @deps, [ $f, "art/textures/ui/" . $civ->{Emblem} ];
 
@@ -393,10 +406,7 @@ sub add_rms
 
         push @roots, $f;
 
-        open my $fh, vfs_to_physical($f) or die "Failed to open '$f': $!";
-        # decode_json expects a UTF-8 string and doesn't handle BOMs, so we strip those
-        # (see http://trac.wildfiregames.com/ticket/1556)
-        my $rms = decode_json(do { local $/; my $file = <$fh>; $file =~ s/^\xEF\xBB\xBF//; $file });
+        my $rms = parse_json_file($f);
 
         push @deps, [ $f, "maps/random/" . $rms->{settings}{Script} ];
 
@@ -405,6 +415,46 @@ sub add_rms
     }
 }
 
+sub add_techs
+{
+    print "Loading techs...\n";
+
+    my @techfiles = find_files('simulation/data/technologies', 'json');
+    for my $f (sort @techfiles)
+    {
+        push @files, $f;
+        push @roots, $f;
+
+        my $tech = parse_json_file($f);
+
+        push @deps, [ $f, "art/textures/ui/session/portraits/technologies/" . $tech->{icon} ] if $tech->{icon};
+        push @deps, [ $f, "simulation/data/technologies/" . $tech->{supersedes} . ".json" ] if $tech->{supersedes};
+    }
+}
+
+sub add_terrains
+{
+    print "Loading terrains...\n";
+
+    my @terrains = find_files('art/terrains', 'xml');
+    for my $f (sort @terrains)
+    {
+        # ignore terrains.xml
+        if ($f !~ /terrains.xml$/)
+        {
+            push @files, $f;
+            
+            my $terrain = XMLin(vfs_to_physical($f), ForceArray => [qw(texture)], KeyAttr => []) or die "Failed to parse '$f': $!";
+
+            for my $texture (@{$terrain->{textures}{texture}})
+            {
+                push @deps, [ $f, "art/textures/terrain/$texture->{file}" ] if $texture->{file};
+            }
+            push @deps, [ $f, "art/materials/$terrain->{material}" ] if $terrain->{material};
+        }
+    }
+}
+
 
 sub check_deps
 {
@@ -462,9 +512,9 @@ sub check_unused
 }
 
 
-add_scenarios_xml() if CHECK_SCENARIOS_XML;
+add_maps_xml() if CHECK_MAPS_XML;
 
-add_scenarios_pmp();
+add_maps_pmp();
 
 add_entities();
 
@@ -486,6 +536,10 @@ add_civs();
 
 add_rms();
 
+add_techs();
+
+add_terrains();
+
 # TODO: add non-skin textures, and all the references to them
 
 print "\n";
